' Licensed Materials - Property of IBM
' 5724-S67
' (c) Copyright IBM Corp. 1998, 2007   All Rights Reserved.
' US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
'***********************************************************BeginFile************
' Time-stamp: 
' Name:
'   getStratusLogs.vbs
' Purpose:
'   retrieve records appended to a log since the previous run
'   and store them sequentially in a new file
' Notes:
'   getStratuslogs.vbs <outfilename> <platformname> <directory> <filename pattern>
'   <outfilename>          Name of temp chunk To be collected
'   <platformname>         unique registry key value To store state of already collected Data
'   <directory>            path of directory To collect
'   <filename pattern>     regex of files To be collected from the directory
'   <eventheader pattern>  regex of the eventheader
' History:
'   2002nov25 0.1 Martin: creation
'   2002dec03 0.2 Martin: make more intelligent, failsafe etc.
'   2002dec06 0.3 Martin: shitty bug recording lastline when EOF char is on last processed line.
'   2004oct28 0.4 VL: WE0407002: VBS actuators do not return exit codes - fixed
'***********************************************************EndFile******
Option Explicit      ' Force explicit variable declaration.
On Error Resume Next

' Variable declaration
Dim args             ' The arguments of this program
Dim iCnt             ' Counter for arguments
Dim WshShell         ' The shell to direct commands to
Dim debugLogObject
Dim debugLogFile

Dim OutFileName        ' The file containing the collected log data
Dim PltfName           ' a unique name of the platform to create the regkey to store information
Dim Directory          ' Directory which contains the log files
Dim FilenamePattern    ' Filename Pattern of logfiles
Dim EventHeaderPattern ' Event Header pattern, recognizes the first line of each event
Dim fso, infile, outfile, record, crmtime, file
Dim tracing
tracing = True

' Constant declaration
Const regKey = "HKLM\Software\CONSUL Risk Management\CEAActuator\"
Const regfirstrec        = "\firstrecord"        ' key to contain first record of last used file
Const reglastline        = "\lastline"           ' key to contain last read linenumber of last used file
Const reglastlineHeader  = "\lastlineheader"     ' key to contain the linenumber of the header of last read event
Const regLastEventLogFilename = "\lastEventLogFilename"
Const debugLog = "..\log\getstratuslogs.log"
Const DEBUGLOG_SIZE = 100000   ' set max size of debug file

Err.Clear
Set wshShell = wscript.CreateObject("WScript.Shell")

'
' debug/tracing INIT
'
If tracing Then
  Set debugLogObject = CreateObject("Scripting.FileSystemObject")

  On Error Resume Next
  ' Check the size of the debug file. If too big remove it.
  Set file = debugLogObject.GetFile(debugLog)
  If Err.number = 0 and file.Size > DEBUGLOG_SIZE Then
    file.Delete 
  End If
  
  Err.Clear

  Set debugLogFile   = debugLogObject.OpenTextFile(debugLog, 8, True)   'ForAppending
  If Err.number <> 0 Then
    wscript.echo "Not able to write to debug log file !  Disabling debugging. (errorcode=" & Err.number & ")"
    tracing = False
  Else
    wscript.echo "Opened log file " & debugLog
    DebugMessage debugLogFile, "******  Start  *******"
  End If
End If
'
' End of debug/tracing INIT
'

'
' Command Line Arguments processsing
'
Set args = wscript.Arguments
If tracing Then
  For iCnt = 0 To args.Count-1
    DebugMessage debugLogFile, "Args " & iCnt & " : " & args(iCnt)
  Next
End If
If args.Count < 5 Then
  If tracing Then
    DebugMessage debugLogFile, "Insufficient number of arguments."
  End If
  Call CleanExit(1)
End If

OutFileName = args(0)
PltfName = args(1)
Directory = args(2)
FilenamePattern = args(3)
EventHeaderPattern = args(4)
'
' End of Command Line Arguments processing
'

'
' STATE information processing
'
If tracing Then
  DebugMessage debugLogFile, "Start reading registry keys:"
End If

' get first record from previous run
Dim firstrecord      ' first record of file, to determine if this is the same file as previous run
firstrecord = wshShell.RegRead(regKey & PltfName & regfirstrec)
If Err.Number <> 0 Then
  firstrecord = "first run"
End If
If tracing Then
  DebugMessage debugLogFile, "  Firstrecord = " & firstrecord & "."
End If

' get last linenumber from previous run
Dim lastline         ' last line read
lastline = wshShell.RegRead(regKey & PltfName & reglastline)
If Err.Number <> 0 Then
  lastline = "1"
End If
If tracing Then
  DebugMessage debugLogFile, "  Lastline = " & lastline & "."
End If

' get linenumber of the header of last read event from previous run
Dim lastlineHeader         ' last header line
lastlineHeader = wshShell.RegRead(regKey & PltfName & reglastlineheader)
If Err.Number <> 0 Then
  lastlineHeader = "2"    ' first line is the fileheader line
End If
If tracing Then
  DebugMessage debugLogFile, "  LastlineHeader = " & lastlineHeader & "."
End If

' get filename of last processed eventlog
Dim lastEventLogFilename
lastEventLogFilename = wshShell.RegRead(regKey & PltfName & regLastEventLogFilename)
If Err.Number <> 0 Then
  lastEventLogFilename = ""
End If
If tracing Then
  DebugMessage debugLogFile, "  LastEventLogFilename = " & lastEventLogFilename & "."
End If

If tracing Then
  DebugMessage debugLogFile, "Read all registry keys."
End If
'
' End of STATE information processing
'


'
'============< start of MAIN >==========================================
'
Err.Clear

Dim filenameMatcher, eventHeaderMatcher
Dim folder, fc, f1count, f1, f1order, f1Idx

'
' Creation of Regular Expression Objects
'
Set filenameMatcher = CreateObject("VBScript.RegExp")
If Err.Number <> 0 Then
  DebugMessage debugLogFile, "Error creating RegExp [1]: " & Err.Number & ": " & Err.Description
  Call CleanExit(1)
End If

filenameMatcher.Pattern = FilenamePattern
filenameMatcher.IgnoreCase = True
Set eventHeaderMatcher = CreateObject("VBScript.RegExp")
If Err.Number <> 0 Then
  DebugMessage debugLogFile, "Error creating RegExp [2]: " & Err.Number & ": " & Err.Description
  Call CleanExit(1)
End If
eventHeaderMatcher.Pattern = EventHeaderPattern
eventHeaderMatcher.IgnoreCase = True

Set fso = CreateObject("Scripting.FileSystemObject")
If Err.Number <> 0 Then
  DebugMessage debugLogFile, "Error creating FileSystemObject: " & Err.Number & ": " & Err.Description
  Call CleanExit(1)
End If

Set crmtime = CreateObject("CRMTime.CRMTimeObject")
If Err.Number <> 0 Then
  DebugMessage debugLogFile, "Error creating CRMTimeObject: " & Err.Number & ": " & Err.Description
  Call CleanExit(1)
End If

' open outfile
Set outfile = fso.CreateTextFile(OutFileName, True)
If Err.Number <> 0 Then
  DebugMessage debugLogFile, "Error opening output file " & OutFileName & ": " & Err.Number & ": " & Err.Description
  Call CleanExit(1)
End If

' get the filenames to process
Set folder = fso.GetFolder(Directory)
If Err.Number <> 0 Then
  DebugMessage debugLogFile, "Error opening folder " & Directory & ": " & Err.Number & ": " & Err.Description
  Call CleanExit(1)
End If

Set fc = folder.Files
If Err.Number <> 0 Then
  DebugMessage debugLogFile, "Error getting files from " & Directory & ": " & Err.Number & ": " & Err.Description
  Call CleanExit(1)
End If

If tracing Then
  DebugMessage debugLogFile, "File(s) to be processed:"
End If

'
' Building list of files to be processed
'
ReDim fls(0)
Dim indexVal, indexStr
f1count = 0
indexVal = 0
indexStr = ""
For Each f1 in fc
  'determine list of files to be processed. Only look to filename and not to file timestamp
  if ComparefileNames (f1.Name, lastEventLogFilename) <> -1  and filenameMatcher.test(f1.Name) Then
    ReDim Preserve fls(f1count)
    Set fls(f1count) = f1
    f1count = f1count + 1
    If tracing Then
      DebugMessage debugLogFile, "  Filename = '" & f1.Name & "'."
    End If    
  End If
Next

If tracing Then
  DebugMessage debugLogFile, "All file(s) determined."
End If
'
' End of building list of files
'

'
' Processing the list of files; open file, process events & write to outputfile
'
If tracing Then
  DebugMessage debugLogFile, "Start processing all indicated file(s):"
End If

' sort the list of file in correct order
f1Order = SortLogfiles(fls)

For f1Idx = 0 To f1count-1

  ' first check if event log file can be read. If not, stop processing.
  ' open infile
  Set infile = fso.OpenTextFile(fls(f1Order(f1Idx)).Path, 1) ' ForReading
  ' catch when read of file fails !
  If Err.number <> 0 Then
    If tracing Then
      DebugMessage debugLogFile, "!!  Not able to read from event log file '" & fls(f1Order(f1Idx)).Path & "'."
      DebugMessage debugLogFile, "!!  Check file permissions of file '" & fls(f1Order(f1Idx)).Path & "'"
      DebugMessage debugLogFile, "!!  Fix problem and perform a new collect !"
      DebugMessage debugLogFile, "!!  Interrupting processing of other event log file(s) !!!"
    End If
    Exit For
  End If

  '  CollectLogfile fls(f1Order(f1Idx))
  CollectLogfile infile, fls(f1Order(f1Idx)) 

  ' record the filename of last processed file
  lastEventLogFilename = fls(f1Order(f1Idx)).Name
      
  ' remove the processed file, keep the last one
  If f1Idx < f1count - 1 Then
    If tracing Then
      DebugMessage debugLogFile, "  Deleting file: '" & fls(f1Order(f1Idx)).Path & "'."
    End If
    fls(f1Order(f1Idx)).Delete
    If Err.number <> 0 Then
      If tracing Then
        DebugMessage debugLogFile, "!!  Not able to delete processed log file '" & fls(f1Order(f1Idx)).Path & "'."
      End If
    End If
  End If 
Next
If tracing Then
  DebugMessage debugLogFile, "Finished the processing of all indicated file(s)."
End If

' close output file
outfile.Close

' store new values in registry
wshShell.RegWrite regKey & PltfName & reglastline, lastline, "REG_DWORD"
wshShell.RegWrite regKey & PltfName & regLastlineHeader, lastlineHeader, "REG_DWORD"
wshShell.RegWrite regKey & PltfName & regfirstrec, firstrecord
wshShell.RegWrite regKey & PltfName & regLastEventLogFilename, lastEventLogFilename

Call CleanExit(0)

'
'============< end of MAIN >==========================================
'


'============< subroutines >==========================================

'
' CollectLogfile(f1)
'    do the actual collection of the new records
'
Sub CollectLogfile(infile, f1)
  Dim currentInputLine, tmpline
  On Error Resume Next
  
  tmpline = 0
  ' read first record from infile
  record = infile.ReadLine
  ' compare if equal to line in registry (same file)
  ' start at first record if different
  If record <> firstrecord Then
    lastline = 1   ' first line is the file header
    lastlineHeader = 2   ' 2 or > line is the first log event header
    firstrecord = record
    outfile.WriteLine record
  End If
        
  If tracing Then
    DebugMessage debugLogFile , ("  " & f1.DateLastModified & " " & f1.Path & " lastline: " & lastline & " lastlineHeader: " & lastlineHeader)
  End If

  ' jump to the line last processed previous run
  Do While infile.Line < lastline and infile.AtEndOfStream <> True
    infile.SkipLine
  Loop

  'wscript.echo "skiplines:"
  'wscript.echo "  infile.Line = " & infile.Line
  'wscript.echo "  lastline    = " & lastline

  ' if file (last file read earlier in previous run) is expanded with new data
  '    
  If lastline > 1 and  infile.Line = lastline  and infile.AtEndOfStream <> True Then
    tmpline = infile.Line

    currentInputLine = infile.ReadLine
    ' Check if the line read is a event header line. If so, then a 
    ' complete new event record added to file
    If eventHeaderMatcher.test(currentInputLine) Then
      ' new events added to file so start output file with a FILE header line
      outfile.WriteLine firstrecord

      lastlineHeader = lastline
      'write the event header
      outfile.WriteLine currentInputLine
      ' check if Readline incremented the current line number (see remark below)
      If tmpline = infile.Line Then
        lastline = infile.Line + 1
      Else
        lastline = infile.Line
      End If
      
    ElseIf currentInputLine = "" Then   ' ignore empty lines as part of previous event
      ' check if Readline incremented the current line number (see remark below)
      If tmpline = infile.Line Then
        lastline = infile.Line + 1
      Else
        lastline = infile.Line
      End If
      
    Else
      ' new events added to file so start output file with a FILE header line
      outfile.WriteLine firstrecord

      ' apparently the last processed event of previous run was not complete
      ' So, goto lastlineheader (go back a few lines to begin of event)
      ' by closing and reopening the file.
      infile.Close
      Set infile = fso.OpenTextFile(f1.Path, 1) ' ForReading
      Do While infile.Line < lastlineHeader
        infile.SkipLine
      Loop
    End If

    'wscript.echo "if expanded with data:"
    'wscript.echo "  infile.Line = " & infile.Line
    'wscript.echo "  lastline    = " & lastline
    'wscript.echo "  tmpline     = " & tmpline    

  End If

  
  ' process file until eof
  Do While infile.AtEndOfStream <> True
    tmpline = infile.Line
    ' read next line
    currentInputLine = infile.ReadLine   'Readline will increment infile.Line with one only if EOF not on this line

    ' recognize event header lines and record the position in file
    If eventHeaderMatcher.test(currentInputLine) Then
      ' record the position of the last event header line in file
      lastlineHeader = infile.Line - 1
    End If

    ' ignore empty lines ??
    'If currentInputLine <> "" Then
    ' write to output file
    outfile.WriteLine currentInputLine
    'End If

    ' REMARK:
    ' infile.file points to the current line of stream
    ' So after a infile.Readline, infile.file points to the next line
    ' Except if the infile.Readline contains the EOF. Then infile.file stays the current line.
    '
    ' record the current line of stream (starting point of next run)
    If tmpline = infile.Line Then
      lastline = infile.Line + 1
    Else
      lastline = infile.Line
    End If
    
    'wscript.echo "process until eof:"
    'wscript.echo "  infile.Line = " & infile.Line
    'wscript.echo "  lastline    = " & lastline
    'wscript.echo "  tmpline     = " & tmpline    
    
  Loop

  infile.Close
End Sub 


'
' Comparefls(f1, f2)
'   returns:
'      1  if  f1 > f2
'     -1  if  f2 > f1
'      0  if  f1 = f2
'
Function Comparefls(f1, f2)
  Dim LastWriteTime1, LastWriteTime2
  On Error Resume Next

  Set file = fso.GetFile(f1.Path)
  LastWriteTime1 = file.DateLastModified
  Set file = fso.GetFile(f2.Path)
  LastWriteTime2 = file.DateLastModified

  If LastWriteTime1 > LastWriteTime2 Then
    Comparefls = 1
  ElseIf LastWriteTime1 < LastWriteTime2 Then
    Comparefls = -1
  ElseIf f1.Name > f2.Name Then
    Comparefls = 1
  ElseIf f1.Name < f2.Name Then
    Comparefls = -1
  Else
    Comparefls = 0
  End If   
End Function  


'
' ComparefileNames(f1, f2)
'   returns:
'      1  if  f1 > f2
'     -1  if  f2 > f1
'      0  if  f1 = f2
' 
Function ComparefileNames(f1, f2)
  Dim f1_index, f2_index
  Dim f1_date, f2_date
  On Error Resume Next

  f1_index = 0
  f2_index = 0
  f1_date = ""
  f2_date = ""

  f1_index = CInt( RegExpTest("\d+$", f1) )
  f2_index = CInt( RegExpTest("\d+$", f2) )
  f1_date = RegExpTest("\d{1,2}-\d{1,2}-\d{1,2}", f1)
  f2_date = RegExpTest("\d{1,2}-\d{1,2}-\d{1,2}", f2)
     
  If f1_date > f2_date Then
    ComparefileNames = 1
  ElseIf f1_date < f2_date Then
    ComparefileNames = -1
  ' date part are equal so check the index part of those
  ElseIf f1_index > f2_index Then
    ComparefileNames = 1
  ElseIf f1_index < f2_index Then
    ComparefileNames = -1
  Else
    ComparefileNames = 0
  End If   
End Function  


'
' SortLogfiles(Strings())
'    In:  fls:     file objects
'    Out: Long:    sorted order by LastWriteTime and name
'
Function SortLogfiles(Strings())
  Dim Count, Order1, Order2
  Dim i1, i1max, i2, i2max, i3, SortedLength
  On Error Resume Next
  Count = UBound(Strings) + 1
  On Error GoTo 0
  If Count > 0 Then
    ReDim Order1(Count - 1)
    ReDim Order2(Count - 1)
    For i1 = 0 To Count - 1      'Start order
      Order1(i1) = i1
    Next
    SortedLength = 1
    While SortedLength < Count
      i1 = 0
      i2 = i1 + SortedLength
      i3 = 0
      While i1 < Count
        i1max = i2
        i2max = i2 + SortedLength
        If i2max > Count Then
          i2max = Count
        End If   
        While i1 < i1max And i2 < i2max
'          Select Case Comparefls(fls(Order1(i1)), fls(Order1(i2)))
          Select Case ComparefileNames( fls(Order1(i1)).Name, fls(Order1(i2)).Name)
          Case 1:
            Order2(i3) = Order1(i2)
            i2 = i2 + 1
          Case Else
            Order2(i3) = Order1(i1)
            i1 = i1 + 1
          End Select
          i3 = i3 + 1
        Wend
        For i1 = i1 To i1max - 1
          Order2(i3) = Order1(i1)
          i3 = i3 + 1
        Next
        For i2 = i2 To i2max - 1
          Order2(i3) = Order1(i2)
          i3 = i3 + 1
        Next
        i1 = i1 + SortedLength
        i2 = i2 + SortedLength
        If i2 > Count Then i2 = Count
      Wend
      Order1 = Order2
      SortedLength = SortedLength * 2
    Wend
    SortLogfiles = Order1
  End If
End Function

'
' DebugMessage()
'
Sub DebugMessage(outputFile, text)
  On Error Resume Next
  wscript.echo "Message: " & text
  outputFile.WriteLine Now & " " & text
End Sub



Function RegExpTest(patrn, strng)
   Dim regEx, Match, Matches, RetVal   ' Create variable.
   Set regEx = New RegExp   ' Create a regular expression.
   regEx.Pattern = patrn   ' Set pattern.
   regEx.IgnoreCase = True   ' Set case insensitivity.
   regEx.Global = True   ' Set global applicability.
   Set Matches = regEx.Execute(strng)   ' Execute search.
   For Each Match in Matches   ' Iterate Matches collection.
      RetVal = Match.Value
   Next
   RegExpTest = RetVal
End Function

'
' CleanExit(error_code)
'
Sub CleanExit(error_code)
   On Error Resume Next
   If tracing Then
      DebugMessage debugLogFile, "****** The End *******"
      debugLogFile.Close()
   End If
   WScript.Quit error_code
End Sub

