REM Licensed Materials - Property of IBM
REM 5724-S67
REM (c) Copyright IBM Corp. 1998, 2007   All Rights Reserved.
REM US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
@echo off
goto comment

    *************************************************************************
    * Actuator batch script for Blue Coat ProxySG Event Source.             *
    *                                                                       *
    * Made 12Oct2005 accordingly Blue_Coat_ProxySG_TDD_1.1.2.doc, EE05S9001 *
    *                                                                       *
    * Dzmitry Paulenka, ES developer                                        *
    *************************************************************************

:comment

rem ----- Receive parameters from the InSight actuator
set temp_file1=%1
set temp_file2=%2
set temp_file3=%3
set temp_file4=%4
set source_folder=%5
set props_file=%6

if %temp_file1%=="" (
    echo ERROR: not all input parameters, enter name of 1 temporary file
    goto EOF
)
if %temp_file2%=="" (
    echo ERROR: not all input parameters, enter name of 2 temporary file
    goto EOF
)
if %temp_file3%=="" (
    echo ERROR: not all input parameters, enter name of 3 temporary file
    goto EOF
)
if %temp_file4%=="" (
    echo ERROR: not all input parameters, enter name of 4 temporary file
    goto EOF
)
if %source_folder%=="" (
    echo ERROR: not all input parameters, enter name of PoP folder
    goto EOF
)
if %props_file%=="" (
    echo ERROR: not all input parameters, enter name of property file
    goto EOF
)

rem ----- Create property file and temporary files
if not exist ..\log mkdir ..\log
type nul > ..\log\%props_file%.log
echo Date: %date% Time: %time% Start processing >> ..\log\%props_file%.log

type nul > %temp_file1%
type nul > %temp_file2%
type nul > %temp_file3%
type nul > %temp_file4%

rem ----- If %source_folder% not exists, finish work with return code 1
if not exist %source_folder% (
    echo ERROR: source folder "%source_folder%" not exist >> ..\log\%props_file%.log
    goto error_1
)

rem ----- Create 2 empty folders
if exist %props_file%.dir1 (
    rmdir /s /q %props_file%.dir1
    if exist %props_file%.dir1 (
        echo ERROR: remove of "%props_file%.dir1" folder end with error >> ..\log\%props_file%.log
        goto error_3
    )
)

if exist %props_file%.dir2 (
    rmdir /s /q %props_file%.dir2
    if exist %props_file%.dir2 (
        echo ERROR: remove of "%props_file%.dir2" folder end with error >> ..\log\%props_file%.log
        goto error_3
    )
)

mkdir %props_file%.dir1
mkdir %props_file%.dir2

rem ----- Move files which name have substring "_main_","_im_","_streaming_" and "_common_ncsa_"
rem       and extensions ".log" or ".gz" from %source_folder% to %props_file%.dir1
if exist %source_folder%\*.log (
    for /f %%a in ('dir %source_folder%\*.log /a:-d /b /o:n') do (
        for /f "tokens=1-3 delims=_" %%b in ("%%a") do (
            if "%%c"=="main"            move /y %source_folder%\%%a %props_file%.dir1
            if "%%c"=="im"              move /y %source_folder%\%%a %props_file%.dir1
            if "%%c"=="streaming"       move /y %source_folder%\%%a %props_file%.dir1
            if "%%c_%%d"=="common_ncsa" move /y %source_folder%\%%a %props_file%.dir1
            if not errorlevel 0 (
                echo Date: %date% Time: %time% File %%a is busy
            )
        )
    )
)

if exist %source_folder%\*.gz (
    for /f %%a in ('dir %source_folder%\*.gz /a:-d /b /o:n') do (
        for /f "tokens=1-3 delims=_" %%b in ("%%a") do (
            if "%%c"=="main"            move /y %source_folder%\%%a %props_file%.dir1
            if "%%c"=="im"              move /y %source_folder%\%%a %props_file%.dir1
            if "%%c"=="streaming"       move /y %source_folder%\%%a %props_file%.dir1
            if "%%c_%%d"=="common_ncsa" move /y %source_folder%\%%a %props_file%.dir1
            if not errorlevel 0 (
                echo Date: %date% Time: %time% File %%a is busy
            )
        )
    )
)

rem ----- Copy all files from "%props_file%.dir1" to "%props_file%.dir2"
if exist %props_file%.dir1\*.log copy /y /b %props_file%.dir1\*.log %props_file%.dir2
if exist %props_file%.dir1\*.gz copy /y /b %props_file%.dir1\*.gz %props_file%.dir2

rem ----- uncompress all files with ".gz" extension in "%props_file%.dir2"
if exist %props_file%.dir2\*.gz (
    for /f %%a in ('dir %props_file%.dir2\*.gz /a:-d /b /o:n') do (
        ..\bin\minigzip.exe -d %props_file%.dir2\%%a
        if not errorlevel 0 (
            move /y %props_file%.dir1\*.* %source_folder%
            echo ERROR: file was not decompressed correctly >> ..\log\%props_file%.log
            goto error_2
        )
    )
)

rem ----- Copy "_main_","_im_","_streaming_" and "_common_ncsa_" files from "%props_file%.dir2"
rem       to temporary file "%temp_file1%", "%temp_file2%", "%temp_file3%", "%temp_file4%" 
if exist %props_file%.dir2\*.log (
    for /f %%a in ('dir %props_file%.dir2\*.log /a:-d /b /o:n') do (
        for /f "tokens=1-3 delims=_" %%b in ("%%a") do (
            if "%%c"=="main" (
                findstr /v /r "^#" %props_file%.dir2\%%a >> %temp_file1%
                echo %%a >> ..\log\%props_file%.log
            )
            if "%%c"=="im" (
                findstr /v /r "^#" %props_file%.dir2\%%a >> %temp_file2%
                echo %%a >> ..\log\%props_file%.log
            )
            if "%%c"=="streaming" (
                findstr /v /r "^#" %props_file%.dir2\%%a >> %temp_file3%
                echo %%a >> ..\log\%props_file%.log
            )
            if "%%c_%%d"=="common_ncsa" (
                findstr /v /r "^#" %props_file%.dir2\%%a >> %temp_file4%
                echo %%a >> ..\log\%props_file%.log
            )
            if not errorlevel 0 (
                type nul > %temp_file1%
                type nul > %temp_file2%
                type nul > %temp_file3%
                type nul > %temp_file4%
                move /y %props_file%.dir1\*.* %source_folder%
                echo ERROR: copy command was not processed correctly >> ..\log\%props_file%.log
                goto error_2
            )
        )
    )
)

rem ----- Remove folder "%props_file%.dir2"
rmdir /s /q %props_file%.dir2
if exist %props_file%.dir2 (
    type nul > %temp_file1%
    type nul > %temp_file2%
    type nul > %temp_file3%
    type nul > %temp_file4%
    move /y %props_file%.dir1\*.* %source_folder%
    echo ERROR: an error occured when deleting "%props_file%.dir2" folder >> ..\log\%props_file%.log
    goto error_3
)

goto end

rem ----- Return error codes
:error_1
    set err=1
    echo Exit with error code = %err% >> ..\log\%props_file%.log
    goto end

:error_2
    set err=2
    echo Exit with error code = %err% >> ..\log\%props_file%.log
    goto end

:error_3
    set err=3
    echo Exit with error code = %err% >> ..\log\%props_file%.log
    goto end

rem ----- End of the program
:end

echo Date: %date% Time: %time% Finish processing >> ..\log\%props_file%.log

:EOF

