rem IBM Confidential
rem OCO Source Material
rem 5724-S67
rem (c) Copyright IBM Corp. 1998, 2011
rem The source code for this program is not published or otherwise divested of
rem its trade secret, irrespective of what has been deposited with the U.S.
rem Copyright Office.

@echo off

rem ######################################################################################
rem Receives parameters from the Actuator
rem ######################################################################################

rem Parameters legend:
rem  1 - original chunk file name
rem  2 - exported to XML chunk file name
rem  3 - user name (only for remote collect required)
rem  4 - user password (only for remote collect required)
rem  5 - name of the Windows log for collect
rem  6 - name of the file with list of providers names. Maximum 22 rows in the list. Empty string if there no need filtering.
rem  7 - log number (0 if only one log will be processed, otherwise ordinal number start from 1)
rem  8 - first log flag (1 if log is first processed, otherwise 0)
rem  9 - last log flag (1 if log is last processed, otherwise 0)
rem 10 - success log collect flag (1 if at least one log was collect successfully, otherwise 0)
rem 11 - switch for logging: 1 - enable, 0 - disable
rem 12 - properties file name

setlocal
set TSIEM_BIN_DIR=%~dp0
set TSIEM_BIN_DIR=%TSIEM_BIN_DIR:~0,-1%
rem -- File for saving tarball with collected original evtx-logfiles
set ORIGINAL_CHUNK=%~1
rem -- File for saving exported events from all collected logfiles in XML format
set EXPORTED_CHUNK=%~2
set USER_NAME=%~3
set USER_PWD=%~4
set WIN_LOG_FILE=%~5
set PROVIDERS_FILE=%~6
set LOG_NUMBER=%~7
set FIRST_LOG=%~8
set LAST_LOG=%~9
shift
set COLLECTED=%~9
shift
set ENABLE_LOG=%~9
for %%I in (%*) do set PROPS_FILE=%%~I
for %%I in (%PROPS_FILE%) do set PROPS_NAME=%%~nI

rem -- sets ES log name
if "%PROPS_FILE%" == "" (
   set ES_LOG="..\log\~n0-error.log"
) else (
   set ES_LOG="..\log\%PROPS_NAME%.log"
)

rem -- clears log file and start logging
if %FIRST_LOG% EQU 1 (
   if %ENABLE_LOG% EQU 1 (
      type nul > %ES_LOG%
   )
)
if %ENABLE_LOG% NEQ 1 (
   set ES_LOG=nul
)

rem ######################################################################################
rem Actuator parameters check
rem ######################################################################################
set EXIT_CODE=2

rem -- Properties file PROPS_FILE argument is missing
if "%PROPS_FILE%" == "" (
   call :print_message "ERROR: Missing properties file argument."
   goto failure
)
if not exist "%PROPS_FILE%" (
   call :print_message "ERROR: Can't find properties file '%PROPS_FILE%'."
   goto failure
)

rem -- one or both logset files arguments are missing
set CHUNKNAME_EMPTY=0
if "%ORIGINAL_CHUNK%" == "" (
   call :print_message "ERROR: Missing output file name argument for sublog #1 of Windows '%WIN_LOG_FILE%' audit logs."
   set CHUNKNAME_EMPTY=1
) else (
   type nul > "%ORIGINAL_CHUNK%" 2>>%ES_LOG%
)
if "%EXPORTED_CHUNK%" == "" (
   call :print_message "ERROR: Missing output file name argument for sublog #2 of Windows '%WIN_LOG_FILE%' audit logs."
   set CHUNKNAME_EMPTY=1
) else (
   type nul > "%EXPORTED_CHUNK%" 2>>%ES_LOG%
)
if %CHUNKNAME_EMPTY% EQU 1 goto failure

if "%TSIEM_TMP_DIR%" == "" (
   set WORK_TMP_DIR=%TEMP%
) else (
   set WORK_TMP_DIR=%TSIEM_TMP_DIR%
)
if "%WORK_TMP_DIR%" == "" (
   set WORK_TMP_DIR=%SystemRoot%\Temp
)
if not "%WORK_TMP_DIR:~-1%" == "\" set WORK_TMP_DIR=%WORK_TMP_DIR%\
set WORK_TMP_DIR=%WORK_TMP_DIR%ES-%PROPS_NAME%
set COLLECT_SCRIPT=eventscollect.bat
set COLLECT_PROGRESS_FILE=collectprogress.txt
set TIMESTAMP_FILE=timestamp.txt
set COLLECT_ERROR_FILE=collecterror.txt
set INITIAL_COLLECT_DAYS=2
set STATE_FILE=%WORK_TMP_DIR%\current_state.txt
set TEMP_PROPS_FILE=%WORK_TMP_DIR%\new_properties.txt
set LOGS_EXT_FILE=logs_extension.txt

if %FIRST_LOG% EQU 1 (
   type nul > "%TEMP_PROPS_FILE%"
   call :print_message "Processing started."
)
if %LOG_NUMBER% EQU 0 (
   set PROPERTY_NAME=TIMESTAMP
) else (
   set PROPERTY_NAME=TIMESTAMP_%LOG_NUMBER%
)

call :parse_props_file

if "%HOST_NAME%" == "" (
   call :print_message "ERROR: 'SystemName' variable is not found in '%PROPS_FILE%' file."
   goto failure
)

if %FIRST_LOG% EQU 1 (
   Rem -- creates temp directory
   call :print_message "Create temporary working directory '%WORK_TMP_DIR%'."
   if exist "%WORK_TMP_DIR%\" (
      rmdir /s /q "%WORK_TMP_DIR%" >nul 2>>%ES_LOG%
      if exist "%WORK_TMP_DIR%\" (
         call :print_message "ERROR: Failure to remove existing temporary working directory '%WORK_TMP_DIR%'."
         goto failure
      )
   )
   mkdir "%WORK_TMP_DIR%" >nul 2>>%ES_LOG%
   if errorlevel 1 (
      call :print_message "ERROR: Failure to create temporary working directory '%WORK_TMP_DIR%'."
      goto failure
   )
   if not "%PROVIDERS_FILE%" == "" move /y "..\run\%PROVIDERS_FILE%" "%WORK_TMP_DIR%\" >nul 2>>%ES_LOG%
   call :check_remote_collect || goto failure
) else (
   for /f "tokens=1,2 delims= " %%a in ('type "%STATE_FILE%"') do (
       set HOST_NAME=%%a
       set REMOTE_COLLECT=%%b
   )
)

if "%HOST_NAME%" == "" (
   call :print_message "ERROR: Failure to read temporary state file '%STATE_FILE%'."
   goto failure
)

if %REMOTE_COLLECT% EQU 1 (
   if "%USER_NAME%" == "" (
      call :print_message "ERROR: Missing username for accessing target system."
      goto failure
   )
)

call :print_message ""
call :print_message "- Processing of the log '%WIN_LOG_FILE%' started."
if "%START_TIMESTAMP%" == "" call :print_message "Initial collect mode use."
call :print_message "-- Start preparing environment for the collect process."
set COLLECT_ERROR=0
call :create_collect_script

if %REMOTE_COLLECT% EQU 0 goto localcollect
   for /f "delims=" %%I in ('..\bin\cmtool.exe -k -d "%USER_PWD%" 2^>^>%ES_LOG%') do set PASSWORD=%%I
   set SHARE_NAME=\\%HOST_NAME%\ADMIN$
   set COLLECT_DIR=%SHARE_NAME%\Temp\ES-%PROPS_NAME%

   if %FIRST_LOG% EQU 1 (
      call :print_message "Map network share of the target system."
      net use "%SHARE_NAME%" "%PASSWORD%" "/u:%USER_NAME%" >nul 2>>%ES_LOG%
      if errorlevel 1 (
         call :print_message "ERROR: Failure during mapping network share of the target system."
         goto failure
      )
      if exist "%COLLECT_DIR%\" (
         rmdir /s /q "%COLLECT_DIR%" >nul 2>>%ES_LOG%
         if exist "%COLLECT_DIR%\" (
            call :print_message "ERROR: Failure to remove existing temporary working directory on target system '%COLLECT_DIR%'."
            goto failure
         )
      )
      call :print_message "Create temporary working directory '%COLLECT_DIR%' on the target system."
      mkdir "%COLLECT_DIR%" >nul 2>>%ES_LOG%
      if errorlevel 1 (
         call :print_message "ERROR: Failure to create temporary working directory on target system '%COLLECT_DIR%'."
         goto failure
      )
   )

   call :print_message "Copy necessary execution files to the target system."
   rem -- Copying collect script to the target system
   call :copy2target "%WORK_TMP_DIR%" "%COLLECT_SCRIPT%" || goto failure

   if not "%PROVIDERS_FILE%" == "" (
      call :copy2target "%WORK_TMP_DIR%" "%PROVIDERS_FILE%" || goto failure
   )

   if %FIRST_LOG% EQU 1 (
      rem -- Copying necessary binary files to the target system
      call :copy2target "%TSIEM_BIN_DIR%" "archive1.dll" "bzip2.dll" "zlib1.dll" "bsdtar.exe" || goto failure
   )

   rem -- Copying date.exe for getting timestamp on target system
   if "%START_TIMESTAMP%" == "" (
      if not exist "%COLLECT_DIR%\date.exe" (
         call :copy2target "%TSIEM_BIN_DIR%" "msvcr71.dll" "date.exe" || goto failure
	  )
   )

   rem -- Flag message for catching error during starting collect script
   type nul > "%COLLECT_DIR%\%COLLECT_ERROR_FILE%"
   call :print_message "ERROR: Failure during start collect script with PSEXEC." "%COLLECT_DIR%\%COLLECT_ERROR_FILE%"

   call :print_message "-- Start the collect process."
   rem -- Start collect script on target system and getting result files from target system

   if %LAST_LOG% EQU 1 (
      (set CM_OPT=)
   ) else (
      set CM_OPT=-k
   )
   ..\bin\cmtool.exe %CM_OPT% -d "%USER_PWD%" | ..\bin\psexec.exe \\%HOST_NAME% -u %USER_NAME% -r -q cmd.exe /c "%%SystemRoot%%\Temp\ES-%PROPS_NAME%\%COLLECT_SCRIPT%" | ..\bin\bsdtar.exe -C "%WORK_TMP_DIR%" -xf - >> %ES_LOG%

   rem -- Copy collect progress log to main logfile
   call :print2log "%COLLECT_DIR%\%COLLECT_PROGRESS_FILE%"

   set EXIT_CODE=1
   rem -- Check for collect script errors on target system
   copy /b /y "%COLLECT_DIR%\%COLLECT_ERROR_FILE%" "%WORK_TMP_DIR%\%COLLECT_ERROR_FILE%"
   if errorlevel 1 (
      call :print_message "ERROR: Failure to copy errors report from the target system."
      goto failure
   ) else (
      for %%I in ("%WORK_TMP_DIR%\%COLLECT_ERROR_FILE%") do (
          if %%~zI GTR 0 set COLLECT_ERROR=1
      )
   )
   if %COLLECT_ERROR% EQU 1 (
      call :print2log "%COLLECT_DIR%\%COLLECT_ERROR_FILE%"
   )

   rem -- Get new timestamp from target system
   if exist "%COLLECT_DIR%\%TIMESTAMP_FILE%" (
      copy /b /y "%COLLECT_DIR%\%TIMESTAMP_FILE%" "%WORK_TMP_DIR%\%TIMESTAMP_FILE%"
      if errorlevel 1 (
         call :print_message "ERROR: Failure to get new timestamp from the target system."
         goto failure
      )
   ) else (
      goto collecterror
   )
   copy /y "%COLLECT_DIR%\%LOGS_EXT_FILE%" "%WORK_TMP_DIR%\%LOGS_EXT_FILE%"
   if %LAST_LOG% EQU 1 (
      if exist "%COLLECT_DIR%\" (
         rmdir /s /q "%COLLECT_DIR%" >nul 2>>%ES_LOG%
         if exist "%COLLECT_DIR%\" (
            call :print_message "ERROR: Failure to remove temporary working directory on target system '%COLLECT_DIR%'."
            goto failure
         )
      )
      net use "%SHARE_NAME%" /delete >nul 2>>%ES_LOG%
      if errorlevel 1 (
         call :print_message "ERROR: Failure to unmap network share of the target system."
      )
   )
   goto finish

:localcollect
   set COLLECT_DIR=%WORK_TMP_DIR%
   set EXIT_CODE=1

   call :print_message "-- Start the collect process."
   rem -- Start collect script
   call "%COLLECT_DIR%\%COLLECT_SCRIPT%"

   rem -- Check for collect script errors
   if exist "%COLLECT_DIR%\%COLLECT_ERROR_FILE%" (
      for %%I in ("%COLLECT_DIR%\%COLLECT_ERROR_FILE%") do (
          if %%~zI GTR 0 set COLLECT_ERROR=1
      )
   )
   if %COLLECT_ERROR% EQU 1 (
      type "%COLLECT_DIR%\%COLLECT_ERROR_FILE%" >> %ES_LOG%
   )
   if not exist "%COLLECT_DIR%\%TIMESTAMP_FILE%" goto collecterror
:finish

call :print_message "-- Start postprocessing stage."
rem -- Calculate amount of collected events
call :print_message "Calculate the amount of the collected events."
for /f "tokens=*" %%I in ('find /C "<Event xmlns" "%WORK_TMP_DIR%\%WIN_LOG_FILE%.xml" 2^>nul') do set EVENTS_NUMBER=%%I
for %%I in (%EVENTS_NUMBER%) do set EVENTS_NUMBER=%%I
call :print_message "The total number of new collected events is %EVENTS_NUMBER%."

call :print_message "Copy original log contents to sublog #1."
rem -- Create tarball from collected events in evtx-files into first logset file
set /p LOG_FILE_EXT=<"%WORK_TMP_DIR%\%LOGS_EXT_FILE%"
set CUR_DIR=%CD%
pushd "%WORK_TMP_DIR%"
"%TSIEM_BIN_DIR%\bsdtar.exe" -cf evtxs.tar *%LOG_FILE_EXT% >nul 2>>"%CUR_DIR%"\%ES_LOG%
popd
if errorlevel 1 (
   call :print_message "ERROR: Failure to create tarball from collected events in evtx-files into first logset file."
   goto failure
)
del /f /q "%WORK_TMP_DIR%\*%LOG_FILE_EXT%"
move /y "%WORK_TMP_DIR%\evtxs.tar" "%ORIGINAL_CHUNK%" >nul 2>>%ES_LOG%
if errorlevel 1 (
   call :print_message "ERROR: Failure to move collected events in XML format into second logset file."
   goto failure
)
call :print_message "Copy XML dump of the original log contents to sublog #2."
rem -- Move collected events in XML format into second logset file
move /y "%WORK_TMP_DIR%\%WIN_LOG_FILE%.xml" "%EXPORTED_CHUNK%" >nul 2>>%ES_LOG%
if errorlevel 1 (
   call :print_message "ERROR: Failure to move collected events in XML format into second logset file."
   goto failure
)

call :print_message "Save the beginning timestamp for the next collect in the properties file."
for /f %%I in ('type "%WORK_TMP_DIR%\%TIMESTAMP_FILE%" 2^>nul') do set NEW_TIMESTAMP=%%I
if "%NEW_TIMESTAMP%" == "" (
   call :print_message "ERROR: Failure to get timestamp for last collected event."
   goto failure
) else (
   echo %PROPERTY_NAME% = %NEW_TIMESTAMP%>> "%TEMP_PROPS_FILE%"
)

call :print_message "-- Processing of the log '%WIN_LOG_FILE%' finished [SUCCESS]."

if %LAST_LOG% EQU 1 call :cleanup
exit /b 0

rem -- Collect script finished with error
:collecterror
if %COLLECT_ERROR% EQU 1 (
   call :print_message "ERROR: Failure during execution collect script."
) else (
   call :print_message "ERROR: There is not exist new timestamp on the target system."
)

rem -- prints failure message to actuator log and finishes the script with errorlevel 1 (FAILURE)
:failure
call :print_message "- Processing of the log '%WIN_LOG_FILE%' finished [FAILURE]."

if %EXIT_CODE% EQU 1 (
   if not "%START_TIMESTAMP%" == "" (
      echo %PROPERTY_NAME% = %NEW_TIMESTAMP%>> "%TEMP_PROPS_FILE%"
   )
   if %LAST_LOG% EQU 1 (
      if %COLLECTED% EQU 0 (
         set EXIT_CODE=2
      ) else (
         call :cleanup
      )
   )
)
if %EXIT_CODE% EQU 2 call :cleanup
exit /b %EXIT_CODE%


rem #########################################################################################
rem Parsing properties file PROPS_FILE for setting state variables
rem #########################################################################################

:parse_props_file

rem -- read value of the 'SystemName' parameter into HOST_NAME
for /f "tokens=2 delims==" %%I in ('findstr /R /C:"^SystemName = .*" "%PROPS_FILE%" 2^>^>%ES_LOG%') do set HOST_NAME=%%I
for %%I in (%HOST_NAME%) do set HOST_NAME=%%I
rem -- read value of the 'TIMESTAMP' parameter into START_TIMESTAMP
for /f "tokens=2 delims==" %%I in ('findstr /R /C:"^%PROPERTY_NAME% = .*" "%PROPS_FILE%" 2^>^>%ES_LOG%') do set START_TIMESTAMP=%%I
for %%I in (%START_TIMESTAMP%) do set START_TIMESTAMP=%%I
exit /b


rem #########################################################################################
rem Check collecting mode and set REMOTE_COLLECT variable
rem #########################################################################################

:check_remote_collect

for /f "tokens=1,* delims=|" %%a in ('cscript.exe //Nologo ..\bin\localorremote.vbs %HOST_NAME%') do (
    set RESULT1=%%a
    set RESULT2=%%b
)
if "%RESULT1%" == "LOCAL" (
   set REMOTE_COLLECT=0
   call :print_message "Collect from the local target system: '%HOST_NAME%'."
) else if "%RESULT1%" == "REMOTE" (
   set REMOTE_COLLECT=1
   set HOST_NAME=%RESULT2%
   call :print_message "Collect from the remote target system: '%HOST_NAME%'."
) else (
   call :print_message "ERROR: Script for determining mode of the collect return failure."
   if "%RESULT2%" == "" (
      call :print_message "%RESULT1%"
   ) else (
      call :print_message "%RESULT2%"
   )
   exit /b 1
)
echo %HOST_NAME% ^%REMOTE_COLLECT%> "%STATE_FILE%"
exit /b 0


rem #########################################################################################
rem Creating collect script COLLECT_SCRIPT in WORK_TMP_DIR
rem #########################################################################################

:create_collect_script

set NEW_BAT_FILE="%WORK_TMP_DIR%\%COLLECT_SCRIPT%"
type nul > %NEW_BAT_FILE% 2>>%ES_LOG%

echo @echo off>> %NEW_BAT_FILE%
echo rem #################################### eventscollect.bat ####################################>> %NEW_BAT_FILE%
echo rem ### on success send in stdout tarball with resulting bziped XML-file and all evtx-files>> %NEW_BAT_FILE%
echo rem ### if failure return code 1 and explanation message in COLLECT_ERROR_FILE>> %NEW_BAT_FILE%
echo rem ###########################################################################################>> %NEW_BAT_FILE%
echo setlocal>> %NEW_BAT_FILE%
if %REMOTE_COLLECT% EQU 1 (
   echo set WORK_TMP_DIR="%%SystemRoot%%\Temp\ES-%PROPS_NAME%">> %NEW_BAT_FILE%
   echo set WORK_TMP_DIR=%%WORK_TMP_DIR:~1,-1%%>> %NEW_BAT_FILE%
   echo set TSIEM_BIN_DIR=%%WORK_TMP_DIR%%>> %NEW_BAT_FILE%
   echo set COLLECT_PROGRESS_FILE="%%WORK_TMP_DIR%%\%COLLECT_PROGRESS_FILE%">> %NEW_BAT_FILE%
   echo if "%%TSIEM_TMP_DIR%%" == "" ^(>> %NEW_BAT_FILE%
   echo    set CHUNKS_TMP_DIR=%%WORK_TMP_DIR%%>> %NEW_BAT_FILE%
   echo ^) else (>> %NEW_BAT_FILE%
   echo    if "%%TSIEM_TMP_DIR:~-1%%" == "\" ^(>> %NEW_BAT_FILE%
   echo       set CHUNKS_TMP_DIR=%%TSIEM_TMP_DIR%%ES-%PROPS_NAME%>> %NEW_BAT_FILE%
   echo    ^) else (>> %NEW_BAT_FILE%
   echo       set CHUNKS_TMP_DIR=%%TSIEM_TMP_DIR%%\ES-%PROPS_NAME%>> %NEW_BAT_FILE%
   echo    ^)>> %NEW_BAT_FILE%
   echo ^)>> %NEW_BAT_FILE%
   echo type nul ^> %%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
) else (
   echo set WORK_TMP_DIR="%WORK_TMP_DIR%">> %NEW_BAT_FILE%
   echo set WORK_TMP_DIR=%%WORK_TMP_DIR:~1,-1%%>> %NEW_BAT_FILE%
   echo set TSIEM_BIN_DIR="%TSIEM_BIN_DIR%">> %NEW_BAT_FILE%
   echo set TSIEM_BIN_DIR=%%TSIEM_BIN_DIR:~1,-1%%>> %NEW_BAT_FILE%
   echo set COLLECT_PROGRESS_FILE=%ES_LOG%>> %NEW_BAT_FILE%
   echo set CHUNKS_TMP_DIR=%%WORK_TMP_DIR%%>> %NEW_BAT_FILE%
)
echo set TIMESTAMP_FILE="%%WORK_TMP_DIR%%\%TIMESTAMP_FILE%">> %NEW_BAT_FILE%
echo set COLLECT_ERROR_FILE="%%WORK_TMP_DIR%%\%COLLECT_ERROR_FILE%">> %NEW_BAT_FILE%
echo type nul ^> %%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
echo if defined PROCESSOR_ARCHITEW6432 (>> %NEW_BAT_FILE%
echo    set TARGET_ARCH=%%PROCESSOR_ARCHITEW6432%%>> %NEW_BAT_FILE%
echo ) else (>> %NEW_BAT_FILE%
echo    if defined PROCESSOR_ARCHITECTURE (>> %NEW_BAT_FILE%
echo       set TARGET_ARCH=%%PROCESSOR_ARCHITECTURE%%>> %NEW_BAT_FILE%
echo    )>> %NEW_BAT_FILE%
echo )>> %NEW_BAT_FILE%
echo if not defined TARGET_ARCH (>> %NEW_BAT_FILE%
echo    call :print_message "ERROR: Cannot define target system processor architecture." %%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
echo    exit /b ^1>> %NEW_BAT_FILE%
echo )>> %NEW_BAT_FILE%
echo for /f "tokens=*" %%%%I in ('echo %%TARGET_ARCH%% ^^^| find /c "64" 2^^^>nul') do set ARCH_64=%%%%I>> %NEW_BAT_FILE%
echo if %%ARCH_64%% EQU 1 (>> %NEW_BAT_FILE%
echo    set WIN32_SYS_DIR=%%SystemRoot%%\sysnative>> %NEW_BAT_FILE%
echo    call :print_message "The operating system on the target system has 'x86-64' architecture.">> %NEW_BAT_FILE%
echo ) else (>> %NEW_BAT_FILE%
echo    set WIN32_SYS_DIR=%%SystemRoot%%\System32>> %NEW_BAT_FILE%
echo    call :print_message "The operating system on the target system has 'x86' architecture.">> %NEW_BAT_FILE%
echo )>> %NEW_BAT_FILE%
echo set WIN_SYS_DIR=%%SystemRoot%%\System32>> %NEW_BAT_FILE%
echo set WEVT_EXE_FILE=%%WIN32_SYS_DIR%%\wevtutil.exe>> %NEW_BAT_FILE%
echo set ARCH_LOGS_LIST=%%WORK_TMP_DIR%%\archivedlogs.lst>> %NEW_BAT_FILE%
echo set LAST_EVENT_FILE="%%WORK_TMP_DIR%%\lastevent.xml">> %NEW_BAT_FILE%
echo set QUERY_FILE="%%WORK_TMP_DIR%%\query.xml">> %NEW_BAT_FILE%
echo set LOGS_EXT_FILE="%%WORK_TMP_DIR%%\%LOGS_EXT_FILE%">> %NEW_BAT_FILE%
echo if exist "%%TIMESTAMP_FILE%%" del /f /q "%%TIMESTAMP_FILE%%">> %NEW_BAT_FILE%
echo rem Setup temporary directory for result chunks if system variable TSIEM_TMP_DIR is defined>> %NEW_BAT_FILE%
echo if /i not "%%CHUNKS_TMP_DIR%%" == "%%WORK_TMP_DIR%%" ^(>> %NEW_BAT_FILE%
echo    call :print_message "Create temporary directory for chunks '%%CHUNKS_TMP_DIR%%'.">> %NEW_BAT_FILE%
echo    if exist "%%CHUNKS_TMP_DIR%%\" ^(>> %NEW_BAT_FILE%
echo       rmdir /s /q "%%CHUNKS_TMP_DIR%%" ^>nul 2^>^>%%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
echo       if exist "%%CHUNKS_TMP_DIR%%\" ^(>> %NEW_BAT_FILE%
echo          call :print_message "ERROR: Failure to remove existing temporary directory for chunks '%%CHUNKS_TMP_DIR%%'." %%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
echo          exit /b ^1>> %NEW_BAT_FILE%
echo       ^)>> %NEW_BAT_FILE%
echo    ^)>> %NEW_BAT_FILE%
echo    mkdir "%%CHUNKS_TMP_DIR%%" ^>nul 2^>^>%%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
echo    if errorlevel 1 ^(>> %NEW_BAT_FILE%
echo       call :print_message "ERROR: Failure to create temporary working directory '%%CHUNKS_TMP_DIR%%'." %%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
echo       exit /b ^1>> %NEW_BAT_FILE%
echo    ^)>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo if not exist "%%WEVT_EXE_FILE%%" ^(>> %NEW_BAT_FILE%
echo    call :print_message "'%%WEVT_EXE_FILE%%' Windows native utility for parsing '%WIN_LOG_FILE%' log not found on the target system. Unable to proceed further with the collect." %%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
echo    exit /b ^1>> %NEW_BAT_FILE%
echo ^) else ^(>> %NEW_BAT_FILE%
echo    call :print_message "Found Windows native utility for parsing '%WIN_LOG_FILE%' log on the target system: '%%WEVT_EXE_FILE%%'.">> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo ^(set LOG_FILE_PATH=^)>> %NEW_BAT_FILE%
echo ^(set LOG_FILE_NAME=^)>> %NEW_BAT_FILE%
echo ^(set LOG_FILE_EXT=^)>> %NEW_BAT_FILE%
echo for /f "tokens=1,* delims=:" %%%%a in ^('wevtutil gl "%WIN_LOG_FILE%" ^^^| find "logFileName:"'^) do ^(>> %NEW_BAT_FILE%
echo     set LOG_FILE_PATH=%%%%b>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo for /f "delims=" %%%%I in ^('echo %%LOG_FILE_PATH%%'^) do ^(>> %NEW_BAT_FILE%
echo     set LOG_FILE_PATH=%%%%~dpI>> %NEW_BAT_FILE%
echo     set LOG_FILE_NAME=%%%%~nI>> %NEW_BAT_FILE%
echo     set LOG_FILE_EXT=%%%%~xI>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo if "%%LOG_FILE_EXT%%" == "" set LOG_FILE_EXT=.>> %NEW_BAT_FILE%
echo set LOG_FILE_PATH=%%LOG_FILE_PATH:~0,-1%%>> %NEW_BAT_FILE%
echo set WIN_LOG_DIR=%%LOG_FILE_PATH%%>> %NEW_BAT_FILE%
echo if %%ARCH_64%% EQU 1 (>> %NEW_BAT_FILE%
echo    set WIN32_LOG_DIR=%%LOG_FILE_PATH:System32\winevt=sysnative\winevt%%>> %NEW_BAT_FILE%
echo ) else (>> %NEW_BAT_FILE%
echo    set WIN32_LOG_DIR=%%LOG_FILE_PATH%%>> %NEW_BAT_FILE%
echo )>> %NEW_BAT_FILE%
echo if not exist "%%WIN32_LOG_DIR%%" ^(>> %NEW_BAT_FILE%
echo    call :print_message "Folder with '%WIN_LOG_FILE%' log files '%%LOG_FILE_PATH%%' not found on the target system. Unable to proceed further with the collect." %%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
echo    exit /b ^1>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo echo %%LOG_FILE_EXT%%^>^>"%%LOGS_EXT_FILE%%">> %NEW_BAT_FILE%
echo call :print_message "Found folder with '%WIN_LOG_FILE%' log on the target system: '%%WIN_LOG_DIR%%'.">> %NEW_BAT_FILE%
if "%START_TIMESTAMP%" == "" (
   echo call :print_message "Calculate the timestamp, from which the events will be collected.">> %NEW_BAT_FILE%
   echo set DATE_ERR=^0>> %NEW_BAT_FILE%
   echo pushd "%%TSIEM_BIN_DIR%%">> %NEW_BAT_FILE%
   echo for /f "tokens=*" %%%%I in ^('"date.exe "+%%%%Y-%%%%m-%%%%dT%%%%H:%%%%m:%%%%S.000Z" -d "%INITIAL_COLLECT_DAYS% days ago"" 2^^^>^^^>%%%%COLLECT_ERROR_FILE%%%%'^) do set START_TIMESTAMP=%%%%I>> %NEW_BAT_FILE%
   echo popd>> %NEW_BAT_FILE%
   echo if errorlevel 1 set DATE_ERR=^1>> %NEW_BAT_FILE%
   echo if "%%START_TIMESTAMP%%" == "" set DATE_ERR=^1>> %NEW_BAT_FILE%
   echo if %%DATE_ERR%% EQU 1 ^(>> %NEW_BAT_FILE%
   echo    call :print_message "ERROR: Failure to get timestamp for initial collect." %%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
   echo    exit /b ^1>> %NEW_BAT_FILE%
   echo ^)>> %NEW_BAT_FILE%
) else (
   echo ^(set START_TIMESTAMP=%START_TIMESTAMP%^)>> %NEW_BAT_FILE%
)
echo call :convert_timestamp "%%START_TIMESTAMP%%">> %NEW_BAT_FILE%
echo call :print_message "Events will be collected from '%%START_TIMESTAMP%%'.">> %NEW_BAT_FILE%
echo rem -- Collecting events from newcreated archived logfiles>> %NEW_BAT_FILE%
echo set FIRST_FILE=^1>> %NEW_BAT_FILE%
echo set NEED_CHECK=^1>> %NEW_BAT_FILE%
echo if not exist "%%WIN32_LOG_DIR%%\Archive-%%LOG_FILE_NAME%%-*%%LOG_FILE_EXT%%" ^(>> %NEW_BAT_FILE%
echo    call :print_message "Archived '%WIN_LOG_FILE%' log files were not found on the target system.">> %NEW_BAT_FILE%
echo    goto end_cycle>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo :label^1>> %NEW_BAT_FILE%
echo set ATTEMPT=^0>> %NEW_BAT_FILE%
echo rem -- Cycle for checking archives created during processing previously existed ^(maximum 100 iteration, then generate error^)>> %NEW_BAT_FILE%
echo :start_cycle>> %NEW_BAT_FILE%
echo     if %%ATTEMPT%% EQU 100 ^(>> %NEW_BAT_FILE%
echo        call :print_message "ERROR: Logfiles archives are produced quicker than they can be collected. Please, change '%WIN_LOG_FILE%' log archiving settings.">> %NEW_BAT_FILE%
echo        call :print_message "INFO: Please increase the size of active logfile to avoid such situation in the future and repeat the collect.">> %NEW_BAT_FILE%
echo        goto final_actions>> %NEW_BAT_FILE%
echo     ^)>> %NEW_BAT_FILE%
echo     set /a ATTEMPT+=^1>> %NEW_BAT_FILE%
echo     call :get_archives_list ^|^| goto end_cycle>> %NEW_BAT_FILE%
echo     for /f "delims=" %%%%I in ^('type "%%ARCH_LOGS_LIST%%" 2^^^>^^^>%%COLLECT_PROGRESS_FILE%%'^) do ^(>> %NEW_BAT_FILE%
echo         set PREVIOUS_ARCHIVE=%%%%I>> %NEW_BAT_FILE%
if not "%START_TIMESTAMP%" == "" (
   echo         call :check_prev_events "%%%%I" ^|^| goto final_actions>> %NEW_BAT_FILE%
)
echo         call :collect_archives "%%%%I" ^|^| goto final_actions>> %NEW_BAT_FILE%
echo     ^)>> %NEW_BAT_FILE%
echo     goto start_cycle>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo :end_cycle>> %NEW_BAT_FILE%
if not "%START_TIMESTAMP%" == "" (
   echo call :check_prev_events "%WIN_LOG_FILE%" ^|^| goto final_actions>> %NEW_BAT_FILE%
)
echo rem -- Collecting events from current active logfile>> %NEW_BAT_FILE%
echo call :print_message "Collect events from the currently active '%WIN_LOG_FILE%' log: '%%WIN_LOG_DIR%%\%%LOG_FILE_NAME%%%%LOG_FILE_EXT%%'.">> %NEW_BAT_FILE%
echo if exist "%%CHUNKS_TMP_DIR%%\%%LOG_FILE_NAME%%%%LOG_FILE_EXT%%" del /f /q "%%CHUNKS_TMP_DIR%%\%%LOG_FILE_NAME%%%%%LOG_FILE_EXT%%%" ^>nul 2^>^>%%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
echo if %%FIRST_FILE%% EQU 1 ^(>> %NEW_BAT_FILE%
echo    call :create_query "%WIN_LOG_FILE%" "%PROVIDERS_FILE%" "%%START_TIMESTAMP%%">> %NEW_BAT_FILE%
echo ^) else ^(>> %NEW_BAT_FILE%
echo    call :create_query "%WIN_LOG_FILE%" "%PROVIDERS_FILE%" "">> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo if not errorlevel 1 ^(>> %NEW_BAT_FILE%
echo    "%%WEVT_EXE_FILE%%" epl %%QUERY_FILE%% /sq:true "%%CHUNKS_TMP_DIR%%\%%LOG_FILE_NAME%%%%LOG_FILE_EXT%%" ^>nul 2^>^>%%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo if errorlevel 1 ^(>> %NEW_BAT_FILE%
echo    call :print_message "ERROR: Failure to collect events from the currently active '%WIN_LOG_FILE%' log.">> %NEW_BAT_FILE%
echo    del /f /q "%%CHUNKS_TMP_DIR%%\%%LOG_FILE_NAME%%%%LOG_FILE_EXT%%">> %NEW_BAT_FILE%
echo    goto final_actions>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo call :get_archives_list>> %NEW_BAT_FILE%
echo if not errorlevel 1 ^(>> %NEW_BAT_FILE%
echo    call :print_message "New archived logfile was created during collect events from the currently active '%WIN_LOG_FILE%' log.">> %NEW_BAT_FILE%
echo    goto start_cycle>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo :final_actions>> %NEW_BAT_FILE%
echo rem -- Export events from collected evtx-logfiles into one XML file>> %NEW_BAT_FILE%
echo call :print_message "Export collected events into XML format.">> %NEW_BAT_FILE%
echo ^(set CURRENT_LOG=^)>> %NEW_BAT_FILE%
echo for /f "delims=" %%%%I in ^('dir /B /ON "%%CHUNKS_TMP_DIR%%\*%%LOG_FILE_EXT%%" 2^^^>^^^>%%COLLECT_PROGRESS_FILE%%'^) do ^(>> %NEW_BAT_FILE%
echo     "%%WEVT_EXE_FILE%%" qe "%%CHUNKS_TMP_DIR%%\%%%%I" /lf /f:xml ^>^> "%%CHUNKS_TMP_DIR%%\%WIN_LOG_FILE%.xml" 2^>^>%%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
echo     if errorlevel 1 ^(>> %NEW_BAT_FILE%
echo        call :print_message "ERROR: Failure to export collected events into XML format from '%%%%I' logfile." %%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
echo        exit /b 1>> %NEW_BAT_FILE%
echo     ^)>> %NEW_BAT_FILE%
echo     set CURRENT_LOG=%%%%I>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo if "%%CURRENT_LOG%%" == "" ^(>> %NEW_BAT_FILE%
echo    call :print_message "ERROR: No logfiles were processed." %%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
echo    exit /b 1>> %NEW_BAT_FILE%
echo ^) else ^(>> %NEW_BAT_FILE%
echo    call :print_message "Calculate the timestamp for the next collect.">> %NEW_BAT_FILE%
echo    rem -- Get last collected event info for timestamp>> %NEW_BAT_FILE%
echo    ^(set EVENT_SYSTEM_TIME=^)>> %NEW_BAT_FILE%
echo    "%%WEVT_EXE_FILE%%" qe "%%CHUNKS_TMP_DIR%%\%%CURRENT_LOG%%" /lf /rd:true /c:1 /f:xml ^> %%LAST_EVENT_FILE%% 2^>^>%%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
echo    if errorlevel 1 ^(>> %NEW_BAT_FILE%
echo       call :print_message "ERROR: Failure to export last event from collected logs." %%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
echo       exit /b ^1>> %NEW_BAT_FILE%
echo    ^)>> %NEW_BAT_FILE%
echo    call :parse_timestamp>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo if "%%EVENT_SYSTEM_TIME%%" == "" ^(>> %NEW_BAT_FILE%
echo    set EVENT_SYSTEM_TIME=%%START_TIMESTAMP%%>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo echo %%EVENT_SYSTEM_TIME%%^> %%TIMESTAMP_FILE%%>> %NEW_BAT_FILE%
echo call :print_message "Timestamp for the next collect is '%%EVENT_SYSTEM_TIME%%'.">> %NEW_BAT_FILE%
if %REMOTE_COLLECT% EQU 1 (
   echo rem -- Get all result files from target system>> %NEW_BAT_FILE%
   echo call :print_message "Send result files to destination system.">> %NEW_BAT_FILE%
   echo set ERROR=^0>> %NEW_BAT_FILE%
   echo pushd "%%CHUNKS_TMP_DIR%%">> %NEW_BAT_FILE%
   echo "%%WORK_TMP_DIR%%\bsdtar.exe" -cjf - "%WIN_LOG_FILE%.xml" "*%%LOG_FILE_EXT%%" 2^>^>%%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
   echo if errorlevel 1 set ERROR=^1>> %NEW_BAT_FILE%
   echo popd>> %NEW_BAT_FILE%
   echo if /i not "%%CHUNKS_TMP_DIR%%" == "%%WORK_TMP_DIR%%" ^(>> %NEW_BAT_FILE%
   echo    call :print_message "Remove temporary directory for chunks '%%CHUNKS_TMP_DIR%%'.">> %NEW_BAT_FILE%
   echo    rmdir /s /q "%%CHUNKS_TMP_DIR%%" ^>nul 2^>^>%%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
   echo    if exist "%%CHUNKS_TMP_DIR%%\" ^(>> %NEW_BAT_FILE%
   echo       call :print_message "ERROR: Failure to remove temporary directory for chunks '%%CHUNKS_TMP_DIR%%'." %%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
   echo       exit /b ^1>> %NEW_BAT_FILE%
   echo    ^)>> %NEW_BAT_FILE%
   echo ^) else ^(>> %NEW_BAT_FILE%
   echo    del /f /q "%%WORK_TMP_DIR%%\*%%LOG_FILE_EXT%%" "%%WORK_TMP_DIR%%\%WIN_LOG_FILE%.xml" ^>nul 2^>^>%%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
   echo    if exist "%%WORK_TMP_DIR%%\*%%LOG_FILE_EXT%%" ^(>> %NEW_BAT_FILE%
   echo       call :print_message "ERROR: Failure to remove temporary log files." %%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
   echo       exit /b ^1>> %NEW_BAT_FILE%
   echo    ^)>> %NEW_BAT_FILE%
   echo ^)>> %NEW_BAT_FILE%
   echo if %%ERROR%% EQU 1 ^(>> %NEW_BAT_FILE%
   echo    call :print_message "ERROR: Failure to send result files to destination system." %%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
   echo    exit /b ^1>> %NEW_BAT_FILE%
   echo ^)>> %NEW_BAT_FILE%
)
echo exit /b ^0>> %NEW_BAT_FILE%
echo.>> %NEW_BAT_FILE%
echo rem #########################################################################################>> %NEW_BAT_FILE%
echo rem Convert timestamp from ISO to archived logfile name format>> %NEW_BAT_FILE%
echo rem %%1  - ISO timestamp>> %NEW_BAT_FILE%
echo rem Result - set ARCH_LOG_TIME variable>> %NEW_BAT_FILE%
echo rem #########################################################################################>> %NEW_BAT_FILE%
echo :convert_timestamp>> %NEW_BAT_FILE%
echo for /f "tokens=1 delims=T" %%%%I in ^("%%~1"^) do set DATE_PART=%%%%I>> %NEW_BAT_FILE%
echo for /f "tokens=2 delims=T" %%%%I in ^("%%~1"^) do set TIME_PART=%%%%I>> %NEW_BAT_FILE%
echo for /f "tokens=1 delims=:" %%%%I in ^("%%TIME_PART%%"^) do set HOURS=%%%%I>> %NEW_BAT_FILE%
echo for /f "tokens=2 delims=:" %%%%I in ^("%%TIME_PART%%"^) do set MINUTES=%%%%I>> %NEW_BAT_FILE%
echo for /f "tokens=3 delims=:" %%%%I in ^("%%TIME_PART%%"^) do set SECOND_PART=%%%%I>> %NEW_BAT_FILE%
echo for /f "tokens=1 delims=." %%%%I in ^("%%SECOND_PART%%"^) do set SECONDS=%%%%I>> %NEW_BAT_FILE%
echo for /f "tokens=2 delims=." %%%%I in ^("%%SECOND_PART%%"^) do set MICROSECONDS=%%%%I>> %NEW_BAT_FILE%
echo set ARCH_LOG_TIME=%%DATE_PART%%-%%HOURS%%-%%MINUTES%%-%%SECONDS%%-%%MICROSECONDS%%>> %NEW_BAT_FILE%
echo exit /b>> %NEW_BAT_FILE%
echo.>> %NEW_BAT_FILE%
echo rem #########################################################################################>> %NEW_BAT_FILE%
echo rem Get list of archived logfiles for processing>> %NEW_BAT_FILE%
echo rem Result - list of necessary archived logfiles in ARCH_LOGS_LIST file>> %NEW_BAT_FILE%
echo rem #########################################################################################>> %NEW_BAT_FILE%
echo :get_archives_list>> %NEW_BAT_FILE%
echo if defined PREVIOUS_ARCHIVE ^(>> %NEW_BAT_FILE%
echo    set AN=%%PREVIOUS_ARCHIVE%%>> %NEW_BAT_FILE%
echo ^) else ^(>> %NEW_BAT_FILE%
echo    set AN=Archive-%%LOG_FILE_NAME%%-%%ARCH_LOG_TIME%%>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo type nul ^> "%%ARCH_LOGS_LIST%%" 2^>^>%%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
echo for %%%%I in ^("%%WIN32_LOG_DIR%%\Archive-%%LOG_FILE_NAME%%-*%%LOG_FILE_EXT%%"^) do ^(>> %NEW_BAT_FILE%
echo     if "%%%%~nI" GTR "%%AN%%" ^(>> %NEW_BAT_FILE%
echo        echo %%%%~nI^>^> "%%ARCH_LOGS_LIST%%">> %NEW_BAT_FILE%
echo     ^) else ^(>> %NEW_BAT_FILE%
echo        set NEED_CHECK=^0>> %NEW_BAT_FILE%
echo     ^)>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo for %%%%I in ^("%%ARCH_LOGS_LIST%%"^) do if %%%%~zI EQU 0 exit /b ^1>> %NEW_BAT_FILE%
echo exit /b ^0>> %NEW_BAT_FILE%
echo.>> %NEW_BAT_FILE%
echo rem #########################################################################################>> %NEW_BAT_FILE%
echo rem Collect events from archived log files>> %NEW_BAT_FILE%
echo rem %%1 - Log name>> %NEW_BAT_FILE%
echo rem #########################################################################################>> %NEW_BAT_FILE%
echo :collect_archives>> %NEW_BAT_FILE%
echo call :print_message "Collect events from '%%WIN_LOG_DIR%%\%%~1%%LOG_FILE_EXT%%' log archive.">> %NEW_BAT_FILE%
echo if %%FIRST_FILE%% EQU 1 ^(>> %NEW_BAT_FILE%
echo    call :create_query "file://%%WIN_LOG_DIR%%\%%~1%%LOG_FILE_EXT%%" "%PROVIDERS_FILE%" "%%START_TIMESTAMP%%">> %NEW_BAT_FILE%
echo ^) else ^(>> %NEW_BAT_FILE%
echo    call :create_query "file://%%WIN_LOG_DIR%%\%%~1%%LOG_FILE_EXT%%" "%PROVIDERS_FILE%" "">> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo if not errorlevel 1 ^(>> %NEW_BAT_FILE%
echo    "%%WEVT_EXE_FILE%%" epl %%QUERY_FILE%% /sq:true "%%CHUNKS_TMP_DIR%%\%%~1%%LOG_FILE_EXT%%" ^>nul 2^>^>%%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo if errorlevel 1 ^(>> %NEW_BAT_FILE%
echo    call :print_message "ERROR: Failure to collect events from '%%WIN_LOG_DIR%%\%%~1%%LOG_FILE_EXT%%' Log.">> %NEW_BAT_FILE%
echo    if exist "%%CHUNKS_TMP_DIR%%\%%~1%%LOG_FILE_EXT%%" del /f /q "%%CHUNKS_TMP_DIR%%\%%~1%%LOG_FILE_EXT%%">> %NEW_BAT_FILE%
echo    exit /b ^1>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo set FIRST_FILE=^0>> %NEW_BAT_FILE%
echo exit /b ^0>> %NEW_BAT_FILE%
if not "%START_TIMESTAMP%" == "" (
   echo.>> %NEW_BAT_FILE%
   echo rem #########################################################################################>> %NEW_BAT_FILE%
   echo rem Check for existing evenst from previous collect>> %NEW_BAT_FILE%
   echo rem %%1 - name of the current processed logfile>> %NEW_BAT_FILE%
   echo rem #########################################################################################>> %NEW_BAT_FILE%
   echo :check_prev_events>> %NEW_BAT_FILE%
   echo if %%NEED_CHECK%% EQU 0 exit /b ^0>> %NEW_BAT_FILE%
   echo if %%1 == "%WIN_LOG_FILE%" ^(>> %NEW_BAT_FILE%
   echo    call :create_query %%1 "%PROVIDERS_FILE%" "">> %NEW_BAT_FILE%
   echo ^) else ^(>> %NEW_BAT_FILE%
   echo    call :create_query "file://%%WIN_LOG_DIR%%\%%~1%%LOG_FILE_EXT%%" "%PROVIDERS_FILE%" "">> %NEW_BAT_FILE%
   echo ^)>> %NEW_BAT_FILE%
   echo if not errorlevel 1 ^(>> %NEW_BAT_FILE%
   echo    "%%WEVT_EXE_FILE%%" qe %%QUERY_FILE%% /sq:true /rd:false /c:1 /f:xml ^> %%LAST_EVENT_FILE%% 2^>^>%%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
   echo ^)>> %NEW_BAT_FILE%
   echo if errorlevel 1 ^(>> %NEW_BAT_FILE%
   echo    if %%1 == "%WIN_LOG_FILE%" ^(>> %NEW_BAT_FILE%
   echo       set LN=current active '%WIN_LOG_FILE%' log>> %NEW_BAT_FILE%
   echo    ^) else ^(>> %NEW_BAT_FILE%
   echo       set LN=archived Log '%%WIN_LOG_DIR%%\%%~1%%LOG_FILE_EXT%%'>> %NEW_BAT_FILE%
   echo    ^)>> %NEW_BAT_FILE%
   echo    call :print_message "ERROR: Failure to collect first event from %%LN%%." %%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
   echo    exit /b ^1>> %NEW_BAT_FILE%
   echo ^)>> %NEW_BAT_FILE%
   echo if not %%1 == "%WIN_LOG_FILE%" goto label^1>> %NEW_BAT_FILE%
   echo for /f "delims=" %%%%I in ^('type "%%ARCH_LOGS_LIST%%" 2^^^>nul'^) do set LAST_ARCH=%%%%I>> %NEW_BAT_FILE%
   echo call :get_archives_list>> %NEW_BAT_FILE%
   echo set NEW_LAST_ARCH=%%LAST_ARCH%%>> %NEW_BAT_FILE%
   echo if not errorlevel 1 ^(>> %NEW_BAT_FILE%
   echo    for /f "delims=" %%%%I in ^('type "%%ARCH_LOGS_LIST%%" 2^^^>nul'^) do set NEW_LAST_ARCH=%%%%I>> %NEW_BAT_FILE%
   echo ^)>> %NEW_BAT_FILE%
   echo if not "%%LAST_ARCH%%" == "%%NEW_LAST_ARCH%%" exit /b ^0>> %NEW_BAT_FILE%
   echo :label^1>> %NEW_BAT_FILE%
   echo set NEED_CHECK=^0>> %NEW_BAT_FILE%
   echo call :parse_timestamp>> %NEW_BAT_FILE%
   echo if "%%EVENT_SYSTEM_TIME%%" GTR "%%START_TIMESTAMP%%" ^(>> %NEW_BAT_FILE%
   echo    call :print_message "WARNING: Events with previous timestamp not found, some events may have been lost.">> %NEW_BAT_FILE%
   echo    call :print_message "INFO: Please collect more often to avoid such situations in the future.">> %NEW_BAT_FILE%
   echo ^)>> %NEW_BAT_FILE%
   echo exit /b ^0>> %NEW_BAT_FILE%
)
echo.>> %NEW_BAT_FILE%
echo rem #########################################################################################>> %NEW_BAT_FILE%
echo rem Parse timestamp from event in given XML-file>> %NEW_BAT_FILE%
echo rem %%1 - XML-file with event>> %NEW_BAT_FILE%
echo rem #########################################################################################>> %NEW_BAT_FILE%
echo :parse_timestamp>> %NEW_BAT_FILE%
echo rem -- Check for ESENT.dll events. They have another System event part format>> %NEW_BAT_FILE%
echo ^(set TOK12= ^)>> %NEW_BAT_FILE%
echo ^(set TOK16= ^)>> %NEW_BAT_FILE%
echo ^(set SYSTIME= ^)>> %NEW_BAT_FILE%
echo set TIME_TAG=TimeCreated>> %NEW_BAT_FILE%
echo for /f "tokens=12,16 delims=^<" %%%%a in ^('type "%%LAST_EVENT_FILE%%" 2^^^>^^^>%%COLLECT_PROGRESS_FILE%%'^) do ^(>> %NEW_BAT_FILE%
echo    set TOK12=%%%%a>> %NEW_BAT_FILE%
echo    set TOK16=%%%%b>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo if "%%TOK12:~0,11%%" == "%%TIME_TAG%%" ^(>> %NEW_BAT_FILE%
echo    set SYSTIME="%%TOK12%%">> %NEW_BAT_FILE%
echo ^) else if "%%TOK16:~0,11%%" == "%%TIME_TAG%%" ^(>> %NEW_BAT_FILE%
echo    set SYSTIME="%%TOK16%%">> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo set EVENT_SYSTEM_TIME=%%SYSTIME:~25,-4%%>> %NEW_BAT_FILE%
echo exit /b>> %NEW_BAT_FILE%
echo.>> %NEW_BAT_FILE%
echo rem ######################################################################################>> %NEW_BAT_FILE%
echo rem Printing routine. Prints a message to log. Takes two parameters:>> %NEW_BAT_FILE%
echo rem   1 - message itself>> %NEW_BAT_FILE%
echo rem   2 - name of a log file. if empty, use default value.>> %NEW_BAT_FILE%
echo rem ######################################################################################>> %NEW_BAT_FILE%
echo :print_message>> %NEW_BAT_FILE%
echo ^(set OUT_FILE=%%~2^)>> %NEW_BAT_FILE%
echo if "%%OUT_FILE%%" == "" set OUT_FILE=%%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
echo echo [%%date%% %%time%%] %%~1^>^> %%OUT_FILE%%>> %NEW_BAT_FILE%
echo exit /b>> %NEW_BAT_FILE%
echo.>> %NEW_BAT_FILE%
echo rem ######################################################################################>> %NEW_BAT_FILE%
echo rem Generate query file for wevtutil utility. Takes three parameters:>> %NEW_BAT_FILE%
echo rem   1 - logname, with 'file://' prefix if collect from archive>> %NEW_BAT_FILE%
echo rem   2 - file name, that contains list of providers>> %NEW_BAT_FILE%
echo rem   3 - start timestamp>> %NEW_BAT_FILE%
echo rem ######################################################################################>> %NEW_BAT_FILE%
echo :create_query>> %NEW_BAT_FILE%
echo echo ^^^<QueryList^^^>^> %%QUERY_FILE%%>> %NEW_BAT_FILE%
echo echo ^^^<Query Id="0" Path="%%~1"^^^>^>^> %%QUERY_FILE%%>> %NEW_BAT_FILE%
echo echo ^^^<Select Path="%%~1"^^^>*^>^> %%QUERY_FILE%%>> %NEW_BAT_FILE%
echo if not "%%~2" == "" ^(>> %NEW_BAT_FILE%
echo    echo [System[^>^> %%QUERY_FILE%%>> %NEW_BAT_FILE%
echo ^) else if not "%%~3" == "" ^(>> %NEW_BAT_FILE%
echo    echo [System[^>^> %%QUERY_FILE%%>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo if not "%%~2" == "" ^(>> %NEW_BAT_FILE%
echo    echo Provider[^>^> %%QUERY_FILE%%>> %NEW_BAT_FILE%
echo    set FF=^1>> %NEW_BAT_FILE%
echo    for /f "delims=" %%%%a in ^('type "%%WORK_TMP_DIR%%\%%~2"'^) do ^(>> %NEW_BAT_FILE%
echo        call :add_provider "%%%%a">> %NEW_BAT_FILE%
echo    ^)>> %NEW_BAT_FILE%
echo    echo ]^>^> %%QUERY_FILE%%>> %NEW_BAT_FILE%
echo    if not "%%~3" == "" ^(>> %NEW_BAT_FILE%
echo       echo and^>^> %%QUERY_FILE%%>> %NEW_BAT_FILE%
echo    ^)>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo if not "%%~3" == "" ^(>> %NEW_BAT_FILE%
echo    echo TimeCreated[@SystemTime ^^^&gt; '%%~3']^>^> %%QUERY_FILE%%>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo if not "%%~2" == "" ^(>> %NEW_BAT_FILE%
echo    echo ]]^>^> %%QUERY_FILE%%>> %NEW_BAT_FILE%
echo ^) else if not "%%~3" == "" ^(>> %NEW_BAT_FILE%
echo    echo ]]^>^> %%QUERY_FILE%%>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo echo ^^^</Select^^^>^>^> %%QUERY_FILE%%>> %NEW_BAT_FILE%
echo echo ^^^</Query^^^>^>^> %%QUERY_FILE%%>> %NEW_BAT_FILE%
echo echo ^^^</QueryList^^^>^>^> %%QUERY_FILE%%>> %NEW_BAT_FILE%
echo exit /b ^0>> %NEW_BAT_FILE%
echo.>> %NEW_BAT_FILE%
echo rem ######################################################################################>> %NEW_BAT_FILE%
echo :add_provider>> %NEW_BAT_FILE%
echo if %%FF%% EQU 1 ^(>> %NEW_BAT_FILE%
echo    echo @Name='%%~1'^>^> %%QUERY_FILE%%>> %NEW_BAT_FILE%
echo ^) else ^(>> %NEW_BAT_FILE%
echo    echo or @Name='%%~1'^>^> %%QUERY_FILE%%>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo set FF=^2>> %NEW_BAT_FILE%
echo exit /b ^0>> %NEW_BAT_FILE%

call :print_message "Target collect script '%COLLECT_SCRIPT%' generated."
exit /b


rem ######################################################################################
rem Printing routine. Prints a message to log. Takes two parameters:
rem   1 - message itself
rem   2 - name of a log file. if empty, use default value.
rem ######################################################################################

:print_message

(set OUT_FILE=%~2)
if "%OUT_FILE%" == "" set OUT_FILE=%ES_LOG%
if "%~1" == "" (
   echo.>> "%OUT_FILE%"
) else (
   echo [%date% %time%] %~1>> "%OUT_FILE%"
)
exit /b


rem ######################################################################################
rem Copy content to logfile
rem ######################################################################################

:print2log

if exist "%~1" (
   for /f "delims=" %%I in ('type "%~1" 2^>nul') do (
       call :print_message "%HOST_NAME%: %%I"
   )
)
exit /b


rem ######################################################################################
rem Copy files to the target system for execution
rem ######################################################################################

:copy2target

set SRC_DIR=%~1
shift
:nextfile
set FILENAME=%~1
if not "%FILENAME%" == "" (
    copy /b /y "%SRC_DIR%\%FILENAME%" "%COLLECT_DIR%"
    if errorlevel 1 (
       call :print_message "ERROR: Failure to copy '%FILENAME%' to the target system."
       exit /b 1
    ) else (
       call :print_message "File '%FILENAME%' successfully copied to the target system."
    )
    shift
    goto nextfile
)
exit /b 0


rem ######################################################################################
rem Cleanup routine. Deleting temp directories and unmaping share
rem ######################################################################################

:cleanup

call :print_message ""
call :print_message "Free and clean up all used system resources."
..\bin\cmtool.exe -d "%USER_PWD%" >nul

if %EXIT_CODE% EQU 2 (
   set RM=FAILURE
) else (
   set RM=SUCCESS
   move /y "%TEMP_PROPS_FILE%" "%PROPS_FILE%" >nul 2>>%ES_LOG%
   if errorlevel 1 (
      call :print_message "ERROR: Failure to save current state to properties file."
      set RM=FAILURE
      set EXIT_CODE=2
   )
)
if not "%COLLECT_DIR%" == "" (
   if exist "%COLLECT_DIR%\" (
      rmdir /s /q "%COLLECT_DIR%" >nul 2>>%ES_LOG%
      if exist "%COLLECT_DIR%\" (
         call :print_message "WARNING: Failure to remove temporary working directory on target system '%COLLECT_DIR%'."
      )
   )
)
if not "%SHARE_NAME%" == "" (
   net use | find "%SHARE_NAME%" >nul
   if not errorlevel 1 (
      net use "%SHARE_NAME%" /delete >nul 2>>%ES_LOG%
      if errorlevel 1 (
         call :print_message "WARNING: Failure to unmap network share of the target system."
      )
   )
)
if not "%WORK_TMP_DIR%" == "" (
   if exist "%WORK_TMP_DIR%\" (
      rmdir /s /q "%WORK_TMP_DIR%" >nul 2>>%ES_LOG%
      if exist "%WORK_TMP_DIR%\" (
         call :print_message "WARNING: Failure to remove existing temporary working directory '%WORK_TMP_DIR%'."
      )
   )
)
call :print_message "Processing finished [%RM%]."
exit /b
