@echo off

REM Licensed Materials - Property of IBM
REM 5724-S67
REM (c) Copyright IBM Corp. 1998, 2008   All Rights Reserved.
REM US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
set TEMP_FILE1=%~1
set TEMP_FILE2=%~2
set CLEANUP=%~3
set AUDIT_NAME=%~4
set FILE_PATH=%~5
set INSTANCE=%~6
set USER_NAME=%~7
set PASSWORD=%~8
for %%I in (%*) do set PROPS_FILE=%%~I
for %%I in (%PROPS_FILE%) do set PROPS_NAME=%%~nI
set WORK_TMP_DIR=%PROPS_NAME%-ES.tmp
set PRINTING=enabled

rem -- sets actuator log name
if "%PROPS_FILE%" == "" (
   set ES_LOG="..\log\gettsmplogs.err"
) else (
   set ES_LOG="..\log\%PROPS_NAME%.log"
)

type nul > %ES_LOG%

if "%TEMP_FILE1%" == "" (
   call :print_message "ERROR: Missing logset #1 name argument." %ES_LOG%
   goto EXIT_ERROR
) else (
   type nul > "%TEMP_FILE1%" 2>>%ES_LOG%
)           
if "%TEMP_FILE2%" == "" (
   call :print_message "ERROR: Missing logset #2 name argument." %ES_LOG%
   goto EXIT_ERROR
) else (
   type nul > "%TEMP_FILE2%" 2>>%ES_LOG%
)

rem -- Property file PROPS_FILE argument is missing
if "%PROPS_FILE%" == "" (
   call :print_message "ERROR: Missing property filename argument." %ES_LOG%
   goto EXIT_ERROR
)
if "%AUDIT_NAME%" == "" (
   call :print_message "ERROR: Missing audit name argument." %ES_LOG%
   goto EXIT_ERROR
)
if "%FILE_PATH%" == "" (
   call :print_message "ERROR: Missing logs file path argument." %ES_LOG%
   goto EXIT_ERROR
)
if "%USER_NAME%" == "" (
   call :print_message "ERROR: Missing username argument." %ES_LOG%
   goto EXIT_ERROR
)

call :print_message "Processing started." %ES_LOG%

if not exist "%FILE_PATH%\" (
   call :print_message "ERROR: Audit logs directory not found." %ES_LOG%
   call :print_message "NOTE: If directory exists, please make sure that Agent process has necessary rights to acces it and perform collect again." %ES_LOG%
   goto EXIT_ERROR
)

if exist "%WORK_TMP_DIR%\" (
   rmdir /S /Q "%WORK_TMP_DIR%" >nul 2>>%ES_LOG%
   if exist "%WORK_TMP_DIR%\" (
      call :print_message "ERROR: Failure to remove existing temporary working directory: '%WORK_TMP_DIR%'." %ES_LOG%
      goto EXIT_ERROR
   )
)
mkdir "%WORK_TMP_DIR%" >nul 2>>%ES_LOG%
if errorlevel 1 (
   call :print_message "ERROR: Failure to create temporary working directory: '%WORK_TMP_DIR%'." %ES_LOG%
   goto EXIT_ERROR
)

rem -------------------------------------------------------------------------------------------------------------------
rem  Creates two lists of audit files: with ascending and  descending  order by date of creating, files are matched without prefix
set PATTEN="%AUDIT_NAME%_[0-9A-Z-]*_[0-9]*_[0-9]*.sqlaudit" 
dir /OD /A-D /B /S /TC "%FILE_PATH%" | findstr %PATTEN% > "%WORK_TMP_DIR%\listing.%PROPS_NAME%" 2>>%ES_LOG%
dir /O-D /A-D /B /S /TC "%FILE_PATH%" | findstr %PATTEN% > "%WORK_TMP_DIR%\reverslist.%PROPS_NAME%" 2>>%ES_LOG%
call :print_message "Created listing of available Audit files." %ES_LOG%

rem check for first collect and assign variables for secondary collecting
(set LINE_NUM=)
(set PREV_FILE=)
if exist "%PROPS_FILE%" (
   for /F "tokens=2 delims==" %%I in ('findstr LINENUM "%PROPS_FILE%" 2^>nul') do (set LINE_NUM=%%I)
   for /F "tokens=2 delims==" %%I in ('findstr PREVFILE "%PROPS_FILE%" 2^>nul') do (set PREV_FILE=%%I)
)

rem select only the data that we care about
set BCP_SELECT=SELECT event_time, action_id, class_type, REPLACE( REPLACE( REPLACE( REPLACE( REPLACE( CONVERT( NVARCHAR(1000), statement), '          ', ' '), '     ', ' '), CHAR(10), ' '), CHAR(9), ' '),CHAR(13), ' '), server_principal_name, succeeded, server_instance_name, database_name, object_name

if "%PREV_FILE%" == "" goto FC
if "%LINE_NUM%" == "" goto FC

rem check: if PREV_FILE is valid
echo "%PREV_FILE%" | findstr %PATTEN% 2>>%ES_LOG%
if errorlevel 1 (
  call :print_message "Warning: Property file is corrupted."  %ES_LOG%
  goto FC
)

rem check: if listing is not empty
for %%I in ("%WORK_TMP_DIR%\listing.%PROPS_NAME%") do (set FILE_SIZE=%%~zI)
if %FILE_SIZE% EQU 0 (
   call :print_message "WARNING: There are no audit files in the pointed directory: '%FILE_PATH%'." %ES_LOG%
   goto EXIT_OK
) else (
   rem Check connection with server
   for /F "delims=," %%I IN ('type "%WORK_TMP_DIR%\listing.%PROPS_NAME%" 2^>nul') do ( 
       if %%~zI NEQ 0 (
          bcp.exe "%BCP_SELECT% FROM sys.fn_get_audit_file ('%%I', default, default)" QUERYOUT "%WORK_TMP_DIR%\tmp.txt" -U"%USER_NAME%" -P"%PASSWORD%" -S"%INSTANCE%" -f../bin/sqlserverauditformat.fmt -F 1 -L 2 >nul 2>>%ES_LOG%
		  
		  goto NEXT1
       )
   )
)
rem if all audit files are empty
call :print_message "WARNING: Could not check connection to MS SQL Server: audit file are empty." %ES_LOG%
goto EXIT_OK

:NEXT1
if errorlevel 1 (
    rem Try to check the connection again in case there is a audit file, that doesn't contain any records
    for /F "delims=," %%I IN ('type "%WORK_TMP_DIR%\listing.%PROPS_NAME%" 2^>nul') do (
        bcp.exe "%BCP_SELECT% FROM sys.fn_get_audit_file ('%%I', default, default)" QUERYOUT "%WORK_TMP_DIR%\tmp.txt" -U"%USER_NAME%" -P"%PASSWORD%" -f../bin/sqlserverauditformat.fmt >nul 2>>%ES_LOG%

		goto NO_CONNECTION1
    )

:NO_CONNECTION1
    if errorlevel 1 (
       call :print_message "ERROR: Could not connect to MS SQL Server." %ES_LOG%
       goto EXIT_ERROR
    )
)
rem End check

rem create temporary list for deleting and packing of files, set old LAST_LINE value
call :print_message "Events will be collected from record number %LINE_NUM% of '%PREV_FILE%' file." %ES_LOG%

type nul > "%WORK_TMP_DIR%\list" 2>>%ES_LOG%
set /a OLD=%LINE_NUM%-1

rem check if exist trace file that is equal to prev_file
for /F "delims=," %%I IN ('type "%WORK_TMP_DIR%\listing.%PROPS_NAME%" 2^>nul') do (
    if "%PREV_FILE%" == "%%I" (
       if %%~zI NEQ 0 (
          goto EXIST_AUDIT
       )
    )
)
rem if there is no trc that is equal to prev_file-- take file that is greather than prev_file and check it for its size
call :print_message "WARNING: Previous last handled audit file was deleted." %ES_LOG%
for /F "delims=," %%I IN ('type "%WORK_TMP_DIR%\listing.%PROPS_NAME%" 2^>nul') do (
    if "%PREV_FILE%" LSS "%%I" ( 
       if %%~zI NEQ 0 (
          set PREV_FILE=%%I
          set /a LINE_NUM=1
          set /a OLD=0
          goto EXIST_AUDIT
       )
    )
)
rem if there are no any trc greather than prev_file
call :print_message "WARNING: Could not find new non empty audit file." %ES_LOG%
goto EXIT_OK

:EXIST_AUDIT
rem obtain records from prev_file: take file that is equal to prev_file and copy records from all files below in listing
set TRCCOPY=0
for /F "delims=," %%I IN ('type "%WORK_TMP_DIR%\listing.%PROPS_NAME%" 2^>nul') do (
    call :ALLOW_COPY
    if errorlevel 1 (
       if %%zI NEQ 0 (
          bcp.exe "%BCP_SELECT% FROM sys.fn_get_audit_file ('%%I', default, default)" QUERYOUT "%WORK_TMP_DIR%\%%~nI.txt" -U"%USER_NAME%" -P"%PASSWORD%" -S"%INSTANCE%" -f../bin/sqlserverauditformat.fmt >"%WORK_TMP_DIR%\bcpout.txt" 2>>%ES_LOG%

		  if errorlevel 1 (
             call :print_message "WARNING: File '%%~nxI' was corrupted." %ES_LOG%
          ) else (
             call :print_message "Collected data from Audit file '%%~nxI'." %ES_LOG%
          )
          echo "%%I">> "%WORK_TMP_DIR%\list"
          copy /b "%TEMP_FILE1%" + "%WORK_TMP_DIR%\%%~nI.txt" "%TEMP_FILE1%" >nul 2>>%ES_LOG%
          if errorlevel 1 (
             call :print_message "ERROR: Failure to copy collected events into first logset file." %ES_LOG%
             goto EXIT_ERROR
          )
          del /Q "%WORK_TMP_DIR%\%%~nI.txt" >nul 2>>%ES_LOG%
          set /a OLD=0
          set LAST_AUDIT_FILE=%%I
       )
    )
    if "%PREV_FILE%" == "%%I" (
       bcp.exe "%BCP_SELECT% FROM sys.fn_get_audit_file ('%%I', default, default)" QUERYOUT "%TEMP_FILE1%" -U"%USER_NAME%" -P"%PASSWORD%" -S"%INSTANCE%" -f../bin/sqlserverauditformat.fmt -F%LINE_NUM% >"%WORK_TMP_DIR%\bcpout.txt" 2>>%ES_LOG%

	   if errorlevel 1 (
          call :print_message "WARNING: File '%%~nxI' was corrupted." %ES_LOG%
       ) else (
          call :print_message "Collected data from Audit file '%%~nxI'." %ES_LOG%
       )
       rem create a list of files need to be archived and deleted
       echo "%%I">> "%WORK_TMP_DIR%\list"
       set LAST_AUDIT_FILE=%%I
       set TRCCOPY=1
    ) 
)

rem take number of copied rows from bcpout file
set /a LINE_NUM=0
for /F "tokens=1" %%I IN ('findstr /C:"rows copied" "%WORK_TMP_DIR%\bcpout.txt" 2^>nul') do (set LINE_NUM=%%I) 

if "%PREV_FILE%" == "%LAST_AUDIT_FILE%" (
   rem only the last file was collected
   if %LINE_NUM% EQU 0 (
      call :print_message "WARNING: There are no new events or audit files were corrupted." %ES_LOG%
      goto EXIT_OK
   )
) 

set /a LINE_NUM=LINE_NUM+OLD+1
echo PREVFILE=%LAST_AUDIT_FILE%> %PROPS_FILE%
echo LINENUM="%LINE_NUM%">> %PROPS_FILE%
goto TAR

rem -------------------------------------------------------------------------------------------------------------------
:FC
rem check if listing file is not empty
call :print_message "Initial collect mode use." %ES_LOG%
for %%I in ("%WORK_TMP_DIR%\listing.%PROPS_NAME%") do (set FILE_SIZE=%%~zI)
 
if %FILE_SIZE% EQU 0 (
   echo PREVFILE=000000000000.sqlaudit>%PROPS_FILE%
   echo LINENUM="1">>%PROPS_FILE%
   call :print_message "WARNING: There are no audit files in the pointed directory: '%FILE_PATH%'." %ES_LOG%
   goto EXIT_OK
) else (
   rem check connection with MS SQL server
   for /F "delims=," %%I IN ('type "%WORK_TMP_DIR%\listing.%PROPS_NAME%" 2^>nul') do ( 
       if %%~zI NEQ 0 (
          bcp.exe "%BCP_SELECT% FROM sys.fn_get_audit_file ('%%I', default, default)" QUERYOUT "%WORK_TMP_DIR%\tmp.txt" -U"%USER_NAME%" -P"%PASSWORD%" -S"%INSTANCE%" -f../bin/sqlserverauditformat.fmt -F 1 -L 2 2>>%ES_LOG%

		  goto NEXT2
       )
   )
)
	  
call :print_message "ERROR: Could not check connection to MS SQL Server: there are no non empty audit in the pointed directory '%FILE_PATH%'." %ES_LOG%
goto EXIT_ERROR

:NEXT2
if errorlevel 1 (
   rem Try to check the connection again in case there is a trace file, that doesn't contain any records
   for /F "delims=," %%I IN ('type "%WORK_TMP_DIR%\listing.%PROPS_NAME%" 2^>nul') do (
       bcp.exe "%BCP_SELECT% FROM sys.fn_get_audit_file ('%%I', default, default)" QUERYOUT "%WORK_TMP_DIR%\tmp.txt" -U"%USER_NAME%" -P"%PASSWORD%" -f../bin/sqlserverauditformat.fmt 2>>%ES_LOG%

	   goto NO_CONNECTION2
   )

:NO_CONNECTION2
   if errorlevel 1 (
      call :print_message "ERROR: Could not connect to MS SQL Server." %ES_LOG%
      goto EXIT_ERROR
   )
)
rem End check
type nul > "%WORK_TMP_DIR%\list" 2>>%ES_LOG%
rem take the latest trace file from listing and if its size is not equal to NULL copy log records to TEMP1
for /F "delims=," %%I IN ('type "%WORK_TMP_DIR%\reverslist.%PROPS_NAME%" 2^>nul') do ( 
    if %%~zI NEQ 0 (
	   bcp.exe "%BCP_SELECT% FROM sys.fn_get_audit_file ('%%I', default, default)" QUERYOUT "%TEMP_FILE1%" -U"%USER_NAME%" -P"%PASSWORD%" -S"%INSTANCE%" -f../bin/sqlserverauditformat.fmt >"%WORK_TMP_DIR%\bcpout.txt" 2>>%ES_LOG%

	   if errorlevel 1 (
          call :print_message "WARNING: File '%%~nxI' was corrupted." %ES_LOG%
       ) else (
          call :print_message "Collected data from Audit file '%%~nxI'." %ES_LOG%
       )
       echo "%%I">> "%WORK_TMP_DIR%\list"
       set LAST_AUDIT_FILE=%%I
       goto NEXT3
    ) else (
       call :print_message "WARNING: File %%I is empty." %ES_LOG%
    )
)

call :print_message "WARNING: Could not find non empty files." %ES_LOG%
goto EXIT_OK

:NEXT3  
for /F "tokens=1" %%I IN ('findstr /C:"rows copied" "%WORK_TMP_DIR%\bcpout.txt" 2^>nul') do (set LINE_NUM=%%I)
rem check for LINE_NUM: line_num must be different from 0
if "%LINE_NUM%" == "" (
   call :print_message "WARNING: There are no events or audit file was corrupted." %ES_LOG%
   set /a LINE_NUM=0
)

echo PREVFILE=%LAST_AUDIT_FILE%>%PROPS_FILE%
set /a LINE_NUM=LINE_NUM+1
echo LINENUM="%LINE_NUM%">>%PROPS_FILE%
goto TAR

rem -------------------------------------------------------------------------------------------------------------------
:TAR
rem Make an archive with trc files
set BSDTAR_OPTS=-cf
for /F "delims=," %%I in ('type "%WORK_TMP_DIR%\list" 2^>nul') do (
    call :ARCHIVE %%I
)
call :print_message "Original logs archive created." %ES_LOG%
goto T

rem -------------------------------------------------------------------------------------------------------------------
:T
if "%CLEANUP%" == "yes" (
   call :print_message "Cleanup requested. Cleanning..." %ES_LOG%
   for /F "delims=," %%I in ('type "%WORK_TMP_DIR%\list" 2^>nul') do (
       if not %%I == "%LAST_AUDIT_FILE%" (
          del /Q /F %%I 2>>%ES_LOG%
          call :print_message "Deleting file %%I..." %ES_LOG%
       )
   )
   call :print_message "Cleanup finished." %ES_LOG%
)
goto EXIT_OK

rem -------------------------------------------------------------------------------------------------------------------
:ARCHIVE
..\bin\bsdtar.exe %BSDTAR_OPTS% "%TEMP_FILE2%" %1 >nul 2>>%ES_LOG%
if errorlevel 1 (
   call :print_message "ERROR: Failure to create tarball from original logs into second logset file." %ES_LOG%
   goto EXIT_ERROR
)
set BSDTAR_OPTS=-rf
exit /b

rem -------------------------------------------------------------------------------------------------------------------
:ALLOW_COPY
if %TRCCOPY% EQU 1 (
   exit /b 1
) else (
   exit /b 0
)

rem -------------------------------------------------------------------------------------------------------------------
:print_message
if "%PRINTING%" == "enabled" @echo [%date%][%time%] %~1>> %2
exit /b

rem -------------------------------------------------------------------------------------------------------------------
:EXIT_ERROR
if exist "%WORK_TMP_DIR%\" (
   rmdir /S /Q "%WORK_TMP_DIR%" >nul 2>>%ES_LOG%
   if exist "%WORK_TMP_DIR%\" (
      call :print_message "WARNING: Failure to remove existing temporary working directory: '%WORK_TMP_DIR%'." %ES_LOG%
   )
)
call :print_message "Processing finished [FAILURE]." %ES_LOG%
exit /b 1

rem -------------------------------------------------------------------------------------------------------------------
:EXIT_OK
if exist "%WORK_TMP_DIR%\" (
   rmdir /S /Q "%WORK_TMP_DIR%" >nul 2>>%ES_LOG%
   if exist "%WORK_TMP_DIR%\" (
      call :print_message "WARNING: Failure to remove existing temporary working directory: '%WORK_TMP_DIR%'." %ES_LOG%
   )
)
call :print_message "Processing finished [SUCCESS]." %ES_LOG%
exit /b 0
