rem IBM Confidential
rem OCO Source Material
rem 5724-S67
rem (c) Copyright IBM Corp. 1998, 2010
rem The source code for this program is not published or otherwise divested of
rem its trade secret, irrespective of what has been deposited with the U.S.
rem Copyright Office.

@echo off

rem ######################################################################################
rem Receives parameters from the Actuator
rem ######################################################################################
rem -- File for saving tarball with collected original evtx-logfiles
setlocal
set ORIGINAL_CHUNK=%~1
rem -- File for saving exported events from all collected logfiles in XML format
set EXPORTED_CHUNK=%~2
set USER_NAME=%~3
set USER_PWD=%~4
set WIN_LOG_FILE=Security
for %%I in (%*) do set PROPS_FILE=%%~I
for %%I in (%PROPS_FILE%) do set PROPS_NAME=%%~nI

rem -- sets ES log name
if "%PROPS_FILE%" == "" (
   set ES_LOG="..\log\~n0-error.log"
) else (
   set ES_LOG="..\log\%PROPS_NAME%.log"
)

rem -- clears log file and start logging
type nul > %ES_LOG%

rem ######################################################################################
rem Actuator parameters check
rem ######################################################################################
set PRINTING=enabled

rem -- one or both logset files arguments are missing
set CHUNKNAME_EMPTY=0
if "%ORIGINAL_CHUNK%" == "" (
   call :print_message "ERROR: Missing output file name argument for sublog #1 of Windows %WIN_LOG_FILE% audit logs."
   set CHUNKNAME_EMPTY=1
) else (
   type nul > "%ORIGINAL_CHUNK%" 2>>%ES_LOG%
)
if "%EXPORTED_CHUNK%" == "" (
   call :print_message "ERROR: Missing output file name argument for sublog #2 of Windows %WIN_LOG_FILE% audit logs."
   set CHUNKNAME_EMPTY=1
) else (
   type nul > "%EXPORTED_CHUNK%" 2>>%ES_LOG%
)
if %CHUNKNAME_EMPTY% EQU 1 goto failure

rem -- Properties file PROPS_FILE argument is missing
if "%PROPS_FILE%" == "" (
   call :print_message "ERROR: Missing properties file argument."
   goto failure
)
if not exist "%PROPS_FILE%" (
   call :print_message "ERROR: Can't find properties file '%PROPS_FILE%'."
   goto failure
)

call :print_message "Processing started."
call :parse_props_file

if "%HOST_NAME%" == "" (
   call :print_message "ERROR: 'SystemName' variable is not found in '%PROPS_FILE%' file."
   goto failure
)

set TSIEM_BIN_DIR=%~dp0%
set TSIEM_BIN_DIR=%TSIEM_BIN_DIR:~0,-1%
if "%TSIEM_TMP_DIR%" == "" (
   set WORK_TMP_DIR=%TEMP%
) else (
   set WORK_TMP_DIR=%TSIEM_TMP_DIR%
)
if "%WORK_TMP_DIR%" == "" (
   set WORK_TMP_DIR=%SystemRoot%\Temp
)
if not "%WORK_TMP_DIR:~-1%" == "\" set WORK_TMP_DIR=%WORK_TMP_DIR%\
set WORK_TMP_DIR=%WORK_TMP_DIR%ES-%PROPS_NAME%
set COLLECT_SCRIPT=eventscollect.bat
set COLLECT_PROGRESS_FILE=collectprogress.txt
set TIMESTAMP_FILE=timestamp.txt
set COLLECT_ERROR_FILE=collecterror.txt
set INITIAL_COLLECT_DAYS=2
set PROC_ID_INFO=procID.txt
set PROC_LIST=processlist.txt
Rem -- creates temp directory
call :print_message "Create temporary working directory '%WORK_TMP_DIR%'."
if exist "%WORK_TMP_DIR%\" (
   rmdir /s /q "%WORK_TMP_DIR%" >nul 2>>%ES_LOG%
   if exist "%WORK_TMP_DIR%\" (
      call :print_message "ERROR: Failure to remove existing temporary working directory '%WORK_TMP_DIR%'."
      goto failure
   )
)
mkdir "%WORK_TMP_DIR%" >nul 2>>%ES_LOG%
if errorlevel 1 (
   call :print_message "ERROR: Failure to create temporary working directory '%WORK_TMP_DIR%'."
   goto failure
)

call :check_remote_collect || goto failure

if %REMOTE_COLLECT% EQU 1 (
   if "%USER_NAME%" == "" (
      call :print_message "ERROR: Missing username for accessing target system."
      goto failure
   )
)

call :print_message "- Start preparing environment for the collect process."

call :create_collect_script

set COLLECT_ERROR=0
if %REMOTE_COLLECT% EQU 0 goto localcollect
   for /f "delims=" %%I in ('..\bin\cmtool.exe -k -d "%USER_PWD%" 2^>^>%ES_LOG%') do set PASSWORD=%%I
   call ..\bin\mapnetworkshare.bat "%HOST_NAME%" "ADMIN$" "%PASSWORD% /u:%USER_NAME%" >> %ES_LOG% 2>&1 || goto failure
   rem SHARE_DISK assigned by previous call
   set COLLECT_DIR=%SHARE_DISK%\Temp\ES-%PROPS_NAME%

   if exist "%COLLECT_DIR%\" (
      rmdir /s /q "%COLLECT_DIR%" >nul 2>>%ES_LOG%
      if exist "%COLLECT_DIR%\" (
         call :print_message "ERROR: Failure to remove existing temporary working directory on target system '%COLLECT_DIR%'."
         goto failure
      )
   )
   call :print_message "Create temporary working directory '%COLLECT_DIR%' on the target system."
   mkdir "%COLLECT_DIR%" >nul 2>>%ES_LOG%
   if errorlevel 1 (
      call :print_message "ERROR: Failure to create temporary working directory on target system '%COLLECT_DIR%'."
      goto failure
   )

   call :print_message "Copy necessary execution files to the target system."
   rem -- Copying collect script to the target system
   call :copy2target "%WORK_TMP_DIR%" "%COLLECT_SCRIPT%" || goto failure

   rem -- Copying necessary binary files to the target system
   call :copy2target "%TSIEM_BIN_DIR%" "archive1.dll bzip2.dll zlib1.dll bsdtar.exe" || goto failure

   rem -- Copying date.exe for getting timestamp on target system
   if "%START_TIMESTAMP%" == "" (
      call :copy2target "%TSIEM_BIN_DIR%" "msvcr71.dll date.exe" || goto failure
   )

   rem -- Flag message for catching error during starting collect script
   call :print_message "ERROR: Failure during start collect script with PSEXEC." "%COLLECT_DIR%\%COLLECT_ERROR_FILE%"

   call :print_message "- Start the collect process."
   rem -- Start collect script on target system and getting result files from target system
   set WMIC=%SystemRoot%\System32\wbem\wmic.exe
   
   if not exist %WMIC% (
     set WMIC=%SystemRoot%\sysnative\wbem\wmic.exe
	 if not exist %WMIC% (
	   call :print_message "ERROR : wmic.exe not found. Unable to proceed with collect.
	   goto failure   
     )
   )	 
 
   %WMIC% /NODE:"%HOST_NAME%" /USER:"%USER_NAME%" /PASSWORD:"%PASSWORD%" PROCESS call create 'cmd.exe /c "%%SystemRoot%%\Temp\ES-%PROPS_NAME%\%COLLECT_SCRIPT%"' > "%PROC_ID_INFO%"
   if errorlevel 1 (  
    call :print_message "ERROR: Failure to execute collect script on remote machine %HOST_NAME%." || goto failure
   )
  
   for /f "tokens=2 delims=;= " %%A in ('findstr ProcessId "%PROC_ID_INFO%"') do (
     set PROC_ID=%%A
   )
   
   set TIMEOUT_COUNT=0
   
   :wmicloop
   timeout /t 1
   
   %WMIC% /NODE:"%HOST_NAME%" /USER:"%USER_NAME%" /PASSWORD:"%PASSWORD%" PROCESS WHERE PROCESSID="%PROC_ID%" > "%PROC_LIST%" 2>&1
   if errorlevel 1 (
     call :print_message "ERROR: Failure to query remote machine for running processes."
	 goto failure
   )
   findstr /c:"No Instance(s) Available." "%PROC_LIST%" >>nul 2>&1
   if errorlevel 1 (
    set /a TIMEOUT_COUNT+=1
	if %TIMEOUT_COUNT% geq 60 (
	 call :print_message "ERROR: Timeout while waiting for collect process to end."
	 goto failure
	)
	goto wmicloop
   )

   rem -- Copy collect progress log to main logfile
   call :print2log "%COLLECT_DIR%\%COLLECT_PROGRESS_FILE%"

   copy /Y "%COLLECT_DIR%\2008.tar" "%WORK_TMP_DIR%" >nul 2>>%ES_LOG%
   ..\bin\bsdtar.exe -C "%WORK_TMP_DIR%" -xf "%WORK_TMP_DIR%/2008.tar" >> %ES_LOG%
   rem -- Check for collect script errors on target system
   copy /Y "%COLLECT_DIR%\%COLLECT_ERROR_FILE%" "%WORK_TMP_DIR%\%COLLECT_ERROR_FILE%" >nul 2>>%ES_LOG%
   if errorlevel 1 (
      call :print_message "ERROR: Failure to copy errors report from the target system."
      goto failure
   ) else (
      for %%I in ("%WORK_TMP_DIR%\%COLLECT_ERROR_FILE%") do (
          if %%~zI GTR 0 set COLLECT_ERROR=1
      )
   )
   if %COLLECT_ERROR% EQU 1 (
      call :print2log "%COLLECT_DIR%\%COLLECT_ERROR_FILE%"
   )

   rem -- Get new timestamp from target system
   if exist "%COLLECT_DIR%\%TIMESTAMP_FILE%" (
      copy /Y "%COLLECT_DIR%\%TIMESTAMP_FILE%" "%WORK_TMP_DIR%\%TIMESTAMP_FILE%" >nul 2>>%ES_LOG%
      if errorlevel 1 (
         call :print_message "ERROR: Failure to get new timestamp from the target system."
         goto failure
      )
   ) else (
      goto collecterror
   )
   if exist "%COLLECT_DIR%\" (
      rmdir /s /q "%COLLECT_DIR%" >nul 2>>%ES_LOG%
      if exist "%COLLECT_DIR%\" (
         call :print_message "ERROR: Failure to remove temporary working directory on target system '%COLLECT_DIR%'."
         goto failure
      )
   )
   goto finish

:localcollect
   set COLLECT_DIR=%WORK_TMP_DIR%

   call :print_message "- Start the collect process."
   rem -- Start collect script
   call "%COLLECT_DIR%\%COLLECT_SCRIPT%"

   rem -- Check for collect script errors
   if exist "%COLLECT_DIR%\%COLLECT_ERROR_FILE%" (
      for %%I in ("%COLLECT_DIR%\%COLLECT_ERROR_FILE%") do (
          if %%~zI GTR 0 set COLLECT_ERROR=1
      )
   )
   if %COLLECT_ERROR% EQU 1 (
      type "%COLLECT_DIR%\%COLLECT_ERROR_FILE%" >> %ES_LOG%
   )
   if not exist "%COLLECT_DIR%\%TIMESTAMP_FILE%" goto collecterror
:finish

call :print_message "- Start postprocessing stage."
rem -- Calculate amount of collected events
call :print_message "Calculate the amount of the collected events."
for /f "tokens=*" %%I in ('find /C "<Event xmlns" "%WORK_TMP_DIR%\%WIN_LOG_FILE%.xml" 2^>nul') do set EVENTS_NUMBER=%%I
for %%I in (%EVENTS_NUMBER%) do set EVENTS_NUMBER=%%I
call :print_message "The total number of new collected events is %EVENTS_NUMBER%."

call :print_message "Copy original log contents to sublog #1."
rem -- Create tarball from collected events in evtx-files into first logset file
for /f %%I in ('cd') do set CUR_DIR=%%I
pushd "%WORK_TMP_DIR%"
"%TSIEM_BIN_DIR%\bsdtar.exe" -cf evtxs.tar *.evtx >nul 2>>"%CUR_DIR%"\%ES_LOG%
popd
if errorlevel 1 (
   call :print_message "ERROR: Failure to create tarball from collected events in evtx-files into first logset file."
   goto failure
)
move /Y "%WORK_TMP_DIR%\evtxs.tar" "%ORIGINAL_CHUNK%" >nul 2>>%ES_LOG%
if errorlevel 1 (
   call :print_message "ERROR: Failure to move collected events in XML format into second logset file."
   goto failure
)
call :print_message "Copy XML dump of the original log contents to sublog #2."
rem -- Move collected events in XML format into second logset file
move /Y "%WORK_TMP_DIR%\%WIN_LOG_FILE%.xml" "%EXPORTED_CHUNK%" >nul 2>>%ES_LOG%
if errorlevel 1 (
   call :print_message "ERROR: Failure to move collected events in XML format into second logset file."
   goto failure
)

call :print_message "Save the beginning timestamp for the next collect in the properties file."
for /f %%I in ('type "%WORK_TMP_DIR%\%TIMESTAMP_FILE%" 2^>nul') do set NEW_TIMESTAMP=%%I
if "%NEW_TIMESTAMP%" == "" (
   call :print_message "ERROR: Failure to get timestamp for last collected event."
   goto failure
) else (
   echo TIMESTAMP = %NEW_TIMESTAMP%> "%PROPS_FILE%"
)

call :cleanup
call :print_message "Processing finished [SUCCESS]."
exit /b 0


rem -- Collect script finished with error
:collecterror
if %COLLECT_ERROR% EQU 1 (
   call :print_message "ERROR: Failure during execution collect script."
) else (
   call :print_message "ERROR: There is not exist new timestamp on the target system."
)

rem -- prints failure message to actuator log and finishes the script with errorlevel 1 (FAILURE)
:failure

call :cleanup
call :print_message "Processing finished [FAILURE]."
exit /b 1


rem #########################################################################################
rem Parsing properties file PROPS_FILE for setting state variables
rem #########################################################################################

:parse_props_file

rem -- read value of the 'SystemName' parameter into HOST_NAME
for /f "tokens=2 delims==" %%I in ('findstr /R /C:"^SystemName = .*" "%PROPS_FILE%" 2^>^>%ES_LOG%') do set HOST_NAME=%%I
for %%I in (%HOST_NAME%) do set HOST_NAME=%%I
rem -- read value of the 'TIMESTAMP' parameter into START_TIMESTAMP
for /f "tokens=2 delims==" %%I in ('findstr /R /C:"^TIMESTAMP = .*" "%PROPS_FILE%" 2^>^>%ES_LOG%') do set START_TIMESTAMP=%%I
for %%I in (%START_TIMESTAMP%) do set START_TIMESTAMP=%%I
if "%START_TIMESTAMP%" == "" call :print_message "Initial collect mode use."
exit /b


rem #########################################################################################
rem Check collecting mode and set REMOTE_COLLECT variable
rem #########################################################################################

:check_remote_collect

for /f "tokens=1,* delims=|" %%a in ('cscript.exe //Nologo ..\bin\localorremote.vbs %HOST_NAME%') do (
    set RESULT1=%%a
    set RESULT2=%%b
)
if "%RESULT1%" == "LOCAL" (
   set REMOTE_COLLECT=0
   call :print_message "Collect from the local target system: '%HOST_NAME%'."
) else if "%RESULT1%" == "REMOTE" (
   set REMOTE_COLLECT=1
   set HOST_NAME=%RESULT2%
   call :print_message "Collect from the remote target system: '%HOST_NAME%'."
) else (
   call :print_message "ERROR: Script for determining mode of the collect return failure."
   if "%RESULT2%" == "" (
      call :print_message "%RESULT1%"
   ) else (
      call :print_message "%RESULT2%"
   )
   exit /b 1
)
exit /b 0


rem #########################################################################################
rem Creating collect script COLLECT_SCRIPT in WORK_TMP_DIR
rem #########################################################################################

:create_collect_script

set NEW_BAT_FILE="%WORK_TMP_DIR%\%COLLECT_SCRIPT%"
type nul > %NEW_BAT_FILE% 2>>%ES_LOG%

echo @echo off>> %NEW_BAT_FILE%
echo rem #################################### eventscollect.bat ####################################>> %NEW_BAT_FILE%
echo rem ### on success send in stdout tarball with resulting bziped XML-file and all evtx-files>> %NEW_BAT_FILE%
echo rem ### if failure return code 1 and explanation message in COLLECT_ERROR_FILE>> %NEW_BAT_FILE%
echo rem ###########################################################################################>> %NEW_BAT_FILE%
echo setlocal>> %NEW_BAT_FILE%
if %REMOTE_COLLECT% EQU 1 (
   echo ^(set WORK_TMP_DIR=%%SystemRoot%%\Temp\ES-%PROPS_NAME%^)>> %NEW_BAT_FILE%
   echo ^(set TSIEM_BIN_DIR=%%WORK_TMP_DIR%%^)>> %NEW_BAT_FILE%
   echo ^(set COLLECT_PROGRESS_FILE=^"%%WORK_TMP_DIR%%\%COLLECT_PROGRESS_FILE%^"^)>> %NEW_BAT_FILE%
   echo if ^"%%TSIEM_TMP_DIR%%^" == ^"^" ^(>> %NEW_BAT_FILE%
   echo    ^(set CHUNKS_TMP_DIR=%%WORK_TMP_DIR%%^)>> %NEW_BAT_FILE%
   echo ^) else (>> %NEW_BAT_FILE%
   echo    if ^"%%TSIEM_TMP_DIR:~-1%%^" == ^"\^" ^(>> %NEW_BAT_FILE%
   echo       set CHUNKS_TMP_DIR=%%TSIEM_TMP_DIR%%ES-%PROPS_NAME%>> %NEW_BAT_FILE%
   echo    ^) else (>> %NEW_BAT_FILE%
   echo       set CHUNKS_TMP_DIR=%%TSIEM_TMP_DIR%%\ES-%PROPS_NAME%>> %NEW_BAT_FILE%
   echo    ^)>> %NEW_BAT_FILE%
   echo ^)>> %NEW_BAT_FILE%
) else (
   echo ^(set WORK_TMP_DIR=%WORK_TMP_DIR%^)>> %NEW_BAT_FILE%
   echo ^(set TSIEM_BIN_DIR=%TSIEM_BIN_DIR%^)>> %NEW_BAT_FILE%
   echo ^(set COLLECT_PROGRESS_FILE=%ES_LOG%^)>> %NEW_BAT_FILE%
   echo ^(set CHUNKS_TMP_DIR=%%WORK_TMP_DIR%%^)>> %NEW_BAT_FILE%
)
echo ^(set TIMESTAMP_FILE=^"%%WORK_TMP_DIR%%\%TIMESTAMP_FILE%^"^)>> %NEW_BAT_FILE%
echo ^(set COLLECT_ERROR_FILE=^"%%WORK_TMP_DIR%%\%COLLECT_ERROR_FILE%^"^)>> %NEW_BAT_FILE%
echo type nul ^> %%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
echo if defined PROCESSOR_ARCHITEW6432 (>> %NEW_BAT_FILE%
echo    set TARGET_ARCH=%%PROCESSOR_ARCHITEW6432%%>> %NEW_BAT_FILE%
echo ) else (>> %NEW_BAT_FILE%
echo    if defined PROCESSOR_ARCHITECTURE (>> %NEW_BAT_FILE%
echo       set TARGET_ARCH=%%PROCESSOR_ARCHITECTURE%%>> %NEW_BAT_FILE%
echo    )>> %NEW_BAT_FILE%
echo )>> %NEW_BAT_FILE%
echo if not defined TARGET_ARCH (>> %NEW_BAT_FILE%
echo    call :print_message ^"ERROR: Cannot define target system processor architecture.^" %%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
echo    exit /b ^1>> %NEW_BAT_FILE%
echo )>> %NEW_BAT_FILE%
echo for /f ^"tokens=*^" %%%%I in (^'echo %%TARGET_ARCH%% ^^^| find /c ^"64^" 2^^^>nul^') do set ARCH_64=%%%%I>> %NEW_BAT_FILE%
echo if %%ARCH_64%% EQU 1 (>> %NEW_BAT_FILE%
if %REMOTE_COLLECT% EQU 1 (
echo    set WIN32_SYS_DIR=%%SystemRoot%%\System32>> %NEW_BAT_FILE%
) else (
echo    set WIN32_SYS_DIR=%%SystemRoot%%\sysnative>> %NEW_BAT_FILE%
)
echo    call :print_message ^"The operating system on the target system has 'x86-64' architecture.^">> %NEW_BAT_FILE%
echo ) else (>> %NEW_BAT_FILE%
echo    set WIN32_SYS_DIR=%%SystemRoot%%\System32>> %NEW_BAT_FILE%
echo    call :print_message ^"The operating system on the target system has 'x86' architecture.^">> %NEW_BAT_FILE%
echo )>> %NEW_BAT_FILE%
echo ^(set WIN_SYS_DIR=%%SystemRoot%%\System32^)>> %NEW_BAT_FILE%
echo ^(set WIN_LOG_DIR=%%WIN_SYS_DIR%%\winevt\Logs^)>> %NEW_BAT_FILE%
echo ^(set WIN32_LOG_DIR=%%WIN32_SYS_DIR%%\winevt\Logs^)>> %NEW_BAT_FILE%
echo ^(set WEVT_EXE_FILE=%%WIN32_SYS_DIR%%\wevtutil.exe^)>> %NEW_BAT_FILE%
echo ^(set ARCH_LOGS_LIST=%%WORK_TMP_DIR%%\archivedlogs.lst^)>> %NEW_BAT_FILE%
echo ^(set LAST_EVENT_FILE=^"%%WORK_TMP_DIR%%\lastevent.xml^"^)>> %NEW_BAT_FILE%
echo rem Setup temporary directory for result chunks if system variable TSIEM_TMP_DIR is defined>> %NEW_BAT_FILE%
echo if /i not ^"%%CHUNKS_TMP_DIR%%^" == ^"%%WORK_TMP_DIR%%^" ^(>> %NEW_BAT_FILE%
echo    call :print_message ^"Create temporary directory for chunks '%%CHUNKS_TMP_DIR%%'.^">> %NEW_BAT_FILE%
echo    if exist ^"%%CHUNKS_TMP_DIR%%\^" ^(>> %NEW_BAT_FILE%
echo       rmdir /s /q ^"%%CHUNKS_TMP_DIR%%^" ^>nul 2^>^>%%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
echo       if exist ^"%%CHUNKS_TMP_DIR%%\^" ^(>> %NEW_BAT_FILE%
echo          call :print_message ^"ERROR: Failure to remove existing temporary directory for chunks '%%CHUNKS_TMP_DIR%%'.^" %%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
echo          exit /b ^1>> %NEW_BAT_FILE%
echo       ^)>> %NEW_BAT_FILE%
echo    ^)>> %NEW_BAT_FILE%
echo    mkdir ^"%%CHUNKS_TMP_DIR%%^" ^>nul 2^>^>%%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
echo    if errorlevel 1 ^(>> %NEW_BAT_FILE%
echo       call :print_message ^"ERROR: Failure to create temporary working directory '%%CHUNKS_TMP_DIR%%'.^" %%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
echo       exit /b ^1>> %NEW_BAT_FILE%
echo    ^)>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo if not exist ^"%%WIN32_LOG_DIR%%\^" ^(>> %NEW_BAT_FILE%
echo    call :print_message ^"Folder ^'%%WIN32_LOG_DIR%%^' with %WIN_LOG_FILE% Logs not found on the target system. Unable to proceed further with the collect.^" %%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
echo    exit /b ^1>> %NEW_BAT_FILE%
echo ^) else ^(>> %NEW_BAT_FILE%
echo    call :print_message ^"Found folder with %WIN_LOG_FILE% Log on the target system: ^'%%WIN_LOG_DIR%%^'.^">> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo if not exist ^"%%WEVT_EXE_FILE%%^" ^(>> %NEW_BAT_FILE%
echo    call :print_message ^"^'%%WEVT_EXE_FILE%%^' Windows native utility for parsing %WIN_LOG_FILE% Log not found on the target system. Unable to proceed further with the collect.^" %%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
echo    exit /b ^1>> %NEW_BAT_FILE%
echo ^) else ^(>> %NEW_BAT_FILE%
echo    call :print_message ^"Found Windows native utility for parsing %WIN_LOG_FILE% Log on the target system: ^'%%WEVT_EXE_FILE%%^'.^">> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
if "%START_TIMESTAMP%" == "" (
   echo call :print_message ^"Calculate the timestamp, from which the events will be collected.^">> %NEW_BAT_FILE%
   echo set DATE_ERR=^0>> %NEW_BAT_FILE%
   echo for /f ^"tokens=*^" %%%%I in ^(^'^"^"%%TSIEM_BIN_DIR%%\date.exe^" ^"+%%%%Y-%%%%m-%%%%dT%%%%H:%%%%M:%%%%S.000Z^" -d ^"%INITIAL_COLLECT_DAYS% days ago^"^" 2^^^>^^^>%%%%COLLECT_ERROR_FILE%%%%^'^) do set START_TIMESTAMP=%%%%I>> %NEW_BAT_FILE%
   echo if errorlevel 1 set DATE_ERR=^1>> %NEW_BAT_FILE%
   echo if ^"%%START_TIMESTAMP%%^" == ^"^" set DATE_ERR=^1>> %NEW_BAT_FILE%
   echo if %%DATE_ERR%% EQU 1 ^(>> %NEW_BAT_FILE%
   echo    call :print_message ^"ERROR: Failure to get timestamp for initial collect.^" %%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
   echo    exit /b ^1>> %NEW_BAT_FILE%
   echo ^)>> %NEW_BAT_FILE%
) else (
   echo ^(set START_TIMESTAMP=%START_TIMESTAMP%^)>> %NEW_BAT_FILE%
)
echo call :convert_timestamp ^"%%START_TIMESTAMP%%^">> %NEW_BAT_FILE%
echo call :print_message ^"Events will be collected from ^'%%START_TIMESTAMP%%^'.^">> %NEW_BAT_FILE%
echo rem -- Collecting events from newcreated archived logfiles>> %NEW_BAT_FILE%
echo set FIRST_LOG=^1>> %NEW_BAT_FILE%
echo set NEED_CHECK=^1>> %NEW_BAT_FILE%
echo if not exist ^"%%WIN32_LOG_DIR%%\Archive-%WIN_LOG_FILE%-*.evtx^" ^(>> %NEW_BAT_FILE%
echo    call :print_message ^"Archived %WIN_LOG_FILE% Log files were not found on the target system.^">> %NEW_BAT_FILE%
echo    goto end_cycle>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo :label^1>> %NEW_BAT_FILE%
echo set ATTEMPT=^0>> %NEW_BAT_FILE%
echo rem -- Cycle for checking archives created during processing previously existed ^(maximum 100 iteration, then generate error^)>> %NEW_BAT_FILE%
echo :start_cycle>> %NEW_BAT_FILE%
echo     if %%ATTEMPT%% EQU 100 ^(>> %NEW_BAT_FILE%
echo        call :print_message ^"ERROR: Logfiles archives are produced quicker than they can be collected. Please, change %WIN_LOG_FILE% Log archiving settings.^">> %NEW_BAT_FILE%
echo        call :print_message ^"INFO: Please increase the size of active logfile to avoid such situation in the future and repeat the collect.^">> %NEW_BAT_FILE%
echo        goto final_actions>> %NEW_BAT_FILE%
echo     ^)>> %NEW_BAT_FILE%
echo     set /a ATTEMPT+=^1>> %NEW_BAT_FILE%
echo     call :get_archives_list ^|^| goto end_cycle>> %NEW_BAT_FILE%
echo     for /f ^"delims=^" %%%%I in ^(^'type ^"%%ARCH_LOGS_LIST%%^" 2^^^>^^^>%%COLLECT_PROGRESS_FILE%%^'^) do ^(>> %NEW_BAT_FILE%
echo         set PREVIOUS_ARCHIVE=%%%%I>> %NEW_BAT_FILE%
if not "%START_TIMESTAMP%" == "" (
   echo         call :check_prev_events ^"%%%%I^" ^|^| goto final_actions>> %NEW_BAT_FILE%
)
echo         call :collect_archives ^"%%%%I^" ^|^| goto final_actions>> %NEW_BAT_FILE%
echo     ^)>> %NEW_BAT_FILE%
echo     goto start_cycle>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo :end_cycle>> %NEW_BAT_FILE%
if not "%START_TIMESTAMP%" == "" (
   echo call :check_prev_events ^"%WIN_LOG_FILE%^" ^|^| goto final_actions>> %NEW_BAT_FILE%
)
echo rem -- Collecting events from current active logfile>> %NEW_BAT_FILE%
echo call :print_message ^"Collect events from the currently active %WIN_LOG_FILE% Log: ^'%%WIN_LOG_DIR%%\%WIN_LOG_FILE%.evtx^'.^">> %NEW_BAT_FILE%
echo if exist ^"%%CHUNKS_TMP_DIR%%\%WIN_LOG_FILE%.evtx^" del /f /q ^"%%CHUNKS_TMP_DIR%%\%WIN_LOG_FILE%.evtx^" ^>nul 2^>^>%%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
echo if %%FIRST_LOG%% EQU 1 ^(>> %NEW_BAT_FILE%
echo    ^"%%WEVT_EXE_FILE%%^" epl ^"%WIN_LOG_FILE%^" ^"/q:*[System[TimeCreated[@SystemTime ^> ^'%%START_TIMESTAMP%%^']]]^" ^"%%CHUNKS_TMP_DIR%%\%WIN_LOG_FILE%.evtx^" ^>nul 2^>^>%%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
echo ^) else ^(>> %NEW_BAT_FILE%
echo    ^"%%WEVT_EXE_FILE%%^" epl ^"%WIN_LOG_FILE%^" ^"%%CHUNKS_TMP_DIR%%\%WIN_LOG_FILE%.evtx^" ^>nul 2^>^>%%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo if errorlevel 1 ^(>> %NEW_BAT_FILE%
echo    call :print_message ^"ERROR: Failure to collect events from the currently active %WIN_LOG_FILE% Log.^">> %NEW_BAT_FILE%
echo    del /f /q ^"%%CHUNKS_TMP_DIR%%\%WIN_LOG_FILE%.evtx^"
echo    goto final_actions>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo call :get_archives_list>> %NEW_BAT_FILE%
echo if not errorlevel 1 ^(>> %NEW_BAT_FILE%
echo    call :print_message ^"New archived logfile was created during collect events from the currently active %WIN_LOG_FILE% Log.^">> %NEW_BAT_FILE%
echo    goto start_cycle>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo :final_actions>> %NEW_BAT_FILE%
echo rem -- Export events from collected evtx-logfiles into one XML file>> %NEW_BAT_FILE%
echo call :print_message ^"Export collected events into XML format.^">> %NEW_BAT_FILE%
echo ^(set CURRENT_LOG=^)>> %NEW_BAT_FILE%
echo for /f ^"delims=^" %%%%I in ^(^'dir /B /ON ^"%%CHUNKS_TMP_DIR%%\*.evtx^" 2^^^>^^^>%%COLLECT_PROGRESS_FILE%%^'^) do ^(>> %NEW_BAT_FILE%
echo     ^"%%WEVT_EXE_FILE%%^" qe ^"%%CHUNKS_TMP_DIR%%\%%%%I^" /lf /f:xml ^>^> ^"%%CHUNKS_TMP_DIR%%\%WIN_LOG_FILE%.xml^" 2^>^>%%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
echo     if errorlevel 1 ^(>> %NEW_BAT_FILE%
echo        call :print_message ^"ERROR: Failure to export collected events into XML format from '%%%%I' logfile.^" %%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
echo        exit /b 1>> %NEW_BAT_FILE%
echo     ^)>> %NEW_BAT_FILE%
echo     set CURRENT_LOG=%%%%I>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo if ^"%%CURRENT_LOG%%^" == ^"^" ^(>> %NEW_BAT_FILE%
echo    call :print_message ^"ERROR: No logfiles were processed.^" %%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
echo    exit /b 1>> %NEW_BAT_FILE%
echo ^) else ^(>> %NEW_BAT_FILE%
echo    call :print_message ^"Calculate the timestamp for the next collect.^">> %NEW_BAT_FILE%
echo    rem -- Get last collected event info for timestamp>> %NEW_BAT_FILE%
echo    ^(set EVENT_SYSTEM_TIME=^)>> %NEW_BAT_FILE%
echo    ^"%%WEVT_EXE_FILE%%^" qe ^"%%CHUNKS_TMP_DIR%%\%%CURRENT_LOG%%^" /lf /rd:true /c:1 /f:xml ^> %%LAST_EVENT_FILE%% 2^>^>%%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
echo    if errorlevel 1 ^(>> %NEW_BAT_FILE%
echo       call :print_message ^"ERROR: Failure to export last event from collected logs.^" %%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
echo       exit /b ^1>> %NEW_BAT_FILE%
echo    ^)>> %NEW_BAT_FILE%
echo    call :parse_timestamp>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo if ^"%%EVENT_SYSTEM_TIME%%^" == ^"^" ^(>> %NEW_BAT_FILE%
echo    set EVENT_SYSTEM_TIME=%%START_TIMESTAMP%%>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo echo %%EVENT_SYSTEM_TIME%%^> %%TIMESTAMP_FILE%%>> %NEW_BAT_FILE%
echo call :print_message ^"Timestamp for the next collect is ^'%%EVENT_SYSTEM_TIME%%^'.^">> %NEW_BAT_FILE%
if %REMOTE_COLLECT% EQU 1 (
   echo rem -- Get all result files from target system>> %NEW_BAT_FILE%
   echo call :print_message ^"Send result files to destination system.^">> %NEW_BAT_FILE%
   echo set ERROR=^0>> %NEW_BAT_FILE%
   echo pushd ^"%%CHUNKS_TMP_DIR%%^">> %NEW_BAT_FILE%
   echo ^"%%WORK_TMP_DIR%%\bsdtar.exe^" -cjf "2008.tar" ^"%WIN_LOG_FILE%.xml^" ^"*.evtx^" 2^>^>%%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
   echo if errorlevel 1 set ERROR=^1>> %NEW_BAT_FILE%
   echo popd>> %NEW_BAT_FILE%
   echo if /i not ^"%%CHUNKS_TMP_DIR%%^" == ^"%%WORK_TMP_DIR%%^" ^(>> %NEW_BAT_FILE%
   echo    call :print_message ^"Remove temporary directory for chunks '%%CHUNKS_TMP_DIR%%'.^">> %NEW_BAT_FILE%
   echo    rmdir /s /q ^"%%CHUNKS_TMP_DIR%%^" ^>nul 2^>^>%%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
   echo    if exist ^"%%CHUNKS_TMP_DIR%%\^" ^(>> %NEW_BAT_FILE%
   echo       call :print_message ^"ERROR: Failure to remove temporary directory for chunks '%%CHUNKS_TMP_DIR%%'.^" %%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
   echo       exit /b ^1>> %NEW_BAT_FILE%
   echo    ^)>> %NEW_BAT_FILE%
   echo ^)>> %NEW_BAT_FILE%
   echo if %%ERROR%% EQU 1 ^(>> %NEW_BAT_FILE%
   echo    call :print_message ^"ERROR: Failure to send result files to destination system.^" %%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
   echo    exit /b ^1>> %NEW_BAT_FILE%
   echo ^)>> %NEW_BAT_FILE%
)
echo exit /b ^0>> %NEW_BAT_FILE%
echo.>> %NEW_BAT_FILE%
echo rem #########################################################################################>> %NEW_BAT_FILE%
echo rem Convert timestamp from ISO to archived logfile name format>> %NEW_BAT_FILE%
echo rem %%1  - ISO timestamp>> %NEW_BAT_FILE%
echo rem Result - set ARCH_LOG_TIME variable>> %NEW_BAT_FILE%
echo rem #########################################################################################>> %NEW_BAT_FILE%
echo :convert_timestamp>> %NEW_BAT_FILE%
echo for /f ^"tokens=1 delims=T^" %%%%I in ^(^"%%~1^"^) do set DATE_PART=%%%%I>> %NEW_BAT_FILE%
echo for /f ^"tokens=2 delims=T^" %%%%I in ^(^"%%~1^"^) do set TIME_PART=%%%%I>> %NEW_BAT_FILE%
echo for /f ^"tokens=1 delims=:^" %%%%I in ^(^"%%TIME_PART%%^"^) do set HOURS=%%%%I>> %NEW_BAT_FILE%
echo for /f ^"tokens=2 delims=:^" %%%%I in ^(^"%%TIME_PART%%^"^) do set MINUTES=%%%%I>> %NEW_BAT_FILE%
echo for /f ^"tokens=3 delims=:^" %%%%I in ^(^"%%TIME_PART%%^"^) do set SECOND_PART=%%%%I>> %NEW_BAT_FILE%
echo for /f ^"tokens=1 delims=.^" %%%%I in ^(^"%%SECOND_PART%%^"^) do set SECONDS=%%%%I>> %NEW_BAT_FILE%
echo for /f ^"tokens=2 delims=.^" %%%%I in ^(^"%%SECOND_PART%%^"^) do set MICROSECONDS=%%%%I>> %NEW_BAT_FILE%
echo set ARCH_LOG_TIME=%%DATE_PART%%-%%HOURS%%-%%MINUTES%%-%%SECONDS%%-%%MICROSECONDS%%>> %NEW_BAT_FILE%
echo exit /b>> %NEW_BAT_FILE%
echo.>> %NEW_BAT_FILE%
echo rem #########################################################################################>> %NEW_BAT_FILE%
echo rem Get list of archived logfiles for processing>> %NEW_BAT_FILE%
echo rem Result - list of necessary archived logfiles in ARCH_LOGS_LIST file>> %NEW_BAT_FILE%
echo rem #########################################################################################>> %NEW_BAT_FILE%
echo :get_archives_list>> %NEW_BAT_FILE%
echo if defined PREVIOUS_ARCHIVE ^(>> %NEW_BAT_FILE%
echo    set AN=%%PREVIOUS_ARCHIVE%%>> %NEW_BAT_FILE%
echo ^) else ^(>> %NEW_BAT_FILE%
echo    set AN=Archive-%WIN_LOG_FILE%-%%ARCH_LOG_TIME%%>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo type nul ^> ^"%%ARCH_LOGS_LIST%%^" 2^>^>%%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
echo for %%%%I in ^(^"%%WIN32_LOG_DIR%%\Archive-%WIN_LOG_FILE%-*.evtx^"^) do ^(>> %NEW_BAT_FILE%
echo     if ^"%%%%~nI^" GTR ^"%%AN%%^" ^(>> %NEW_BAT_FILE%
echo        echo %%%%~nI^>^> ^"%%ARCH_LOGS_LIST%%^">> %NEW_BAT_FILE%
echo     ^) else ^(>> %NEW_BAT_FILE%
echo        set NEED_CHECK=^0>> %NEW_BAT_FILE%
echo     ^)>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo for %%%%I in ^(^"%%ARCH_LOGS_LIST%%^"^) do if %%%%~zI EQU 0 exit /b ^1>> %NEW_BAT_FILE%
echo exit /b ^0>> %NEW_BAT_FILE%
echo.>> %NEW_BAT_FILE%
echo rem #########################################################################################>> %NEW_BAT_FILE%
echo rem Collect events from archived log files>> %NEW_BAT_FILE%
echo rem %%1 - Log name>> %NEW_BAT_FILE%
echo rem #########################################################################################>> %NEW_BAT_FILE%
echo :collect_archives>> %NEW_BAT_FILE%
echo call :print_message ^"Collect events from ^'%%WIN_LOG_DIR%%\%%~1.evtx^' %WIN_LOG_FILE% Log archive.^">> %NEW_BAT_FILE%
echo if %%FIRST_LOG%% EQU 1 ^(>> %NEW_BAT_FILE%
echo    ^"%%WEVT_EXE_FILE%%^" epl ^"%%WIN_LOG_DIR%%\%%~1.evtx^" /lf ^"/q:*[System[TimeCreated[@SystemTime ^> ^'%%START_TIMESTAMP%%^']]]^" ^"%%CHUNKS_TMP_DIR%%\%%~1.evtx^" ^>nul 2^>^>%%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
echo ^) else ^(>> %NEW_BAT_FILE%
echo    ^"%%WEVT_EXE_FILE%%^" epl ^"%%WIN_LOG_DIR%%\%%~1.evtx^" /lf ^"%%CHUNKS_TMP_DIR%%\%%~1.evtx^" ^>nul 2^>^>%%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo if errorlevel 1 ^(>> %NEW_BAT_FILE%
echo    call :print_message ^"ERROR: Failure to collect events from ^'%%WIN_LOG_DIR%%\%%~1.evtx^' Log.^">> %NEW_BAT_FILE%
echo    if exist ^"%%CHUNKS_TMP_DIR%%\%%~1.evtx^" del /f /q ^"%%CHUNKS_TMP_DIR%%\%%~1.evtx^"
echo    exit /b ^1>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo set FIRST_LOG=^0>> %NEW_BAT_FILE%
echo exit /b ^0>> %NEW_BAT_FILE%
if not "%START_TIMESTAMP%" == "" (
   echo.>> %NEW_BAT_FILE%
   echo rem #########################################################################################>> %NEW_BAT_FILE%
   echo rem Check for existing evenst from previous collect>> %NEW_BAT_FILE%
   echo rem %%1 - name of the current processed logfile>> %NEW_BAT_FILE%
   echo rem #########################################################################################>> %NEW_BAT_FILE%
   echo :check_prev_events>> %NEW_BAT_FILE%
   echo if %%NEED_CHECK%% EQU 0 exit /b ^0>> %NEW_BAT_FILE%
   echo if %%1 == ^"%WIN_LOG_FILE%^" ^(>> %NEW_BAT_FILE%
   echo    ^"%%WEVT_EXE_FILE%%^" qe %%1 /rd:false /c:1 /f:xml ^> %%LAST_EVENT_FILE%% 2^>^>%%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
   echo    if errorlevel 1 ^(>> %NEW_BAT_FILE%
   echo       call :print_message ^"ERROR: Failure to collect first event from current active %WIN_LOG_FILE% Log.^" %%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
   echo       exit /b ^1>> %NEW_BAT_FILE%
   echo    ^)>> %NEW_BAT_FILE%
   echo ^) else ^(>> %NEW_BAT_FILE%
   echo    ^"%%WEVT_EXE_FILE%%^" qe ^"%%WIN_LOG_DIR%%\%%~1.evtx^" /lf /rd:false /c:1 /f:xml ^> %%LAST_EVENT_FILE%% 2^>^>%%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
   echo    if errorlevel 1 ^(>> %NEW_BAT_FILE%
   echo       call :print_message ^"ERROR: Failure to collect first event from archived Log ^'%%WIN_LOG_DIR%%\%%~1.evtx^'.^" %%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
   echo       exit /b ^1>> %NEW_BAT_FILE%
   echo    ^)>> %NEW_BAT_FILE%
   echo    goto label^1>> %NEW_BAT_FILE%
   echo ^)>> %NEW_BAT_FILE%
   echo for /f ^"delims=^" %%%%I in ^(^'type ^"%%ARCH_LOGS_LIST%%^" 2^^^>nul^'^) do set LAST_ARCH=%%%%I>> %NEW_BAT_FILE%
   echo call :get_archives_list>> %NEW_BAT_FILE%
   echo set NEW_LAST_ARCH=%%LAST_ARCH%%
   echo if not errorlevel 1 ^(>> %NEW_BAT_FILE%
   echo    for /f ^"delims=^" %%%%I in ^(^'type ^"%%ARCH_LOGS_LIST%%^" 2^^^>nul^'^) do set NEW_LAST_ARCH=%%%%I>> %NEW_BAT_FILE%
   echo ^)>> %NEW_BAT_FILE%
   echo if not ^"%%LAST_ARCH%%^" == ^"%%NEW_LAST_ARCH%%^" exit /b ^0>> %NEW_BAT_FILE%
   echo :label^1>> %NEW_BAT_FILE%
   echo set NEED_CHECK=^0>> %NEW_BAT_FILE%
   echo call :parse_timestamp>> %NEW_BAT_FILE%
   echo if ^"%%EVENT_SYSTEM_TIME%%^" GTR ^"%%START_TIMESTAMP%%^" ^(>> %NEW_BAT_FILE%
   echo    call :print_message ^"WARNING: Events with previous timestamp not found, some events may have been lost.^">> %NEW_BAT_FILE%
   echo    call :print_message ^"INFO: Please collect more often to avoid such situations in the future.^">> %NEW_BAT_FILE%
   echo ^)>> %NEW_BAT_FILE%
   echo exit /b ^0>> %NEW_BAT_FILE%
)
echo.>> %NEW_BAT_FILE%
echo rem #########################################################################################>> %NEW_BAT_FILE%
echo rem Parse timestamp from event in given XML-file>> %NEW_BAT_FILE%
echo rem %%1 - XML-file with event>> %NEW_BAT_FILE%
echo rem #########################################################################################>> %NEW_BAT_FILE%
echo :parse_timestamp>> %NEW_BAT_FILE%
echo SETLOCAL ENABLEDELAYEDEXPANSION>> %NEW_BAT_FILE%
echo set TIME_TAG=TimeCreated>> %NEW_BAT_FILE%
echo ^(set SYSTIME= ^)>> %NEW_BAT_FILE%
echo for /F ^"usebackq tokens=1,^* delims=^^^<^" %%%%a in ^(^`type ^"%%LAST_EVENT_FILE%%^" 2^^^>^^^>%%COLLECT_PROGRESS_FILE%%^`^) do ^(>> %NEW_BAT_FILE%
echo   set MATCH_STRING=%%%%a>> %NEW_BAT_FILE%
echo   set REST_STRING=%%%%b>> %NEW_BAT_FILE%
echo :loop>> %NEW_BAT_FILE%
echo   set FIELD_NAME=^!MATCH_STRING^:^~0^,11^!>> %NEW_BAT_FILE%
echo   if ^"^!FIELD_NAME^!^"==^"%%TIME_TAG%%^" ^(>> %NEW_BAT_FILE%
echo      set SYSTIME=^!MATCH_STRING^!>> %NEW_BAT_FILE%
echo      goto endofsearch>> %NEW_BAT_FILE%
echo   ^)>> %NEW_BAT_FILE%
echo   for /F ^"tokens=1,^* delims=^^^<^" %%%%c in ^(^"^!REST_STRING^!^"^) do ^(>> %NEW_BAT_FILE%
echo      set MATCH_STRING=%%%%c>> %NEW_BAT_FILE%
echo      set REST_STRING=%%%%d>> %NEW_BAT_FILE%
echo      goto loop>> %NEW_BAT_FILE%
echo   ^)>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo :endofsearch>> %NEW_BAT_FILE%
echo set FOUND_TIME=^!SYSTIME^:^~24^,-3^!>> %NEW_BAT_FILE%
echo ^(ENDLOCAL>> %NEW_BAT_FILE%
echo set EVENT_SYSTEM_TIME=%%FOUND_TIME%%>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo exit /b>> %NEW_BAT_FILE%
echo.>> %NEW_BAT_FILE%
echo rem ######################################################################################>> %NEW_BAT_FILE%
echo rem printing routine. prints a message to log.>> %NEW_BAT_FILE%
echo rem takes two parameters:>> %NEW_BAT_FILE%
echo rem   1 - message itself>> %NEW_BAT_FILE%
echo rem   2 - name of a log file. if empty, use default value.>> %NEW_BAT_FILE%
echo rem ######################################################################################>> %NEW_BAT_FILE%
echo :print_message>> %NEW_BAT_FILE%
echo ^(set OUT_FILE=%%~2^)>> %NEW_BAT_FILE%
echo if ^"%%OUT_FILE%%^" == ^"^" set OUT_FILE=%%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
echo echo [%%date%% %%time%%] %%~1^>^> %%OUT_FILE%%>> %NEW_BAT_FILE%
echo exit /b>> %NEW_BAT_FILE%

call :print_message "File '%COLLECT_SCRIPT%' generated."
exit /b


rem ######################################################################################
rem printing routine. prints a message to log. 
rem takes two parameters:
rem   1 - message itself
rem   2 - name of a log file. if empty, use default value.
rem ######################################################################################

:print_message

(set OUT_FILE=%~2)
if "%OUT_FILE%" == "" set OUT_FILE=%ES_LOG%
if "%PRINTING%" == "enabled" echo [%date% %time%] %~1>> "%OUT_FILE%"
exit /b


rem ######################################################################################
rem Copy content to logfile
rem ######################################################################################

:print2log

if exist "%~1" (
  for /f "delims=" %%I in ('type "%~1" 2^>nul') do (
    call :print_message "%HOST_NAME%: %%I"
  )
)
exit /b


rem ######################################################################################
rem Copy files to the target system for execution
rem ######################################################################################

:copy2target

for %%I in (%~2) do (
    copy /b /y "%~1\%%I" "%COLLECT_DIR%" >nul 2>>%ES_LOG%
    if errorlevel 1 (
       call :print_message "ERROR: Failure to copy '%%I' to the target system."
       exit /b 1
    ) else (
       call :print_message "File '%%I' successfully copied to the target system."
    )
)
exit /b 0


rem ######################################################################################
rem Cleanup routine. Deleting temp directories and unmaping share
rem ######################################################################################

:cleanup

call :print_message "Free and clean up all used system resources."
if not "%COLLECT_DIR%" == "" (
   if exist "%COLLECT_DIR%\" (
      rmdir /s /q "%COLLECT_DIR%" >nul 2>>%ES_LOG%
      if exist "%COLLECT_DIR%\" (
         call :print_message "WARNING: Failure to remove temporary working directory on target system '%COLLECT_DIR%'."
      )
   )
)
if not "%SHARE_DISK%" == "" (
   if exist "%SHARE_DISK%\" (
      net use "%SHARE_DISK%" /delete >nul 2>>%ES_LOG%
      if exist "%SHARE_DISK%\" (
         call :print_message "WARNING: Failure to unmap network share of the target system."
      )
   )
)
if not "%WORK_TMP_DIR%" == "" (
   if exist "%WORK_TMP_DIR%\" (
      rmdir /s /q "%WORK_TMP_DIR%" >nul 2>>%ES_LOG%
      if exist "%WORK_TMP_DIR%\" (
         call :print_message "WARNING: Failure to remove existing temporary working directory '%WORK_TMP_DIR%'."
      )
   )
)
exit /b
