rem IBM Confidential
rem OCO Source Material
rem 5724-S67
rem (c) Copyright IBM Corp. 1998, 2010
rem The source code for this program is not published or otherwise divested of
rem its trade secret, irrespective of what has been deposited with the U.S.
rem Copyright Office.
rem version 2011-04-09
@echo off


rem ######################################################################################
rem Receives parameters from the Actuator
rem ######################################################################################
rem -- File for saving tarball with collected original evtx-logfiles
setlocal
set ORIGINAL_CHUNK=%~1
rem -- File for saving exported events from all collected logfiles in XML format
set EXPORTED_CHUNK=%~2
set USER_NAME=%~3
set USER_PWD=%~4
set WIN_LOG_FILE=%~5
set REMOTE_TSIEM_TMP_DIR=%~6

for %%I in (%*) do set PROPS_FILE=%%~I
for %%I in (%PROPS_FILE%) do set PROPS_NAME=%%~nI
rem -- sets actuator log name
if "%PROPS_FILE%" == "" (
   set ES_LOG="..\log\~n0-error.log"
) else (
   set ES_LOG="..\log\%PROPS_NAME%.log"
)

rem -- clears log file and start logging
type nul > %ES_LOG%

rem ######################################################################################
rem Actuator parameters check
rem ######################################################################################
set PRINTING=enabled
	
rem -- one or both logset files arguments are missing
set CHUNKNAME_EMPTY=0
if "%ORIGINAL_CHUNK%" == "" (
   call :print_message "ERROR: Missing output file name argument for sublog #1 of Windows %WIN_LOG_FILE% audit logs."
   set CHUNKNAME_EMPTY=1
) else (
   type nul > "%ORIGINAL_CHUNK%" 2>>%ES_LOG%
)
if "%EXPORTED_CHUNK%" == "" (
   call :print_message "ERROR: Missing output file name argument for sublog #2 of Windows %WIN_LOG_FILE% audit logs."
   set CHUNKNAME_EMPTY=1
) else (
   type nul > "%EXPORTED_CHUNK%" 2>>%ES_LOG%
)
if %CHUNKNAME_EMPTY% EQU 1 goto failure

rem -- Properties file PROPS_FILE argument is missing
if "%PROPS_FILE%" == "" (
   call :print_message "ERROR: Missing properties file argument."
   goto failure
)
if not exist "%PROPS_FILE%" (
   call :print_message "ERROR: Can't find properties file '%PROPS_FILE%'."
   goto failure
)

call :print_message "Processing started."
call :parse_props_file

if "%HOST_NAME%" == "" (
   call :print_message "ERROR: 'SystemName' variable is not found in '%PROPS_FILE%' file."
   goto failure
)

set TSIEM_BIN_DIR=%~dp0%
set TSIEM_BIN_DIR=%TSIEM_BIN_DIR:~0,-1%
set PS_EXE=%TSIEM_BIN_DIR%\psexec.exe
set PS_OPTIONS=-u %USER_NAME% -p %USER_PWD% -q
set NET_OPTIONS=%USER_PWD% /u:%USER_NAME%
set LOCREMDET_SCRIPT=%TSIEM_BIN_DIR%\localorremote.vbs
call :check_remote_collect || goto failure
if not "%REMOTE_TSIEM_TMP_DIR%" == "" (
    if %REMOTE_COLLECT% EQU 1 (
		set WORK_TMP_DIR=%CD%
	) else (
		set WORK_TMP_DIR=%REMOTE_TSIEM_TMP_DIR%
	)	
) else (
   set WORK_TMP_DIR=%CD%
)
if "%WORK_TMP_DIR%" == "" (
   set WORK_TMP_DIR=%SystemRoot%\Temp
)
if not "%WORK_TMP_DIR:~-1%" == "\" set WORK_TMP_DIR=%WORK_TMP_DIR%\
set WORK_TMP_DIR=%WORK_TMP_DIR%ES-%PROPS_NAME%
set COLLECT_SCRIPT=eventscollect.bat
set COLLECT_PROGRESS_FILE=collectprogress.txt
set WEVTUTIL2003_LOG=wevtutil2003.log
set TIMESTAMP_FILE=timestamp.txt
set COLLECT_ERROR_FILE=collecterror.txt
set INITIAL_COLLECT_DAYS=2


Rem -- creates temp directory
call :print_message "Create temporary working directory '%WORK_TMP_DIR%'."
if exist "%WORK_TMP_DIR%\" (
   rmdir /s /q "%WORK_TMP_DIR%" >nul 2>>%ES_LOG%
   if exist "%WORK_TMP_DIR%\" (
      call :print_message "ERROR: Failure to remove existing temporary working directory '%WORK_TMP_DIR%'."
      goto failure
   )
)
mkdir "%WORK_TMP_DIR%" >nul 2>>%ES_LOG%
if errorlevel 1 (
   call :print_message "ERROR: Failure to create temporary working directory '%WORK_TMP_DIR%'."
   goto failure
)

if %REMOTE_COLLECT% EQU 1 (
   if "%USER_NAME%" == "" (
      call :print_message "ERROR: Missing username for accessing target system."
      goto failure
   )
   if "%USER_PWD%" == "" (
      call :print_message "ERROR: Missing password for accessing target system."
      goto failure
   )
)

call :print_message "- Start preparing environment for the collect process."

call :create_collect_script

set COLLECT_ERROR=0
if %REMOTE_COLLECT% EQU 0 goto localcollect
	if "%REMOTE_TSIEM_TMP_DIR%" == "" (
		call :print_message "The collect script will use default share ADMIN$ ."
		set SHARE_NAME=ADMIN$
	)else  ( 	
		set SHARE_NAME=%REMOTE_TSIEM_TMP_DIR%
		rem -- call :getShareName %REMOTE_TSIEM_TMP_DIR%
	)	
   call :print_message "Find free drive letter for mapping network share." %ES_LOG%   
   call :find_free_disk || goto failure
   call :print_message "Map network share of the target system on the drive '%SHARE_DISK%' with '\\%HOST_NAME%\%SHARE_NAME%'." %ES_LOG%     
   net use %SHARE_DISK% /delete >nul 2>1   
   net use %SHARE_DISK% "\\%HOST_NAME%\%SHARE_NAME%" %NET_OPTIONS% >nul 2>>%ES_LOG%   
   if errorlevel 1 (
      call :print_message "ERROR: Failure to map '%SHARE_NAME%' to '%HOST_NAME%'."
      goto failure
   )    
   set COLLECT_DIR=%SHARE_DISK%\Temp\ES-%PROPS_NAME%AD
   if exist "%COLLECT_DIR%\" (
      rmdir /s /q "%COLLECT_DIR%" >nul 2>>%ES_LOG%
      if exist "%COLLECT_DIR%\" (
         call :print_message "ERROR: Failure to remove existing temporary working directory on target system '%COLLECT_DIR%'."
         goto failure
      )
   )   
   call :print_message "Create temporary working directory '%COLLECT_DIR%' on the target system."
   mkdir "%COLLECT_DIR%" >nul 2>>%ES_LOG%     
   if errorlevel 1 (
      call :print_message "ERROR: Failure to create temporary working directory on target system '%COLLECT_DIR%'."
      goto failure
   )  
  if exist "%SHARE_DISK%\Temp\bin\" (
      rmdir /s /q "%SHARE_DISK%\Temp\bin" >nul 2>>%ES_LOG%
      if exist "%SHARE_DISK%\Temp\bin\" (
         call :print_message "ERROR: Failure to remove existing temporary working directory on target system '%SHARE_DISK%\bin'."
         goto failure
      )
   )   
   call :print_message "Create temporary working directory '%SHARE_DISK%\Temp\bin' on the target system."
   mkdir "%SHARE_DISK%\Temp\bin" >nul 2>>%ES_LOG%
     if errorlevel 1 (
      call :print_message "ERROR: Failure to create temporary working directory on target system  '%SHARE_DISK%\Temp\bin'."
      goto failure
   )
   copy /Y "%TSIEM_BIN_DIR%\getwinlocale.bat" ""%SHARE_DISK%\Temp\bin"" >nul 2>>%ES_LOG%  
   if errorlevel 1 (
      call :print_message "ERROR: Failure to copy getwinlocale.bat on target system '%COLLECT_DIR%'."
      goto failure
   )
      
   call :print_message "Copy necessary execution files to the target system."
   rem -- Copying collect script to the target system
   call :copy2target "%WORK_TMP_DIR%" "%COLLECT_SCRIPT%" || goto failure
   rem -- Copying props file to the target system
   cd 
   call :copy2target "..\run" "%PROPS_NAME%.props" || goto failure

    
   rem -- Copying necessary binary files to the target system
   call :copy2target "%TSIEM_BIN_DIR%" "archive1.dll bzip2.dll zlib1.dll bsdtar.exe icuin40.dll icuuc40.dll icudt40.dll icuio40.dll icule40.dll iculx40.dll icutu40.dll icuuc40.dll " || goto failure
   
   rem -- Copying necessary wevtutil2003  to the target system
   call :copy2target "%TSIEM_BIN_DIR%" "wevtutil2003.exe" || goto failure
   
   rem -- Copying date.exe for getting timestamp on target system
   if "%START_TIMESTAMP%" == "" (
      call :copy2target "%TSIEM_BIN_DIR%" "msvcr71.dll date.exe" || goto failure
   )
   call :print_message "- Start the collect process."

   rem -- Start collect script on target system and getting result files from target system   
   if "%REMOTE_TSIEM_TMP_DIR%" == ""  (
	"%PS_EXE%" \\%HOST_NAME% %PS_OPTIONS% cmd.exe /c "%%SystemRoot%%\Temp\ES-%PROPS_NAME%AD\%COLLECT_SCRIPT%">nul 2>> %ES_LOG%
   ) else (     
	 "%PS_EXE%" \\%HOST_NAME% %PS_OPTIONS% cmd.exe /c "\\%HOST_NAME%\%SHARE_NAME%\Temp\ES-%PROPS_NAME%AD\%COLLECT_SCRIPT%" >nul 2 >> %ES_LOG%		
   )
   rem -- Copy collect progress log to main logfile   
   call :print2log "%COLLECT_DIR%\%COLLECT_PROGRESS_FILE%"
   copy /Y "%COLLECT_DIR%\%WEVTUTIL2003_LOG%" "%WORK_TMP_DIR%\%HOST_NAME%-%WEVTUTIL2003_LOG%" >nul 2>>%ES_LOG%
   call :print_message "- Result of wevtutil2003.exe for system : %HOST_NAME%."
   type  "%WORK_TMP_DIR%\%HOST_NAME%-%WEVTUTIL2003_LOG%" >>%ES_LOG%
   
   rem -- Check for collect script errors on target system |
   copy /Y "%COLLECT_DIR%\%COLLECT_ERROR_FILE%" "%WORK_TMP_DIR%\%COLLECT_ERROR_FILE%" >nul 2>>%ES_LOG%
   if errorlevel 1 (
      call :print_message "ERROR: Failure to copy errors report from the target system."
      goto failure
   ) else (
      for %%I in ("%WORK_TMP_DIR%\%COLLECT_ERROR_FILE%") do (
          if %%~zI GTR 0 set COLLECT_ERROR=1
      )
   )
   
   if %COLLECT_ERROR% EQU 1 (   
      call :print_message "ERROR: Failure during execution collect script."
      call :print2log "%COLLECT_DIR%\%COLLECT_ERROR_FILE%"
	  goto failure
   )
   
   rem -- Copy collected events from target system
   copy /Y "%COLLECT_DIR%\%PROPS_NAME%.props" "..\run" > nul 2>>%ES_LOG%
   if errorlevel 1 (
	 call :print_message "ERROR: Failure to get new LPI from the target system."
	 goto failure
	)
	
    rem -- Copy collected events from target system 
    copy /Y "%COLLECT_DIR%\1.*" "%WORK_TMP_DIR%" >nul 2>>%ES_LOG%
	copy /Y "%COLLECT_DIR%\2.*" "%WORK_TMP_DIR%" >nul 2>>%ES_LOG%
	if errorlevel 1 (
	 call :print_message "ERROR: Failure to get the collected events from the target system."
	 goto failure
	)
   
   if exist "%SHARE_DISK%\Temp\bin\" (
      rmdir /s /q "%SHARE_DISK%\Temp\bin" >nul 2>>%ES_LOG%
      if exist "%SHARE_DISK%\Temp\bin" (
         call :print_message "ERROR: Failure to remove temporary working directory on target system '%SHARE_DISK%\Temp\bin'."
         goto failure
      )
   )
   if exist "%COLLECT_DIR%\" (
      rmdir /s /q "%COLLECT_DIR%" >nul 2>>%ES_LOG%
      if exist "%COLLECT_DIR%\" (
         call :print_message "ERROR: Failure to remove temporary working directory on target system '%COLLECT_DIR%'."
         goto failure
      )
   )
   net use %SHARE_DISK% /delete >nul 2>>%ES_LOG%
   if errorlevel 1 (
      call :print_message "ERROR: Failure to unmap network share of the target system."
   )  
   goto finish

:localcollect
   set COLLECT_DIR=%WORK_TMP_DIR%
   call :print_message "- Start the collect process. "
   rem -- Start collect script
   call "%COLLECT_DIR%\%COLLECT_SCRIPT%"
   rem -- Check for collect script errors
   if exist "%COLLECT_DIR%\%COLLECT_ERROR_FILE%" (	  
      for %%I in ("%COLLECT_DIR%\%COLLECT_ERROR_FILE%") do (	  
          if %%~zI GTR 0 set COLLECT_ERROR=1
      )
   )
   if %COLLECT_ERROR% EQU 1 (
      call :print_message "ERROR: Failure during execution collect script."
      type "%COLLECT_DIR%\%COLLECT_ERROR_FILE%" >> %ES_LOG%
	  goto failure
   )
   for /f "tokens=2 delims==" %%I in ('findstr /R /C:"^TIMESTAMP = .*" "%PROPS_FILE%" 2^>^>%ES_LOG%') do set NEW_START_TIMESTAMP=%%I
   echo %NEW_START_TIMESTAMP%> "%COLLECT_DIR%\%TIMESTAMP_FILE%"
   if not exist "%COLLECT_DIR%\%TIMESTAMP_FILE%" (
		call :print_message "ERROR: There is not exist new timestamp on the target system."
		goto failure
   )   
   
:finish

call :print_message "- Start postprocessing stage."
call :print_message "Copy not original log contents to sublog #1."
for /f %%I in ('cd') do set CUR_DIR=%%I
for /F "tokens=*" %%I in ('dir /B /ON ^"%WORK_TMP_DIR%\1.*^"')do (
	rename "%WORK_TMP_DIR%\%%I" "%%I.gz" >nul 2>>%ES_LOG%
	"%TSIEM_BIN_DIR%\gunzip.exe" -d "%WORK_TMP_DIR%\%%I.gz">nul 2>>"%CUR_DIR%"\%ES_LOG%		
	rename "%WORK_TMP_DIR%\%%I" "%%I.noOrig" >nul 2>>%ES_LOG%
)
type nul >"%WORK_TMP_DIR%\objvaldump.txt"	
for /f "tokens=*" %%I in ('dir /B /ON ^"%WORK_TMP_DIR%\*.noOrig^"')do (	
	type "%WORK_TMP_DIR%\%%I">>"%WORK_TMP_DIR%\objvaldump.txt"	
)

move /Y "%WORK_TMP_DIR%\objvaldump.txt" "%EXPORTED_CHUNK%" >nul 2>>%ES_LOG%
if errorlevel 1 (
   call :print_message "ERROR: Failure to move collected events in Objval format into first logset file."
   goto failure
)

call :print_message "Copy the original log contents to sublog #2."
for /f "tokens=*" %%I in ('dir /B /ON ^"%WORK_TMP_DIR%\2.*^"')do (
    rename "%WORK_TMP_DIR%\%%I" "%%I.gz" >nul 2>>%ES_LOG%	
    "%TSIEM_BIN_DIR%\gunzip.exe" -d "%WORK_TMP_DIR%\%%I.gz">nul 2>>"%CUR_DIR%"\%ES_LOG%		
    rename "%WORK_TMP_DIR%\%%I" "%%I.evt" >nul 2>>%ES_LOG%		
)
pushd "%WORK_TMP_DIR%"
"%TSIEM_BIN_DIR%\bsdtar.exe" -cf evts.tar *.evt >nul 2>>"%CUR_DIR%"\%ES_LOG%
popd
rem -- Move collected events into second logset file
move /Y "%WORK_TMP_DIR%\evts.tar" "%ORIGINAL_CHUNK%" >nul 2>>%ES_LOG%
if errorlevel 1 (
   call :print_message "ERROR: Failure to create tarball from collected events in evt-files into first logset file."
   goto failure
)
call :cleanup
call :print_message "Processing finished [SUCCESS]."
exit /b 0

rem -- prints failure message to actuator log and finishes the script with errorlevel 1 (FAILURE)
:failure

call :cleanup
call :print_message "Processing finished [FAILURE]."
exit /b 1


rem #########################################################################################
rem Parsing properties file PROPS_FILE for setting state variables
rem #########################################################################################

:parse_props_file

rem -- read value of the 'SystemName' parameter into HOST_NAME
for /f "tokens=2 delims==" %%I in ('findstr /R /C:"^SystemName = .*" "%PROPS_FILE%" 2^>^>%ES_LOG%') do set HOST_NAME=%%I
for %%I in (%HOST_NAME%) do set HOST_NAME=%%I
rem -- read value of the 'TIMESTAMP' parameter into START_TIMESTAMP
for /f "tokens=2 delims==" %%I in ('findstr /R /C:"^TIMESTAMP = .*" "%PROPS_FILE%" 2^>^>%ES_LOG%') do set START_TIMESTAMP=%%I
for %%I in (%START_TIMESTAMP%) do set START_TIMESTAMP=%%I
call :print_message "START_TIMESTAMP :%START_TIMESTAMP%"
if "%START_TIMESTAMP%" == "" (
  call :print_message "Initial collect mode use."
) else (
  call :print_message "Last collect time %START_TIMESTAMP%."
)
call :print_message "##################################"
exit /b


rem #########################################################################################
rem Check collecting mode and set REMOTE_COLLECT variable
rem #########################################################################################

:check_remote_collect

for /f "tokens=1,* delims=|" %%a in ('cscript.exe //Nologo "%LOCREMDET_SCRIPT%" %HOST_NAME%') do (
    set RESULT1=%%a
    set RESULT2=%%b
)
if "%RESULT1%" == "LOCAL" (
   set REMOTE_COLLECT=0
   call :print_message "Collect from the local target system: '%HOST_NAME%'."
) else if "%RESULT1%" == "REMOTE" (
   set REMOTE_COLLECT=1
   set HOST_NAME=%RESULT2%
   call :print_message "Collect from the remote target system: '%HOST_NAME%'."
) else (
   call :print_message "ERROR: Script for determining mode of the collect return failure."
   if "%RESULT2%" == "" (
      call :print_message "%RESULT1%"
   ) else (
      call :print_message "%RESULT2%"
   )
   exit /b 1
)
exit /b 0


rem #########################################################################################
rem Creating collect script COLLECT_SCRIPT in WORK_TMP_DIR
rem #########################################################################################

:create_collect_script
set NEW_BAT_FILE="%WORK_TMP_DIR%\%COLLECT_SCRIPT%"
type nul > %NEW_BAT_FILE% 2>>%ES_LOG%
echo @echo off>> %NEW_BAT_FILE%
echo rem #################################### eventscollect.bat ####################################>> %NEW_BAT_FILE%
echo rem ### on success send in stdout tarball with resulting bziped objVal-file and all evt-files>> %NEW_BAT_FILE%
echo rem ### if failure return code 1 and explanation message in COLLECT_ERROR_FILE>> %NEW_BAT_FILE%
echo rem ###########################################################################################>> %NEW_BAT_FILE%
echo setlocal>> %NEW_BAT_FILE%
if "%REMOTE_TSIEM_TMP_DIR%" == ""  (
	echo set SHARE_PATH=%%SystemRoot%%>> %NEW_BAT_FILE%
) else (  
	if %REMOTE_COLLECT% EQU 1 (   
		echo set SHARENAME=%REMOTE_TSIEM_TMP_DIR%>> %NEW_BAT_FILE%
		echo FOR /F ^"tokens=* skip=1 delims= ^" %%%%A IN ^(^'NET SHARE ^^^"%%SHARENAME%%^^^"^'^) DO ^( >> %NEW_BAT_FILE%
		echo    echo %%%%A^|find ^"Path^" ^>nul >> %NEW_BAT_FILE%
		echo    if errorlevel 0 ^( 	>> %NEW_BAT_FILE%
		echo		call :metoda ^"%%%%A^" 	>> %NEW_BAT_FILE%
		echo		goto :findsharepath >> %NEW_BAT_FILE%
		echo     ^) >> %NEW_BAT_FILE%
		echo ^) >> %NEW_BAT_FILE%
		echo :findsharepath >> %NEW_BAT_FILE%
		echo for /F ^"tokens=2* delims=[^"  %%%%I in ^(%%V_RESULT%%^) do set SHAREPATH_TEMP=%%%%I >> %NEW_BAT_FILE%
		echo for /F ^"tokens=* delims= ^" %%%%a in ^(^^^"%%SHAREPATH_TEMP%%^^^"^) do set SHARE_PATH=%%%%a>> %NEW_BAT_FILE%
		echo call :trim %%SHARE_PATH%%>> %NEW_BAT_FILE%
		echo :trim>> %NEW_BAT_FILE%
		echo set SHARE_PATH=%%*>> %NEW_BAT_FILE%
		echo exit /b 0>> %NEW_BAT_FILE%
		echo :metoda >> %NEW_BAT_FILE%		
		echo set TEMP_VAR=%%1 >> %NEW_BAT_FILE%
		echo set V_RESULT=%%TEMP_VAR:Path=Path[%% >> %NEW_BAT_FILE%
		echo exit /b 0>> %NEW_BAT_FILE%
	) else (
		echo set SHARE_PATH=%REMOTE_TSIEM_TMP_DIR%>> %NEW_BAT_FILE%
	)	
)
if %REMOTE_COLLECT% EQU 1 (   
   echo ^(set WORK_TMP_DIR=%%SHARE_PATH%%\Temp\ES-%PROPS_NAME%AD^)>> %NEW_BAT_FILE%	
   echo ^(set TSIEM_BIN_DIR=%%WORK_TMP_DIR%%^)>> %NEW_BAT_FILE%
   echo ^(set COLLECT_PROGRESS_FILE=^"%%WORK_TMP_DIR%%\%COLLECT_PROGRESS_FILE%^"^)>> %NEW_BAT_FILE%
   echo ^(set WEVT2003_LOG_FILE=%%WORK_TMP_DIR%%\wevtutil2003.log^)>> %NEW_BAT_FILE%         
) else (
   echo ^(set WORK_TMP_DIR=%WORK_TMP_DIR%^)>> %NEW_BAT_FILE%
   echo ^(set TSIEM_BIN_DIR=%TSIEM_BIN_DIR%^)>> %NEW_BAT_FILE%
   echo ^(set COLLECT_PROGRESS_FILE=%ES_LOG%^)>> %NEW_BAT_FILE%   
   echo ^(set WEVT2003_LOG_FILE=%%COLLECT_PROGRESS_FILE%%^)>> %NEW_BAT_FILE%    
)
echo ^(set CHUNKS_TMP_DIR=%%WORK_TMP_DIR%%^)>> %NEW_BAT_FILE%   
echo ^(set ARCHIVES_TMP_DIR=%%WORK_TMP_DIR%%\Archives^)>> %NEW_BAT_FILE%
echo mkdir "%%ARCHIVES_TMP_DIR%%" >> %NEW_BAT_FILE%
echo ^(set TIMESTAMP_FILE=^"%%WORK_TMP_DIR%%\%TIMESTAMP_FILE%^"^)>> %NEW_BAT_FILE%
echo ^(set COLLECT_ERROR_FILE=^"%%WORK_TMP_DIR%%\%COLLECT_ERROR_FILE%^"^)>> %NEW_BAT_FILE%
echo type nul ^> %%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
echo if defined PROCESSOR_ARCHITEW6432 (>> %NEW_BAT_FILE%
echo    set TARGET_ARCH=%%PROCESSOR_ARCHITEW6432%%>> %NEW_BAT_FILE%
echo ) else (>> %NEW_BAT_FILE%
echo    if defined PROCESSOR_ARCHITECTURE (>> %NEW_BAT_FILE%
echo       set TARGET_ARCH=%%PROCESSOR_ARCHITECTURE%%>> %NEW_BAT_FILE%
echo    )>> %NEW_BAT_FILE%
echo )>> %NEW_BAT_FILE%
echo if not defined TARGET_ARCH (>> %NEW_BAT_FILE%
echo    call :print_message ^"ERROR: Cannot define target system processor architecture.^" %%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
echo    exit /b ^1>> %NEW_BAT_FILE%
echo )>> %NEW_BAT_FILE%
echo for /f ^"tokens=*^" %%%%I in (^'echo %%TARGET_ARCH%% ^^^| find /c ^"64^" 2^^^>nul^') do set ARCH_64=%%%%I>> %NEW_BAT_FILE%
rem -- If if x64 Arch use sysnative . See http://support.microsoft.com/kb/942589
echo if %%ARCH_64%% EQU 1 (>> %NEW_BAT_FILE%
echo    set WIN_SYS_DIR=%%SystemRoot%%\sysnative>> %NEW_BAT_FILE%
echo    call :print_message ^"The operating system on the target system has 'x86-64' architecture.^">> %NEW_BAT_FILE%
echo ) else (>> %NEW_BAT_FILE%
echo    set WIN_SYS_DIR=%%SystemRoot%%\system32>> %NEW_BAT_FILE%
echo    call :print_message ^"The operating system on the target system has 'x86' architecture.^">> %NEW_BAT_FILE%
echo )>> %NEW_BAT_FILE%
echo for /F ^"tokens=* skip=2^" %%%%A IN (^'REG.EXE QUERY ^"HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Services\Eventlog\%WIN_LOG_FILE%^" /v ^"File^"^') DO SET WIN_LOG_DIR=%%%%~A>> %NEW_BAT_FILE%
echo set WIN_LOG_DIR=%%WIN_LOG_DIR:REG_SZ=REG_SZ[%%>> %NEW_BAT_FILE%
echo for /F ^"tokens=2* delims=[^"  %%%%I in ^(^^^"%%WIN_LOG_DIR%%^^^"^) do set WIN_LOG_DIR=%%%%I>> %NEW_BAT_FILE% 
echo for /F ^"tokens=* delims= ^" %%%%a in ^(^^^"%%WIN_LOG_DIR%%^^^"^) do set WIN_LOG_DIR=%%%%a>> %NEW_BAT_FILE% 
echo echo ^"%%WIN_LOG_DIR%%^"^|find /I ^"%%SystemRoot%%\system32\config^" ^>nul>> %NEW_BAT_FILE%
echo if errorlevel 1  set ISDEFAULT="FALSE">> %NEW_BAT_FILE%
echo if not errorlevel 1  set ISDEFAULT="TRUE">> %NEW_BAT_FILE%
echo if %%ARCH_64%% EQU 1 (>> %NEW_BAT_FILE%
echo 	if %%ISDEFAULT%% EQU ^"TRUE^" (>> %NEW_BAT_FILE%
echo    	set WIN_LOG_DIR=%%SystemRoot%%\sysnative\config\>> %NEW_BAT_FILE%
echo 	) else (>> %NEW_BAT_FILE%
echo 		call :getFilePath ^"%%WIN_LOG_DIR%%^">> %NEW_BAT_FILE%
echo 	) >> %NEW_BAT_FILE%
echo ) else (>> %NEW_BAT_FILE%
echo 		call :getFilePath ^"%%WIN_LOG_DIR%%^">> %NEW_BAT_FILE%
echo ) >> %NEW_BAT_FILE%
echo call :print_message ^"Windows event log location is : %%WIN_LOG_DIR%%.^">> %NEW_BAT_FILE%
echo ^(set WEVT2003_EXE_FILE=%%TSIEM_BIN_DIR%%\wevtutil2003.exe^)>> %NEW_BAT_FILE%
echo ^(set ARCH_LOGS_LIST=%%WORK_TMP_DIR%%\archivedlogs.lst^)>> %NEW_BAT_FILE%
echo if not exist ^"%%WIN_LOG_DIR%%^" ^(>> %NEW_BAT_FILE%
echo    call :print_message ^"File or folder ^'%%WIN_LOG_DIR%%^' with %WIN_LOG_FILE% Logs not found on the target system. Unable to proceed further with the collect.^" %%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
echo    exit /b ^1>> %NEW_BAT_FILE%
echo ^) else ^(>> %NEW_BAT_FILE%
echo    call :print_message ^"Found folder with %WIN_LOG_FILE% Log on the target system: ^'%%WIN_LOG_DIR%%^'.^">> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo if not exist ^"%%WEVT2003_EXE_FILE%%^" ^(>> %NEW_BAT_FILE%
echo    call :print_message ^"^'%%WEVT2003_EXE_FILE%%^' Windows 2003 utility for parsing %WIN_LOG_FILE% Log not found on the target system. Unable to proceed further with the collect.^" %%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
echo    exit /b ^1>> %NEW_BAT_FILE%
echo ^) else ^(>> %NEW_BAT_FILE%
echo    call :print_message ^"Found Windows 2003 utility for parsing %WIN_LOG_FILE% Log on the target system: ^'%%WEVT2003_EXE_FILE%%^'.^">> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE% 
if "%START_TIMESTAMP%" == "" (      
   echo call :print_message ^"Initial collect mode use.^" >> %NEW_BAT_FILE%
   echo call :print_message ^"Calculate the timestamp, from which the events will be collected.^">> %NEW_BAT_FILE%
   echo set DATE_ERR=^0>> %NEW_BAT_FILE%
   echo for /f  %%%%I in ^(^'^"^"%%TSIEM_BIN_DIR%%\date.exe^" ^"+%%%%Y-%%%%m-%%%%dT%%%%H:%%%%m:%%%%S.000^" -d ^"%INITIAL_COLLECT_DAYS% days ago^"^" 2^^^>^^^>%%%%COLLECT_ERROR_FILE%%%%^'^) do set START_TIMESTAMP=%%%%I>> %NEW_BAT_FILE%
   echo call :print_message ^"START_TIMESTAMP. %%START_TIMESTAMP%%^">> %NEW_BAT_FILE%
   rem -- echo if errorlevel 1 set DATE_ERR=^1>> %NEW_BAT_FILE%
   echo if %%START_TIMESTAMP%%== ^"^" set DATE_ERR=^1>> %NEW_BAT_FILE%   
   echo if %%DATE_ERR%% EQU 1 ^(>> %NEW_BAT_FILE%
   echo    call :print_message ^"ERROR: Failure to get timestamp for initial collect.^" %%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
   echo    exit /b ^1>> %NEW_BAT_FILE%
   echo ^)>> %NEW_BAT_FILE%
   if %REMOTE_COLLECT% EQU 1 (
		echo echo TIMESTAMP = %%START_TIMESTAMP%%^>^>"%%WORK_TMP_DIR%%\%PROPS_FILE%" >> %NEW_BAT_FILE%
   ) else (
		echo echo TIMESTAMP = %%START_TIMESTAMP%%^>^>"..\run\%PROPS_FILE%" >> %NEW_BAT_FILE%
   )
) else (
   echo ^(set START_TIMESTAMP=%START_TIMESTAMP%^)>> %NEW_BAT_FILE%
)
rem -- echo call :convert_timestamp ^"%%START_TIMESTAMP%%^">> %NEW_BAT_FILE%
rem -- Convert Last Collected time from LPI into UTC format . Autobackup logs for WIN 2003 creates archives containing archived time in UTC format
echo set WEVT2003_EXE_FILE_TMP=^"%%WEVT2003_EXE_FILE%%^">> %NEW_BAT_FILE%
echo for /f %%%%I in (^'%%%%WEVT2003_EXE_FILE_TMP%%%% -converttime %%%%START_TIMESTAMP%%%% -logfile ^^^"%%%%WEVT2003_LOG_FILE%%%%^^^"^') do set UTCSTARTTIMESTAMP=%%%%I>> %NEW_BAT_FILE%
echo call :print_message ^"UTC Start timestamp: %%UTCSTARTTIMESTAMP%%.^" >> %NEW_BAT_FILE%
echo call :convert_timestamp ^"%%UTCSTARTTIMESTAMP%%^">> %NEW_BAT_FILE%
echo rem -- Collecting events from new created archived logfiles>> %NEW_BAT_FILE%
echo if not exist ^"%%WIN_LOG_DIR%%Archive-%WIN_LOG_FILE%-*.evt^" ^(>> %NEW_BAT_FILE%
echo    call :print_message ^"Archived %WIN_LOG_FILE% Log files were not found on the target system. %%WIN_LOG_DIR%%Archive-%WIN_LOG_FILE%-*.evt^">> %NEW_BAT_FILE%
echo    goto end_cycle>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo :label^1>> %NEW_BAT_FILE%
echo set ATTEMPT=^0>> %NEW_BAT_FILE%
echo rem -- Cycle for checking archives created during processing previously existed ^(maximum 100 iteration, then generate error^)>> %NEW_BAT_FILE%
echo :start_cycle>> %NEW_BAT_FILE%
echo     if %%ATTEMPT%% EQU 100 ^(>> %NEW_BAT_FILE%
echo        call :print_message ^"ERROR: Logfiles archives are produced quicker than they can be collected. Please, change %WIN_LOG_FILE% Log archiving settings.^">> %NEW_BAT_FILE%
echo        call :print_message ^"INFO: Please increase the size of active logfile to avoid such situation in the future and repeat the collect.^">> %NEW_BAT_FILE%
echo        goto final_actions>> %NEW_BAT_FILE%
echo     ^)>> %NEW_BAT_FILE%
echo     set /a ATTEMPT+=^1>> %NEW_BAT_FILE%
echo     call :get_archives_list ^|^| goto end_cycle>> %NEW_BAT_FILE%
echo     for /f ^"delims=^" %%%%I in ^(^'type ^"%%ARCH_LOGS_LIST%%^" 2^^^>^^^>%%COLLECT_PROGRESS_FILE%%^'^) do ^(>> %NEW_BAT_FILE%
echo         set PREVIOUS_ARCHIVE=%%%%I>> %NEW_BAT_FILE%
echo         call :collect_archives ^"%%%%I^" ^|^| goto final_actions>> %NEW_BAT_FILE%
echo     ^)>> %NEW_BAT_FILE%
echo :end_cycle>> %NEW_BAT_FILE%
echo rem -- Collecting events from current active logfile: wevtutil2003 -tempchunkpath ... -logfile ...>> %NEW_BAT_FILE%
if %REMOTE_COLLECT% EQU 1 (
	echo cd ^"%%WORK_TMP_DIR%%^">> %NEW_BAT_FILE%
)
echo ^"%%WEVT2003_EXE_FILE%%^" -tempchunkpath ^"%%CHUNKS_TMP_DIR%%^" -logfile ^"%%WEVT2003_LOG_FILE%%^" -properties LogPlatformInstance %PROPS_NAME% EventLog ^"%WIN_LOG_FILE%^" SystemName %HOST_NAME%>>%NEW_BAT_FILE%
echo if errorlevel 1 ^(>> %NEW_BAT_FILE%
echo    call :print_message ^"ERROR: Failure to collect events from the currently active %WIN_LOG_FILE% Log.^">> %NEW_BAT_FILE%
echo    goto final_actions>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
rem -- rename the chuncks
echo move ^"%%CHUNKS_TMP_DIR%%\*1.^" ^"%%WORK_TMP_DIR%%\1.%WIN_LOG_FILE%NotOrig^" ^>nul 2^>^>%%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%	
echo move ^"%%CHUNKS_TMP_DIR%%\*2.^" ^"%%WORK_TMP_DIR%%\2.%WIN_LOG_FILE%Orig^" ^>nul 2^>^>%%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
echo call :get_archives_list>> %NEW_BAT_FILE%
echo if not errorlevel 1 ^(>> %NEW_BAT_FILE%
echo    call :print_message ^"New archived logfile was created during collect events from the currently active %WIN_LOG_FILE% Log.^">> %NEW_BAT_FILE%
echo    goto start_cycle>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
if %REMOTE_COLLECT% EQU 1 (
   echo call :print_message ^"Send result files to destination system.^">> %NEW_BAT_FILE%
   echo set ERROR=^0>> %NEW_BAT_FILE%
   echo pushd ^"%%CHUNKS_TMP_DIR%%^">> %NEW_BAT_FILE%  
   echo if errorlevel 1 set ERROR=^1>> %NEW_BAT_FILE%
   echo popd>> %NEW_BAT_FILE%
   echo if /i not ^"%%CHUNKS_TMP_DIR%%^" == ^"%%WORK_TMP_DIR%%^" ^(>> %NEW_BAT_FILE%
   echo    call :print_message ^"Remove temporary directory for chunks '%%CHUNKS_TMP_DIR%%'.^">> %NEW_BAT_FILE%
   echo    rmdir /s /q ^"%%CHUNKS_TMP_DIR%%^" ^>nul 2^>^>%%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
   echo    if exist ^"%%CHUNKS_TMP_DIR%%\^" ^(>> %NEW_BAT_FILE%
   echo       call :print_message ^"ERROR: Failure to remove temporary directory for chunks '%%CHUNKS_TMP_DIR%%'.^" %%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
   echo       exit /b ^1>> %NEW_BAT_FILE%
   echo    ^)>> %NEW_BAT_FILE%
   echo ^)>> %NEW_BAT_FILE%
   echo if %%ERROR%% EQU 1 ^(>> %NEW_BAT_FILE%
   echo    call :print_message ^"ERROR: Failure to send result files to destination system.^" %%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
   echo    exit /b ^1>> %NEW_BAT_FILE%
   echo ^)>> %NEW_BAT_FILE%
)
echo exit /b ^0>> %NEW_BAT_FILE%
echo.>> %NEW_BAT_FILE%
echo rem #########################################################################################>> %NEW_BAT_FILE%
echo rem Get file path >> %NEW_BAT_FILE%
echo rem %%1  fullpath for the file>> %NEW_BAT_FILE%
echo rem Result - set WIN_LOG_DIR variable>> %NEW_BAT_FILE%
echo rem #########################################################################################>> %NEW_BAT_FILE%
echo :getFilePath >> %NEW_BAT_FILE%
echo set rootFolder=%%~d1>> %NEW_BAT_FILE%
echo set eventPath=%%~p1>> %NEW_BAT_FILE%
echo set WIN_LOG_DIR=%%rootFolder%%%%eventPath%%>> %NEW_BAT_FILE%
echo exit /b>> %NEW_BAT_FILE%
echo.>> %NEW_BAT_FILE%
echo rem #########################################################################################>> %NEW_BAT_FILE%
echo rem Convert timestamp from ISO to archived logfile name format>> %NEW_BAT_FILE%
echo rem %%1  - ISO timestamp>> %NEW_BAT_FILE%
echo rem Result - set ARCH_LOG_TIME variable>> %NEW_BAT_FILE%
echo rem #########################################################################################>> %NEW_BAT_FILE%
echo :convert_timestamp>> %NEW_BAT_FILE%
echo for /f ^"tokens=1 delims=T^" %%%%I in ^(^"%%~1^"^) do set DATE_PART=%%%%I>> %NEW_BAT_FILE%
echo for /f ^"tokens=2 delims=T^" %%%%I in ^(^"%%~1^"^) do set TIME_PART=%%%%I>> %NEW_BAT_FILE%
echo for /f ^"tokens=1 delims=:^" %%%%I in ^(^"%%TIME_PART%%^"^) do set HOURS=%%%%I>> %NEW_BAT_FILE%
echo for /f ^"tokens=2 delims=:^" %%%%I in ^(^"%%TIME_PART%%^"^) do set MINUTES=%%%%I>> %NEW_BAT_FILE%
echo for /f ^"tokens=3 delims=:^" %%%%I in ^(^"%%TIME_PART%%^"^) do set SECOND_PART=%%%%I>> %NEW_BAT_FILE%
echo for /f ^"tokens=1 delims=.^" %%%%I in ^(^"%%SECOND_PART%%^"^) do set SECONDS=%%%%I>> %NEW_BAT_FILE%
echo for /f ^"tokens=2 delims=.^" %%%%I in ^(^"%%SECOND_PART%%^"^) do set MICROSECONDS=%%%%I>> %NEW_BAT_FILE%
echo set ARCH_LOG_TIME=%%DATE_PART%%-%%HOURS%%-%%MINUTES%%-%%SECONDS%%-%%MICROSECONDS%%>> %NEW_BAT_FILE%
echo exit /b>> %NEW_BAT_FILE%
echo.>> %NEW_BAT_FILE%
echo rem #########################################################################################>> %NEW_BAT_FILE%
echo rem Get list of archived logfiles for processing>> %NEW_BAT_FILE%
echo rem Result - list of necessary archived logfiles in ARCH_LOGS_LIST file>> %NEW_BAT_FILE%
echo rem #########################################################################################>> %NEW_BAT_FILE%
echo :get_archives_list>> %NEW_BAT_FILE%
echo if defined PREVIOUS_ARCHIVE ^(>> %NEW_BAT_FILE%
echo    set AN=%%PREVIOUS_ARCHIVE%%>> %NEW_BAT_FILE%
echo ^) else ^(>> %NEW_BAT_FILE%
echo    set AN=Archive-%WIN_LOG_FILE%-%%ARCH_LOG_TIME%%>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo type nul ^> ^"%%ARCH_LOGS_LIST%%^" 2^>^>%%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
echo set TEMP_VAR=%%WIN_LOG_DIR%%Archive-%WIN_LOG_FILE%-*.evt>> %NEW_BAT_FILE% 
echo for /F "tokens=*" %%%%I in ^(^'dir /B /ON ^"%%TEMP_VAR%%^"^' ^) do ^(>> %NEW_BAT_FILE%
echo     if ^"%%%%~nI^" GTR ^"%%AN%%^" ^(>> %NEW_BAT_FILE%
echo        echo %%%%~nI^>^> ^"%%ARCH_LOGS_LIST%%^">> %NEW_BAT_FILE%
echo     ^) else ^(>> %NEW_BAT_FILE%
echo        set NEED_CHECK=^0>> %NEW_BAT_FILE%
echo     ^)>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo for %%%%I in ^(^"%%ARCH_LOGS_LIST%%^"^) do ^(>> %NEW_BAT_FILE%
echo 	if %%%%~zI EQU 0 exit /b ^1>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo exit /b ^0>> %NEW_BAT_FILE%
echo.>> %NEW_BAT_FILE%
echo rem #########################################################################################>> %NEW_BAT_FILE%
echo rem Collect events from archived log files>> %NEW_BAT_FILE%
echo rem %%1 - Log name>> %NEW_BAT_FILE%
echo rem #########################################################################################>> %NEW_BAT_FILE%
echo :collect_archives>> %NEW_BAT_FILE%

if %REMOTE_COLLECT% EQU 1 (
	echo cd ^"%%WORK_TMP_DIR%%^">> %NEW_BAT_FILE%
)	
echo copy ^"%%WIN_LOG_DIR%%\%%~1.evt^" ^"%%ARCHIVES_TMP_DIR%%^" ^>nul 2^>^>%%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%	
echo ^"%%WEVT2003_EXE_FILE%%^" -tempchunkpath ^"%%CHUNKS_TMP_DIR%%^" -logfile ^"%%WEVT2003_LOG_FILE%%^" -properties LogPlatformInstance %PROPS_NAME% EventLog ^"%%ARCHIVES_TMP_DIR%%\%%~1.evt^" SystemName %HOST_NAME%>>%NEW_BAT_FILE% 
echo if errorlevel 1 ^(>> %NEW_BAT_FILE%
echo    call :print_message ^"ERROR: Failure to collect events from ^'%%WIN_LOG_DIR%%%%~1.evt^' Log.^">> %NEW_BAT_FILE%
echo    exit /b ^1>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
rem -- rename  chuncks 
echo move ^"%%CHUNKS_TMP_DIR%%\*1^" ^"%%WORK_TMP_DIR%%\1.%%~1NotOrig^" ^>nul 2^>^>%%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
echo move ^"%%CHUNKS_TMP_DIR%%\*2^" ^"%%WORK_TMP_DIR%%\2.%%~1Orig^" ^>nul 2^>^>%%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%	
echo del ^"%%ARCHIVES_TMP_DIR%%\%%~1.evt^">> %NEW_BAT_FILE%	
echo exit /b ^0>> %NEW_BAT_FILE%
echo.>> %NEW_BAT_FILE%
echo rem ######################################################################################>> %NEW_BAT_FILE%
echo rem printing routine. prints a message to log.>> %NEW_BAT_FILE%
echo rem takes two parameters:>> %NEW_BAT_FILE%
echo rem   1 - message itself>> %NEW_BAT_FILE%
echo rem   2 - name of a log file. if empty, use default value.>> %NEW_BAT_FILE%
echo rem ######################################################################################>> %NEW_BAT_FILE%
echo :print_message>> %NEW_BAT_FILE%
echo ^(set OUT_FILE=%%~2^)>> %NEW_BAT_FILE%
echo if ^"%%OUT_FILE%%^" == ^"^" set OUT_FILE=%%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
echo echo [%%date%% %%time%%] %%~1^>^> %%OUT_FILE%%>> %NEW_BAT_FILE%
echo exit /b>> %NEW_BAT_FILE%
rem ######################################################################################
rem printing routine. prints a message to log. 
rem takes two parameters:
rem   1 - message itself
rem   2 - name of a log file. if empty, use default value.
rem ######################################################################################

:print_message

(set OUT_FILE=%~2)
if "%OUT_FILE%" == "" set OUT_FILE=%ES_LOG%
if "%PRINTING%" == "enabled" echo [%date% %time%] %~1>> "%OUT_FILE%"
exit /b

rem ######################################################################################
rem Searching for unused drive letter for mapping shared resource
rem ######################################################################################

:find_free_disk

for %%I in (Z Y X W V U T S R Q P O N M L K) do (
    if not exist "%%I:\" ( 
       set SHARE_DISK=%%I:
       exit /b 0
    )
)
call :print_message "ERROR: Can't found unused drive letter for mapping target network share." %ES_LOG%
exit /b 1

rem ######################################################################################
rem Copy content to logfile
rem ######################################################################################

:print2log

if exist "%~1" (
  for /f "tokens=* delims=" %%I in ('type "%~1" 2^>nul') do (
    call :print_message "%HOST_NAME%: %%I"
  )
)
exit /b


rem ######################################################################################
rem Copy files to the target system for execution
rem ######################################################################################

:copy2target

for %%I in (%~2) do (
    copy /b /y "%~1\%%I" "%COLLECT_DIR%" >nul 2>>%ES_LOG%
    if errorlevel 1 (
       call :print_message "ERROR: Failure to copy '%%I' to the target system."
       exit /b 1
    ) else (
       call :print_message "File '%%I' successfully copied to the target system."
    )
)
exit /b 0

rem #########################################################################################>>
rem Get share name >> %NEW_BAT_FILE%
rem %%1  fullpath for the the shared folder>> %NEW_BAT_FILE%
rem Result - set SHARE_NAME variable>> %NEW_BAT_FILE%
rem #########################################################################################>> 
:getShareName
set SHARE_NAME=%~n1
exit /b


rem ######################################################################################
rem Cleanup routine. Deleting temp directories and unmaping share
rem ######################################################################################

:cleanup

call :print_message "Free and clean up all used system resources."
if not "%COLLECT_DIR%" == "" (
   if exist "%COLLECT_DIR%\" (   
   rmdir /s /q "%COLLECT_DIR%" >nul 2>>%ES_LOG%
      if exist "%COLLECT_DIR%\" (
         call :print_message "WARNING: Failure to remove temporary working directory on target system '%COLLECT_DIR%'."
      )
   )
)
if not "%SHARE_DISK%" == "" (
   if exist "%SHARE_DISK%\" (
   net use "%SHARE_DISK%" /delete >nul 2>>%ES_LOG%
      if exist "%SHARE_DISK%\" (
         call :print_message "WARNING: Failure to unmap network share of the target system."
      )
   )
)
if not "%WORK_TMP_DIR%" == "" (
   if exist "%WORK_TMP_DIR%\" (
   rmdir /s /q "%WORK_TMP_DIR%" >nul 2>>%ES_LOG%
      if exist "%WORK_TMP_DIR%\" (
         call :print_message "WARNING: Failure to remove existing temporary working directory '%WORK_TMP_DIR%'."
      )
   )
)
exit /b
