' Licensed Materials - Property of IBM
' 5724-S67
' (c) Copyright IBM Corp. 1998, 2007   All Rights Reserved.
' US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
'***********************************************************BeginFile************ 
' Time-stamp: <010611 edwin 15:25 getnewlogs.vbs>
' Name:
'   getnewlogs.vbs
' Purpose:
'   retrieve files modified since the previous run.
'   and store them sequentially in a new file
' Notes:
'   getnewlogs.vbs <filename> <directory> <registry key> <regexp> <props>
'   <registry key> is used to store the date of the run
'   <props> is unique ID which represents platform instance
' History:
'    990712 <version> mostert <ID>:
'    010528 0.0.10 edwin QE105014: fixed selection of files that are New.
'    031106 EE0307005 armen: added possibility to create different registry key
'    for different platform instance (depending on <props>)
'      050224 AB WE0502064: correction have been made
'        - ReadRegistry corrected
'        - actuator script corrected to conforms current actutor script
'          functionality - on first run it takes log files 2 days ago
'        - identation corrected
'        - incorrect save of last time stamp fixed
'        - incorrect check for correct reading of registery fixed 
'        - additional error handling added
'        - additional tracing messages added
'      050224 AB EE0501007: -d option added
'      050224 AB EE0501007: CheckFile function added (it check do the current file
'        locked by another process or not) 
'***********************************************************EndFile******
Option Explicit   ' Force explicit variable declaration.
On Error Resume Next

Dim PropsFile
Dim debugLogFile

Dim deleteFile
Dim getOldFiles
Dim argOffset

Const regSec = "HKLM\Software\CONSUL Risk Management\CEAActuator\"
Const debugLog = "..\log\getrsalogs_"
Const tracing  = True
Const appendMode = 8

Dim filename, directory, regkey, regexp

Dim args
Set args = Wscript.Arguments

If args.count < 4 then
  wscript.quit 1
End If

If (Left(args(0),2) = "-d") Then
  deleteFile = True
  getOldFiles = True
  argOffset = 1
Else
  deleteFile = False
  getOldFiles = False
  argOffset = 0
End IF

filename = args(0 + argOffset)
directory = args(1 + argOffset)

regkey = args(2 + argOffset)
regexp = "^" & args(3 + argOffset) & "$"

If args.Count > (4 + argOffset) Then
  PropsFile = args(4 + argOffset)
End If

' Initiate logging file
If tracing Then
  Dim debugLogObject, logFileName
  logFileName = Replace(PropsFile, ".props", "")
  logFileName = debugLog & logFileName & ".log"

  Set debugLogObject = CreateObject("Scripting.FileSystemObject")
  Set debugLogFile   = debugLogObject.OpenTextFile(logFileName, appendMode, True)
  If Err.Number <> 0 Then
    wscript.echo "Failed to open log file " & logFileName
    Call CleanExit(1)
  Else
    wscript.echo "Opened log file " & logFileName
  End If
End If

If tracing Then
  debugLogFile.WriteLine "" ' Log Separator

  Dim iCnt
  For iCnt = 0 To args.Count-1
    infoMessage debugLogFile, "Args " & iCnt & " : " & args(iCnt)
  Next
End If

Err.clear

Dim WshShell
Set WshShell=WScript.CreateObject("WScript.Shell")

Dim previous, d
'previous = wshShell.RegRead("HKLM\Software\CONSUL Risk Management\CEAActuator\" & regkey) 
previous = ReadRegistry
If previous = "" Then
  d = DateAdd("d", -2, Now ())	'050222 WE0502064 AB: 2 days ago 
Else
  d = CDate(previous)
End If

If tracing Then
  infoMessage debugLogFile, "Last date from previous run: " & CStr(d)
End If


Dim fso, f, f1, fc, MyFile, matcher

Set fso = CreateObject("Scripting.FileSystemObject")
Set MyFile = fso.CreateTextFile(filename, True)

If Err.Number <> 0 Then
  If tracing Then
    infoMessage debugLogFile, "Failed to create file '" & filename & "', Error number: " & Err.Number & ", Error: " & Err.Description
  End If

  Call CleanExit(1)
End If

Set f = fso.GetFolder(directory)  
If Err.Number <> 0 Then
  If tracing Then
    infoMessage debugLogFile, "Failed to get folder '" & directory & "'."
  End IF
  
  Call CleanExit(1)
End If


Dim lastModifiedDate  '050224 WE0502064 AB: added

Set fc = f.Files
Set matcher = WScript.CreateObject("VBScript.RegExp")
If Err.Number <> 0 Then
  If tracing Then
    infoMessage debugLogFile, "Could not create VBScript.RegExp object"
  End IF
  
  Call CleanExit(1)
End If

matcher.Pattern = regexp
matcher.IgnoreCase = True
For Each f1 in fc
  lastModifiedDate = f1.DateLastModified 

  If (getOldFiles or (d < lastModifiedDate)) _
     And matcher.test(f1.Name) _
     And CheckFile(fso, f1) Then

    Dim f1str
    Set f1str = f1.OpenAsTextStream(1, -2)
      Do While f1str.AtEndOfStream <> True    
        MyFile.WriteLine f1str.ReadLine
      Loop
      f1str.Close

      If tracing Then 
        infoMessage debugLogFile, "File processed: " & f1.Path
      End If

      If deleteFile Then 
        If tracing Then 
          infoMessage debugLogFile, "Deleting file: " & f1.Path
        End If

        f1.Delete (True)
      End If
  Else
    If tracing Then
      infoMessage debugLogFile, "File skipped: " & f1.Path
    End If

  End If
Next
MyFile.Close

'050222 EE0501007 AB: corrected to store modify date of last processed file
If lastModifiedDate <> Empty Then
  previous = FormatDateTime (lastModifiedDate, 0)
Else
  previous = FormatDateTime (Now, 0)
End If

WriteRegistry previous

If tracing Then
  infoMessage debugLogFile, "Completed successfully on " & previous
End If

Call CleanExit(0)

' ReadRegistry
' EE0307025: read LPI record if possible, or generic one otheriwise
' note: must On Error GoTo 0 after the function
Function ReadRegistry
  On Error Resume Next ' no On Error GoTo 0 here

  If PropsFile <> "" then 
    ReadRegistry = wshShell.RegRead(regSec & regkey & "\" & PropsFile & "\previous")
  Else
    ReadRegistry = wshShell.RegRead(regSec & regkey)
  End If

  If Err.Number <> 0 Then
   If tracing Then
    infoMessage debugLogFile, "Failed to read registry (could happen on first run of actuator script)"
   End If

   'Call CleanExit(1) 050224 WE0502064 AB: do not need to do CleanExit here, 
   'because the registry key could be absent on first run
  End If
End Function

' WriteRegistry
' EE0307025: read LPI record if possible, or generic one otheriwise
' note: must On Error GoTo 0 after the function
Sub WriteRegistry (lastModifiledDate)
  On Error Resume Next ' no On Error GoTo 0 here

  'previous = FormatDateTime(now, 0) 050222 EE0501007 AB: removed 
  If PropsFile <> "" Then
    wshShell.RegWrite regSec & regkey & "\" & PropsFile & "\previous", lastModifiledDate
  Else
    wshShell.RegWrite regSec & regkey, lastModifiledDate 'write standart format
  End If

  If Err.Number <> 0 Then
    If tracing Then
      infoMessage debugLogFile, "Failed to write registry"
    End If

    Call CleanExit(1)
  End If
End Sub

' infoMessage()
'
Sub infoMessage(outputFile, text)
  On Error Resume Next
  wscript.echo "Message: " & text
  outputFile.WriteLine Now & " " & text
End Sub

'
' CheckFile (file)
'
' Check: is file opened for writing by some other process
'
Function CheckFile (fso, file)
  On Error Resume Next 

  Dim tempName, filePath

  filePath = file.Path
  tempName = GetTempName (fso, filePath)

  file.Move (tempName)
  If Err.Number = 0 Then
    'rename file back
    file.Move (filePath)
    CheckFile = True
    Exit Function
  End If

  CheckFile = False

End Function

'
' GetTempName (fso, path)
'
' Return temporary name
'
Function GetTempName (fso, path)
  On Error Resume Next 

  Dim tempName

  tempName = fso.GetTempName         
  While True
    If (fso.FileExists (path & tempName)) Then
      WScript.Echo path & tempName
      tempName = fso.GetTempName
    Else
      GetTempName = path & tempName
      Exit Function
    End If
  WEnd

End Function 

'
' CleanExit(error_code)
'
Sub CleanExit(error_code)
  On Error Resume Next

  If tracing Then
    debugLogFile.Close
  End If

  WScript.Quit error_code	
End Sub

