rem IBM Confidential
rem OCO Source Material
rem 5724-S67
rem (c) Copyright IBM Corp. 1998, 2010
rem The source code for this program is not published or otherwise divested of
rem its trade secret, irrespective of what has been deposited with the U.S.
rem Copyright Office.

@echo off

rem ################################## getmswinseclog.bat ################################
rem Receives parameters from the Actuator
rem ######################################################################################

setlocal
set TKLM_LOGSET=%~1
set LOGDIR=%~2
set LOGMASK=%~3
set USER_NAME=%~4
set USER_PWD=%~5
for %%I in (%*) do set PROPS_FILE=%%~I
for %%I in (%PROPS_FILE%) do set PROPS_NAME=%%~nI
set PLATFORM=Windows

set Path=%TSIEM_HOME%\jre\jre\bin;%Path%
rem -- sets ES log name
if "%PROPS_FILE%" == "" (
   set ES_LOG="..\log\gettklmlogs.err"
) else (
   set ES_LOG="..\log\%PROPS_NAME%.log"
)

rem -- clears log file and start logging
type nul > %ES_LOG%

rem ######################################################################################
rem Actuator parameters check
rem ######################################################################################
set PRINTING=enabled



rem -- one or both logset files arguments are missing
set LOGSETNAME_EMPTY=0
if "%TKLM_LOGSET%" == "" (
   call :print_message "ERROR: Missing output file name argument for TKLM audit logs."
   set LOGSETNAME_EMPTY=1
) else (
   type nul > "%TKLM_LOGSET%"
)
if %LOGSETNAME_EMPTY% EQU 1 goto failure

if "%PROPS_FILE%" == "" (
   call :print_message "ERROR: Missing property file argument."
   goto failure
)
if not exist "%PROPS_FILE%" (
   call :print_message "ERROR: Can't find property file '%PROPS_FILE%'."
   goto failure
)
call :print_message "Processing started."

set TKLM_COLLECT=1
if "%LOGDIR%" == "" (
   call :print_message "WARNING: Missing log directory path name argument for TKLM audit log."
   set TKLM_COLLECT=0
)
if "%LOGMASK%" == "" (
   call :print_message "WARNING: Missing template of the log file name argument for TKLM audit log."
   set TKLM_COLLECT=0
)
if %TKLM_COLLECT% EQU 0 (
   call :print_message "ERROR: TKLM audit log cannot be processed due to missing required parameter."
   goto failure
)

call :parse_props_file
if "%HOST_NAME%" == "" (
   call :print_message "ERROR: 'SystemName' variable is not found in '%PROPS_FILE%' file."
   goto failure
)


set TSIEM_BIN_DIR=%~dp0%
set TSIEM_BIN_DIR=%TSIEM_BIN_DIR:~0,-1%
set JAR_FILE=gettklmlogs.jar
set TKLM_TMP_LOGSET=tklm_logset.txt

call :check_remote_collect || goto failure

if %REMOTE_COLLECT% EQU 1 (
   if "%USER_NAME%" == "" (
      call :print_message "ERROR: Missing username for accessing target system."
      goto failure
   )
   if "%USER_PWD%" == "" (
      call :print_message "ERROR: Missing password for accessing target system."
      goto failure
   )
)

call :print_message "- Start preparing environment for the collect process."


if %REMOTE_COLLECT% EQU 0 goto localcollect
   for /f "delims=" %%I in ('..\bin\cmtool.exe -k -d "%USER_PWD%" 2^>^>%ES_LOG%') do set PASSWORD=%%I
   call ..\bin\mapnetworkshare.bat "%HOST_NAME%" "ADMIN$" "%PASSWORD% /u:%USER_NAME%" >> %ES_LOG% 2>&1 || goto failure
   rem SHARE_DISK assigned by previous call
   set COLLECT_DIR=%SHARE_DISK%\Temp\ES-%PROPS_NAME%

   if exist "%COLLECT_DIR%\" (
      rmdir /S /Q "%COLLECT_DIR%" >nul 2>>%ES_LOG%
      if exist "%COLLECT_DIR%\" (
         call :print_message "ERROR: Failure to remove existing temporary working directory on target system: '%COLLECT_DIR%'."
         goto failure
      )
   )
   call :print_message "Create temporary working directory '%COLLECT_DIR%' on the target system."
   mkdir "%COLLECT_DIR%" >nul 2>>%ES_LOG%
   if errorlevel 1 (
      call :print_message "ERROR: Failure to create temporary working directory on target system: '%COLLECT_DIR%'."
      goto failure
   )

   call :print_message "Copy necessary execution files to the target system."
     
   rem -- Copying propsfile  to the target system
   copy /b /y "%PROPS_FILE%" "%COLLECT_DIR%" > nul 2>>%ES_LOG%
   if errorlevel 1 (
       call :print_message "ERROR: Failure to copy '%PROPS_FILE%' to the target system."
       goto failure
    ) else (
       call :print_message "File '%PROPS_FILE%' successfully copied to the target system."
    )
   rem -- Copying necessary binary files to the target system
   call :copy2target "%TSIEM_BIN_DIR%" "%JAR_FILE%" || goto failure
   
   call :print_message "- Start the collect process."

   SET TARGET_DIR=%%SystemRoot%%\Temp\ES-%PROPS_NAME%
   echo @java.exe -Dfile.encoding=UTF-8 -classpath ".;%TARGET_DIR%\%JAR_FILE%" com.ibm.tcim.cea.gensub.itklm.java.GetTKLMLogs "%LOGDIR%" "%LOGMASK%" "%TARGET_DIR%\%TKLM_TMP_LOGSET%" "%TARGET_DIR%\%PROPS_NAME%.props" "%PLATFORM%"> "%COLLECT_DIR%\collectscript.bat"
   echo @if errorlevel 1 echo.^> "%TARGET_DIR%\collect_error">> "%COLLECT_DIR%\collectscript.bat"

   rem -- Start collect script on target system and getting result files from target system
   ..\bin\cmtool.exe -d "%USER_PWD%" | ..\bin\psexec.exe \\%HOST_NAME% -u %USER_NAME% -r -q cmd.exe /c "%TARGET_DIR%\collectscript.bat" >>%ES_LOG% 2>&1
   
   rem -- Check for collect script errors on target system
   if exist "%COLLECT_DIR%\collect_error" (
      call :print_message "ERROR: Failure during execution collect script."
      goto failure
   )
   if exist "%COLLECT_DIR%\%PROPS_NAME%.props" (
   copy /B /Y "%COLLECT_DIR%\%PROPS_NAME%.props" "%PROPS_FILE%" > nul
   if errorlevel 1 (
         call :print_message "ERROR: Failure to get new propsfile from the target system."
         goto failure
      )
   ) else (
      call :print_message "ERROR: There is not exist new propsfile on the target system."
      goto failure
   )
   if exist "%COLLECT_DIR%\%TKLM_TMP_LOGSET%" (
      copy /B /Y "%COLLECT_DIR%\%TKLM_TMP_LOGSET%" "%TKLM_LOGSET%" > nul
      if errorlevel 1 (
         call :print_message "ERROR: Failure to copy TKLM collected logset from the target system."
         goto failure
      )
   ) else (
      call :print_message "WARNING: There is not exist TKLM collected logset on target system."
      call :print_message "Possible reason: new events are not presented in logs."
   )
   goto finish
   
:localcollect
 
 call :print_message "- Start the local collect process."
   rem -- Start collect script
    
   java -Dfile.encoding=UTF-8 -classpath ".;%TSIEM_BIN_DIR%\gettklmlogs.jar" com.ibm.tcim.cea.gensub.itklm.java.GetTKLMLogs "%LOGDIR%" "%LOGMASK%" "%TKLM_LOGSET%" "%PROPS_FILE%" %PLATFORM% 1>> %ES_LOG% 2>&1
    if errorlevel 1 (
      call :print_message "ERROR: Collect process failed."
      goto failure
  )
  
:finish

call :cleanup
call :print_message "Processing finished [SUCCESS]."
exit /b 0

:failure

call :cleanup
call :print_message "Processing finished [FAILURE]."
exit /b 1


:parse_props_file
rem -- read value of the 'SystemName' parameter into HOST_NAME
for /f "tokens=2 delims==" %%I in ('findstr /R /C:"^SystemName = .*" "%PROPS_FILE%" 2^>nul') do set HOST_NAME=%%I
for /f %%I in ("%HOST_NAME%") do set HOST_NAME=%%I
exit /b

rem #########################################################################################
rem Check collecting mode and set REMOTE_COLLECT variable
rem #########################################################################################

:check_remote_collect

for /f "tokens=1,* delims=|" %%a in ('cscript.exe //Nologo ..\bin\localorremote.vbs %HOST_NAME%') do (
    set RESULT1=%%a
    set RESULT2=%%b
)
if "%RESULT1%" == "LOCAL" (
   set REMOTE_COLLECT=0
   call :print_message "Collect from the local target system: '%HOST_NAME%'."
) else (
  if "%RESULT1%" == "REMOTE" (
     set REMOTE_COLLECT=1
     set HOST_NAME=%RESULT2%
     call :print_message "Collect from the remote target system: '%HOST_NAME%'."
  ) else (
     call :print_message "ERROR: Script for determining mode of the collect return failure."
     if "%RESULT2%" == "" (
        call :print_message "%RESULT1%"
     ) else (
     call :print_message "%RESULT2%"
     )
     exit /b 1
  )
)
exit /b 0
exit /b

rem ######################################################################################
rem printing routine
rem prints a message to actuator log. 
rem takes two parameters: %1 - message itself, %2 - name of a log file
rem ######################################################################################

:print_message

(set OUT_FILE=%~2)
if "%OUT_FILE%" == "" set OUT_FILE=%ES_LOG%
if "%PRINTING%" == "enabled" echo [%date% %time%] %~1>> "%OUT_FILE%"
exit /b


rem ######################################################################################
rem Copy content to logfile
rem ######################################################################################

:print2log

if exist "%~1" (
  for /f "delims=" %%I in ('type "%~1" 2^>nul') do (
    call :print_message "%HOST_NAME%: %%I"
  )
)
exit /b


rem ######################################################################################
rem Copy files to the target system for execution
rem ######################################################################################

:copy2target

for %%I in (%~2) do (
    copy /b /y "%~1\%%I" "%COLLECT_DIR%" >nul 2>>%ES_LOG%
    if errorlevel 1 (
       call :print_message "ERROR: Failure to copy '%%I' to the target system."
       exit /b 1
    ) else (
       call :print_message "File '%%I' successfully copied to the target system."
    )
)
exit /b 0


rem ######################################################################################
rem Cleanup routine. Deleting temp directories and unmaping share
rem ######################################################################################

:cleanup

call :print_message "Free and clean up all used system resources."
if not "%COLLECT_DIR%" == "" (
   if exist "%COLLECT_DIR%\" (
      rmdir /S /Q "%COLLECT_DIR%" >nul 2>>%ES_LOG%
      if exist "%COLLECT_DIR%\" (
         call :print_message "WARNING: Failure to remove temporary working directory on target system: '%COLLECT_DIR%'."
      )
   )
)
if not "%SHARE_DISK%" == "" (
   if exist "%SHARE_DISK%\" (
      net use "%SHARE_DISK%" /delete >nul 2>>%ES_LOG%
      if exist "%SHARE_DISK%\" (
         call :print_message "WARNING: Failure to unmap network share of the target system."
      )
   )
)
exit /b
