REM Licensed Materials - Property of IBM
REM 5724-S67
REM (c) Copyright IBM Corp. 1998, 2007   All Rights Reserved.
REM US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
@echo off

rem TEMP_FILE1 - temporary file name for sublog1 (audit trail)
rem TEMP_FILE2 - temporary file name for sublog2 (passwd file)
rem TEMP_FILE3 - temporary file name for sublog3 (group file)
rem TEMP_FILE4 - temporary file name, not used
rem CLEANUP - if set to 'yes', forces deletion of already collected files from their original location
rem FTP_SERVER - ftp server name
rem TRAIL_PATH - path and pattern for audit trail files
rem ID_LOCATION - location of passwd and group files
rem USERNAME - username for ftp
rem PASSWORD - password for ftp
rem PROPS_FILE - props file name

set TEMP_FILE1=%1
set TEMP_FILE2=%2
set TEMP_FILE3=%3
set TEMP_FILE4=%4
set CLEANUP=%5
set FTP_SERVER=%6
set TRAIL_PATH=%7
set ID_LOCATION=%8
set USERNAME=%9
shift
set PASSWORD=%9
shift
set PROPS_FILE=%9

type nul > ..\log\%PROPS_FILE%.log

if %TEMP_FILE1% == "" (
  @echo %date% %time% "FAIL: Missing argument" >> ..\log\%PROPS_FILE%.log
  exit /b 88
)			
if %TEMP_FILE2% == "" (
  @echo %date% %time% "FAIL: Missing argument" >> ..\log\%PROPS_FILE%.log
  exit /b 88
)
if %TEMP_FILE3% == "" (
  @echo %date% %time% "FAIL: Missing argument" >> ..\log\%PROPS_FILE%.log
  exit /b 88
)
if %TRAIL_PATH% == "" (
  @echo %date% %time% "FAIL: Missing argument" >> ..\log\%PROPS_FILE%.log 
  exit /b 88
)
if %ID_LOCATION% == "" (
  @echo %date% %time% "FAIL: Missing argument" >> ..\log\%PROPS_FILE%.log 
  exit /b 88
)
if %PROPS_FILE% == "" (
  @echo %date% %time% "FAIL: Missing argument" >> ..\log\%PROPS_FILE%.log 
  exit /b 88
)

type nul > %TEMP_FILE1%
type nul > %TEMP_FILE2%
type nul > %TEMP_FILE3%

mkdir %PROPS_FILE%.dir

type nul > %PROPS_FILE%.dir\%PROPS_FILE%.lst
type nul > %PROPS_FILE%.dir\%PROPS_FILE%_out.lst

@echo %date% %time% "Processing started" >> ..\log\%PROPS_FILE%.log

rem check type of collect: local or remote collect
if %FTP_SERVER% == "" (
  @echo %date% %time% "Collect from local folder" >> ..\log\%PROPS_FILE%.log
  goto local_copy
) else (
  @echo %date% %time% "Collect from ftp-server" >> ..\log\%PROPS_FILE%.log
  goto ftp_transfer
)


:local_copy
FOR /F %%I IN ('@echo %TRAIL_PATH%') DO (set TRAIL_DIR=%%~dpI)
dir /ON /B %TRAIL_DIR%
if %errorlevel% NEQ 0 (
  @echo %date% %time% "ERROR: The data location could not be accessed (Return code 2)" >> ..\log\%PROPS_FILE%.log
  exit /b 2
)

dir /ON /A-D /B %TRAIL_PATH% | findstr /R /C:"^auditlog\..*\.[0-9][0-9][0-9]\.gz$" /R /C:"^auditlog\..*\.[0-9][0-9][0-9]\.Z$" /R /C:"^auditlog\..*\.[0-9][0-9][0-9]$" > %PROPS_FILE%.dir\%PROPS_FILE%.lst
if %errorlevel% NEQ 0 (
  @echo %date% %time% "There are no audit files" >> ..\log\%PROPS_FILE%.log
  goto eof
) else (
  ..\bin\sorttru64fls.exe %PROPS_FILE%.dir\%PROPS_FILE%.lst %PROPS_FILE%.dir\%PROPS_FILE%_out.lst %PROPS_FILE%
  @echo %date% %time% "Files successfully sorted" >> ..\log\%PROPS_FILE%.log
  goto local_cleanup
)

:local_cleanup
FOR /F %%I IN ('@echo %TRAIL_PATH%') DO (set TRAIL_DIR=%%~dpI)
FOR /F %%I IN ('@echo ////%TRAIL_PATH%') DO (set TRAIL_FILE=%%~nxI)
if "%CLEANUP%" EQU "yes" (
  FOR /F %%I IN ('type %PROPS_FILE%.dir\%PROPS_FILE%_out.lst') DO (
    FOR /F %%J IN ('findstr %%I %PROPS_FILE%.dir\%PROPS_FILE%.lst') DO (
      move %TRAIL_DIR%\%%J %PROPS_FILE%.dir
      @echo %date% %time% "File %%J collected by moving" >> ..\log\%PROPS_FILE%.log
    )
  )
) else (
  FOR /F %%I IN ('type %PROPS_FILE%.dir\%PROPS_FILE%_out.lst') DO (
    FOR /F %%J IN ('findstr %%I %PROPS_FILE%.dir\%PROPS_FILE%.lst') DO (
      copy %TRAIL_DIR%\%%J %PROPS_FILE%.dir
      @echo %date% %time% "File %%J collected by copying" >> ..\log\%PROPS_FILE%.log
    )
  )
)
if %errorlevel% NEQ 0 (
  @echo %date% %time% "ERROR: The data location could not be accessed (Return code 2)" >> ..\log\%PROPS_FILE%.log
  exit /b 2
) else (
  cd %PROPS_FILE%.dir
  FOR /F %%I IN ('type %PROPS_FILE%.lst') DO if '%%~xI' EQU '.gz' (..\..\bin\minigzip.exe -d %%I)
  FOR /F %%I IN ('type %PROPS_FILE%.lst') DO if '%%~xI' EQU '.Z' (..\..\bin\minigzip.exe -d %%I)
  cd ..
)
cd %PROPS_FILE%.dir
FOR /F %%I IN ('@echo ////%TRAIL_PATH%') DO (set TRAIL_FILE=%%~nI)
dir /ON /A-D /B /S .\%TRAIL_FILE%.* > %PROPS_FILE%_out1.lst
..\..\bin\prcsstru64logs.exe ..\..\run\%PROPS_FILE%.dir\%PROPS_FILE%_out1.lst ..\%PROPS_FILE% ..\%TEMP_FILE1% ..\..\log\%PROPS_FILE%.log
set RET=%ERRORLEVEL%
del ..\temp1
cd ..
if %errorlevel% NEQ 0 (
  @echo %date% %time% "ERROR: Data processing failed (Exit)" >> ..\log\%PROPS_FILE%.log
  exit /b 88
)

if "%CLEANUP%" EQU "yes" (
  move %ID_LOCATION%\passwd %TEMP_FILE2%
  move %ID_LOCATION%\group %TEMP_FILE3%
  if errorlevel 1 (
    @echo %date% %time% "ERROR: ID files unavailable" >> ..\log\%PROPS_FILE%.log
  ) else (
    @echo %date% %time% "ID files collected by moving" >> ..\log\%PROPS_FILE%.log
  )
) else (
  copy %ID_LOCATION%\passwd %TEMP_FILE2%
  copy %ID_LOCATION%\group %TEMP_FILE3%
  if ERRORLEVEL 1 (
    @echo %date% %time% "ERROR: ID files unavailable" >> ..\log\%PROPS_FILE%.log
  ) else (
    @echo %date% %time% "ID files collected by copying" >> ..\log\%PROPS_FILE%.log
  )
)

if "%CLEANUP%" NEQ "yes" (
  rmdir /S /Q %PROPS_FILE%.dir
  goto eof
) else (
  if "%RET%" EQU "0" (
    rmdir /S /Q %PROPS_FILE%.dir
  )
  goto eof
)



:ftp_transfer
FOR /F %%I IN ('@echo ////%TRAIL_PATH%') DO (set TRAIL_DIR=%%~pI)
set TRAIL_DIR=%TRAIL_DIR:\=/%
FOR /F %%I IN ('@echo ////%TRAIL_PATH%') DO (set TRAIL_FILE=%%~nxI)
..\bin\ncftpls.exe -x "a" -u %USERNAME% -p %PASSWORD% ftp://%FTP_SERVER%/
if %errorlevel% EQU 1 (
  @echo %date% %time% "ERROR: Could not connect to the FTP server (Return code 1)" >> ..\log\%PROPS_FILE%.log
  exit /b 1
)
..\bin\ncftpls.exe -x "a" -u %USERNAME% -p %PASSWORD% ftp://%FTP_SERVER%/"%%2F"%TRAIL_DIR%/
if %errorlevel% EQU 1 (
  @echo %date% %time% "ERROR: Could not connect to the FTP server (Return code 1)" >> ..\log\%PROPS_FILE%.log
  exit /b 1
)
if %errorlevel% EQU 2 (
  @echo %date% %time% "ERROR: The data location could not be accessed (Return code 2)" >> ..\log\%PROPS_FILE%.log
  exit /b 1
)
if %errorlevel% EQU 5 (
  @echo %date% %time% "ERROR: The data location could not be accessed (Return code 2)" >> ..\log\%PROPS_FILE%.log
  exit /b 2
)
if %errorlevel% EQU 7 (
  @echo %date% %time% "ERROR: The data location could not be accessed (Return code 2)" >> ..\log\%PROPS_FILE%.log
  exit /b 2
)

..\bin\ncftpls.exe -x "a" -u %USERNAME% -p %PASSWORD% ftp://%FTP_SERVER%/"%%2F"%TRAIL_DIR%/ | findstr /R /C:"^auditlog\..*\.[0-9][0-9][0-9]\.gz$" /R /C:"^auditlog\..*\.[0-9][0-9][0-9]\.Z$" /R /C:"^auditlog\..*\.[0-9][0-9][0-9]$" > %PROPS_FILE%.dir\%PROPS_FILE%.lst
if %errorlevel% NEQ 0 (
  @echo %date% %time% "There are no audit files" >> ..\log\%PROPS_FILE%.log
  goto eof
) else (
  ..\bin\sorttru64fls.exe %PROPS_FILE%.dir\%PROPS_FILE%.lst %PROPS_FILE%.dir\%PROPS_FILE%_out.lst %PROPS_FILE%
  @echo %date% %time% "Files successfully sorted" >> ..\log\%PROPS_FILE%.log
  goto ftp_cleanup
)

:ftp_cleanup
if "%CLEANUP%" EQU "yes" (
  cd %PROPS_FILE%.dir
  FOR /F %%I IN ('type %PROPS_FILE%_out.lst') DO (
    FOR /F %%J IN ('findstr %%I %PROPS_FILE%.lst') DO (
      ..\..\bin\ncftpget.exe -DD -u %USERNAME% -p %PASSWORD% %FTP_SERVER% . %TRAIL_DIR%"%%J"
      @echo %date% %time% "File %%J collected by moving" >> ..\..\log\%PROPS_FILE%.log
    )
  )
  cd ..
) else (
  cd %PROPS_FILE%.dir
  FOR /F %%I IN ('type %PROPS_FILE%_out.lst') DO (
    FOR /F %%J IN ('findstr %%I %PROPS_FILE%.lst') DO (
      ..\..\bin\ncftpget.exe -u %USERNAME% -p %PASSWORD% %FTP_SERVER% . %TRAIL_DIR%"%%J"
      @echo %date% %time% "File %%J collected by copying" >> ..\..\log\%PROPS_FILE%.log
    )
  )
  cd ..
)
if %errorlevel% EQU 1 (
  @echo %date% %time% "ERROR: Could not connect to the FTP server (Return code 1)" >> ..\log\%PROPS_FILE%.log
  exit /b 1
)
if %errorlevel% EQU 0 (
  cd %PROPS_FILE%.dir
  FOR /F %%I IN ('type %PROPS_FILE%.lst') DO if '%%~xI' EQU '.gz' (..\..\bin\minigzip.exe -d %%I)
  FOR /F %%I IN ('type %PROPS_FILE%.lst') DO if '%%~xI' EQU '.Z' (..\..\bin\minigzip.exe -d %%I)
  cd ..
)

cd %PROPS_FILE%.dir
FOR /F %%I IN ('@echo ////%TRAIL_PATH%') DO (set TRAIL_FILE=%%~nI)
dir /ON /A-D /B /S .\%TRAIL_FILE%.* > %PROPS_FILE%_out1.lst
..\..\bin\prcsstru64logs.exe ..\..\run\%PROPS_FILE%.dir\%PROPS_FILE%_out1.lst ..\%PROPS_FILE% ..\%TEMP_FILE1% ..\..\log\%PROPS_FILE%.log
set RET=%ERRORLEVEL%
del ..\temp1
cd ..
if %errorlevel% NEQ 0 (
  @echo %date% %time% "ERROR: Data processing failed (Exit)" >> ..\log\%PROPS_FILE%.log
  exit /b 88
)
if "%CLEANUP%" EQU "yes" (
  cd %PROPS_FILE%.dir
  ..\..\bin\ncftpget.exe -DD -u %USERNAME% -p %PASSWORD% %FTP_SERVER% . %ID_LOCATION%/passwd
  ..\..\bin\ncftpget.exe -DD -u %USERNAME% -p %PASSWORD% %FTP_SERVER% . %ID_LOCATION%/group
  move passwd ..\%TEMP_FILE2%
  move group ..\%TEMP_FILE3% 
  if errorlevel 1 (
    @echo %date% %time% "ERROR: ID files unavailable" >> ..\..\log\%PROPS_FILE%.log
  ) else (
    @echo %date% %time% "ID files collected by moving" >> ..\..\log\%PROPS_FILE%.log
  )
  cd ..
) else (
  cd %PROPS_FILE%.dir
  ..\..\bin\ncftpget.exe -u %USERNAME% -p %PASSWORD% %FTP_SERVER% . %ID_LOCATION%/passwd
  ..\..\bin\ncftpget.exe -u %USERNAME% -p %PASSWORD% %FTP_SERVER% . %ID_LOCATION%/group
  move passwd ..\%TEMP_FILE2%
  move group ..\%TEMP_FILE3%
  if errorlevel 1 (
    @echo %date% %time% "ERROR: ID files unavailable"  >> ..\..\log\%PROPS_FILE%.log
  ) else (
    @echo %date% %time% "ID files collected by copying" >> ..\..\log\%PROPS_FILE%.log
  )
  cd ..
)
if "%CLEANUP%" NEQ "yes" (
  rmdir /S /Q %PROPS_FILE%.dir
  goto eof
) else (
  if "%RET%" EQU "0" (
    rmdir /S /Q %PROPS_FILE%.dir
  )
  goto eof
)

:eof
@echo %date% %time% "Finish processing (Return code 0)" >> ..\log\%PROPS_FILE%.log
exit /b 0