rem IBM Confidential
rem OCO Source Material
rem 5724-S67
rem (c) Copyright IBM Corp. 1998, 2010
rem The source code for this program is not published or otherwise divested of
rem its trade secret, irrespective of what has been deposited with the U.S.
rem Copyright Office.

@echo off

rem ################################## getmswinaddslog.bat ###############################
rem Receives parameters from the Actuator
rem ######################################################################################

setlocal enabledelayedexpansion
set TSIEM_BIN_DIR=%~dp0%
set TSIEM_BIN_DIR=%TSIEM_BIN_DIR:~0,-1%

set LOGSET1=%~1
set LOGSET2=%~2
set LOGSET3=%~3
set LOGSET4=%~4
set LOGSET5=%~5
set LOG_DIR=%~6
set FILE_PATTERN1=%~7
set FILE_PATTERN2=%~8
set FILE_PATTERN3=%~9
shift
set FILE_PATTERN4=%~9
shift
set FILE_PATTERN5=%~9
for %%I in (%*) do set PROPS_FILE=%%~I
for %%I in (%PROPS_FILE%) do set ES_NUMBER=%%~nI

rem ######################################################################################
rem Actuator parameters check
rem ######################################################################################
set PRINTING=enabled

rem -- sets actuator log name
if "%PROPS_FILE%" == "" (
   set ES_LOG="..\log\gettsiemserverlogs-error.log"
) else (
   set ES_LOG="..\log\%ES_NUMBER%.log"
)

type nul > %ES_LOG%

rem -- Property file PROPS_FILE argument is missing
if "%PROPS_FILE%" == "" (
   call :print_message "ERROR: Missing property file argument." %ES_LOG%
   goto failure
)
if not exist "%PROPS_FILE%" (
   type nul > "%PROPS_FILE%" 2>>%ES_LOG%
   if errorlevel 1 (
      call :print_message "ERROR: "Failure creating propsfile '!PROPS_FILE!'." %ES_LOG%
      goto failure
   )
)

if "%LOG_DIR%" == "" (
   call :print_message "ERROR: Missing log directory path argument." %ES_LOG%
   goto failure
) else (
   if not exist "%LOG_DIR%\" (
      call :print_message "ERROR: Specified in ES properties directory with logfiles does not exist: '%LOG_DIR%'." %ES_LOG%
      goto failure
   )
)
call :print_message "Processing started." %ES_LOG%

pushd "%LOG_DIR%"
set LOG_DIR=%CD%
popd
call :print_message "Source log directory path is '%LOG_DIR%'." %ES_LOG%

set WORK_TMP_DIR=%SystemRoot%\Temp\ES-%ES_NUMBER%
set GROUPS_LIST="%WORK_TMP_DIR%\groups.lst"
set LOGS_LIST="%WORK_TMP_DIR%\logs.lst"
set TEMP_LOG="%WORK_TMP_DIR%\temp.log"
set DDIN_FILE="%WORK_TMP_DIR%\ddin.tmp"
set DDOUT_FILE="%WORK_TMP_DIR%\ddout.tmp"
set NEW_PROPS="%WORK_TMP_DIR%\%ES_NUMBER%.props"
set FILEMODTIME_SCRIPT=%TSIEM_BIN_DIR%\filemodtime.vbs
set DD_EXE=%TSIEM_BIN_DIR%\smalldd.exe

call :print_message "Create temporary working directory." %ES_LOG%
if exist "%WORK_TMP_DIR%\" (
   rmdir /S /Q "%WORK_TMP_DIR%" >nul 2>>%ES_LOG%
   if exist "%WORK_TMP_DIR%\" (
      call :print_message "ERROR: Failure removing existing temporary working directory '%WORK_TMP_DIR%'." %ES_LOG%
      goto failure
   )
)
mkdir "%WORK_TMP_DIR%" >nul 2>>%ES_LOG%
if not exist "%WORK_TMP_DIR%\" (
   call :print_message "ERROR: Failure creating temporary working directory '%WORK_TMP_DIR%'." %ES_LOG%
   goto failure
)

set COLLECT_PERFORMED=0

call :collect_log "%LOGSET1%" BlueBook    "%FILE_PATTERN1%" || goto failure
call :collect_log "%LOGSET2%" AuditCtl    "%FILE_PATTERN2%" || goto failure
call :collect_log "%LOGSET3%" MainMapper  "%FILE_PATTERN3%" || goto failure
call :collect_log "%LOGSET4%" AgentSystem "%FILE_PATTERN4%" || goto failure
call :collect_log "%LOGSET5%" AuthDaemon  "%FILE_PATTERN5%" || goto failure

echo.>> %ES_LOG%
if %COLLECT_PERFORMED% equ 0 goto failure

call :print_message "Saving state information in properties file." %ES_LOG%
copy /b /y "%NEW_PROPS%" "%PROPS_FILE%" >nul 2>>%ES_LOG%
if errorlevel 1 (
   call :print_message "ERROR: Failure saving new state information." %ES_LOG%
   goto failure
)
call :cleanup
call :print_message "Processing finished [SUCCESS]." %ES_LOG%
exit /b 0


:failure

call :cleanup
call :print_message "Processing finished [FAILURE]." %ES_LOG%
exit /b 1


rem ######################################################################################
rem procedure for collect events for specified log
rem  %1 - log type
rem ######################################################################################

:collect_log

set LOGSET=%1
set LOG_TYPE=%~2
set FILE_PATTERN=%~3

echo.>> %ES_LOG%
call :print_message "*****************************************" %ES_LOG%
call :print_message "  Start processing %LOG_TYPE% log" %ES_LOG%
call :print_message "*****************************************" %ES_LOG%

if %LOGSET% == "" (
   call :print_message "ERROR: Missing output file name argument for sublog of %LOG_TYPE% logs." %ES_LOG%
   exit /b 1
) else (
   type nul > %LOGSET% 2>>%ES_LOG%
   if errorlevel 1 (
      call :print_message "ERROR: Failure creating/clearing logset." %ES_LOG%
      exit /b 1
   )
)

if "%FILE_PATTERN%" == "" (
   call :print_message "ERROR: Missing regular expression of the logfiles name argument." %ES_LOG%
   exit /b 1
) else (
   call :print_message "Regular expression of the logfiles name is '%FILE_PATTERN%'." %ES_LOG%
)

call :create_group_list

(set READ_BYTES=)
set RESULT_CODE=0
for /f %%G in ('type %GROUPS_LIST%') do (
    set LOGS_GROUP=%%G
    call :print_message "- Collect for group '!LOGS_GROUP!' started." %ES_LOG%
    call :print_message "Reading state information from '%PROPS_FILE%'." %ES_LOG%
    call :parse_props_file
    set LAST_LTS=!PREV_LTS!
    set FIRST_LOG=1
    call :attempts_cycle
    if !RESULT_CODE! equ 0 (
       if "!LAST_LTS!" == "" (
          call :print_message "Calculate the properties for the next collect." %ES_LOG%
          for /f "tokens=*" %%J in ('""%TSIEM_BIN_DIR%\date.exe" "+%%Y-%%m-%%dT%%H:%%m:%%S"" 2^>^>%ES_LOG%') do set PREV_LTS=%%J
          if errorlevel 1 set DATE_ERR=1
          if "!PREV_LTS!" == "" set DATE_ERR=1
          if !DATE_ERR! equ 1 (
             call :print_message "ERROR: Failure getting current timestamp." %ES_LOG%
             exit /b 1
          )
          set READ_BYTES=0
          set FIRST_LINE=noline
       ) else (
          call :get_file_ts %TEMP_LOG%
          PREV_LTS=!FILE_MODTIME!
          if "!PREV_LTS!" gtr "!CURR_LTS!" (
             set PREV_LTS=!CURR_LTS!
          )
       )
    )
    call :print_message "Timestamp for the next collect is '!PREV_LTS!'." %ES_LOG%
    for %%F in (%LOGSET%) do set LOGSET_SIZE=%%~zF
    call :print_message "The total size of new collected events is !LOGSET_SIZE! bytes." %ES_LOG%
    call :print_message "Saving current state information." %ES_LOG%
    echo !LOGS_GROUP!_MODTIME=!PREV_LTS!>> "%NEW_PROPS%"
    echo !LOGS_GROUP!_BYTESCOUNT=!READ_BYTES!>> "%NEW_PROPS%"
    echo !LOGS_GROUP!_FIRSTPART="!FIRST_LINE!">> "%NEW_PROPS%"
    set FLG1=0
    if !RESULT_CODE! equ 0 set FLG1=1
    if !LOGSET_SIZE! gtr 0 set FLG1=1
    if !FLG1! equ 1 (
       set COLLECT_PERFORMED=1
       call :print_message "- Collect for group '!LOGS_GROUP!' finished [SUCCESS]." %ES_LOG%
    ) else (
       call :print_message "- Collect for group '!LOGS_GROUP!' finished [FAILURE]." %ES_LOG%
    )
)

if !RESULT_CODE! equ 0 (
   if "%READ_BYTES%" == "" call :print_message "ERROR: Logfiles for '%LOG_TYPE%' log not found." %ES_LOG%
)
exit /b 0


rem ######################################################################################
rem nested cycle by rollover attempts. maximum 100 attempts
rem ######################################################################################

:attempts_cycle

set ATTEMPT=0

:start_atcl
if !ATTEMPT! equ 100 goto end_atcl
    set /a ATTEMPT+=1
    call :get_control_attr
    call :get_logs_list || exit /b 1
    if !LOGS_NUMBER! equ 1 (
       if not "!CONTROL_ATTR!" == "" (
          call :print_message "ERROR: Failure getting attributes of required logfiles for the group '!LOGS_GROUP!'." %ES_LOG%
          set RESULT_CODE=1
          exit /b
       )
    )
    if !LOGS_NUMBER! gtr 1 (
       if "!CONTROL_ATTR!" == "" (
          print_message "INFO: Rollover accured during logfiles list creation. Recreate list." "${ES_LOG}"
          goto start_atcl
       )
    )
    set FOUND_PREV=0
    if "!PREV_LTS!" == "" set FOUND_PREV=1
    if "!FIRST_LINE!" == "noline" set FOUND_PREV=1
    set ROTATED=0
    (set LAST_ATTR=)
    call :logs_cycle
    if !RESULT_CODE! neq 0 (
       call :print_message "Collect failure for logfile '%CURR_LOG%'." %ES_LOG%
       exit /b
    )
    if !ROTATED! equ 0 (
       exit /b
    ) else (
       call :print_message "INFO: Rollover accured during collect process. Collecting newly created logfiles." %ES_LOG%
    )
    goto start_atcl
:end_atcl
call :print_message "ERROR: Logfiles archives are produced quicker than they can be collected. Please, change log archiving settings." %ES_LOG%
call :print_message "INFO: Please increase the size of active logfile to avoid such situation in the future and repeat the collect." %ES_LOG%
set RESULT_CODE=1
exit /b


rem ######################################################################################
rem second nested cycle by logfiles for current group
rem ######################################################################################

:logs_cycle

for /f "delims=" %%L in ('type "%LOGS_LIST%"') do (
    set LOG_NAME=%%~L
    set CURR_LOG=%LOG_DIR%\!LOG_NAME!
    call :get_file_ts "!CURR_LOG!" || exit /b
    set CURR_LTS=!FILE_MODTIME!
    set FLG2=0
    if "!CURR_LTS!" gtr "!LAST_LTS!" set FLG2=1
    if "!CURR_LTS!" == "!LAST_LTS!"  set FLG2=1
    if !FLG2! equ 1 (
       call :get_first_line "!CURR_LOG!" || exit /b
       set FLG3=0
       if !FIRST_LOG! equ 0 (
          if "!CURR_LTS!" == "!LAST_LTS!" (
             if "!FIRST_LINE!" == "!CURR_FL!" set FLG3=1
          )
       )
       if !FLG3! equ 1 (
          set FOUND_PREV=1
          type nul > %TEMP_LOG%
       ) else (
          if !FOUND_PREV! equ 0 (
             if not "!FIRST_LINE!" == "!CURR_FL!" (
                call :print_message "WARNING: Last collected logfile from previous collect was not found, some events may have been lost." %ES_LOG%
                call :print_message "INFO: Please collect more often to avoid such situations in the future." %ES_LOG%
                call :print_message "INFO: Collect everything available for logfiles group '!LOGS_GROUP!'." %ES_LOG%
                set FOUND_PREV=1
             )
          )
          call :print_message "Collect events from logfile '!CURR_LOG!'." %ES_LOG%
          copy /b /y "!CURR_LOG!" %TEMP_LOG% >nul 2>>%ES_LOG%
          if errorlevel 1 (
             call :print_message "ERROR: Failure copying logfile to temporary working file '!CURR_LOG!'." %ES_LOG%
             set RESULT_CODE=1
             exit /b
          )
          set LAST_ATTR=!CONTROL_ATTR!
          if !LOGS_NUMBER! equ 1 (
             call :get_logs_list || exit /b
             if !LOGS_NUMBER! equ 1 (
                (set CONTROL_ATTR=)
             ) else (
                set CONTROL_ATTR=1
             )
          ) else (
             call :get_control_attr
          )
          if not "!CONTROL_ATTR!" == "!LAST_ATTR!" (
             set ROTATED=1
             exit /b
          )
       )
       for %%I in (%TEMP_LOG%) do set TEMP_LOG_SIZE=%%~zI
       if !TEMP_LOG_SIZE! neq 0 (
          set FLG4=0
          if !FOUND_PREV! equ 0 (
             if "!FIRST_LINE!" == "!CURR_FL!" set FLG4=1
          )
          if !FLG4! equ 1 (
             set FOUND_PREV=1
             if !TEMP_LOG_SIZE! gtr !READ_BYTES! (
                "%DD_EXE%" %READ_BYTES% EOF %TEMP_LOG% %DDOUT_FILE% 2>>%ES_LOG%
                if errorlevel 1 set RESULT_CODE=1
                copy /b /y %LOGSET%+%DDOUT_FILE% %LOGSET% 2>>%ES_LOG%
                if errorlevel 1 set RESULT_CODE=1
             )
          ) else (
             copy /b /y %LOGSET%+%TEMP_LOG% %LOGSET% 2>>%ES_LOG%
             if errorlevel 1 set RESULT_CODE=1
          )
          if !RESULT_CODE! equ 1 (
             call :print_message "ERROR: Failure copying temporary logfile to result logset file '!TEMP_LOG!'." %ES_LOG%
             # may be purge logset file and return properties to values before collect started
             exit /b
          )
          set PREV_LTS=!CURR_LTS!
          set READ_BYTES=!TEMP_LOG_SIZE!
          set FIRST_LINE=!CURR_FL!
       )
       set LAST_LTS=!CURR_LTS!
       set FIRST_LOG=0
    )
)
exit/b


rem #########################################################################################
rem Parsing properties file PROPS_FILE for setting state variables for current group
rem of logfiles LOGS_GROUP
rem #########################################################################################

:parse_props_file

(set PREV_LTS=)
for /f "tokens=2 delims==" %%I in ('findstr /r /c:"^%LOGS_GROUP%_MODTIME=.*" "%PROPS_FILE%" 2^>^>%ES_LOG%') do set PREV_LTS=%%I
for %%I in (%PREV_LTS%) do set PREV_LTS=%%I
(set READ_BYTES=)
for /f "tokens=2 delims==" %%I in ('findstr /r /c:"^%LOGS_GROUP%_BYTESCOUNT=.*" "%PROPS_FILE%" 2^>^>%ES_LOG%') do set READ_BYTES=%%I
for %%I in (%READ_BYTES%) do set READ_BYTES=%%I
(set FIRST_LINE=)
for /f "tokens=2 delims==" %%I in ('findstr /r /c:"^%LOGS_GROUP%_FIRSTPART=.*" "%PROPS_FILE%" 2^>^>%ES_LOG%') do set FIRST_LINE=%%~I

set INITIAL_MODE=0
if "%PREV_LTS%" == "" set INITIAL_MODE=1
if "%READ_BYTES%" == "" set INITIAL_MODE=1
if "!FIRST_LINE!" == "" set INITIAL_MODE=1
if %INITIAL_MODE% EQU 1 (
   (set PREV_LTS=)
   call :print_message "Initial collect mode use." %ES_LOG%
) else (
   call :print_message "Events will be collected from '%PREV_LTS%'" %ES_LOG%
)
exit /b


rem ######################################################################################
rem create logfiles groups list file with name GROUPS_LIST
rem ######################################################################################

:create_group_list

type nul > %GROUPS_LIST%
(set OLD_PREFIX=)
for /f %%I in ('dir /b /a-d /on "%LOG_DIR%" ^| findstr /r /c:"^%FILE_PATTERN%$"') do (
    for /f "tokens=1 delims=." %%J in ("%%I") do set CUR_PREFIX=%%J
    if not "!CUR_PREFIX!" == "!OLD_PREFIX!" (
       echo !CUR_PREFIX!>> %GROUPS_LIST%
       set OLD_PREFIX=!CUR_PREFIX!
    )
)
exit /b


rem ######################################################################################
rem set variable FILE_MODTIME to modification timestamp of required file
rem   %1 - filename
rem ######################################################################################

:get_file_ts

for /f "tokens=1,* delims=|" %%a in ('cscript.exe //Nologo "%FILEMODTIME_SCRIPT%" "%~1"') do (
    set RESULT1=%%a
    set RESULT2=%%b
)
if "%RESULT1%" == "SUCCESS" (
   if "%RESULT2%" == "" (
      call :print_message "ERROR: Failure getting timestamp for logfile '%CURR_LOG%'." %ES_LOG%
      set RESULT_CODE=1
      exit /b 1
   )
   set FILE_MODTIME=%RESULT2%
) else (
   (set FILE_MODTIME=)
   call :print_message "ERROR: Script for determining file timestamp return failure." %ES_LOG%
   if "%RESULT2%" == "" (
      call :print_message "%RESULT1%" %ES_LOG%
   ) else (
      call :print_message "%RESULT2%" %ES_LOG%
   )
   set RESULT_CODE=1
   exit /b 1
)
exit /b 0


rem ######################################################################################
rem put list of logfiles for current group LOGS_GROUP into LOGS_LIST file and set variable
rem LOGS_NUMBER to number of found logfiles
rem ######################################################################################

:get_logs_list

type nul > %LOGS_LIST%
set LOGS_NUMBER=0
for /f "delims=" %%F in ('dir /b /a-d /od /t:w "%LOG_DIR%" ^| findstr /r /c:"^%FILE_PATTERN%$" ^| findstr /r /c:"!LOGS_GROUP!\."') do (
    echo "%%F">> %LOGS_LIST%
    set /a LOGS_NUMBER+=1
)
if %LOGS_NUMBER% equ 0 (
   call :print_message "ERROR: Failure getting list of required logfiles for the group '%~1'." %ES_LOG%
   set RESULT_CODE=1
   exit /b 1
)
exit /b 0


rem ######################################################################################
rem Get attributes for the second file of the logfiles list for current group,
rem sorted by descend of modification timestamp (newest - first).
rem This algorithm used specially for cover two types of logs rotations:
rem  - insert/shift/rename
rem  - append/replace cyclic
rem ######################################################################################

:get_control_attr

(set CONTROL_ATTR=)
set N=0
for /f "delims=" %%F in ('dir /a-d /o-d /t:w "%LOG_DIR%" ^| findstr /r /c:"^.*[0-9][0-9]* %FILE_PATTERN%$" ^| findstr /r /c:"!LOGS_GROUP!\."') do (
    set /a N+=1
    if !N! equ 2 (
       set CONTROL_ATTR=%%F
       exit /b
    )
)
exit /b


rem ######################################################################################
rem get first 100 bytes of the first line
rem   %1 - filename
rem ######################################################################################

:get_first_line

set /p CURR_FL=<%1
echo "!CURR_FL!"> %DDIN_FILE%
"%DD_EXE%" BOF 100 %DDIN_FILE% %DDOUT_FILE% 2>>%ES_LOG%
if errorlevel 1 (
   call :print_message "ERROR: Failure reading first 100 bytes of the first line of current logfile '!CURR_LOG!'." %ES_LOG%
   set RESULT_CODE=1
   exit /b 1
)
for /f "delims=" %%I in ('type %DDOUT_FILE%') do set CURR_FL=%%~I

exit /b 0


rem ######################################################################################
rem printing routine
rem prints a message to actuator log.
rem takes two parameters: %1 - message itself, %2 - name of a log file
rem ######################################################################################

:print_message

if "%PRINTING%" == "enabled" @echo [%date%][%time%] %~1>> %2
exit /b


rem ######################################################################################
rem Cleanup routine. Deleting temp directories and unmaping share
rem ######################################################################################

:cleanup

call :print_message "Free and clean up all used system resources." %ES_LOG%
if not "%WORK_TMP_DIR%" == "" (
   if exist "%WORK_TMP_DIR%\" (
      call :print_message "Remove temporary working directory." %ES_LOG%
      rmdir /S /Q "%WORK_TMP_DIR%" >nul 2>>%ES_LOG%
      if exist "%WORK_TMP_DIR%\" (
         call :print_message "WARNING: Failure to remove existing temporary working directory: '%WORK_TMP_DIR%'." %ES_LOG%
      )
   )
)
exit /b
