/// CiscoAsa7x.src.gml - Licensed Materials ­ Property of IBM
/// CiscoAsa7x.src.gml - 5724-S67
/// CiscoAsa7x.src.gml - © Copyright IBM Corp. 1998, 2007   All Rights Reserved.
/// CiscoAsa7x.src.gml - US Government Users Restricted Right ­ Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

key PlatformRefKey(platformtype, platformname)
key OriginRefKey(platformtype, platformname)
key TargetRefKey(platformtype, platformname)
key ObjectRefKey(objecttype, objectpath, objectname)
key StreamRefKey(description)
key SourceRefKey(realname, logonname)
key EventDetailKey(message,platformeventendtime)

first state Setup
  description:="Activity"
  equality:="host"
  data:=""
  period:=GetIniProperty("ActivityPeriod","600000") 
  threshold:=GetIniProperty("ActivityThreshold","1200")
  activityalerter:=GetListener(SumCounter)
  Listen(activityalerter,Setup)

  description := "Aggregator"

  equality := "eventmainclass,eventclass,successclass,realname,logonname,objecttype,objectpath,objectname,origin,target" 
  targets := ""
  separator := "."
  period := GetIniProperty("AggregationPeriod","600000")
  threshold := GetIniProperty("AggregationThreshold","1")
  aggregator := GetListener(Aggregator)
  Listen(aggregator,Setup)

  description:="HostScan"
  equality:="host,origin,objectname"
  data:="target"
  period:=GetIniProperty("HostScanPeriod","300000") 
  threshold:=GetIniProperty("HostScanThreshold","4")
  hostscandetector:=GetListener(DataCounter)

  description:="PortScan"
  equality:="host,origin,target"
  data:="serviceport"
  period:=GetIniProperty("PortScanPeriod","300000") 
  threshold:=GetIniProperty("PortScanThreshold","4")
  portscandetector:=GetListener(DataCounter)

  MoveTo(ListenState)
endstate 

last state Shutdown
  if aggregator ne null then 
    StopListening(aggregator)
    StopListening(activityalerter)
    StopListening(portscandetector)
    StopListening(hostscandetector)
  endif
endstate

state ListenState(platformeventcount eq null)
  Listen(aggregator,ListenState)
  Listen(activityalerter,ListenState)
  if ListenState.audit eq "yes" then
    Listen(hostscandetector,ListenState)
    Listen(portscandetector,ListenState)
  endif
endstate

initial final state ASAEvent
  (( (platformeventcount ne null) and
     (description eq "Aggregator")  ))

    platformname:=ASAEvent.host
    platformtype:="Cisco-ASA"
    platform:=GetPlatform(PlatformRefKey)

    objecttype := Trim(ASAEvent.objecttype)
    objectpath := Trim(ASAEvent.objectpath)
    objectname := Trim(ASAEvent.objectname)

    if (objecttype eq "SYSTEM") then
      objectpath := Trim(objectpath,".")
      if (objectname eq "-") then
        objectname := null
      endif    
      objectname := Trim(objectname,platformname,GetChunkProperty(ASAEvent,"platformname"))
    else
      objectname := Trim(objectname,"-")
    endif

    if ( ((objectpath eq null) or (objectpath eq "") or (objectpath eq "-")) and (objecttype ne "CONNECTION")) then
      objectpath := Trim(Path(objectname), "-")
      objectname := Trim(Filename(objectname),"-")
    else
      objectpath := "-"
    endif

    object:=GetObject(ObjectRefKey)

    if ((ASAEvent.target ne null) and (Trim(ASAEvent.target) ne "")) then
      platformname:=ASAEvent.target
      platformtype:="-"
    else
      platformname:=ASAEvent.host
      platformtype:="Cisco-ASA"
    endif
    target:=GetTarget(TargetRefKey)

    description:=Trim(ASAEvent.logonname,"System")
    stream:=GetStream(StreamRefKey) 

    logonname := Trim(ASAEvent.logonname,"System")
    realname  := Trim(ASAEvent.realname,"System")
    source:=GetSource(SourceRefKey)

    if ((ASAEvent.origin ne null) and (Trim(ASAEvent.origin) ne "")) then
      platformname:=ASAEvent.origin
      platformtype:="-"
    else
      platformname:=ASAEvent.host
      platformtype:="Cisco-ASA"    
    endif
    origin:=GetOrigin(OriginRefKey)

    eventclass:=ASAEvent.eventclass
    eventmainclass:=ASAEvent.eventmainclass
    successclass:=Trim(ASAEvent.successclass, "Success")
    eventtimestamp:=ASAEvent.platformeventtime
    chunk:=ASAEvent
    recordtype:="Cisco-ASA"
    if ASAEvent.platformeventcount ne null then
      eventcount := ASAEvent.platformeventcount
    endif
    message := ASAEvent.message
    if (ASAEvent.platformeventendtime ne null) then
      platformeventendtime:=Append("\"",ASAEvent.platformeventendtime,"\"")
    endif
    event:=GetEvent(EventDetailKey)
endstate

initial final state ASAActivity
  (( (platformeventcount ne null) and (description eq "Activity")  ))

    platformname:=ASAActivity.host
    platformtype:="Cisco-ASA"
    platform:=GetPlatform(PlatformRefKey)
    origin:=GetOrigin(OriginRefKey)
    target:=GetTarget(TargetRefKey)

    objecttype := "Activity"
    objectpath := ASAActivity.period
    objectname := ASAActivity.platformeventcount
    object:=GetObject(ObjectRefKey)

    description:=ASAActivity.host
    stream:=GetStream(StreamRefKey) 

    realname:="System"
    logonname:="System"
    source:=GetSource(SourceRefKey)

    eventmainclass:="Alert"
    eventclass:="Activity"
    successclass:="Success"
    eventtimestamp:=ASAActivity.platformeventtime
    chunk:=ASAActivity
    recordtype:="ASAActivity"
    if ASAActivity.platformeventcount ne null then
      eventcount := ASAActivity.platformeventcount
    endif
    message:="Activity"
    if ( ASAActivity.platformeventendtime ne null) then
      platformeventendtime:=Append("\"",ASAActivity.platformeventendtime,"\"")
    endif
    event:=GetEvent(EventDetailKey)
endstate 

initial state HostScan
  (( (platformeventcount ne null) and (description eq "HostScan")  ))

    platformname:=HostScan.host
    platformtype:="Cisco-ASA"
    platform:=GetPlatform(PlatformRefKey)

    objecttype := "Net"
    objectpath := "-"
    objectname := HostScan.platformeventcount
    object:=GetObject(ObjectRefKey)

    platformname:=HostScan.host
    platformtype:="Cisco-ASA"
    target:=GetTarget(TargetRefKey)

    description:=HostScan.logonname
    stream:=GetStream(StreamRefKey)

    realname:="Anonymous"
    logonname:="Anonymous"
    source:=GetSource(SourceRefKey)

    platformname:=HostScan.origin
    platformtype:="Host"
    origin:=GetOrigin(OriginRefKey)

    eventmainclass:="Alert"
    eventclass:="HostScan"
    successclass:="Success"
    eventtimestamp:=HostScan.platformeventtime
    chunk:=HostScan
    recordtype:="HostScan"
    if HostScan.platformeventcount ne null then
      eventcount := HostScan.platformeventcount
    endif
    message:=HostScan.message 
    if (HostScan.platformeventendtime ne null) then
      platformeventendtime:=Append("\"",HostScan.platformeventendtime,"\"")
    endif
    event:=GetEvent(EventDetailKey)
endstate 

initial state PortScan
  (( (platformeventcount ne null) and (description eq "PortScan")  ))

    platformname:=PortScan.host
    platformtype:="Cisco-ASA"
    platform:=GetPlatform(PlatformRefKey)

    objecttype := "Host"
    objectpath := PortScan.target
    objectname := PortScan.platformeventcount
    object:=GetObject(ObjectRefKey)

    platformname:=PortScan.target
    platformtype:="Host"
    target:=GetTarget(TargetRefKey)

    description:=PortScan.logonname
    stream:=GetStream(StreamRefKey)

    realname:="Anonymous"
    logonname:="Anonymous"
    source:=GetSource(SourceRefKey)

    platformname:=PortScan.origin
    platformtype:="Host"
    origin:=GetOrigin(OriginRefKey)

    eventmainclass:="Alert"
    eventclass:="PortScan"
    successclass:="Success"
    eventtimestamp:=PortScan.platformeventtime
    chunk:=PortScan
    recordtype:="PortScan"
    if PortScan.platformeventcount ne null then
      eventcount := PortScan.platformeventcount
    endif
    message:=PortScan.message 
    if (PortScan.platformeventendtime ne null) then
      platformeventendtime:=Append("\"",PortScan.platformeventendtime,"\"")
    endif
    event:=GetEvent(EventDetailKey)
endstate 
