/// actdir.src.gml - Licensed Materials ­ Property of IBM
/// actdir.src.gml - 5724-S67
/// actdir.src.gml - © Copyright IBM Corp. 1998, 2007   All Rights Reserved.
/// actdir.src.gml - US Government Users Restricted Right ­ Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

key PlatformRefKey(platformtype, platformname)
key ObjectRefKey(objecttype, objectpath, objectname)
key StreamRefKey(description)

key SourceRefKey(logonname)
key SourceAspectKey(originator)

key OriginRefKey(platformtype, platformname)
key TargetRefKey(platformtype, platformname)
key EventDetailKey(recordtype, logontype, recnum, platformeventendtime, authenticationpackage, result, reason, attribute, value, access, newvalue, arguments, id, user, protocol, Target, Source, message)

first state Setup
  description := "Aggregator"
  equality := "emainclass,eclass,successclass,objectname,objectpath,objecttype,usersid"
  targets := ""
  separator := "."
  period := GetIniProperty("AggregationPeriod","2000")
  threshold := GetIniProperty("AggregationThreshold","1")
  aggregator := GetListener(Aggregator)
  Listen(aggregator,Setup)
  MoveTo(Listening)
endstate

state Listening (platformeventcount eq null)
  Listen(aggregator,Listening)
endstate

last state Shutdown
 if aggregator ne null then
     StopListening(aggregator)
     aggregator := null
 endif
endstate

initial final state ADEvent (( (platformeventcount ne null) and (description eq "Aggregator") ))
  _platformtype := "Active Directory"
  epiname := Trim(ADEvent.epiname)
  domain := Trim(ADEvent.domain)
  if ((domain ne "") and (domain ne null)) then
    _platformname := Append(domain, "\\", epiname)
  else
    _platformname := epiname
  endif

  platformtype := _platformtype
  platformname := _platformname
  platform := GetPlatform(PlatformRefKey)
  wherefrom := Trim(ADEvent.wherefrom)
  if ((wherefrom ne "") and (wherefrom ne null)) then
   platformtype := "-"
   platformname := wherefrom
  endif
  origin := GetOrigin(OriginRefKey)
  whereto := Trim(ADEvent.whereto)
  if ((whereto ne "") and (whereto ne null)) then
    platformtype := "-"
    platformname := whereto
  else
    platformtype := _platformtype
    platformname := _platformname
  endif
  target:=GetTarget(TargetRefKey)

  objecttype := Trim(ADEvent.objecttype, "OBJECT")
  objectpath := Trim(ADEvent.objectpath, "-")
  objectname := Trim(ADEvent.objectname, "-")
  if (objectname ne "-") then
    objectpathfromname := Trim(Path(objectname), ".")
    objectnamefromname := Trim(Filename(objectname))
    if objectpathfromname ne "." then
      objectpath := objectpathfromname
      objectname := objectnamefromname
    endif
    if objectpath eq "." then
      if objectname ne "-" then
        objectpath := Trim(Path(objectname), ".")
        objectname := Trim(Filename(objectname), "-")
      endif
    endif
  else if objecttype eq "SYSTEM" then
    objectname := epiname
  endif
  endif

  object:=GetObject(ObjectRefKey)

  sid:=ADEvent.usersid
  case sid of
    "S-1-1-0":
      logonname := "SYSTEM"
    "S-1-5-7", "\\":
      logonname := "ANONYMOUS"
    "S-1-5-18":
      logonname := "SYSTEM"
    otherwise
      logonname := Trim(ADEvent.who, GetLogonName(sid))
  endcase

  if (sid eq "NOSID") then
    sid := GetOriginator(logonname)
  endif

  if (logonname eq "NOSID") then
    logonname := "SYSTEM"
  endif

  originator := sid
  realname := Trim(logonname)

  source   := GetSource(SourceRefKey, SourceAspectKey)

  description := Append(domain, "/", epiname, "/", logonname);
  stream := GetStream(StreamRefKey)

  message0 := ""
  if ADEvent.message ne "" then
    message0 := Append(message0, ADEvent.message)
  endif
  if message0 ne "" then
    message := Trim(message0)
  endif

  eventclass:=ADEvent.eclass
  eventmainclass := ADEvent.emainclass
  successclass := ADEvent.successclass
  eventtimestamp := ADEvent.platformeventtime
  chunk     := ADEvent
  recordtype:= Trim(ADEvent.recordtype, null)
  result    := Trim(ADEvent.result, null)
  reason    := Trim(ADEvent.reason, null)
  attribute := Trim(ADEvent.attribute, null)
  value     := Trim(ADEvent.value, null)
  newvalue  := Trim(ADEvent.newvalue, null)
  access    := Trim(ADEvent.access, null)
  id        := Trim(ADEvent.id, null)
  user      := Trim(ADEvent.user, null)
  protocol  := Trim(ADEvent.protocol, null)
  arguments := Trim(ADEvent.arguments, null)
  Target    := Trim(ADEvent.target, null)
  Source    := Trim(ADEvent.source, null)

  if ADEvent.platformeventcount ne null then
    eventcount := ADEvent.platformeventcount
  endif

  if (ADEvent.platformeventendtime ne null) then
    platformeventendtime:=Append("\"",ADEvent.platformeventendtime,"\"")
  endif

  event := GetEvent(EventDetailKey)
endstate

initial final state OverwriteDetect
  chunkpath := GetChunkProperty(OverwriteDetect,"chunkpath")
  chunkpath := Trim(chunkpath,"chunkpath_IS_NULL")

  if Recall("chunkpath") ne chunkpath then
     Remember("chunkpath", chunkpath)
     if GetChunkProperty(OverwriteDetect, "Overwrite") eq "Y" then
          eventtimestamp := OverwriteDetect.platformeventtime
          wherefrom := Trim(OverwriteDetect.wherefrom, null)
          whereto   := Trim(OverwriteDetect.whereto, null)
          chunk := OverwriteDetect
          eventclass := "AUDIT"
          eventmainclass := "Overwrite"
          objecttype := "AUDITLOG"
          objectpath := OverwriteDetect.objectpath
          objectname := chunkpath
          epiname := Trim(OverwriteDetect.epiname)
          platformname := epiname
          platformtype := "Active Directory"
          object := GetObject(ObjectRefKey)
          origin := GetOrigin(OriginRefKey)
          platform := GetPlatform(PlatformRefKey)

          logonname :="SYSTEM"
          originator :=GetOriginator(logonname, "TSIEM")
          realname := originator
          source := GetSource(SourceRefKey,SourceAspectKey)
          description := "The original events log was overwritten."
          stream := GetStream(StreamRefKey)
          successclass :="Warning"
          target := GetTarget(TargetRefKey)
          event :=GetEvent(EventDetailKey)
     endif
  endif 
endstate

