/// cisco.src.gml - Licensed Materials ­ Property of IBM
/// cisco.src.gml - 5724-S67
/// cisco.src.gml - © Copyright IBM Corp. 1998, 2007   All Rights Reserved.
/// cisco.src.gml - US Government Users Restricted Right ­ Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

key PlatformRefKey(platformtype, platformname)
key OriginRefKey(platformtype, platformname)
key TargetRefKey(platformtype, platformname)
key ObjectRefKey(objecttype, objectpath, objectname)
key StreamRefKey(description)
key SourceRefKey(realname, logonname)
key EventDetailKey(message,count,platformeventendtime,command_source,command_source_type,config_source,config_source_type,config_target,config_target_type,ips_signature,ips_subsignature,severity,category,origin_port,target_port,origtype,newvalue,profile,Source,Target,Sourcetype,Targettype)

first state Setup
  description:="Activity"
  equality:="host"
  data:=""
  period:=GetIniProperty("ActivityPeriod","600000") 
  threshold:=GetIniProperty("ActivityThreshold","1200")
  activityalerter:=GetListener(SumCounter)
  Listen(activityalerter,Setup)

  description := "Aggregator"
  equality := "eventmainclass,eventclass,successclass,realname,logonname,objecttype,objectpath,objectname,command_source,command_source_type,config_source,config_source_type,config_target,config_target_type,origin_ip,host,ips_signature,ips_subsignature,severity,category,origin_port,origin_mac,target_port,target_ip,newvalue,profile,Source,Target,recordtype"
  targets := ""
  separator := "."
  period := GetIniProperty("AggregationPeriod","600000")
  threshold := GetIniProperty("AggregationThreshold","1")
  aggregator := GetListener(Aggregator)
  Listen(aggregator,Setup)

  description:="HostScan"
  equality:="host,logonname,objectname"
  data:="objectpath"
  period:=GetIniProperty("HostScanPeriod","300000") 
  threshold:=GetIniProperty("HostScanThreshold","4")
  hostscandetector:=GetListener(DataCounter)

  description:="PortScan"
  equality:="host,logonname,objectpath"
  data:="serviceport"
  period:=GetIniProperty("PortScanPeriod","300000") 
  threshold:=GetIniProperty("PortScanThreshold","4")
  portscandetector:=GetListener(DataCounter)

  MoveTo(ListenState)
endstate 

last state Shutdown
  if aggregator ne null then 
    StopListening(aggregator)
    StopListening(activityalerter)
    StopListening(portscandetector)
    StopListening(hostscandetector)
  endif
endstate

state ListenState(platformeventcount eq null)
  Listen(aggregator,ListenState)
  Listen(activityalerter,ListenState)
  if ListenState.audit eq "yes" then
    Listen(hostscandetector,ListenState)
    Listen(portscandetector,ListenState)
  endif
endstate

initial final state IOSEvent
  (( (platformeventcount ne null) and
     (description eq "Aggregator")  ))

    platformname:=IOSEvent.host
    platformtype:="Cisco-IOS"
    platform:=GetPlatform(PlatformRefKey)

    objecttype := IOSEvent.objecttype
    objectpath := IOSEvent.objectpath
    objectname := IOSEvent.objectname
    object:=GetObject(ObjectRefKey)
    platformname:=Trim(IOSEvent.target_ip,objectpath)
    platformtype:="Host"
    target:=GetTarget(TargetRefKey)

    description:=IOSEvent.logonname
    stream:=GetStream(StreamRefKey) 

    if( (IOSEvent.logonname ne "") and (IOSEvent.logonname ne null) )
    then
      logonname := IOSEvent.logonname
      realname  := IOSEvent.logonname
    else
      logonname := "Anonymous"
      realname  := "Anonymous"
    endif
    source:=GetSource(SourceRefKey)

    platformname:=Trim(IOSEvent.origin_ip,IOSEvent.origin_mac,logonname)
    platformtype:="Host"
    origin:=GetOrigin(OriginRefKey)

    eventclass:=IOSEvent.eventclass
    eventmainclass:=IOSEvent.eventmainclass
    successclass:=IOSEvent.successclass
    eventtimestamp:=IOSEvent.platformeventtime
    chunk:=IOSEvent
    recordtype:="CISCO-IOS"
    if IOSEvent.platformeventcount ne null then
      eventcount := IOSEvent.platformeventcount
    endif
    if (IOSEvent.platformeventendtime ne null) then
      platformeventendtime:=Append("\"",IOSEvent.platformeventendtime,"\"")
    endif
    message := IOSEvent.message

    if( (IOSEvent.command_source  ne "")      and (IOSEvent.command_source       ne null) ) then command_source      := IOSEvent.command_source      endif
    if( (IOSEvent.command_source_type  ne "") and (IOSEvent.command_source_type  ne null) ) then command_source_type := IOSEvent.command_source_type endif

    if( (IOSEvent.config_source  ne "")      and (IOSEvent.config_source       ne null) ) then config_source      := IOSEvent.config_source      endif
    if( (IOSEvent.config_source_type  ne "") and (IOSEvent.config_source_type  ne null) ) then config_source_type := IOSEvent.config_source_type endif

    if( (IOSEvent.config_target  ne "")      and (IOSEvent.config_target       ne null) ) then config_target      := IOSEvent.config_target      endif
    if( (IOSEvent.config_target_type  ne "") and (IOSEvent.config_target_type  ne null) ) then config_target_type := IOSEvent.config_target_type endif

    if( (IOSEvent.ips_signature    ne "") and (IOSEvent.ips_signature    ne null) ) then ips_signature    := IOSEvent.ips_signature    endif
    if( (IOSEvent.ips_subsignature ne "") and (IOSEvent.ips_subsignature ne null) ) then ips_subsignature := IOSEvent.ips_subsignature endif
    if( (IOSEvent.severity         ne "") and (IOSEvent.severity         ne null) ) then severity         := IOSEvent.severity         endif
    if( (IOSEvent.category         ne "") and (IOSEvent.category         ne null) ) then category         := IOSEvent.category         endif
    if( (IOSEvent.origin_port      ne "") and (IOSEvent.origin_port      ne null) ) then origin_port      := IOSEvent.origin_port      endif
    if( (IOSEvent.target_port      ne "") and (IOSEvent.target_port      ne null) ) then target_port      := IOSEvent.target_port      endif
    if( (IOSEvent.origtype         ne "") and (IOSEvent.origtype         ne null) ) then origtype         := IOSEvent.origtype         endif
    if( (IOSEvent.newvalue         ne "") and (IOSEvent.newvalue         ne null) ) then newvalue         := IOSEvent.newvalue         endif
    if( (IOSEvent.profile          ne "") and (IOSEvent.profile          ne null) ) then profile          := IOSEvent.profile          endif
    if( (IOSEvent.Source           ne "") and (IOSEvent.Source           ne null) ) then Source           := IOSEvent.Source           endif
    if( (IOSEvent.Target           ne "") and (IOSEvent.Target           ne null) ) then Target           := IOSEvent.Target           endif
    if( (IOSEvent.Sourcetype       ne "") and (IOSEvent.Sourcetype       ne null) ) then Sourcetype       := IOSEvent.Sourcetype       endif
    if( (IOSEvent.Targettype       ne "") and (IOSEvent.Targettype       ne null) ) then Targettype       := IOSEvent.Targettype       endif

    event:=GetEvent(EventDetailKey)
endstate

initial final state IOSActivity
  (( (platformeventcount ne null) and (description eq "Activity")  ))

    platformname:=IOSActivity.host
    platformtype:="Cisco-IOS"
    platform:=GetPlatform(PlatformRefKey)
    origin:=GetOrigin(OriginRefKey)
    target:=GetTarget(TargetRefKey)

    objecttype := "ACTIVITY"
    objectpath := IOSActivity.period
    objectname := IOSActivity.platformeventcount
    object:=GetObject(ObjectRefKey)

    description:=IOSActivity.host
    stream:=GetStream(StreamRefKey) 

    realname:="System"
    logonname:="System"
    source:=GetSource(SourceRefKey)

    eventmainclass:="Alert"
    eventclass:="Activity"
    successclass:="Success"
    eventtimestamp:=IOSActivity.platformeventtime
    chunk:=IOSActivity
    recordtype:="IOSActivity"
    if IOSActivity.platformeventcount ne null then
      eventcount := IOSActivity.platformeventcount
    endif
    message:="Activity"
    if (IOSActivity.platformeventendtime ne null) then
      platformeventendtime:=Append("\"",IOSActivity.platformeventendtime,"\"")
    endif
    event:=GetEvent(EventDetailKey)
endstate 

initial state HostScan
  (( (platformeventcount ne null) and (description eq "HostScan")  ))

    platformname:=HostScan.host
    platformtype:="Cisco-IOS"
    platform:=GetPlatform(PlatformRefKey)

    objecttype := "NET"
    objectpath := "-"
    objectname := HostScan.platformeventcount
    object:=GetObject(ObjectRefKey)
    platformname:=HostScan.host
    platformtype:="Cisco-IOS"
    target:=GetTarget(TargetRefKey)

    description:=HostScan.logonname
    stream:=GetStream(StreamRefKey)

    realname:="Anonymous"
    logonname:="Anonymous"
    source:=GetSource(SourceRefKey)
    platformname:=HostScan.logonname
    platformtype:="Host"
    origin:=GetOrigin(OriginRefKey)

    eventmainclass:="Alert"
    eventclass:="HostScan"
    successclass:="Success"
    eventtimestamp:=HostScan.platformeventtime
    chunk:=HostScan
    recordtype:="HostScan"
    if HostScan.platformeventcount ne null then
      eventcount := HostScan.platformeventcount
    endif
    message:="HostScan"
    if (HostScan.platformeventendtime ne null) then
      platformeventendtime:=Append("\"",HostScan.platformeventendtime,"\"")
    endif
    event:=GetEvent(EventDetailKey)
endstate 

initial state PortScan
  (( (platformeventcount ne null) and (description eq "PortScan")  ))

    platformname:=PortScan.host
    platformtype:="Cisco-IOS"
    platform:=GetPlatform(PlatformRefKey)

    objecttype := "HOST"
    objectpath := PortScan.objectpath
    objectname := PortScan.platformeventcount
    object:=GetObject(ObjectRefKey)
    platformname:=PortScan.objectpath
    platformtype:="Host"
    target:=GetTarget(TargetRefKey)

    description:=PortScan.logonname
    stream:=GetStream(StreamRefKey)

    realname:="Anonymous"
    logonname:="Anonymous"
    source:=GetSource(SourceRefKey)
    platformname:=PortScan.logonname
    platformtype:="Host"
    origin:=GetOrigin(OriginRefKey)

    eventmainclass:="Alert"
    eventclass:="PortScan"
    successclass:="Success"
    eventtimestamp:=PortScan.platformeventtime
    chunk:=PortScan
    recordtype:="PortScan"
    if PortScan.platformeventcount ne null then
      eventcount := PortScan.platformeventcount
    endif
    message:="PortScan"
    if (PortScan.platformeventendtime ne null) then
      platformeventendtime:=Append("\"",PortScan.platformeventendtime,"\"")
    endif
    event:=GetEvent(EventDetailKey)
endstate 
