/// db2udb.src.gml - Licensed Materials ­ Property of IBM
/// db2udb.src.gml - 5724-S67
/// db2udb.src.gml - © Copyright IBM Corp. 1998, 2007   All Rights Reserved.
/// db2udb.src.gml - US Government Users Restricted Right ­ Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
key ObjectRefKey(objecttype,objectpath,objectname)
key OriginRefKey(platformtype,platformname) 
key PlatformRefKey(platformtype,platformname)
key TargetRefKey(platformtype,platformname)
key SourceRefKey(logonname)
key StreamRefKey(description)

key SourceAspectKey(originator)
key EventAspectsAudit(dummy, category)
key EventAspectsChecking(accesstype, reason, category, application_id)
key ObjectAspectsChecking(value)
key EventAspectsSecmaint(privilegetype, objecttype, objectschema, objectname, category, application_id)
key EventAspectsSysadmin(auditevent, category, application_id)
key EventAspectsValidate(auditevent, authenticator, authtype, category, application_id)
key EventAspectsContext(auditevent, database, command, db_user_name, category, query)

/// AUDIT

initial state AUDIT 
((    (category eq "AUDIT")    ))

    eventmainclass := AUDIT.eventmainclass
    eventclass := AUDIT.eventclass
    if(AUDIT.event_status >= "0")
    then
        successclass := "Success"
    else
        successclass := "Failure"
    endif

    eventsourcetype := Trim(GetChunkProperty(AUDIT,"PlatformType"), "Unavailable")
    nameOfPlatform  := Trim(GetChunkProperty(AUDIT,"SystemName"), "Unavailable")
    inst_name       := Trim(GetChunkProperty(AUDIT,"Option2"), "Unavailable")
    auditedmachinetype := Trim(GetChunkProperty(AUDIT,"auditedmachinetype"), "Unavailable")

    objecttype := AUDIT.objecttype
    typeOfPlatform2 := eventsourcetype
    case auditedmachinetype of
        "13":
            typeOfPlatform := "Windows"
            typeOfPlatform2 := "IBM DB2"            
            program_folder := Trim(GetChunkProperty(AUDIT,"Option1"), "Unavailable")
            objectpath     := NormalizeString(Append(program_folder,"\\",inst_name,"\\security"))
            objectname     := NormalizeString(AUDIT.objectname)
        "11":
            typeOfPlatform := "AIX"
            typeOfPlatform2 := "IBM DB2"            
            objectpath := Trim(GetChunkProperty(AUDIT,"_logpath"),Append("/home/",inst_name,"/sqllib/security"))
            objectname := AUDIT.objectname
        "14":
            typeOfPlatform := "SUN"
            typeOfPlatform2 := "IBM DB2"            
            objectpath := Trim(GetChunkProperty(AUDIT,"_logpath"),Append("/home/",inst_name,"/sqllib/security"))
            objectname := AUDIT.objectname
        otherwise
            typeOfPlatform := "Host"
            typeOfPlatform2 := "IBM DB2"            
            objectpath := Trim(GetChunkProperty(AUDIT,"_logpath"),Append("/home/",inst_name,"/sqllib/security"))
            objectname := AUDIT.objectname
    endcase
    object     := GetObject(ObjectRefKey)

    platformtype := typeOfPlatform2
    platformname := nameOfPlatform

    origin := GetOrigin(OriginRefKey)
    platform     := GetPlatform(PlatformRefKey)
    target       := GetTarget(TargetRefKey)

    user_id      := Trim(AUDIT.user_id, "Unavailable")
    if (RegexMatch(user_id,"(.+)\\s$") > "0") then
        user_id      := RegexRef("1")
    endif

    ldapenable := Trim(Uppercase(GetChunkProperty(AUDIT,"_ldapenable")), "Unavailable")
    if (ldapenable eq "YES")
    then
        originator := GetOriginator("",Lowercase(user_id))
        logonname  := Lowercase(user_id)
        realname   := logonname
    else
        case typeOfPlatform of
            "Windows":
                case user_id of
                    "Unavailable":
                        logonname  := "Unavailable"
                    otherwise
                        domainname := Trim(GetChunkProperty(AUDIT,"_domainname"), "Unavailable")
                        logonname  := Uppercase(Append(domainname, "\\", user_id))
                endcase
            "AIX":
                logonname  := user_id
            "SUN":
                logonname  := user_id
            otherwise
                logonname  := user_id
        endcase
        realname   := logonname

        orig := GetOriginator(logonname)
        if(orig ne logonname)
        then
            originator := orig
        endif
    endif

    source     := GetSource(SourceRefKey, SourceAspectKey)

    description := logonname
    stream      := GetStream(StreamRefKey)

    chunk := AUDIT
    eventtimestamp := AUDIT.platformeventtime
    category:="AUDIT"
    event := GetEvent(EventAspectsAudit)

endstate

/// CHECKING

initial final state CHECKING 
((    (category eq "CHECKING")    ))

    if(CHECKING.event_status >= "0")
    then
        successclass := "Success"
    else
        successclass := "Failure"
    endif
    case CHECKING.access_result of
        "ACCESS_DENIED":
            eventmainclass := CHECKING.eventmainclass
            successclass   := "Failure"
        "SYSADMIN_CATEGORY":
            case CHECKING.eventmainclass of
                "Unavailable": eventmainclass := "Unavailable"
                otherwise      eventmainclass := NormalizeString(Append("Ask", CHECKING.eventmainclass))
            endcase
        otherwise
            eventmainclass := CHECKING.eventmainclass
    endcase
    case CHECKING.access_attempted of
        "0x0000000000020000": eventclass := "Dbcomment"
        "0x0000000010000000": eventclass := "Su"
        otherwise
            case CHECKING.eventclass of
                "Unavailable": eventclass := "Dbmsobject"
                otherwise      eventclass := CHECKING.eventclass
            endcase
    endcase

    instance_name  := Trim(GetChunkProperty(CHECKING,"Option2"), "Unavailable")
    database_name  := Trim(CHECKING.db_name, "Unavailable")
    object_schema  := Trim(CHECKING.object_schema, "Unavailable")

    if(Trim(CHECKING.aspect_value, "Unavailable") ne "Unavailable")
    then
        value := CHECKING.aspect_value
    endif

    case CHECKING.objecttype of
        "UNAVAILABLE": objecttype := "DBMSOBJECT"
        otherwise      objecttype := CHECKING.objecttype
    endcase
    case CHECKING.objectpath of
        "2":
            objectpath := "-"
        "3":
            objectpath := instance_name
        otherwise
            objectpath := instance_name
            if(database_name ne "Unavailable")
            then
                objectpath := Append(objectpath, "\\", database_name)
                if(object_schema ne "Unavailable")
                then
                    objectpath := Append(objectpath, ":", object_schema)
                endif
            endif
    endcase
    case objecttype of
        "DBINSTANCE": objectname := instance_name
        otherwise     objectname := Trim(CHECKING.objectname, "Unavailable")
    endcase

    eventsourcetype := Trim(GetChunkProperty(CHECKING,"PlatformType"), "Unavailable")
    auditedmachinetype := Trim(GetChunkProperty(CHECKING,"auditedmachinetype"), "Unavailable")

    typeOfPlatform2 := eventsourcetype
    case auditedmachinetype of
        "13":
            typeOfPlatform := "Windows"
            typeOfPlatform2 := "IBM DB2"            
            objectpath     := NormalizeString(objectpath)
            objectname     := NormalizeString(objectname)
        "11":
            typeOfPlatform := "AIX"
            typeOfPlatform2 := "IBM DB2"            
        "14":
            typeOfPlatform := "SUN"
            typeOfPlatform2 := "IBM DB2"            
        otherwise
            typeOfPlatform := "Host"
            typeOfPlatform2 := "IBM DB2"            
    endcase

    object     := GetObject(ObjectRefKey, ObjectAspectsChecking)

    nameOfPlatform := Trim(GetChunkProperty(CHECKING,"SystemName"), "Unavailable")

    platformtype := typeOfPlatform2
    platformname := nameOfPlatform

    platform     := GetPlatform(PlatformRefKey)
    target       := GetTarget(TargetRefKey)

    if (CHECKING.origin_ip ne "") then
      platformname := CHECKING.origin_ip
      platformtype := "HOST"
    endif
    origin := GetOrigin(OriginRefKey)

    user_id      := Trim(CHECKING.user_id, "Unavailable")
    if (RegexMatch(user_id,"(.+)\\s$") > "0") then
        user_id      := RegexRef("1")
    endif

    ldapenable := Trim(Uppercase(GetChunkProperty(CHECKING,"_ldapenable")), "Unavailable")
    if (ldapenable eq "YES")
    then
        originator := GetOriginator("",Lowercase(user_id))
        logonname  := Lowercase(user_id)
        realname   := logonname
    else
        case typeOfPlatform of
            "Windows":
                case user_id of
                    "Unavailable":
                        logonname  := "Unavailable"
                    otherwise
                        domainname := Trim(GetChunkProperty(CHECKING,"_domainname"), "Unavailable")
                        logonname  := Uppercase(Append(domainname, "\\", user_id))
                endcase
            "AIX":
                logonname  := user_id
            "SUN":
                logonname  := user_id
            otherwise
                logonname  := user_id
        endcase
        realname   := logonname

        orig := GetOriginator(logonname)
        if(orig ne logonname)
        then
            originator := orig
        endif
    endif

    source     := GetSource(SourceRefKey, SourceAspectKey)

    description := logonname
    stream      := GetStream(StreamRefKey)

    accesstype  := CHECKING.aspect_accesstype
    case CHECKING.aspect_reason of
        "Unavailable":
            if(Trim(CHECKING.access_approval, "Unavailable") ne "Unavailable")
            then
                reason := CHECKING.access_approval
            endif

        otherwise
            reason := CHECKING.aspect_reason
    endcase

    application_id := CHECKING.application_id;

    chunk := CHECKING
    eventtimestamp := CHECKING.platformeventtime
    category:="CHECKING"
    event := GetEvent(EventAspectsChecking)

endstate

/// SECMAINT

initial state SECMAINT 
((    (category eq "SECMAINT")    ))

    eventmainclass := SECMAINT.eventmainclass
    eventclass := SECMAINT.eventclass
    if(SECMAINT.event_status >= "0")
    then
        successclass := "Success"
    else
        successclass := "Failure"
    endif

    user_id := Trim(SECMAINT.user_id, "Unavailable")
    if (RegexMatch(user_id,"(.+)\\s$") > "0") then
        user_id      := RegexRef("1")
    endif

    case SECMAINT.obj_cond of
        "o1":
            objecttype := SECMAINT.objecttype
            objectname := Trim(SECMAINT.grantee, "Unavailable")
        "o2":
            objecttype := "USER"
            objectname := user_id
        otherwise
            objecttype := "UNAVAILABLE"
            objectname := "Unavailable"
    endcase
    objectpath := SECMAINT.objectpath

    eventsourcetype := Trim(GetChunkProperty(SECMAINT,"PlatformType"), "Unavailable")
    auditedmachinetype := Trim(GetChunkProperty(SECMAINT,"auditedmachinetype"), "Unavailable")

    typeOfPlatform2 := eventsourcetype
    case auditedmachinetype of
        "13":
            typeOfPlatform := "Windows"
            typeOfPlatform2 := "IBM DB2"            
            objectpath     := NormalizeString(objectpath)
            objectname     := NormalizeString(objectname)
        "11":
            typeOfPlatform := "AIX"
            typeOfPlatform2 := "IBM DB2"            
        "14":
            typeOfPlatform := "SUN"
            typeOfPlatform2 := "IBM DB2"            
        otherwise
            typeOfPlatform := "Host"
            typeOfPlatform2 := "IBM DB2"            
    endcase

    object     := GetObject(ObjectRefKey)

    nameOfPlatform := Trim(GetChunkProperty(SECMAINT,"SystemName"), "Unavailable")

    platformtype := typeOfPlatform2
    platformname := nameOfPlatform

    platform     := GetPlatform(PlatformRefKey)
    target       := GetTarget(TargetRefKey)

    if (SECMAINT.origin_ip ne "") then
      platformname := SECMAINT.origin_ip
      platformtype := "HOST"
    endif
    origin := GetOrigin(OriginRefKey)

    ldapenable := Trim(Uppercase(GetChunkProperty(SECMAINT,"_ldapenable")), "Unavailable")
    if (ldapenable eq "YES")
    then
        originator := GetOriginator("",Lowercase(user_id))
        logonname  := Lowercase(user_id)
        realname   := logonname
    else
        case typeOfPlatform of
            "Windows":
                case user_id of
                    "Unavailable":
                        logonname  := "Unavailable"
                    otherwise
                        domainname := Trim(GetChunkProperty(SECMAINT,"_domainname"), "Unavailable")
                        logonname  := Uppercase(Append(domainname, "\\", user_id))
                endcase
            "AIX":
                logonname  := user_id
            "SUN":
                logonname  := user_id
            otherwise
                logonname  := user_id
        endcase
        realname   := logonname

        orig := GetOriginator(logonname)
        if(orig ne logonname)
        then
            originator := orig
        endif
    endif
    source     := GetSource(SourceRefKey, SourceAspectKey)

    description := logonname
    stream      := GetStream(StreamRefKey)

    case SECMAINT.obj_cond of
        "o1":
            case SECMAINT.aspect_privilegetype of
                "Unavailable":
                    if(Trim(SECMAINT.privilege_authority, "Unavailable") ne "Unavailable")
                    then
                        privilegetype := SECMAINT.privilege_authority
                    endif
                otherwise
                    privilegetype := SECMAINT.aspect_privilegetype
            endcase

            if(Trim(SECMAINT.aspect_objecttype, "Unavailable") ne "Unavailable")
            then
                objecttype := SECMAINT.aspect_objecttype
            endif
            if(Trim(SECMAINT.aspect_objectschema, "Unavailable") ne "Unavailable")
            then
                objectschema := SECMAINT.aspect_objectschema
            endif
            if(Trim(SECMAINT.aspect_objectname, "Unavailable") ne "Unavailable")
            then
                objectname := SECMAINT.aspect_objectname
            endif
    endcase

    application_id := SECMAINT.application_id;

    chunk := SECMAINT
    eventtimestamp := SECMAINT.platformeventtime
    category:="SECMAINT"
    event := GetEvent(EventAspectsSecmaint)

endstate

/// SYSADMIN

initial state SYSADMIN 
((    (category eq "SYSADMIN")
  and (event_status ne "-1092")
  and (event_status ne "-551")
  and (event_status ne "-552")
  and (event_status ne "-30060")   ))

    eventmainclass := SYSADMIN.eventmainclass
    eventclass := SYSADMIN.eventclass
    if( (SYSADMIN.event_status >= "0") or (SYSADMIN.event_status = "-1063")
        or (SYSADMIN.event_status = "-1064") ) 
    then
        successclass := "Success"
    else
        successclass := "Failure"
    endif

    instance_name  := Trim(GetChunkProperty(SYSADMIN,"Option2"), "Unavailable")
    database_name  := Trim(SYSADMIN.db_name, "Unavailable")

    nameOfPlatform := Trim(GetChunkProperty(SYSADMIN,"SystemName"), "Unavailable")

    case SYSADMIN.obj_cond of
        "n1":
            case database_name of
                "Unavailable":
                    objecttype := "DBINSTANCE"
                    objectpath := "-"
                    objectname := instance_name
                otherwise
                    objecttype := "DATABASE"
                    objectpath := instance_name
                    objectname := database_name
            endcase
        "n2":
            objecttype := "SYSTEM"
            objectpath := "-"
            objectname := nameOfPlatform
        "n3":
            objecttype := "DBINSTANCE"
            objectpath := "-"
            objectname := instance_name
        "n4":
            objecttype := DATABASE
            objectpath := instance_name
            objectname := database_name
        otherwise
            objecttype := "UNAVAILABLE"
            objectpath := "Unavailable"
            objectname := "Unavailable"
    endcase

    eventsourcetype := Trim(GetChunkProperty(SYSADMIN,"PlatformType"), "Unavailable")
    auditedmachinetype := Trim(GetChunkProperty(SYSADMIN,"auditedmachinetype"), "Unavailable")

    typeOfPlatform2 := eventsourcetype
    case auditedmachinetype of
        "13":
            typeOfPlatform := "Windows"
            typeOfPlatform2 := "IBM DB2"            
            objectpath     := NormalizeString(objectpath)
            objectname     := NormalizeString(objectname)
        "11":
            typeOfPlatform := "AIX"
            typeOfPlatform2 := "IBM DB2"            
        "14":
            typeOfPlatform2 := "IBM DB2"            
            typeOfPlatform := "SUN"
        otherwise
            typeOfPlatform := "Host"
            typeOfPlatform2 := "IBM DB2"            
    endcase

    object     := GetObject(ObjectRefKey)

    platformtype := typeOfPlatform2
    platformname := nameOfPlatform

    platform     := GetPlatform(PlatformRefKey)
    target       := GetTarget(TargetRefKey)
    if (SYSADMIN.origin_ip ne "") then
      platformname := SYSADMIN.origin_ip
      platformtype := "HOST"
    endif
    origin := GetOrigin(OriginRefKey)

    user_id        := Trim(SYSADMIN.user_id, "Unavailable")
    if (RegexMatch(user_id,"(.+)\\s$") > "0") then
        user_id      := RegexRef("1")
    endif

    ldapenable := Trim(Uppercase(GetChunkProperty(SYSADMIN,"_ldapenable")), "Unavailable")
    if (ldapenable eq "YES")
    then
        originator := GetOriginator("",Lowercase(user_id))
        logonname  := Lowercase(user_id)
        realname   := logonname
    else
        case typeOfPlatform of
            "Windows":
                case user_id of
                    "Unavailable":
                        logonname  := "Unavailable"
                    otherwise
                        domainname := Trim(GetChunkProperty(SYSADMIN,"_domainname"), "Unavailable")
                        logonname  := Uppercase(Append(domainname, "\\", user_id))
                endcase
            "AIX":
                logonname  := user_id
            "SUN":
                logonname  := user_id
            otherwise
                logonname  := user_id
        endcase
        realname   := logonname

        orig := GetOriginator(logonname)
        if(orig ne logonname)
        then
            originator := orig
        endif
    endif

    source     := GetSource(SourceRefKey, SourceAspectKey)

    description := logonname
    stream      := GetStream(StreamRefKey)

    auditevent := SYSADMIN.aspect_auditevent

    application_id := SYSADMIN.application_id

    chunk := SYSADMIN
    eventtimestamp := SYSADMIN.platformeventtime
    category:="SYSADMIN"
    event := GetEvent(EventAspectsSysadmin)

endstate

/// VALIDATE

initial state VALIDATE 
((    (category eq "VALIDATE")    ))

    eventmainclass := VALIDATE.eventmainclass
    eventclass := VALIDATE.eventclass
    if(VALIDATE.event_status >= "0")
    then
        successclass := "Success"
    else
        successclass := "Failure"
    endif

    instance_name  := Trim(GetChunkProperty(VALIDATE,"Option2"), "Unavailable")
    database_name  := Trim(VALIDATE.db_name, "Unavailable")
    user_id        := Trim(VALIDATE.user_id, "Unavailable")
    if (RegexMatch(user_id,"(.+)\\s$") > "0") then
        user_id      := RegexRef("1")
    endif

    case VALIDATE.obj_cond of
        "m1":
            case database_name of
                "Unavailable":
                    objecttype := "DBINSTANCE"
                    objectpath := "-"
                    objectname := instance_name
                otherwise
                    objecttype := "DATABASE"
                    objectpath := instance_name
                    objectname := database_name
            endcase
        "m2":
            objecttype := "USER"
            objectpath := "-"
            objectname := user_id
        otherwise
            objecttype := "UNAVAILABLE"
            objectpath := "Unavailable"
            objectname := "Unavailable"
    endcase

    eventsourcetype := Trim(GetChunkProperty(VALIDATE,"PlatformType"), "Unavailable")
    auditedmachinetype := Trim(GetChunkProperty(VALIDATE,"auditedmachinetype"), "Unavailable")

    typeOfPlatform2 := eventsourcetype
    case auditedmachinetype of
        "13":
            typeOfPlatform := "Windows"
            typeOfPlatform2 := "IBM DB2"            
            objectpath     := NormalizeString(objectpath)
            objectname     := NormalizeString(objectname)
        "11":
            typeOfPlatform := "AIX"
            typeOfPlatform2 := "IBM DB2"            
        "14":
            typeOfPlatform := "SUN"
            typeOfPlatform2 := "IBM DB2"            
        otherwise
            typeOfPlatform := "Host"
            typeOfPlatform2 := "IBM DB2"            
    endcase

    object     := GetObject(ObjectRefKey)

    nameOfPlatform := Trim(GetChunkProperty(VALIDATE,"SystemName"), "Unavailable")

    platformtype := typeOfPlatform2
    platformname := nameOfPlatform

    platform     := GetPlatform(PlatformRefKey)
    target       := GetTarget(TargetRefKey)
    if (VALIDATE.origin_ip ne "") then
      platformname := VALIDATE.origin_ip
      platformtype := "HOST"
    endif
    origin := GetOrigin(OriginRefKey)

    ldapenable := Trim(Uppercase(GetChunkProperty(VALIDATE,"_ldapenable")), "Unavailable")
    if (ldapenable eq "YES")
    then
        originator := GetOriginator("",Lowercase(user_id))
        logonname  := Lowercase(user_id)
        realname   := logonname
    else
        case typeOfPlatform of
            "Windows":
                case user_id of
                    "Unavailable":
                        logonname  := "Unavailable"
                    otherwise
                        domainname := Trim(GetChunkProperty(VALIDATE,"_domainname"), "Unavailable")
                        logonname  := Uppercase(Append(domainname, "\\", user_id))
                endcase
            "AIX":
                logonname  := user_id
            "SUN":
                logonname  := user_id
            otherwise
                logonname  := user_id
        endcase
        realname   := logonname

        orig := GetOriginator(logonname)
        if(orig ne logonname)
        then
            originator := orig
        endif
    endif

    source     := GetSource(SourceRefKey, SourceAspectKey)

    description := logonname
    stream      := GetStream(StreamRefKey)

    auditevent := VALIDATE.aspect_auditevent
    case Trim(VALIDATE.asp_cond, "Unavailable") of
        "a2":
            if(Trim(VALIDATE.plug_name, "Unavailable") ne "Unavailable")
            then
                authenticator := VALIDATE.plug_name
            endif
            if(Trim(VALIDATE.auth_type, "Unavailable") ne "Unavailable")
            then
                authtype := VALIDATE.auth_type
            endif
    endcase

    application_id := VALIDATE.application_id

    chunk := VALIDATE
    eventtimestamp := VALIDATE.platformeventtime
    category:="VALIDATE"
    event := GetEvent(EventAspectsValidate)

endstate

/// CONTEXT

initial state CONTEXT
((   (category eq "CONTEXT")   ))

    auditedmachinetype := Trim(GetChunkProperty(CONTEXT, "auditedmachinetype"), "Unavailable")
    eventsourcetype := Trim(GetChunkProperty(CONTEXT,"PlatformType"), "Unavailable")
    source_system_name := Trim(GetChunkProperty(CONTEXT,"SystemName"), "Unavailable")

    typeOfPlatform2 := eventsourcetype
    case auditedmachinetype of
        "13":
            typeOfPlatform := "Windows"
            typeOfPlatform2 := "IBM DB2"
        "11":
            typeOfPlatform := "AIX"
            typeOfPlatform2 := "IBM DB2"
        "14":
            typeOfPlatform2 := "IBM DB2"
            typeOfPlatform := "SUN"
        otherwise
            typeOfPlatform := "Host"
            typeOfPlatform2 := "IBM DB2"
    endcase

    platformtype := typeOfPlatform2
    platformname := source_system_name

    db2_udb_folder := Trim(GetChunkProperty(CONTEXT, "Option1"), "Unavailable")
    instance_name  := Trim(GetChunkProperty(CONTEXT, "Option2"), "Unavailable")

    if ((CONTEXT.aspect_auditevent eq "CONNECT") or
        (CONTEXT.aspect_auditevent eq "CONNECT_RESET") or
        (CONTEXT.aspect_auditevent eq "ATTACH") or
        (CONTEXT.aspect_auditevent eq "DETACH") or
        (CONTEXT.aspect_auditevent eq "UNQUIESCE_TABLESPACE") or
        (CONTEXT.aspect_auditevent eq "RUNSTATS") or
        (CONTEXT.aspect_auditevent eq "ROLLBACK") or
        (CONTEXT.aspect_auditevent eq "REQUEST_ROLLBACK")
    ) then
        objectname := Trim(CONTEXT.aspect_database, instance_name)
    endif

    if ((CONTEXT.aspect_auditevent eq "DARI_START") or
        (CONTEXT.aspect_auditevent eq "DARI_STOP")
    ) then
        objectname := source_system_name
    endif

    if (CONTEXT.aspect_auditevent eq "LOAD_MSG_FILE") then

        objectname := "Unavailable"
    endif

    if ((CONTEXT.aspect_auditevent eq "DBM_CFG_OPERATION") or
        (CONTEXT.aspect_auditevent eq "DISCOVER") or
        (CONTEXT.aspect_auditevent eq "REBIND") or
        (CONTEXT.aspect_auditevent eq "REORG") or
        (CONTEXT.aspect_auditevent eq "REDISTRIBUTE")
    ) then
        if (CONTEXT.aspect_database eq "") then
            objectname := "-"
        else
            objectname := instance_name
        endif
    endif

    if ((CONTEXT.aspect_auditevent eq "OPEN_CURSOR") or
        (CONTEXT.aspect_auditevent eq "CLOSE_CURSOR") or
        (CONTEXT.aspect_auditevent eq "FETCH_CURSOR") or
        (CONTEXT.aspect_auditevent eq "EXECUTE") or
        (CONTEXT.aspect_auditevent eq "EXECUTE_IMMEDIATE") or
        (CONTEXT.aspect_auditevent eq "PREPARE") or
        (CONTEXT.aspect_auditevent eq "DESCRIBE") or
        (CONTEXT.aspect_auditevent eq "BIND") or
        (CONTEXT.aspect_auditevent eq "COMMIT") or
        (CONTEXT.aspect_auditevent eq "IMPLICIT_REBIND") or
        (CONTEXT.aspect_auditevent eq "EXTERNAL_CANCEL") or
        (CONTEXT.aspect_auditevent eq "GET_DB_CFG") or
        (CONTEXT.aspect_auditevent eq "GET_DFLT_CFG")
    ) then

        if ((CONTEXT.aspect_database eq NULL) or (CONTEXT.aspect_database eq "")) then
            objectname := instance_name
        else
            if ((CONTEXT.object_schema eq NULL) or (CONTEXT.object_schema eq "")) then
                objectname := Append(istance_name, "\\", CONTEXT.aspect_database)
            else
                objectname := Append(instance_name, "\\", CONTEXT.aspect_database, ":", CONTEXT.object_schema)
            endif        
        endif
    endif

    if (CONTEXT.aspect_auditevent eq "UPDATE_AUDIT") then
        objectname := "db2audit.cfg"
    endif

    if (CONTEXT.aspect_auditevent eq "SWITCH_USER") then
        objectname := CONTEXT.aspect_db_user_name
    endif

    if ((CONTEXT.aspect_auditevent eq "OPEN_HISTORY_FILE") or
        (CONTEXT.aspect_auditevent eq "CLOSE_HISTORY_FILE") or
        (CONTEXT.aspect_auditevent eq "FETCH_HISTORY_FILE")
    ) then
        objectname := "db2audit.log"
    endif  

    path_to_logs_on_windows := Append(db2_udb_folder, "\\", instance_name, "\\", security)
    path_to_logs_on_unix := Append("/home/", instance_name, "/sqllib/security")

    case CONTEXT.aspect_auditevent of
        "DARI_START":
            if (CONTEXT.aspect_database eq "") then
                objectpath := ""
            else
                objectpath := instance_name
            endif

        "DARI_STOP":
            if (CONTEXT.aspect_database eq "") then
                objectpath := ""
            else
                objectpath := instance_name
            endif

        "LOAD_MSG_FILE":
            if (CONTEXT.aspect_database eq "") then
                objectpath := ""
            else
                objectpath := instance_name
            endif

        "UPDATE_AUDIT":
            if (typeOfPlatform eq "Windows") then
                objectpath := path_to_logs_on_windows
            else
                objectpath := path_to_logs_on_unix
            endif

        "OPEN_HISTORY_FILE":
            if (typeOfPlatform eq "Windows") then
                objectpath := path_to_logs_on_windows
            else
                objectpath := path_to_logs_on_unix
            endif

        "CLOSE_HISTORY_FILE":
            if (typeOfPlatform eq "Windows") then
                objectpath := path_to_logs_on_windows
            else
                objectpath := path_to_logs_on_unix
            endif

        "FETCH_HISTORY_FILE":
            if (typeOfPlatform eq "Windows") then
                objectpath := path_to_logs_on_windows
            else
                objectpath := path_to_logs_on_unix
            endif

        otherwise
            objectpath := "-"
    endcase

    objecttype := CONTEXT.objecttype

    if (auditedmachinetype eq "13") then
        objectpath     := NormalizeString(objectpath)
        objectname     := NormalizeString(objectname)
    endif

    object := GetObject(ObjectRefKey)

    platformname := source_system_name

    origin := GetOrigin(OriginRefKey)

    platform := GetPlatform(PlatformRefKey)

    target := GetTarget(TargetRefKey)

    user_id := CONTEXT.aspect_db_user_name

    ldapenable := Trim(Uppercase(GetChunkProperty(CONTEXT,"_ldapenable")), "Unavailable")
    if (ldapenable eq "YES")
    then
        originator := GetOriginator("",Lowercase(user_id))
        logonname  := Lowercase(user_id)
        realname   := logonname
    else
        case typeOfPlatform of
            "Windows":
                case user_id of
                    "Unavailable":
                        logonname  := "Unavailable"
                    otherwise
                        domainname := Trim(GetChunkProperty(CONTEXT,"_domainname"), "Unavailable")
                        logonname  := Uppercase(Append(domainname, "\\", user_id))
                endcase
            "AIX":
                logonname  := user_id
            "SUN":
                logonname  := user_id
            otherwise
                logonname  := user_id
        endcase
        realname   := logonname

        orig := GetOriginator(logonname)
        if(orig ne logonname)
        then
            originator := orig
        endif
    endif

    source := GetSource(SourceRefKey, SourceAspectKey)

    description := logonname
    stream := GetStream(StreamRefKey)

    eventtimestamp := CONTEXT.platformeventtime

    chunk := CONTEXT
    category:="CONTEXT"
    eventclass := CONTEXT.eventclass
    eventmainclass := CONTEXT.eventmainclass
    successclass := "Success"

    if Trim(CONTEXT.aspect_auditevent, "null") ne "null"             then auditevent := CONTEXT.aspect_auditevent endif
    if Trim(CONTEXT.aspect_database, "null") ne "null"               then database := CONTEXT.aspect_database endif
    if Trim(CONTEXT.aspect_command, "null") ne "null"                then command := CONTEXT.aspect_command endif
    if Trim(CONTEXT.aspect_db_user_name, "null") ne "null"           then db_user_name := CONTEXT.aspect_db_user_name endif
    if Trim(CONTEXT.aspect_query, "null") ne "null"                  then query := CONTEXT.aspect_query endif

    event := GetEvent(EventAspectsContext)

endstate
