/// iis.src.gml - Licensed Materials ­ Property of IBM
/// iis.src.gml - 5724-S67
/// iis.src.gml - © Copyright IBM Corp. 1998, 2007   All Rights Reserved.
/// iis.src.gml - US Government Users Restricted Right ­ Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

//

//

//

//

key EpiCreateKey(platformname,platformtype)
key ObjectRefKey(objectname,objectpath)
key SourceRefKey(logonname)
key StreamRefKey(description)
key FTPStreamRefKey(clientip,sessionnr,entrypoint)
key OriginRefKey(platformname,platformtype)
key TargetRefKey(platformname,platformtype)
key EventDetailKey(miscellaneous,reason,logrecordtype,command)

initial final state IISEvent   
  ((   srvtype eq "W3SVC" ))   

  epiname:=Trim(IISEvent.computername,GetChunkProperty(IISEvent,"platformname"));
  epname:=Trim(IISEvent.service,"Unavailable")
  platformname:=epiname
  if epname eq "Unavailable" then
    platformtype:=RegexReplace(GetChunkProperty(IISEvent,IISEvent.logdiroption), ".*[\\\\/]([^\\\\/]+)$", $1);
  else
    platformtype:=epname
  endif
  epi:=GetPlatform(EpiCreateKey)
  platform:=epi

  objectpath:=Path(IISEvent.target)
  objectname:=Filename(IISEvent.target)
  objecttype:="FILE"
  if objectname eq "" then
    objectname:=Filename(objectpath)
    objectpath:=Path(objectpath)
    objecttype:="DIRECTORY"
  endif
  objectname := Trim(objectname,"*")
  objectpath := Trim(objectpath,".")

  object:=GetObject(ObjectRefKey)

  clientip:=Trim(IISEvent.clientip,"Unavailable")
  username:=Trim(IISEvent.username,"Unavailable")
  if (IISEvent.username eq "-") then

    username:="Anonymous"
  endif
  realname:=username
  logonname:=username
  source:=GetSource(SourceRefKey)

  entrypoint:=Trim(IISEvent.clientip,"Unavailable")
  description:=entrypoint
  stream:=GetStream(StreamRefKey)

  platformtype:="-"
  target:=GetTarget(TargetRefKey)
  platformname:=entrypoint
  origin:=GetOrigin(OriginRefKey)

  originator:=Append(clientip,"/",username) 
  subject:=originator
  subjectname:=username
  eventclass:=NormalizeString(objecttype) 

  if (IISEvent.status eq null) then
    successclass:="Unavailable"
  else if ( (IISEvent.status>="100" and IISEvent.status<"300") or
       (IISEvent.status eq "304") )
    then
      successclass:="Success"
    else
      successclass:="Failure"
    endif
  endif

  case IISEvent.operation of
    "DELETE":
      eventmainclass:="Delete"
    "GET","get":
      eventmainclass:="Read"
    "HEAD","OPTIONS", "TRACE":
      eventmainclass:="Inspect"
    "PUT", "POST":
      eventmainclass:="Write"
    "SEARCH","PROPFIND","INDEX":
      eventmainclass:="Seek"
  otherwise
    if IISEvent.operation eq null then
      eventmainclass:="Unavailable"
      eventclass:="Unavailable"
    else
      eventmainclass:="Application"
      eventclass:="Record"
    endif
  endcase

  eventtimestamp:=IISEvent.platformeventtime

  chunk:=IISEvent
  command:=Append("HTTP:",Trim(IISEvent.operation,"Unavailable"))
  workunit:=Append(IISEvent.status,":",IISEvent.ntstatus)
  program:=IISEvent.service
  logrecordtype:=Append("IIS:",IISEvent.service)
  miscellaneous:=Append(IISEvent.statusdescription,":",IISEvent.parameter)

  event:=GetEvent(EventDetailKey)
endstate 

initial state FTPUser
  ((  ( srvtype eq "MSFTPSVC" )  and ( operation eq "USER" ) ))

  epiname:=Trim(FTPUser.computername,GetChunkProperty(FTPUser,"platformname"));
  epname:=Trim(FTPUser.service,"Unavailable")
  platformname:=epiname
  platformtype:=epname
  epi:=GetPlatform(EpiCreateKey)
  platform:=epi

  clientip:=Trim(FTPUser.clientip,"Unavailable")
  username:=Trim(FTPUser.username,"Unavailable")
  sessionnr:=Trim(FTPUser.sessionnr,"Unavailable")
  origin:=epi
  originator:=Append(clientip,"/",username) 
  realname:=originator
  logonname:=originator
  source:=GetSource(SourceRefKey)
  MoveTo(FTPPass) 
endstate 

final state DevNull
  //
endstate 

state FTPPass
 (( ( srvtype eq "MSFTPSVC" ) and ( operation eq "PASS" ) ))
  if ( (clientip  eq FTPPass.clientip) and 
       (sessionnr eq FTPPass.sessionnr )) then

   objectname:=Trim(FTPPass.service,"Unavailable")
   objectpath:=Trim(FTPPass.computername,GetChunkProperty(FTPPass,"platformname"));

   objecttype:="SYSTEM"
   object:=GetObject(ObjectRefKey)

   entrypoint:=FTPPass.clientip
   description:=entrypoint
   streamtimestamp:=FTPPass.platformeventtime
   stream:=GetStream(FTPStreamRefKey) 

   platformtype:="-"
   platformname:=Trim(FTPPass.clientip,epiname)
   origin:=GetOrigin(OriginRefKey)
   platformname:=epiname
   target:=GetTarget(TargetRefKey)

   subject:=originator
   subjectname:=username
   reason := ""

   eventmainclass := "Logon"
   eventclass := "Ftp"
   if FTPPass.ntstatus eq null then
     if (FTPPass.status eq null) then
       successclass:="Unavailable"
     else if ( (FTPPass.status>="100" and FTPPass.status<"300") or
       (FTPPass.status eq "304") )
       then
         successclass:="Success"
       else
         successclass:="Failure"
       endif
     endif
   else
     if (FTPPass.ntstatus = "0" ) then
        successclass := "Success"
        reason:=FTPPass.statusdescription
     else
       if (FTPPass.status = "332") then 
         reason := FTPPass.statusdescription
       else
         reason:=FTPPass.ntstatusdescription
       endif
       successclass:="Failure"
     endif
   endif

   eventtimestamp:=FTPPass.platformeventtime

   chunk:=FTPPass
   command:=Append("FTP:",FTPPass.operation)
   workunit:=Append("[",FTPPass.sessionnr,"]")
   program:=FTPPass.service
   logrecordtype:=Append("IIS:",FTPPass.service)
   logrecordtimestamp:=FTPPass.platformeventtime
   miscellaneous:=Append(FTPPass.parameter,":",reason)
   event:=GetEvent(EventDetailKey)

   MoveTo(DevNull)
 endif
endstate 

initial state FTPEvent
 ((srvtype eq "MSFTPSVC"
   and not ( operation eq "USER" or  operation eq "PASS" )
 )) 

  epiname:=Trim(FTPEvent.computername,GetChunkProperty(FTPEvent,"platformname"));
  epname:=Trim(FTPEvent.service,"Unavailable")
  platformname:=epiname
  platformtype:=epname
  epi:=GetPlatform(EpiCreateKey)
  platform:=epi

  sessionnr := FTPEvent.sessionnr
  origin:=epi
  clientip := Trim(FTPEvent.clientip,"Unavailable")
  username:=Trim(FTPEvent.username,"Unavailable")
  if (FTPEvent.username eq "-") then
    username:="Anonymous"
  endif
  originator:=Append(clientip,"/",username)
  logonname:=originator
  realname:=originator
  source:=GetSource(SourceRefKey)

  description:="Unknown"
  stream:=GetStream(FTPStreamRefKey)

  successclass := "Success"
  reason := ""
  operation := FTPEvent.operation
  //
  if FTPEvent.operation eq null then
    eventmainclass:="Unavailable"
    eventclass:="Unavailable"
  else
    case FTPEvent.operation of
      "created":
        eventmainclass:="Create"
        eventclass:="File"        
     "deleted":
        eventmainclass:="Delete"
        eventclass:="File"
     "DELE":
        eventmainclass:="Delete"
        eventclass:="File"
     "sent":
        eventmainclass:="Send"
        eventclass:="File"
     "appended":
        eventmainclass:="Write"
        eventclass:="File"
     "MKD":
        eventmainclass:="Create"
        eventclass:="Directory"
     "RMD": 
        eventmainclass:="Delete"
        eventclass:="Directory"
     "QUIT","closed","ABORT":
        eventmainclass:="Logoff"
        eventclass := "Ftp"
    endcase
  endif
  if (eventmainclass eq null) then 
    MoveTo(DevNull) 
  else
    MoveTo(FinishFTPEvent)
  endif
endstate

final state FinishFTPEvent

  if ( eventmainclass eq "Logoff" ) then
    objectname:=Trim(FTPEvent.service,"Unavailable")
    objectpath:=epiname
    objecttype:="SYSTEM"
  else
    objectname:=Trim(Filename(FTPEvent.target),"*")
    objectpath:=Trim(Path(FTPEvent.target),".")
    objecttype:="FILE"
  endif
  object:=GetObject(ObjectRefKey)

  platformname:=Trim(clientip,epiname)
  platformtype:="-"
  origin:=GetOrigin(OriginRefKey)
  platformname:=epiname
  target:=GetTarget(TargetRefKey)

  subject:=originator
  subjectname:=username
  reason := ""
  reason := Append(FTPEvent.status," : ",FTPEvent.statusdescription)
  case FTPEvent.status of
  "421":

    if (eventmainclass eq "Logoff") then
      successclass := "Success" 
      if FTPEvent.ntstatus = "121" then
        reason := Append(FTPEvent.ntstatus," : ")
        reason := Append(reason,FTPEvent.ntstatusdescription)
      endif 
    else
      successclass := "Failure" 
      reason := Append(reason, FTPEvent.ntstatusdescription)     
    endif
  otherwise
    if FTPEvent.status eq null then
      successclass:="Unavailable"
    else if (eventmainclass eq "Logoff") then 
        successclass := "Success"
      else
        if (FTPEvent.status<"421")
        then
          successclass:="Success"
        else  
          successclass:="Failure"
        endif
      endif
    endif
  endcase

  eventtimestamp:=FTPEvent.platformeventtime

  chunk:=FTPEvent
  command:=Append("FTP:",FTPEvent.operation)
  workunit:=Append("[",FTPEvent.sessionnr,"]")
  program:=FTPEvent.service
  logrecordtype:=Append("IIS:",FTPEvent.service)
  logrecordtimestamp:=FTPEvent.platformeventtime
  miscellaneous:=Append(FTPEvent.operation," ",FTPEvent.parameter," ",reason);
  event:=GetEvent(EventDetailKey)
endstate 

