/// itfim.src.gml - Licensed Materials ­ Property of IBM
/// itfim.src.gml - 5724-S67
/// itfim.src.gml - © Copyright IBM Corp. 1998, 2007   All Rights Reserved.
/// itfim.src.gml - US Government Users Restricted Right ­ Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

key ObjectRefKey(objecttype, objectname, objectpath)
key OriginRefKey(platformname, platformtype)
key PlatformRefKey(platformname, platformtype)
key PlatformAspectKey(environment)
key TargetRefKey(platformname, platformtype)
key SourceRefKey(logonname)
key SourceAspectKey(originator)
key StreamRefKey(description)
key EventDetailRefKey(recordtype, instance_id, thread_id, subcomponent, authenticator, description, user, authtype, command, flowid, issuer, protectedresource, scope, granttype)

first state Setup

  description := "Remove Duplicates"
  equality := "event_id,instance_id,logonname,emainclass,eclass,successclass,objectname,objectpath,objecttype"
  targets := ""
  period := "1" 
  strategy := "directory"
  removedups := GetListener(Aggregator)
  Listen(removedups,Setup)
  MoveTo(RemoveDuplicates,Setup)
endstate

last state Shutdown
  if removedups ne null then
    StopListening(removedups)
    removedups := null
  endif
endstate

state RemoveDuplicates (platformeventcount eq null)
  Listen(removedups,RemoveDuplicates)
endstate

initial final state itfim (description eq "Remove Duplicates")

  if (itfim.extensionname eq "IBM_SECURITY_MGMT_POLICY") then
    case itfim.command of
      "Createfederationpartner", "Createwebservicepartner":
        objectname := Remember("id2name", itfim.partnerid, itfim.objectname)
        if Trim(objectname) eq "" then
          objectname := Trim(itfim.partnerid, "Unavailable")
        endif
      "Deletefederationpartner":
        objectname := Forget("id2name", itfim.partnerid)
        objectname := Trim(itfim.objectname, "Unavailable")
        if Trim(objectname) eq "" then
          objectname := Trim(itfim.partnerid, "Unavailable")
        endif
      "Modifyfederationpartner", "Modifywebservicepartner":
        objectname := Recall("id2name", itfim.partnerid)
        if Trim(objectname) eq "" then
          objectname := Trim(itfim.partnerid, "Unavailable")
        endif
      otherwise
        objectname := Trim(itfim.objectname, "Unavailable")
    endcase
  else
    if (itfim.extensionname eq "IBM_SECURITY_TRUST") then
      case itfim.action of
        "Map":
          if (itfim.map_oauth eq "true") then 
            objectname := "System"
          else 
            if (RegexMatch(itfim.tokeninfo, "@") > "0") then
              objectname := Remember("email2name", Trim(itfim.tokeninfo), Trim(itfim.objectname))
            else
              objectname := Remember("uid2name", Trim(itfim.tokenvalues), Trim(itfim.objectname))
            endif
          endif
        otherwise
          if (RegexMatch(itfim.tokenvalues2, "@") > "0") then
            objectname := Trim(Recall("email2name", Trim(itfim.tokenvalues2)), Trim(itfim.tokenvalues2))
          else
            objectname := Trim(Recall("uid2name", Trim(itfim.tokenvalues2)), Trim(itfim.tokenvalues2))
          endif
      endcase
    else
      if (itfim.extensionname eq "IBM_SECURITY_TRUST_OAUTH10") then

        if (itfim.flowid eq "Not Available") then
          objectname := Trim(itfim.objectname, "Unavailable")
        else
          if (itfim.objectname ne "") then
            objectname := Remember("oauth10objectname", itfim.flowid, itfim.objectname)
          else
            objectname := Trim(Recall("oauth10objectname", itfim.flowid), "Unavailable")
          endif
        endif
      else
        if (itfim.extensionname eq "IBM_SECURITY_TRUST_OAUTH20") then

          if (itfim.flowid eq "Not Available") then
            objectname := Trim(itfim.objectname, "Unavailable")
          else
            if (itfim.objectname ne "") then
              objectname := Remember("oauth20objectname", itfim.flowid, itfim.objectname)
            else
              objectname := Trim(Recall("oauth20objectname", itfim.flowid), "Unavailable")
            endif
          endif
        else

          if (itfim.extensionname eq "IBM_SECURITY_SELFCARE") then
             objectname := Trim(itfim.user, "Unavailable")
          else
             objectname := Trim(itfim.objectname, "Unavailable")
          endif

        endif
      endif
    endif
  endif

  objecttype := Trim(itfim.objecttype, "Unavailable")
  if itfim.objectpath eq "Not Available" then
    objectpath := "Unavailable"
  else
    objectpath := Trim(itfim.objectpath, "Unavailable")
  endif
  object := GetObject(ObjectRefKey)

  if ((Trim(itfim.location) eq Trim(itfim.userlocation)) or
      (Trim(itfim.userlocation) eq "") or
      (Trim(itfim.userlocation) eq null)) then
    platformname := Trim(itfim.location, Trim(GetChunkProperty(itfim, "SystemName"), "Unavailable"))
    platformtype := "IBM TFIM"
  else
    platformname := Trim(itfim.userlocation, Trim(GetChunkProperty(itfim, "SystemName"), "Unavailable"))
    platformtype := Trim(itfim.userlocationtype, "Unavailable")
  endif

  if (itfim.extensionname eq "IBM_SECURITY_SELFCARE") then
    platformname := Trim(itfim.platformname, "Unavailable")
    platformtype := Trim(itfim.platformtype, "Unavailable")
  endif

  origin := GetOrigin(OriginRefKey)

  platformname := Trim(itfim.location, Trim(GetChunkProperty(itfim, "SystemName"), "Unavailable"))
  platformtype := "IBM TFIM"
  if itfim.environment ne "" then
    environment := itfim.environment
  endif
  platform := GetPlatform(PlatformRefKey, PlatformAspectKey)

  platformname := Trim(itfim.location, Trim(GetChunkProperty(itfim, "SystemName"), "Unavailable"))
  platformtype := "IBM TFIM"
  target := GetTarget(TargetRefKey)

  if (itfim.extensionname eq "IBM_SECURITY_TRUST") then
    if Trim(objectname) eq "" then
      logonname := "System"
    else
      logonname := objectname
    endif
  else
    if (itfim.extensionname eq "IBM_SECURITY_TRUST_OAUTH10") then
      logonname := objectname
    else
      if (itfim.extensionname eq "IBM_SECURITY_TRUST_OAUTH20") then
        logonname := objectname
      else
        logonname := Trim(itfim.logonname, "Unavailable")
      endif
    endif
  endif
  if logonname eq "Not Available" then

    if (itfim.extensionname eq "IBM_SECURITY_SELFCARE") then
      if (itfim.objectname_aux eq "Not Available")
      then
         logonname := Trim(itfim.user, "Unavailable")
      else
         logonname := Trim(itfim.logonname, "Unavailable")
      endif
    else
      logonname := "System"
    endif

  endif
  originator := logonname
  if logonname eq "System" then
    realname := "System"
  else
    realname := logonname
  endif
  source := GetSource(SourceRefKey, SourceAspectKey)

  description := itfim.message
  stream := GetStream(StreamRefKey)

  chunk := itfim
  eventmainclass := Trim(itfim.emainclass, "Unavailable")
  eventclass := Trim(itfim.eclass, "Unavailable")
  case itfim.success of
    "SUCCESSFUL":
      successclass := "Success"
    "TRUE": 
      successclass := "Success"
    "UNSUCCESSFUL":
      successclass := "Failure"
    "FALSE": 
      successclass := "Failure"
    otherwise
      successclass := "Unavlbl"
  endcase

  eventtimestamp := itfim.platformeventtime
  if itfim.recordtype ne ""        then recordtype := itfim.recordtype endif
  if itfim.instance_id ne ""       then instance_id := itfim.instance_id endif
  if itfim.thread_id ne ""         then thread_id := itfim.thread_id endif
  if itfim.subcomponent ne ""      then subcomponent := itfim.subcomponent endif
  if itfim.authenticator ne ""     then authenticator := itfim.authenticator endif
  if itfim.message ne ""           then description := itfim.message endif

  if itfim.user ne ""              then user := itfim.user endif
  if itfim.authtype ne ""          then authtype := itfim.authtype endif
  if itfim.command ne ""           then command := itfim.command endif

  if itfim.flowid ne ""            then flowid := itfim.flowid endif
  if itfim.granttype ne ""         then granttype := itfim.granttype endif
  if itfim.issuer ne ""            then issuer := itfim.issuer endif
  if itfim.scope ne ""             then scope := itfim.scope endif
  if itfim.protectedresource ne "" then protectedresource := itfim.protectedresource endif

  event := GetEvent(EventDetailRefKey)
endstate
