/// novell.src.gml - Licensed Materials ­ Property of IBM
/// novell.src.gml - 5724-S67
/// novell.src.gml - © Copyright IBM Corp. 1998, 2007   All Rights Reserved.
/// novell.src.gml - US Government Users Restricted Right ­ Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

//

//

//

//

//

key EpiCreateKey(platformname,platformtype)
key ObjectRefKey(objectname,objectpath,objecttype)
key SourceRefKey(origin,originator)
key SourceAspectKey(originator)
key StreamTimeKey(streamtimestamp)
key StreamRefKey(description)
key EventCreateKey(miscellaneous)
key OriginRefKey(platformname,platformtype)
key TargetRefKey(platformname,platformtype)

initial state Novell
  case Novell.platformevent of
    "newvolume": MoveTo(NewVolume)
    "newcontainer": MoveTo(NewContainer)
    "volumepseudologon": MoveTo(VolumePseudoLogon)
    "containerpseudologon": MoveTo(ContainerPseudoLogon)
    "pseudoopenfile": MoveTo(PseudoOpenFile)
    "volumelogon": MoveTo(VolumeLogon)
    "containerlogon": MoveTo(ContainerLogon)
    "volume": MoveTo(Volume)
    "container": MoveTo(Container)
    "join_partition": MoveTo(JoinPartition1)
  endcase
endstate

final state NewVolume

 path := RegexReplace(Novell.path,"[^.]+=","") 
 Remember("volumename", path) 
endstate 

final state NewContainer

 container := RegexReplace(Novell.containerdn,"[^.]+=","") 
 Remember("containername", container) 
 Remember("containertype", Novell.schemaclassname) 
endstate

final state VolumePseudoLogon

 username := RegexReplace(Novell.username,"[^.]+=","") 
 networkaddress := IntToNetworkAddress(Novell.nwatype,Novell.nwaddr)
 Remember("volumeuid2name", Novell.userid, username)
 Remember("volumecid2uid", Novell.connectionid, Novell.userid)
 Remember("volumeuid2addr", Novell.userid, networkaddress)
endstate

final state ContainerPseudoLogon

 username := RegexReplace(Novell.username,"[^.]+=","")
 Remember("containeruid2name", Novell.userid, username)
endstate

final state PseudoOpenFile

 Remember("fh2fname",Novell.handle,Novell.path)
endstate

state VolumeLogon

 username := RegexReplace(Novell.username,"[^.]+=","") 
 if Novell.connectionid ne null then
   Remember("volumecid2uid", Novell.connectionid, Novell.userid)
 endif
 Remember("volumeuid2name", Novell.userid, username)
 originator := username
 logonname := username
 objectname := Recall("volumename")
 objecttype := "SYSTEM"
 epiname := objectname
 MoveTo(Finish)
endstate

state ContainerLogon

 username := RegexReplace(Novell.username,"[^.]+=","") 
 Remember("containeruid2name", Novell.userid, username)
 originator := username
 logonname := username
 objectname := Recall("containername")
 objecttype := Recall("containertype")
 epiname := objectname
 MoveTo(Finish)
endstate

state Volume

 epiname := Recall("volumename")

 if Novell.originator ne null then
   originator := Novell.originator 
 else if Novell.connectionid = "0" then

   originator := "System"
 else   
   originator := Recall("volumecid2uid",Novell.connectionid) 
 endif endif
 if Novell.logonname ne null then
   logonname := Novell.logonname 
 else if Novell.connectionid = "0" then
   logonname := "System"
 else if originator eq "" then 
   originator := Append("cid:",Novell.connectionid)
   logonname := originator
 else
   logonname := Recall("volumeuid2name",originator)
 endif endif endif

 if Novell.handle ne null then 
   if Novell.filename ne null then 
     objectname := Remember("fh2fname",Novell.handle,Novell.filename)
   else                         
     objectname := Recall("fh2fname",Novell.handle)
   endif
   objecttype := "FILE"
 else if Novell.cid ne null then 
   uid := Recall("volumecid2uid", Novell.cid)
   if uid ne "" then
     objectname := Recall("volumeuid2name", uid)
   else 
     objectname := Append("cid:",Novell.cid)
   endif
   objecttype := "USER"
 else if Novell.objectname ne null then

   objectname := RegexReplace(Novell.objectname,"\\.[^.+:]+=","\\.")
   objecttype := Novell.recordtype
 else
   objectname := Recall("volumename")
   objecttype := "VOLUME"
 endif endif endif
 MoveTo(Finish)
endstate

state Container

epiname := Recall("containername")

 if Novell.originator ne null then
   originator := Novell.originator
 else
   originator := Novell.userid
 endif
 if Novell.logonname ne null then
   logonname := Novell.logonname
 else
   logonname := Recall("containeruid2name", originator)
   if logonname eq "" then      
     logonname := Append("uid:",originator)
   endif     
 endif

 if Novell.objectname ne null then
   objectname := RegexReplace(Novell.objectname,"[^.]+=","")
 else
   objectname := Recall("containername")
 endif 
 if Novell.recordtype ne null then
   objecttype := Novell.recordtype
 else
   objecttype := Recall("containertype")
 endif
 MoveTo(Finish)
endstate

state JoinPartition1

 objectname := Novell.objectname
 MoveTo(JoinPartition2)
endstate

state JoinPartition2 

 epiname := Recall("containername")

 originator := Novell.userid
 logonname := Recall("containeruid2name", originator)

 objecttype := "PARTITION"
 miscellaneous := Append("merged NDS partition into ",objectname)
 MoveTo(Finish)
endstate 

final state Finish

 epname := "Novell NetWare"
 platformname:=Trim(epiname,"Unavailable")
 platformtype:=epname
 epi := GetPlatform(EpiCreateKey)
 platform:=epi

 if Novell.objectpath ne null then
   objectpath := Novell.objectpath
 else
   if objectname ne "" then
     if RegexMatch(objectname, "^(.*)\\\\(.*)$") <> "0" then
       objectpath := RegexRef("1")
       objectname := RegexRef("2")
     else if RegexMatch(objectname, "^\\.?([^.]+)\\.(.*)$") <> "0" then
       objectpath := RegexRef("2")
       objectname := RegexRef("1")
     else
       objectpath := ""
     endif endif
   else
     objectpath := ""
   endif
 endif
 if objectname eq "" then
   objectname := "-" 
 endif
 if objectpath eq "" then
   objectpath := "-" 
 endif   
 object := GetObject(ObjectRefKey)

 origin := epi
 realname:=originator
 source := GetSource(SourceRefKey,SourceAspectKey)

 streamtimestamp := Novell.platformeventtime
 if Novell.entrypoint ne null then
   entrypoint := Novell.entrypoint
 else if Novell.nwatype ne null then
   entrypoint := IntToNetworkAddress(Novell.nwatype, Novell.nwaddr)
 else   
   entrypoint := epiname
 endif endif
 description:=Append(entrypoint, "/", origin, "/", originator)
 stream := GetStream(StreamRefKey)

 target:=GetTarget(TargetRefKey)
 origin:=GetOrigin(OriginRefKey)

 subject := originator
 subjectname := logonname

 if Novell.eventmainclass ne null then
   eventmainclass := Novell.eventmainclass
 else if Novell.emainclass ne null then
   eventmainclass := Novell.emainclass
 else
   eventmainclass := "*"
 endif endif
 if Novell.eventclass ne null then
   eventclass := Novell.eventclass
 else if Novell.eclass ne null then
   eventclass := Novell.eclass
 else
   eventclass := "*"
 endif endif
 if Novell.successclass ne null then
   successclass := Novell.successclass
 else if Novell.sclass eq "" then
   if Novell.status = "0" then  
     successclass := "Success"
   else
     successclass := "Failure"
   endif
 else
   successclass := Novell.sclass
 endif endif

 eventtimestamp := Novell.platformeventtime

 chunk := Novell
 if Novell.msg ne null then
   miscellaneous := Novell.msg
 else
   miscellaneous := Append(Novell.type,": ",Novell.description)
   if Novell.msgadd ne null then
     miscellaneous := Append(miscellaneous,Novell.msgadd)
   endif
   if Novell.status <> "0" then
     miscellaneous := Append(miscellaneous," (exit status = ",Novell.status,")")
   endif
 endif
 logrecordtype := Novell.recordtype
 logrecordtimestamp := Novell.platformeventtime
 event := GetEvent(EventCreateKey)

endstate
