/// novell_nsure.src.gml - Licensed Materials ­ Property of IBM
/// novell_nsure.src.gml - 5724-S67
/// novell_nsure.src.gml - © Copyright IBM Corp. 1998, 2007   All Rights Reserved.
/// novell_nsure.src.gml - US Government Users Restricted Right ­ Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

//

//

key PlatformRefKey(platformtype, platformname)
key ObjectRefKey(objecttype, objectpath, objectname)
key StreamRefKey(description)
key SourceRefKey(realname, logonname)
key OriginRefKey(platformnameFrom, platformtype)
key TargetRefKey(platformnameTo, platformtype)
key EventDetailKey(recordtype, Source, Target, attribute, authenticator, difference, duplicate, id, 
                    newname, newvalue, oldvalue, privilege, protocol, reason, request, user,
                    value, message, platformeventendtime, platformversion, object_focus)

first state Setup
  description:= "Aggregator"
  equality   := "recordtype,eventid,successclass,emainclass,eclass,objecttype,objectpath,objectname,username,Source,Target,sourceip,object_focus,origin_hostname,Originator"  
  period     := "300000" 
  aggregator := GetListener("Aggregator")
  Listen(aggregator,Setup)
  MoveTo(ListenState)
endstate

state ListenState ( platformeventcount eq null )
  Listen(aggregator,ListenState)
endstate

last state Shutdown
  if aggregator ne null then 
    StopListening(aggregator)
  endif
endstate

initial final state Finish ( platformeventcount ne null )
  eventid := Finish.eventid

  if (Finish.agentip ne null) then
    platformname := Uppercase(Trim(Finish.sourceip,Finish.agentip))
  else
    platformname := Uppercase(Trim(Finish.sourceip,GetChunkProperty(Finish,"platformname")))
  endif

  platformtype := "Novell (Nsure) Audit"
  platform     := GetPlatform(PlatformRefKey)

  if( (Finish.SessionID eq null) and
      (Finish.Originator eq null) and
      (Finish.OriginatorType eq null) and
      (Finish.Target eq null) and
      (Finish.TargetType eq null) and
      (Finish.SubTarget eq null) and
      (Finish.Text3 eq null) and
      (Finish.Value3 eq null) ) 
  then
     objectname := Trim(Finish.objectname, "-")          
     newname    := Trim(Finish.newname,null)
     logonname  := Trim(Finish.username, "System")
     Target     := Trim(Finish.Target101,null)
     platformversion := "1.0.1"
  else
     objectname := Trim(Trim(Finish.PrimaryTarget, Finish.Target), "-")  
     newname    := Trim(Finish.Text1,null)
     logonname  := Trim(Finish.Originator, "System")
     Target     := Trim(Finish.Target,null)
     platformversion := "greater 1.0.1"
  endif

  objectpath := Trim(Finish.objectpath, "-")
  objecttype := Trim(Finish.objecttype)

  if (objectpath eq "-" and (objecttype eq "FILE" or objecttype eq "DIRECTORY"
                             or objecttype eq "AUDITSERVICE")) 
  then
    objectpath := Trim(Path(objectname), ".")
    objectname := Trim(Filename(objectname), "-")   
  endif

  case objecttype of
     "SYSTEM":
       objectname := platformname
       objectpath := "-"         
  endcase
  object := GetObject(ObjectRefKey)

  description    := Trim(Finish.desc)
  stream         := GetStream(StreamRefKey)

  logonname      := RegexReplace(logonname, "^\\.(.+?)\\.?$", "$1")
  realname       := logonname
  source         := GetSource(SourceRefKey)

  platformnameFrom := Trim(Finish.whereFrom,platformname)
  platformnameTo   := Trim(Finish.whereTo,platformname)
  target           := GetTarget(TargetRefKey)

  if( (Finish.origin_hostname ne "") and (Finish.origin_hostname ne null) )
  then
    platformtype     := "Host"
    platformname     := Finish.origin_hostname
    platformnameFrom := Finish.origin_hostname
  endif
  origin := GetOrigin(OriginRefKey)

  eventclass     := Finish.eclass
  eventmainclass := Finish.emainclass
  successclass   := Trim(Finish.successclass,"Success")

  Source        := Trim(Finish.Source,null)

  attribute     := Trim(Trim(Finish.attribute,Finish.attribute2),null)
  authenticator := Trim(Finish.authenticator,null)
  difference    := Trim(Finish.difference,null)
  duplicate     := Trim(Finish.duplicate,null)
  id            := Trim(Finish.id,null)

  newvalue      := Trim(Finish.newvalue,null)
  oldvalue      := Trim(Finish.oldvalue,null)
  privilege     := Trim(Finish.privilege,null)
  protocol      := Trim(Finish.protocol,null)
  reason        := Trim(Finish.reason,null)
  request       := Trim(Finish.request,null)
  user          := Trim(Finish.user,null)
  value         := Trim(Finish.value,null)
  component     := Trim(Finish.component,null)
  reason        := Trim(Finish.reason,null)
  object_focus  := Trim(Finish.object_focus,null)

  eventtimestamp:= Finish.platformeventtime
  recordtype    := Trim(Finish.recordtype,null)
  chunk         := Finish

  if (Finish.platformeventendtime ne null) then
    platformeventendtime := Append("", Finish.platformeventendtime)
  endif  

  if Finish.platformeventcount ne null then
    eventcount := Finish.platformeventcount
  endif 

  msgadd       := Trim(Finish.msgadd, null)
  messagetext  := Trim(Finish.messagetext,null)
  if (messagetext ne null) then

    value1 := Trim(Finish.value1,null)
    value2 := Trim(Finish.value2,null)
    text1  := Trim(Finish.text1,null)
    text2  := Trim(Finish.text2,null)
    group  := Trim(Finish.group,null)
    data   := Trim(Finish.data,null)

   if ((value1 eq null) and (value2 eq null) and (text1 eq null) 
            and (text2 eq null) and (group eq null) and (data eq null)) then

      msgadd := messagetext
   endif   
  endif

  if (msgadd ne null) then
     message := Append(Finish.desc,"; ",msgadd);
  else
     message := Finish.desc 
  endif 
  if (component ne null) then
    message := Append(message,"; component: ",component)
  endif

  event := GetEvent(EventDetailKey)      

endstate
