/// nt.src.gml - Licensed Materials ­ Property of IBM
/// nt.src.gml - 5724-S67
/// nt.src.gml - © Copyright IBM Corp. 1998, 2007   All Rights Reserved.
/// nt.src.gml - US Government Users Restricted Right ­ Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

key PlatformRefKey(platformtype, platformname)
key ObjectRefKey(objecttype, objectpath, objectname)
key StreamRefKey(description)
key SourceRefKey(logonname)
key SourceAspectKey(originator)
key OriginRefKey(platformtype, platformname)
key TargetRefKey(platformtype, platformname)
key EventDetailKey(recordtype, message, logontype, reason, changes, access, recnum, group, platformeventendtime, 
        authenticationpackage, newvalue, oldvalue, Target,Source, attribute, user,validfrom,
        validto,agent,result,privilege,origtype)

first state Setup

 description := "Directory Scan"
 if GetIniProperty("directory_scan","yes") ne "yes" then
   equality := "emainclass,eclass,successclass,name1,name2,objectname,objectpath,objecttype,pid,sid,usersid,logonid"
   targets := ""
 else
   equality := "name1,name2,emainclass,eclass,successclass,objectname,objectpath,objecttype,sid,usersid,logonid,handleid"
   targets := "recordtype,objectname,pid,msgadd,eventid,successclass,epiname,domain,eventdescription"
 endif
 separator := "\\" 
 period := "5000" 

 strategy := "directory"
 directoryscan := GetListener(Aggregator)
 if ((Setup.skipdirscan ne "yes" and Setup.eclass ne "User" and Setup.eclass ne "Group")) then
   Listen(directoryscan,Setup)
 endif
 CopyTo(DirectoryScan)

 equality := "emainclass,eclass,successclass,name1,name2,objectname,objectpath,objecttype,pid,sid,usersid,handleid"
 description := "Ready"
 targets := "recordtype,pid,msgadd,eventid,successclass,epiname,domain,eventdescription,logonid"
 period := "2000" 
 strategy := "directory" 
 removedups := GetListener(Aggregator)
 if ((Setup.skipdirscan eq "yes" and Setup.eclass ne "User" and Setup.eclass ne "Group" and Setup.eclass ne "Password")) then
   Listen(removedups,Setup)
 endif
 directoryscan:=null 
 MoveTo(RemoveDuplicates)
endstate

last state Shutdown

 if directoryscan ne null then
     StopListening(directoryscan)
     Remember("DirectoryScan","status","shutting down")
     directoryscan := null
 endif
 if removedups ne null then
   if (Recall("DirectoryScan","status") eq "shutting down") then
     Remember("DirectoryScan","status","down")

     CopyTo(RemoveDuplicates)
   else

     if (Recall("DirectoryScan","status") eq "down") then
       StopListening(removedups)
       Forget("DirectoryScan","status")
       removedups := null
     else

       CopyTo(RemoveDuplicates)
     endif
   endif
 endif
endstate

state DirectoryScan ((platformeventcount eq null and emainclass ne "Application"
                     and eclass ne "User" and eclass ne "Group"))
  if DirectoryScan.skipdirscan ne "yes" then
    Listen(directoryscan,DirectoryScan)
  endif
endstate

state RemoveDuplicates
  ((((description eq "Directory Scan")
    or (skipdirscan eq "yes" and platformeventcount eq null))
    and eclass ne "User" and eclass ne "Group"))

  Listen(removedups,RemoveDuplicates)
endstate  

initial final state NTEvent (((eclass eq "User" or eclass eq "Group")
                            or (description eq "Ready" or emainclass eq "Application")))

      case NTEvent.platformevent of
        "handleid":
          MoveTo(HandleID)
        "pid":
          MoveTo(PID)
        otherwise
          MoveTo(Finish)
      endcase

endstate

initial final state OverwriteDetect
  chunkpath := GetChunkProperty(OverwriteDetect,"chunkpath")
  chunkpath := Trim(chunkpath,"chunkpath_IS_NULL")

  if Recall("chunkpath") ne chunkpath then
     Remember("chunkpath", chunkpath)
     if GetChunkProperty(OverwriteDetect, "Overwrite") eq "Y" then
          eventtimestamp := OverwriteDetect.platformeventtime
          validfrom := Trim(OverwriteDetect.validfrom, null)
          validto   := Trim(OverwriteDetect.validto, null)
          chunk := OverwriteDetect
          eventclass := "AUDIT"
          eventmainclass := "Overwrite"
          objecttype := "AUDITLOG"
          objectpath := OverwriteDetect.objectpath
          objectname := chunkpath
          epiname := Trim(OverwriteDetect.epiname)
          platformname := epiname
          platformtype := "Windows"
          object := GetObject(ObjectRefKey)
          origin := GetOrigin(OriginRefKey)
          platform := GetPlatform(PlatformRefKey)

          logonname :="SYSTEM"
          originator :=GetOriginator(logonname, "TSIEM")
          realname := originator
          source := GetSource(SourceRefKey,SourceAspectKey)
          description := "The original events log was overwritten."
          stream := GetStream(StreamRefKey)
          successclass :="Warning"
          target := GetTarget(TargetRefKey)
          event :=GetEvent(EventDetailKey)
     endif
  endif 
endstate

state HandleID

  handleid := Sum(NTEvent.handleid, "0")
  if ( NTEvent.handleid < "0") then

    handleid := Sum("2147483648", NTEvent.handleid) 
  endif

 if NTEvent.objectname ne null then
   objectname := Remember("handle2name", handleid, NTEvent.objectname)
   objectpath := Remember("handle2path", handleid, NTEvent.objectpath)
   objecttype := Remember("handle2type", handleid, NTEvent.objecttype)
 else
   if (NTEvent.eventid eq "562") then
     objectname := Forget("handle2name", handleid)
     objectpath := Forget("handle2path", handleid)
     objecttype := Forget("handle2type", handleid)
   else
     objectname := Recall("handle2name", handleid)
     objectpath := Recall("handle2path", handleid)
     objecttype := Recall("handle2type", handleid)
   endif
 endif
 objecttype := Trim(objecttype, "OBJECT")
 if Trim(objectname) eq "" then
   objectname := Append("Handle", NTEvent.handleid)
   objecttype := Append(objecttype, "ID")
 endif
 objectpath := Trim(objectpath, ".")
 if NTEvent.eventid ne "562" then 
   MoveTo(Finish)
 else
   MoveTo(Exit)
 endif
endstate 

final state Exit
 a:="1"
endstate

state PID
 if NTEvent.objectname ne null then
   objectname := Remember("pid2imagename", NTEvent.pid, NTEvent.objectname)
 else
   objectname := Forget("pid2imagename", NTEvent.pid)
 endif
 objecttype := NTEvent.objecttype
 if Trim(objectname) eq "" then
   objectname := Append("Process", NTEvent.pid)
   objecttype := Append(objecttype, "ID")
 endif
 objectpath := "."
 MoveTo(Finish)
endstate

final state Finish

  matchsid := "(?i)((%\\{)?(s-[-\\d]{3,})}?)"
  if ((NTEvent.eventid eq "528") or (NTEvent.eventid eq "540")) then
    if NTEvent.logonid ne "*" then 
      Remember("logonid2name",NTEvent.logonid,NTEvent.name1)
      Remember("logonid2domain",NTEvent.logonid,NTEvent.srcdomain)
    endif
  endif
  if (NTEvent.eventid eq "576") then
    objectname := Trim(NTEvent.objectname,Recall("logonid2name",NTEvent.logonid),"Unavailable")
    objectpath := Trim(NTEvent.objectpath,Recall("logonid2domain",NTEvent.logonid),"Unavailable")
  endif

  sid:=Trim(NTEvent.sid,NTEvent.usersid)
  if (NTEvent.name2 eq "\\" or NTEvent.name2 eq null) then
    case sid of
      "S-1-5-7", "\\":
        logonname := "ANONYMOUS"
      "S-1-5-18":
        logonname := "SYSTEM"
      otherwise

        if (RegexMatch(sid,matchsid) > "0") then
          logonname:=GetLogonName(RegexRef("3"),"Unavailable")
        else
          logonname := sid
        endif
    endcase
  else
    logonname:=Uppercase(Trim(NTEvent.name2,NTEvent.usersid))
    if (RegexMatch(logonname,matchsid) > "0") then
      logonname:=GetLogonName(RegexRef("3"),"Unavailable")
    endif
  endif

  if (logonname eq "NOSID") then
    logonname := "Unavailable"
  endif

  if (sid eq "NOSID") then    
    sid := GetOriginator(logonname, "Unavailable")
  else
    if (RegexMatch(sid,matchsid) eq "0") then
      sid := GetOriginator(sid,"Unavailable")
    endif
  endif
  originator := sid
  realname := NormalizeString(Trim(NTEvent.name1,logonname))
  if (RegexMatch(realname,matchsid) > "0") then
    realname:=GetLogonName(RegexRef("3"),"Unavailable")
  endif

   sid := GetOriginator(logonname, "Unavailable")
   if (sid ne "Unavailable") then
   originator:=sid
   endif

  logonnameUIS:=GetLogonName(originator,"S-1-5-7")
  if (logonnameUIS ne "S-1-5-7") then
    logonname:=logonnameUIS
  else
    if (RegexMatch(realname, "^([^@]*?)@") > "0") then
      logonname:=realname
      realname:=RegexRef("1") 
    endif 
  endif

  if (realname eq "Unavailable") then
    if (logonname eq "Unavailable") then
      logonname := originator
      realname := originator
    endif 
  endif

  source   := GetSource(SourceRefKey,SourceAspectKey)

  platformtype := "Windows"
  epiname := Trim(NTEvent.epiname)
  domain := Trim(NTEvent.domain)
  srcdomain := Trim(NTEvent.srcdomain)
  workstation := Trim(NTEvent.workstation)
  ipworkstation := Trim(NTEvent.ipworkstation)
  logonid := NTEvent.logonid

  if ((workstation ne null) and (workstation ne "")) then
    Remember("workstation", Append(epiname, "|", domain, "|", logonname, "|", logonid),
             workstation)

    if ((srcdomain ne null) and (srcdomain ne "")) then
        Remember("srcdomain", Append(epiname, "|", domain, "|", logonname, "|", logonid),
                srcdomain)
    if (workstation eq "*") then
	platformname := ipworkstation
    else
        platformname := Append(srcdomain, "\\", workstation)
    endif
    else
        if ((domain ne null) and (domain ne "")) then
            if (workstation eq "*") then
	        platformname := ipworkstation
            else
                platformname := Append(domain, "\\", workstation)
            endif
        else
            platformname := workstation
        endif
    endif
  else
    recalled_workstation := Trim(Recall("workstation", Append(epiname, "|", domain, "|", logonname, "|", logonid)), "")
    recalled_srcdomain := Trim(Recall("srcdomain", Append(epiname, "|", domain, "|", logonname, "|", logonid)), "")
    if (recalled_workstation ne "") then
        if (recalled_srcdomain ne "") then
            if ((srcdomain ne null) and (srcdomain ne "") and (srcdomain ne recalled_srcdomain)) then
                platformname := srcdomain
            else
                platformname := Append(recalled_srcdomain, "\\", recalled_workstation)
            endif
        else
            platformname := Trim(domain, recalled_workstation);
            if (platformname eq domain) then
                platformname := Append(domain, "\\", recalled_workstation)
            endif
        endif
    else
        if((srcdomain ne null) and (srcdomain ne "")) then
            platformname := srcdomain
        else
            platformname := Trim(domain, epiname, "Unavaliable")
            if ((platformname eq domain) and (epiname ne null) and (epiname ne "")) then
                platformname := Append(domain, "\\", epiname)                
            endif
        endif
    endif
  endif

  if ((NTEvent.eventid >= "672") and (NTEvent.eventid <= "677")) then
      if ((NTEvent.target ne "") and (NTEvent.target ne null)) then
        platformname := Trim(NTEvent.target, platformname)
      endif
  endif

  origin := GetOrigin(OriginRefKey)

  domain := Trim(NTEvent.domain)
  if ((domain ne "") and (domain ne null)) then
    platformname := Append(domain, "\\", epiname)
  else
    platformname := epiname
  endif

  platform := GetPlatform(PlatformRefKey)

  target := GetTarget(TargetRefKey)

  objecttype := Trim(objecttype,Trim(NTEvent.objecttype,"OBJECT"))
  if objecttype eq "*" then
    objecttype:="OBJECT"
  endif

  case NTEvent.eventid of
  "769", "770", "771":
    objectname := Trim(NTEvent.TopLevelName, Trim (NTEvent.DNSName, NTEvent.NetBIOS))
  endcase

  objectname := Trim(objectname, Trim(NTEvent.objectname, "-"))
  objectpath := Trim(objectpath, Trim(NTEvent.objectpath, "."))
  objecttype := Trim(NTEvent.objecttype, "OBJECT")

  if (objecttype eq "SYSTEM") then
    objectpath := Trim(objectpath,".")
    if (objectname eq "-") then
      objectname := null
    endif    
    objectname := Trim(objectname,platformname,GetChunkProperty(NTEvent,"platformname"))
  endif

  objectpathfromname := Trim(Path(objectname), ".")
  objectnamefromname := Trim(Filename(objectname))
  if objectpathfromname ne "." then
    objectpath := objectpathfromname
    objectname := objectnamefromname
  endif
  if objectpath eq "." then
    if objectname ne "-" then
      objectpath := Trim(Path(objectname), ".")
      objectname := Trim(Filename(objectname), "-")
    endif
  endif

  if ((NTEvent.eclass eq "User" and (NTEvent.emainclass eq "Add" or NTEvent.emainclass eq "Remove"))) then
    if (RegexMatch(objectname,matchsid) > "0") then
      objectname := GetLogonName(RegexRef("3"))
       if (RegexMatch(objectname,"\\\\") > "0") then
         objectname := RegexRef("1")
       endif
    endif
    objecttype := "GROUP-USER"
    objectpath := Trim(NTEvent.group,".")
    objectname := NormalizeString(objectname)
  else
    group := NTEvent.group
  endif
  case objecttype of                         
     "USER", "GROUP", "ALIAS", "OBJECT", "FILE", "PROCESS", "KEY":
       if (RegexMatch(objectname,matchsid) > "0") then
         objectname := GetLogonName(RegexRef("3"))
         if (RegexMatch(objectname,"\\\\") > "0") then
           objectname := RegexRef("1")
           objectpath := RegexRef("-1")
         endif
       endif
       objectname := NormalizeString(objectname)
       objectpath := NormalizeString(objectpath)
  endcase

  object := GetObject(ObjectRefKey)

  description := Append(domain, "/", epiname, "/", logonname);
  stream := GetStream(StreamRefKey)

  message0 := ""
  sid2name := ""
  if NTEvent.eventdescription ne "" then
    message0 := Append(message0, NTEvent.eventdescription)
  endif
  message0 := Append(message0," ", NTEvent.msgadd)
  if message0 ne "" then
    if (RegexMatch(message0,matchsid) > "0") then
      sid2name := GetLogonName(RegexRef("3"),"Unavailable")
      if (RegexMatch(sid2name,"\\\\") > "0") then
        sid2name := RegexReplace(sid2name,"\\\\","\\\\\\\\")
      endif
      message0 := RegexReplace(message0,matchsid,sid2name)
    endif
    message := message0
  endif

  eventclass := NTEvent.eclass
  eventmainclass := NTEvent.emainclass
  if eventclass eq "Object" then
    case objecttype of
      "FILE":
        eventclass := NormalizeString(objecttype)
    endcase
  endif
  successclass := NTEvent.successclass
  if (NTEvent.sclass eq "Failure" and successclass eq "Success") then
    successclass := "Failure"
  endif

  eventtimestamp := NTEvent.platformeventtime
  chunk     := NTEvent
  recordtype:= Trim(NTEvent.recordtype, null)
  reason    := Trim(NTEvent.reason, null)
  changes   := Trim(NTEvent.changes, null)
  recnum    := Trim(NTEvent.recnum, null)
  newvalue  := Trim(NTEvent.newvalue, null)
  oldvalue  := Trim(NTEvent.oldvalue, null)
  Target    := Trim(NTEvent.target, null)
  Source    := Trim(NTEvent.source, null)
  attribute := Trim(NTEvent.attribute, null)
  user      := Trim(NTEvent.user, null)
  validfrom := Trim(NTEvent.validfrom, null)
  validto   := Trim(NTEvent.validto, null)
  agent     := Trim(NTEvent.agent, null)
  result    := Trim(NTEvent.result, null)
  access    := Trim(NTEvent.access, null)
  changes   := Trim(NTEvent.changes, null)
  privilege := Trim(NTEvent.privilege, null)
  origtype  := Trim(NTEvent.origtype, null)

  authenticationpackage := Trim(NTEvent.authenticationpackage, null)

  if NTEvent.platformeventcount ne null then
    eventcount := NTEvent.platformeventcount
  endif

  if (NTEvent.platformeventendtime ne null) then
    platformeventendtime:=Append("\"",NTEvent.platformeventendtime,"\"")
  endif

  event := GetEvent(EventDetailKey)
endstate
