/// oracle.src.gml - Licensed Materials ­ Property of IBM
/// oracle.src.gml - 5724-S67
/// oracle.src.gml - © Copyright IBM Corp. 1998, 2007   All Rights Reserved.
/// oracle.src.gml - US Government Users Restricted Right ­ Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

//

key PlatformRefKey(platformtype, platformname)
key ObjectRefKey(objecttype, objectpath, objectname)
key StreamRefKey(description)
key SourceRefKey(realname, logonname)
key OriginRefKey(platformtype, platformname)
key TargetRefKey(platformtype, platformname)
key EventDetailKey(description, platformeventendtime, os_user_name, db_user_name, privilege, sql_statement)

first state Start

 sql_reporting_mode := NormalizeString(GetIniProperty("SYSDBA_SQL_STATEMENTS_REPORTING", "None"))
 case sql_reporting_mode of
 "None":
   equality := "emainclass,eclass,objectname,objecttype,objectpath,realname,logonname,objcreator,terminal,successclass,sessionid,comment"
 "Low":
   equality := "emainclass,eclass,objectname,objecttype,objectpath,realname,logonname,objcreator,terminal,successclass,sessionid,comment,sql_statement_low"
 "High":
   equality := "emainclass,eclass,objectname,objecttype,objectpath,realname,logonname,objcreator,terminal,successclass,sessionid,comment,sql_statement_high"
 endcase

 description := "RemoveDuplicates"
 targets := ""
 period := "600000" 
 threshold := "0"
 nodups := GetListener("Aggregator")
 Listen(nodups,Start)
 MoveTo(Filter)
endstate

state Filter((platformeventcount eq null and RegexMatch(objectname,"\\$")="0"))
  Listen(nodups,Filter)
endstate

last state Finish
  if nodups ne null then
    StopListening(nodups)
  endif
endstate

initial final state Oracle
 (( emainclass ne "NOMAPPING" and platformeventcount ne null))

  platformname := Trim(Oracle.computer,GetChunkProperty(Oracle,"platformname"))
  platformtype := "Oracle"
  platform     := GetPlatform(PlatformRefKey)
  OSUserAsLogonname := GetIniProperty("OSUserAsLogonname", "0")

  sessionid := Oracle.sessionid

  if (is_sysdba_format_event eq "FALSE") then
    if Recall("sessions",sessionid) eq "" then
      Remember("sessions",sessionid,sessionid)

      Remember("logons",sessionid,Trim(Oracle.logonname))
      Remember("names",sessionid,Trim(Oracle.realname))
      Remember("origins",sessionid,Trim(Oracle.terminal))
    endif
  endif

  platformtype := "-"
  target       := GetTarget(TargetRefKey)
  platformname := Trim(Oracle.terminal,
                       Trim(Recall("origins",sessionid),platformname))
  origin       := GetOrigin(OriginRefKey)

  OSName := Trim(Recall("names",sessionid), Trim(Oracle.realname,"Unavailable"))
  DBName := Trim(Recall("logons",sessionid), Trim(Oracle.logonname,"Unavailable"))

  os_user_name := OSName
  db_user_name := DBName
  privilege := Oracle.privilege

  if (OSUserAsLogonname ne "1") then
   logonname := DBName
  else
    logonname := OSName
  endif

  realname := logonname

  source := GetSource(SourceRefKey)

  objectpath := Trim(Oracle.objectpath,".")  

  objecttype := Trim(Oracle.objecttype,"OBJECT")
  objectname := Trim(Oracle.objectname,"Unavailable")

  case Oracle.emainclass of
   "Logon":
     objecttype := "LOGON"
     objectname := objectpath
     objectpath := Trim(Oracle.computer,GetChunkProperty(Oracle,"platformname"))
   "Logoff":
     objecttype := "LOGOFF"
     objectname := objectpath
     objectpath := Trim(Oracle.computer,GetChunkProperty(Oracle,"platformname"))
  endcase

  object := GetObject(ObjectRefKey)

  description := Append("session:",sessionid)
  stream      := GetStream(StreamRefKey) 

  eventmainclass := Trim(Oracle.emainclass,"Unavailable")
  eventclass     := Trim(Oracle.eclass,"Unavailable")

  successclass   := Oracle.successclass
  eventtimestamp := Oracle.platformeventtime
  description    := Oracle.comment
  chunk          := Oracle
  eventcount := Oracle.platformeventcount
  if (Oracle.platformeventendtime ne null) then 
    platformeventendtime := Append("\"",Oracle.platformeventendtime,"\"")
  endif

  sql_reporting_mode := NormalizeString(GetIniProperty("SYSDBA_SQL_STATEMENTS_REPORTING", "None"))
  case sql_reporting_mode of
  "Low":
    if ((Oracle.sql_statement_low ne "") and (Oracle.sql_statement_low ne null)) then sql_statement := Oracle.sql_statement_low endif
  "High":
    if ((Oracle.sql_statement_high ne "") and (Oracle.sql_statement_high ne null)) then sql_statement := Oracle.sql_statement_high endif
  endcase

  if ((emainclass ne "Unavailable") and (eventclass ne "Unavailable")) then
    GetEvent(EventDetailKey)
  endif

  if (is_sysdba_format_event eq "FALSE") then
    if emainclass eq "Logoff" then
      Forget("sessions",sessionid)
      Forget("logons",sessionid)
      Forget("names",sessionid)
      Forget("origins",sessionid)
    endif
  endif
endstate 
