/// oracle_portal.src.gml - Licensed Materials ­ Property of IBM
/// oracle_portal.src.gml - 5724-S67
/// oracle_portal.src.gml - © Copyright IBM Corp. 1998, 2007   All Rights Reserved.
/// oracle_portal.src.gml - US Government Users Restricted Right ­ Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

//

//
key PlatformRefKey(platformtype, platformname)
key OriginRefKey(platformtype, platformname)
key TargetRefKey(platformtype, platformname)
key ObjectRefKey(objecttype, objectpath, objectname)
key StreamRefKey(description)
key SourceRefKey(realname, logonname)

key LdapEventDetailKey(auditmessage,reason,description,eventtype)
key LoginsEventDetailKey(site,message,action_code)

key ActivityEventDetailKey(browser,domain,subdomain,information,url)

initial state Event
  platformname := GetChunkProperty(Event,"platformname")
  chunk        := Event

  case Event.platformevent of
    "ldap"    : MoveTo(LDAP)
    "logins"  : MoveTo(Logins)
    "activity": MoveTo(Activity)
  endcase
endstate

final state Logins

  objectpath   := "Oracle Login Server"
  objecttype   := "SYSTEM"
  objectname   := platformname
  object       := GetObject(ObjectRefKey)

  platformtype := objectpath
  platform     := GetPlatform(PlatformRefKey)

  platformtype := "-"
  target       := GetTarget(TargetRefKey)

  platformname := Trim(Event.IP_ADDRESS,"-")
  origin       := GetOrigin(OriginRefKey)

  logonname    := Trim(Event.USER_NAME,"Anonymous")
  realname     := Trim(Event.USER_NAME,"-")
  source       := GetSource(SourceRefKey)

  description  := Trim(logonname,"-")
  stream       := GetStream(StreamRefKey)

  if Event.ACTION eq "LOGIN" then
    eventmainclass := "Logon"
  else
    eventmainclass := NormalizeString(Event.ACTION)
  endif

  eventclass     := "User"
  if Event.MESSAGE eq "Login successful" then
    successclass := "Success"
  else
    successclass := "Failure"
  endif

  eventtimestamp := Event.platformeventtime

  site := Event.APP_SITE
  message := Event.MESSAGE
  action_code := Event.ACTION_CODE

  GetEvent(LoginsEventDetailKey)
endstate

final state Activity

  platformtype := "Oracle Web Portal"
  platform     := GetPlatform(PlatformRefKey)

  platformtype := "-"
  target       := GetTarget(TargetRefKey)

  platformname := Event.HTTP_REMOTE_ADDRESS
  origin       := GetOrigin(OriginRefKey)

  if (Event.ACTION eq "login" or
      Event.ACTION eq "logout") then
    objecttype := "SYSTEM"
    objectpath := "Oracle Web Portal"
    objectname := GetChunkProperty(Event,"platformname")
  else
    objectpath := Trim(Event.objectpath,Trim(Path(Event.NAME),"-"))
    objectname := Trim(Event.objectname,Trim(Filename(Event.NAME),"-"))
    objecttype := Trim(Event.objecttype,"Object")
  endif

  object       := GetObject(ObjectRefKey)

  logonname := Trim(Event.USERID)
  case Event.action of
   "create_user group",
   "create user":

      Remember("user2creator",Event.NAME,Event.USERID);
   "add_to_group group",
   "add_to_group user",
   "delete_from_group user",
   "delete_from_group group": 
      logonname := Trim(Recall("user2creator",Event.USERID),"Anonymous")

  endcase
  realname     := logonname
  source       := GetSource(SourceRefKey)

  description  := Trim(logonname,"-")
  stream       := GetStream(StreamRefKey)

  case Event.ACTION of
  "login":
    eventmainclass := "Logon"
    eventclass := "User"
  "logout":
    eventmainclass := "Logoff"
    eventclass := "User"
  otherwise
    eventmainclass := Trim(Event.mainclass,Trim(NormalizeString(Event.ACTION),
                           "Unknown"))
    eventclass     := Trim(Event.eventclass,NormalizeString(objecttype))
  endcase

  successclass   := "Success"

  eventtimestamp := Event.platformeventtime

  browser := Event.HTTP_USER_AGENT
  domain  := Event.DOMAIN
  subdomain := Event.SUB_DOMAIN
  information := Event.INFORMATION
  url := Event.URL

  GetEvent(ActivityEventDetailKey)
endstate

final state LDAP

  platformtype := "Oracle LDAP"
  platform     := GetPlatform(PlatformRefKey)
  target       := GetTarget(TargetRefKey)
  platformtype := "-" 
  origin       := GetOrigin(OriginRefKey)

  objectpath   := Trim(Event.objectpath, "-")
  objecttype   := Trim(Event.objecttype, "OBJECT")
  if objecttype eq "SYSTEM" then
    objectpath := "Oracle LDAP"
    objectname := platformname
  else
    objectname := Trim(Event.objectname, "-")
  endif
  object       := GetObject(ObjectRefKey)

  realname     := Trim(Event.username,"-")
  logonname    := Trim(Event.username,"Anonymous")
  source       := GetSource(SourceRefKey)

  description  := Trim(Event.username,"-")
  stream       := GetStream(StreamRefKey)

  eventmainclass := Trim(Event.mainclass)
  eventclass     := Trim(Event.eventclass,"-")
  successclass   := Trim(Event.orclopresult,"*")
  if successclass ne "Success" then
    successclass := "Failure"
  endif
  eventtimestamp := Event.platformeventtime

  auditmessage := Event.orclauditmessage
  reason := Trim(Event.orclopresult,"-")
  description := Event.description
  eventtype := Event.orcleventtype  
  event := GetEvent(LdapEventDetailKey)
endstate
