/// raptor.src.gml - Licensed Materials ­ Property of IBM
/// raptor.src.gml - 5724-S67
/// raptor.src.gml - © Copyright IBM Corp. 1998, 2007   All Rights Reserved.
/// raptor.src.gml - US Government Users Restricted Right ­ Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

key PlatformRefKey(platformtype, platformname)
key ObjectRefKey(objecttype, objectpath, objectname)
key StreamRefKey(description)
key SourceRefKey(realname, logonname)
key OriginRefKey(platformname, platformtype)
key TargetRefKey(platformname, platformtype)
key EventDetailKey(recordtype,eventcount,bytes,platformeventendtime)

first state Setup
  description := "RemoveDuplicates"
  equality    := "username,proto,sourcename,targetname"
  period      := "600000"
  threshold   := "0"

  nodups  := GetListener("Aggregator")
  MoveTo(ListenState)
endstate

state ListenState
(( (platformeventcount eq null) and (eventid eq "121") ))
  Listen(nodups,ListenState)
endstate

last state Shutdown
  if nodups ne null then 
    StopListening(nodups)
  endif
endstate

initial state StatCounter
(( (platformeventcount eq null) and (eventid eq "121") ))

  cursendcnt    := StatCounter.sendcnt
  curreceivecnt := StatCounter.receivecnt
  uniqid := Append(StatCounter.username,StatCounter.proto,StatCounter.sourcename,StatCounter.targetname)
  sendcnt    := Recall("Send",uniqid)
  receivecnt := Recall("Receive",uniqid)
  sendcnt    := Trim(sendcnt,"0")
  receivecnt := Trim(receivecnt,"0")
  sendcnt    := Sum(sendcnt, cursendcnt) 
  receivecnt := Sum(receivecnt, curreceivecnt) 
  Remember("Send",uniqid,sendcnt)   
  Remember("Receive",uniqid,receivecnt)
endstate

initial state Init 
(( ((platformeventcount ne null) and (eventid eq "121")) or (eventid ne "121") ))

  eventid := Init.eventid

  platformname := GetChunkProperty(Init,"platformname")
  platformtype := "Raptor"
  platform     := GetPlatform(PlatformRefKey)

  if (Init.objectame ne null and Init.objectpath eq "yes") then
    objectpath := Path(Init.objectname)
    objectname := Filename(Init.objectname)
  else
    objectname := Init.objectname
    objectpath := Init.objectpath
  endif
  objectname   := Trim(objectname, "-")
  objectpath   := Trim(objectpath, "-")
  if (objectpath eq "yes") then
    objectpath := "-"
  endif
  objecttype     := Trim(Init.objecttype,"Unknown")                                 
  object         := GetObject(ObjectRefKey)

  description := "generic"
  stream         := GetStream(StreamRefKey)

  logonname      := Trim(Init.logonname,Init.username)  
  logonname      := Trim(logonname, "System")
  realname       := logonname
  source         := GetSource(SourceRefKey)

  platformname   := Trim(Init.sourcename,"-")
  if platformname eq "-" then
    if ((objectname eq "auditlog") or (eventid eq "109")) then
      platformtype := "Raptor"
      platformname := GetChunkProperty(Init,"platformname")
    else
      platformtype := "Raptor"
      platformname := "Unavailable"
    endif
  else
    platformtype := "Host"
  endif
  origin         := GetOrigin(OriginRefKey)

  platformname   := Trim(Init.targetname,"-")
  if platformname eq "-" then
    if ((objectname eq "auditlog") or (eventid eq "109")) then
      platformtype := "Raptor"
      platformname := GetChunkProperty(Init,"platformname")
    else
      platformtype := "Raptor"
      platformname := "Unavailable"
    endif
  else
    platformtype := "Host"
  endif
  target         := GetTarget(TargetRefKey)

  eventclass     := Init.eclass
  successclass   := Init.successclass
  eventtimestamp := Init.platformeventtime
  chunk          := Init
  recordtype     := Init.recordtype
  eventmainclass := Init.emainclass

  if (Init.platformeventendtime ne null) then
    platformeventendtime:=Append("\"",Init.platformeventendtime,"\"")
  endif

  if eventid eq "121" then
    if Init.platformeventcount ne null then
      uniqid     := Append(Init.username,Init.proto,Init.sourcename,Init.targetname)
      sendcnt    := Recall("Send",uniqid)
      receivecnt := Recall("Receive",uniqid)
      Forget("Send",uniqid)
      Forget("Receive",uniqid)

      if (sendcnt <> "0") then
        bytes := sendcnt
        CopyTo(Send)
      endif
      if (receivecnt <> "0") then
        bytes := receivecnt
        MoveTo(Receive)
      endif
    endif
  else
    message := description
    MoveTo(Finish) 
  endif
endstate

state Receive
  eventmainclass := "Receive"
  MoveTo(Finish)  
endstate

state Send
  eventmainclass := "Send"
  MoveTo(Finish)
endstate

final state Finish
  if ((eventcount ne null) or (bytes ne null) or (eventid ne "121")) then
    recordtype := Init.eventid
    event := GetEvent(EventDetailKey)
  endif
endstate

