/// sapnetweaver.src.gml - Licensed Materials ­ Property of IBM
/// sapnetweaver.src.gml - 5724-S67
/// sapnetweaver.src.gml - © Copyright IBM Corp. 1998, 2007   All Rights Reserved.
/// sapnetweaver.src.gml - US Government Users Restricted Right ­ Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

key PlatformCreateKey(platformname,platformtype)
key ObjectRefKey(objectname,objectpath,objecttype)
key ObjectAspectKey(origtype)
key SourceRefKey(sapsrv,username)
key StreamTimeKey(streamtimestamp)
key StreamRefKey(description)
key OriginRefKey(platformname,platformtype)
key TargetRefKey(platformname,platformtype)
key EventDetailKey(recordtype,transactioncode,programname,Target,
                   Targettype,oldvalue,newvalue,request,profile,
                   reason,miscellaneous)

initial state SapEvent

  platformname := Trim(SapEvent.terminal, GetChunkProperty(SapEvent, "SystemName"))
  if (Trim(SapEvent.terminal, "") ne "") then
    platformtype := "SAP WAS Client"
  else
    platformtype := "SAP WAS"
  endif
  origin := GetOrigin(OriginRefKey)

  platformname := Trim(GetChunkProperty(SapEvent, "SystemName"))
  sap_inst := Trim(GetChunkProperty(SapEvent, "MapperOption1"))

  if ((sap_inst ne "") and (sap_inst ne null)) then platformname := Append(platformname, ".", sap_inst) endif
  if ((SapEvent.client ne "") and (SapEvent.client ne null)) then platformname := Append(platformname, ".", SapEvent.client) endif

  platformtype := "SAP WAS"
  platform := GetPlatform(PlatformCreateKey)
  target := GetTarget(TargetRefKey)

  sapsrv:=platformname

  username:=Trim(SapEvent.userid,"Anonymous")
  originator:=username
  realname:=originator
  logonname:=username
  source:=GetSource(SourceRefKey)

  entrypoint:=Trim(SapEvent.terminal,sapsrv)

  description:=Append(sapsrv,username,entrypoint)
  stream:=GetStream(StreamRefKey)

  miscellaneous:=Append("mode ",SapEvent.mode)

  successclass:="Success"
  eventtimestamp:=SapEvent.platformeventtime
  command:=Trim(SapEvent.transactioncode,"Unavailable")
  chunk:=SapEvent

  case SapEvent.platformevent of
  "logon":
     MoveTo(LogonLogoff)
  "rfccall":
     MoveTo(RfcCall)
  "transaction":
     MoveTo(Transaction)
  "report":
     MoveTo(Report)
  "usermasterchange":
     MoveTo(UMC)
  "download":
     MoveTo(Download)
  "signature":
     MoveTo(Signature)
  "application":
     MoveTo(Application)
  "audit":
     MoveTo(Audit)
  "auditpolicy":
     MoveTo(AuditPolicy)
  "security":
     MoveTo(Security)
  "module":
     MoveTo(Module)
  "virusscan":
     MoveTo(Virusscan)
  endcase
endstate

final state Virusscan

  objectname:="Virus Scan"
  objectpath:=sapsrv
  objecttype:="SYSTEM"
  object:=GetObject(ObjectRefKey)

  if SapEvent.eventcode eq "BU8" then
    eventmainclass:="Virusdetect"
    eventclass:="System"
    successclass:="Success"
  else
    eventmainclass:="Internalwarning"
    eventclass:="Virusscanner"
    successclass:="Warning"
  endif

  recordtype:=SapEvent.eventcode
  if SapEvent.usertype ne "" then profile:=SapEvent.usertype endif
  if SapEvent.identifier ne "" then reason:=SapEvent.identifier endif
  event:=GetEvent(EventDetailKey)
endstate 

final state Module

  case SapEvent.eventcode of
   "BU5":
     objectname:=SapEvent.usertype
     objectpath:="."
     objecttype:="PROCESS"
   "BU6":
     objectname:=SapEvent.identifierpath
     objectpath:="."
     objecttype:="PROCESS"
   "BU7":
     objectname:="ICF Recorder"
     objectpath:=sapsrv
     objecttype:="MODULE"
  endcase
  if SapEvent.eventcode ne "BU7" then
    origtype:="ICF Recorder entry"
    object:=GetObject(ObjectRefKey, ObjectAspectKey)
  else
    object:=GetObject(ObjectRefKey)
  endif

  if SapEvent.eventcode eq "BU7" then
    eventmainclass:="Modify"
    eventclass:="Module"
  else
    eventmainclass:="Execute"
    eventclass:="Process"
  endif
  successclass:="Success"

  if SapEvent.transactioncode ne "" then transactioncode := SapEvent.transactioncode endif
  if SapEvent.programname ne "" then programname := SapEvent.programname endif
  recordtype:=SapEvent.eventcode
  if SapEvent.eventcode eq "BU5" then
    Target:=SapEvent.reasoncode
    Targettype:="User"
  endif
  event:=GetEvent(EventDetailKey) 
endstate 

final state Security

  objectname:="Security"
  objectpath:=sapsrv
  objecttype:="SYSTEM"
  object:=GetObject(ObjectRefKey)

  if SapEvent.eventcode eq "BU3" then
    eventmainclass:="Modify"
    eventclass:="Security"
    successclass:="Success"
  else
    eventmainclass:="Securityexposure"
    eventclass:="System"
    successclass:="Warning"
    request:=SapEvent.request
  endif

  recordtype:=SapEvent.eventcode
  if SapEvent.oldvalue ne "" then oldvalue := SapEvent.oldvalue endif
  if SapEvent.newvalue ne "" then newvalue := SapEvent.newvalue endif
  if SapEvent.request ne "" then request := SapEvent.request endif
  if SapEvent.transactioncode ne "" then transactioncode := SapEvent.transactioncode endif
  if SapEvent.programname ne "" then programname := SapEvent.programname endif
  event:=GetEvent(EventDetailKey)
endstate 

final state LogonLogoff

  if SapEvent.eventcode eq "AUC" then
    objectname:="Logoff"
  else
    objectname:="Logon"
  endif
  objectpath:=sapsrv
  objecttype:="SYSTEM"
  object:=GetObject(ObjectRefKey)

  eventmainclass:=objectname  
  eventclass:="User"
  case SapEvent.eventcode of
   "AUO":
     eventmainclass:="Authorize"
	 case SapEvent.reasoncode of
	   "0","4":
	     successclass:="Success"
	 otherwise
	   successclass:="Failure"
	 endcase
   "AU5":
     eventmainclass:=Append("[",objectname,"]")
   "AU6":
     eventmainclass:=Append("[",objectname,"]")
	 case SapEvent.reasoncode of
	   "0","4":
	     successclass:="Success"
	 otherwise
	   successclass:="Failure"
	 endcase
   "AU2":
     case SapEvent.reasoncode of
       "1","2","10","13":
         eventmainclass:="Authorize"
         successclass:="Failure"
       "3","5","6","7","8","9","10","11","12":
         eventmainclass:="Authenticate"
         successclass:="Failure"
       "0":
         eventmainclass:="Authorize"
         successclass:="Success"
       "4":
         eventmainclass:="Authenticate"
         successclass:="Success"
     otherwise
       eventmainclass:="Authorize"
       successclass:="Failure"
     endcase
   "BU1":
      eventmainclass:="Authenticate"
      successclass:="Failure"
  endcase

  case SapEvent.eventcode of
   "AUC","AU1","AU5":
     successclass:="Success"
  endcase

  workunit:=Append(Trim(SapEvent.sessionnr)," ",SapEvent.processid)
  if SapEvent.transactioncode ne "" then transactioncode := SapEvent.transactioncode endif
  if SapEvent.programname ne "" then programname := SapEvent.programname endif
  if (RegexMatch(SapEvent.eventcode,"(AU2|AU6|AUO)") > "0") then
    recordtype:=Append(RegexRef("1"),".",SapEvent.reasoncode)
  else
    recordtype:=SapEvent.eventcode
  endif

  if SapEvent.reason eq "EMPTY" then
    if SapEvent.usertype ne "EMPTY" then
      miscellaneous:=Append(miscellaneous,"; User type : ",SapEvent.usertype)
    endif
  else
    if SapEvent.eventcode ne "BU1" then
      miscellaneous:=Append(miscellaneous
                          ,"; User type : ",SapEvent.usertype
                          ,", Reason : ",SapEvent.reasoncode)
	endif
    reason:=SapEvent.reason
  endif
  event:=GetEvent(EventDetailKey)
endstate 

final state RfcCall 

  objectname:=Trim(SapEvent.identifier,"Unavailable")
  objectpath:=Trim(SapEvent.functiongroup,"Unavailable")
  objecttype:="FUNCTION"
  object:=GetObject(ObjectRefKey)

  eventmainclass:="Call"
  eventclass:="Function"
  if (SapEvent.eventcode eq "AUK") then
    successclass:="Success"
  else  
    successclass:="Failure"
  endif

  workunit:=Append(Trim(SapEvent.sessionnr)," ",SapEvent.processid)
  if SapEvent.transactioncode ne "" then transactioncode := SapEvent.transactioncode endif
  if SapEvent.programname ne "" then programname := SapEvent.programname endif
  recordtype:=SapEvent.eventcode
  event:=GetEvent(EventDetailKey)
endstate 

final state Transaction 

  objectname:=Trim(SapEvent.transactioncode,"Unavailable")
  if (SapEvent.mod_fine eq ".") then
    objectpath:=SapEvent.mod_coarse
  else
    objectpath:=SapEvent.mod_fine
  endif
  objecttype:="TRANSACTION"
  object:=GetObject(ObjectRefKey)

  case SapEvent.eventcode of
   "AU3","AU4":
     eventmainclass:="Start"
   "AUQ":
     eventmainclass:="Unlock"
   "AUP":
     eventmainclass:="Lock"
  endcase
  eventclass:="Transaction"
  case SapEvent.eventcode of
   "AU4":
     successclass:="Failure"
   "AU3","AUP","AUQ":
    successclass:="Success"
  endcase

  workunit:=Append(Trim(SapEvent.sessionnr)," ",SapEvent.processid)
  if SapEvent.transactioncode ne "" then transactioncode := SapEvent.transactioncode endif
  if SapEvent.programname ne "" then programname := SapEvent.programname endif
  if ( SapEvent.eventcode eq "AUQ" and Trim(SapEvent.identifier) ne "") then
    miscellaneous:=Append(miscellaneous
                          ,"; Transaction "
                          ,SapEvent.transactioncode
                          ," unlocked using "
                          ,Trim(SapEvent.identifier)
                          )
  else
    if (SapEvent.tcode_desc eq "*" ) then
      description := SapEvent.mod_desc
    else
      description := SapEvent.tcode_desc
    endif
    miscellaneous:=Append( miscellaneous, ", ", description)
  endif
  recordtype:=SapEvent.eventcode
  event:=GetEvent(EventDetailKey)
endstate 

final state Report 

  objectname:=SapEvent.identifier
  objectpath:="."
  objecttype:="REPORT"
  object:=GetObject(ObjectRefKey)

  eventmainclass:="Start"
  eventclass:="Report"
  case SapEvent.eventcode of
   "AUW":
      successclass:="Success"
   "AUX":
      successclass:="Failure"
  endcase

  workunit:=Append(Trim(SapEvent.sessionnr)," ",SapEvent.processid)
  if SapEvent.transactioncode ne "" then transactioncode := SapEvent.transactioncode endif
  if SapEvent.programname ne "" then programname := SapEvent.programname endif
  if SapEvent.reason ne "" then reason:=SapEvent.reason endif
  miscellaneous:=Append(miscellaneous,"; ",Trim(SapEvent.reason))
  recordtype:=SapEvent.eventcode
  event:=GetEvent(EventDetailKey)
endstate 

final state UMC 

  case SapEvent.eventcode of
   "AU7","AU8","AU9","AUA","AUB","AUD","AUM","AUN","BU2":
     objectpath:="."
     objectname:=Trim(SapEvent.identifier,"Unavailable")
     objecttype:="USER"
   "AUR","AUS","AUT","AUU":
     if SapEvent.identtype eq "Profile" then
       objectpath:="."
       objectname:=SapEvent.identifier
       objecttype:="PROFILE"
     else
       objectpath:=SapEvent.identifierpath
       objectname:=SapEvent.identifier
       objecttype:="AUTHORIZATION"
     endif
  otherwise
     objectpath:="-"
     objectname:="Unavailable"
  endcase
  object:=GetObject(ObjectRefKey)

  case SapEvent.eventcode of
   "AU8","AUR","AUS":
     eventmainclass:="Delete"
   "AU9","AUM":
     eventmainclass:="Lock"
   "AUA","AUN":
     eventmainclass:="Unlock"
   "AU7","AUR":
     eventmainclass:="Create"
   "AUB","AUD","AUT":
     eventmainclass:="Modify"
   "AUU":
     eventmainclass:="Activate"
   "BU2":
     eventmainclass:="Change"
     eventclass:="Password"
  endcase
  case SapEvent.eventcode of
    "AU7","AU8","AU9","AUA","AUB","AUD","AUM","AUN":
       eventclass:="User"
    "AUR","AUS","AUT","AUU":
       eventclass:=SapEvent.identtype

  endcase

  workunit:=Append(Trim(SapEvent.sessionnr)," ",SapEvent.processid)
  case SapEvent.eventcode of
   "AUM":
     miscellaneous:=Append(miscellaneous
                          ,"; User Locked After Incorrect Logon")
   "AUN":
     miscellaneous:=Append(miscellaneous
                          ,"; User Lock Because of Incorrect Logon Removed")
   "AUD":
     miscellaneous:=Append(miscellaneous
                          ,"; User Master Record of User '",objectname,"' Changed")
   "AUB":
     miscellaneous:=Append(miscellaneous
                          ,"; Authorizations for User '",objectname,"' Changed")
  endcase
  if (RegexMatch(SapEvent.eventcode,"(AUR|AUS|AUT|AUU)") > "0") then
    recordtype:=Append(RegexRef("1"),".",SapEvent.identtype)
  else
    recordtype:=SapEvent.eventcode
  endif
  if SapEvent.transactioncode ne "" then transactioncode := SapEvent.transactioncode endif
  if SapEvent.programname ne "" then programname := SapEvent.programname endif
  event:=GetEvent(EventDetailKey)
endstate 

final state Application 

  objectname:=Trim(SapEvent.programname,"Unavailable")
  objectpath:="."
  objecttype:="PROGRAM"
  object:=GetObject(ObjectRefKey)

  if SapEvent.eventcode eq "AUG" then
    eventmainclass:="Start"
  else
    eventmainclass:="Stop"
  endif
  eventclass:="Process"

  workunit:=Append(Trim(SapEvent.sessionnr)," ",SapEvent.processid)
  if SapEvent.transactioncode ne "" then transactioncode := SapEvent.transactioncode endif
  if SapEvent.programname ne "" then programname := SapEvent.programname endif
  recordtype:=SapEvent.eventcode
  event:=GetEvent(EventDetailKey)
endstate 

final state Download 

  objectname:=Trim(SapEvent.identifier, "Unavailable")

  if (Trim(SapEvent.terminal, "") ne "") then
    objectpath := Trim(SapEvent.terminal)
  else
    objectpath := sapsrv
  endif 

  objecttype:="FILE"
  object:=GetObject(ObjectRefKey)

  eventmainclass:="Create"
  eventclass:="File"

  workunit:=Append(Trim(SapEvent.sessionnr)," ",SapEvent.processid)
  if SapEvent.transactioncode ne "" then transactioncode := SapEvent.transactioncode endif
  if SapEvent.programname ne "" then programname := SapEvent.programname endif
  recordtype:=SapEvent.eventcode
  miscellaneous:=Append(miscellaneous, "; ",
                       SapEvent.filetype, "-file '", objectname,
                       "', ", SapEvent.filesize, " bytes",
                       " created on ", objectpath)

  event:=GetEvent(EventDetailKey)
endstate 

final state Signature 

  objectname:=Trim(SapEvent.identifier,"Unavailable")
  objectpath:="."
  objecttype:="SIGNATURE"
  object:=GetObject(ObjectRefKey)

  eventmainclass:="Authenticate"
  eventclass:="User"
  successclass:="Failure"

  workunit:=Append(Trim(SapEvent.sessionnr)," ",SapEvent.processid)
  if SapEvent.transactioncode ne "" then transactioncode := SapEvent.transactioncode endif
  if SapEvent.programname ne "" then programname := SapEvent.programname endif
  recordtype:=SapEvent.eventcode
  case SapEvent.reason of
   "PROFILE LOCKED":
     miscellaneous:=Append(miscellaneous,"; The SSF profile is locked")
   "INVALID PASSWORD":
     miscellaneous:=Append(miscellaneous,"; Incorrect password specified")
   "VERIFY FAILED":
     miscellaneous:=Append(miscellaneous,"; User signature could not be verified by the system")
  endcase
  event:=GetEvent(EventDetailKey)
endstate 

final state Audit

  objectname:="Audit"
  objectpath:=sapsrv
  objecttype:="SYSTEM"
  object:=GetObject(ObjectRefKey)

  if SapEvent.action eq "1" then
    eventmainclass:="Start"
  else
    eventmainclass:="Stop"
  endif
  eventclass:="Audit"

  workunit:=Append(Trim(SapEvent.sessionnr)," ",SapEvent.processid)
  if SapEvent.transactioncode ne "" then transactioncode := SapEvent.transactioncode endif
  if SapEvent.programname ne "" then programname := SapEvent.programname endif
  recordtype:=SapEvent.eventcode
  event:=GetEvent(EventDetailKey)
endstate 

final state AuditPolicy

  objectname:="Audit"
  objectpath:=sapsrv
  objecttype:="SYSTEM"
  object:=GetObject(ObjectRefKey)

  eventmainclass:="Modify"
  eventclass:="Auditpolicy"

  workunit:=Append(Trim(SapEvent.sessionnr)," ",SapEvent.processid)
  if SapEvent.programname ne "" then programname := SapEvent.programname endif
  if SapEvent.transactioncode ne "" then transactioncode := SapEvent.transactioncode endif
  recordtype:=SapEvent.eventcode
  if SapEvent.aud_class eq "0" then
    miscellaneous:=Append
        ( miscellaneous,"; "
        , "Slot "  ,SapEvent.aud_slot
        ,": Weight ",SapEvent.aud_weight
        ,", User "  ,Trim(SapEvent.aud_user)
        ,", Client ",SapEvent.aud_client
        ,", Msg. "  ,Trim(SapEvent.aud_msg))
  else
    miscellaneous:=Append
        (miscellaneous,"; "
        ,  "Slot "  ,SapEvent.aud_slot
        ,": Class " ,SapEvent.aud_class
        ,", Weight ",SapEvent.aud_weight
        ,", User "  ,Trim(SapEvent.aud_user)
        ,", Client ",SapEvent.aud_client
        )
  endif
  event:=GetEvent(EventDetailKey)
endstate 
