/// sapr3.src.gml - Licensed Materials ­ Property of IBM
/// sapr3.src.gml - 5724-S67
/// sapr3.src.gml - © Copyright IBM Corp. 1998, 2007   All Rights Reserved.
/// sapr3.src.gml - US Government Users Restricted Right ­ Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

//

//

key PlatformCreateKey(platformname,platformtype)
key ObjectRefKey(objectname,objectpath,objecttype)
key SourceRefKey(sapsrv,username)
key StreamTimeKey(streamtimestamp)
key StreamRefKey(description)
key OriginRefKey(platformname,platformtype)
key TargetRefKey(platformname,platformtype)
key EventDetailKey(miscellaneous,logrecordtype)

initial state SapEvent
//
  sapsrv:=Trim(SapEvent.client,GetChunkProperty(SapEvent,"platformname"))
  platformname:=sapsrv
  platformtype:="SAP R/3"
  platform:=GetPlatform(PlatformCreateKey)

  username:=Trim(SapEvent.userid,"Anonymous")

  originator:=username
  realname:=originator
  logonname:=username
  source:=GetSource(SourceRefKey)

  entrypoint:=Trim(SapEvent.terminal,sapsrv)

  description:=Append(sapsrv,username,entrypoint)
  stream:=GetStream(StreamRefKey)

  miscellaneous:=Append("mode ",SapEvent.mode)

  platformtype:="-"
  target:=GetTarget(TargetRefKey)
  platformname:=entrypoint
  origin:=GetOrigin(OriginRefKey)

  successclass:="Success"
  eventtimestamp:=SapEvent.platformeventtime
  command:=Trim(SapEvent.transactioncode,"Unavailable")
  chunk:=SapEvent

  case SapEvent.platformevent of
  "logon":
     MoveTo(LogonLogoff)
  "rfccall":
     MoveTo(RfcCall)
  "transaction":
     MoveTo(Transaction)
  "report":
     MoveTo(Report)
  "usermasterchange":
     MoveTo(UMC)
  "download":
     MoveTo(Download)
  "signature":
     MoveTo(Signature)
  "application":
     MoveTo(Application)
  "audit":
     MoveTo(Audit)
  "auditpolicy":
     MoveTo(AuditPolicy)
  endcase
endstate  

final state LogonLogoff 
//

  if SapEvent.eventcode eq "AUC" then
    objectname:="Logoff"
  else
    objectname:="Logon"
  endif
  objectpath:=sapsrv
  objecttype:="SYSTEM"
  object:=GetObject(ObjectRefKey)

  eventmainclass:=objectname  

  case SapEvent.eventcode of
   "AUC","AU1":
     eventclass:="User"
   "AUO":
     eventmainclass:="Authorize"
     eventclass:="User"
   "AU5","AU6":
     eventmainclass:=Append("[",objectname,"]")
     eventclass:="User"
   "AU2":
     case SapEvent.reasoncode of
      "0","1","2","10","13":
         eventmainclass:="Authorize"
         eventclass:="User"
      "3","4","5","6","7","8","9","10","11","12":
         eventmainclass:="Authenticate"
         eventclass:="User"
     otherwise
         eventmainclass:="Authorize"
         eventclass:="User"
     endcase
  endcase

  case SapEvent.eventcode of
   "AUC","AU1","AU5":
     successclass:="Success"
   "AUO","AU2","AU6":
     successclass:="Failure"
  endcase

  workunit:=Append(Trim(SapEvent.sessionnr)," ",SapEvent.processid)
  program:=Trim(SapEvent.programname)
  logrecordtype:=SapEvent.eventcode

  if SapEvent.reason eq "EMPTY" then
    if SapEvent.usertype ne "EMPTY" then
      miscellaneous:=Append(miscellaneous,"; User type : ",SapEvent.usertype)
    endif
  else
    miscellaneous:=Append(miscellaneous
                          ,"; User type : ",SapEvent.usertype
                          ,", Reason : ",SapEvent.reason)
  endif
  event:=GetEvent(EventDetailKey)

  //
endstate 

final state RfcCall 

  objectname:=Trim(SapEvent.identifier,"Unavailable")
  objectpath:=Trim(SapEvent.functiongroup,"Unavailable")
  objecttype:="FUNCTION"
  object:=GetObject(ObjectRefKey)

  eventmainclass:="Call"
  eventclass:="Function"
  if (SapEvent.eventcode eq "AUK") then
    successclass:="Success"
  else  
    successclass:="Failure"
  endif

  workunit:=Append(Trim(SapEvent.sessionnr)," ",SapEvent.processid)
  program:=Trim(SapEvent.programname)

  logrecordtype:=SapEvent.eventcode
  event:=GetEvent(EventDetailKey)

  //
endstate 

final state Transaction 

  objectname:=Trim(SapEvent.transactioncode,"Unavailable")

  if (SapEvent.mod_fine eq ".") then
    objectpath:=SapEvent.mod_coarse
  else
    objectpath:=SapEvent.mod_fine
  endif
  objecttype:="TRANSACTION"
  object:=GetObject(ObjectRefKey)

  case SapEvent.eventcode of
   "AU3","AU4":
     eventmainclass:="Start"
   "AUQ":
     eventmainclass:="Unlock"
   "AUP":
     eventmainclass:="Lock"
  endcase
  eventclass:="Transaction"
  case SapEvent.eventcode of
   "AU4":
     successclass:="Failure"
   "AU3","AUP","AUQ":
    successclass:="Success"
  endcase

  workunit:=Append(Trim(SapEvent.sessionnr)," ",SapEvent.processid)
  program:=Trim(SapEvent.programname)
  if ( SapEvent.eventcode eq "AUQ" and
       Trim(SapEvent.identifier) ne ""
     ) then
     miscellaneous:=Append(miscellaneous
                          ,"; Transaction "
                          ,SapEvent.transactioncode
                          ," unlocked using "
                          ,Trim(SapEvent.identifier)
                          )
  else
    if (SapEvent.tcode_desc eq "*" ) then
      description := SapEvent.mod_desc
    else
      description := SapEvent.tcode_desc
    endif
    miscellaneous:=Append
        ( miscellaneous
        , ", "
        , description
        )
  endif
  logrecordtype:=SapEvent.eventcode
  event:=GetEvent(EventDetailKey)

  //
endstate 

final state Report 

  objectname:=SapEvent.identifier
  objectpath:="."
  objecttype:="REPORT"
  object:=GetObject(ObjectRefKey)

  eventmainclass:="Start"
  eventclass:="Report"
  case SapEvent.eventcode of
   "AUW":
      successclass:="Success"
   "AUX":
      successclass:="Failure"
  endcase

  workunit:=Append(Trim(SapEvent.sessionnr)," ",SapEvent.processid)
  program:=SapEvent.programname
  logrecordtype:=SapEvent.eventcode
  if SapEvent.eventcode eq "AUX" then
    miscellaneous:=Append(miscellaneous,"; ",Trim(SapEvent.reason))
  endif
  event:=GetEvent(EventDetailKey)

  if SapEvent.eventcode eq "AUX" then
    eventmainclass:="Authorize"
    eventclass:="User"
    successclass:="Failure"

    workunit:=Append(Trim(SapEvent.sessionnr)," ",SapEvent.processid)
    program:=SapEvent.programname
    logrecordtype:=SapEvent.eventcode
    miscellaneous:=Append(miscellaneous,"; ",SapEvent.reason)
    event:=GetEvent(EventDetailKey)

  endif

//
endstate 

final state UMC 

  case SapEvent.eventcode of
   "AU7","AU8","AU9","AUA","AUB","AUD","AUM","AUN":
     objectpath:="."
     objectname:=Trim(SapEvent.identifier,"Unavailable")
     objecttype:="USER"
   "AUR","AUS","AUT","AUU":
     if SapEvent.identtype eq "Profile" then
       objectpath:="."
       objectname:=SapEvent.identifier
       objecttype:="PROFILE"
     else
       objectpath:=SapEvent.identifierpath
       objectname:=SapEvent.identifier
       objecttype:="AUTHORIZATION"
     endif
   otherwise
     objectpath:="-"
     objectname:="Unavailable"
  endcase
  object:=GetObject(ObjectRefKey)

  case SapEvent.eventcode of
   "AU8","AUR":
     eventmainclass:="Delete"
   "AU9","AUM":
     eventmainclass:="Lock"
   "AUA","AUN":
     eventmainclass:="Unlock"
   "AU7","AUR":
     eventmainclass:="Create"
   "AUB","AUD","AUT":
     eventmainclass:="Modify"
   "AUU":
     eventmainclass:="Activate"
  endcase
  case SapEvent.eventcode of
    "AU7","AU8","AU9","AUA","AUB","AUD","AUM","AUN":
       eventclass:="User"
    "AUR","AUS","AUT","AUU":
       eventclass:=SapEvent.identtype

  endcase

  workunit:=Append(Trim(SapEvent.sessionnr)," ",SapEvent.processid)
  program:=SapEvent.programname
  case SapEvent.eventcode of
   "AUM":
     miscellaneous:=Append(miscellaneous
                          ,"; User Locked After Incorrect Logon")
   "AUN":
     miscellaneous:=Append(miscellaneous
                          ,"; User lock because of incorrect logon removed")
   "AUD":
     miscellaneous:=Append(miscellaneous
                          ,"; User Master Record of user '",objectname,"' Changed")
   "AUB":
     miscellaneous:=Append(miscellaneous
                          ,"; Authorizations for user '",objectname,"' Changed")
  endcase
  logrecordtype:=SapEvent.eventcode
  event:=GetEvent(EventDetailKey)

  //
endstate 

final state Application 

  objectname:=Trim(SapEvent.programname,"Unavailable")
  objectpath:="."
  objecttype:="PROGRAM"
  object:=GetObject(ObjectRefKey)

  if SapEvent.eventcode eq "AUG" then
    eventmainclass:="Start"
  else
    eventmainclass:="Stop"
  endif
  eventclass:="Process"

  workunit:=Append(Trim(SapEvent.sessionnr)," ",SapEvent.processid)
  program:=SapEvent.programname
  logrecordtype:=SapEvent.eventcode
  event:=GetEvent(EventDetailKey)

  //
endstate 

final state Download 

  objectname:=Trim(SapEvent.identifier, "Unavailable")
  objectpath:=Trim(SapEvent.terminal, ".")
  objecttype:="FILE"
  object:=GetObject(ObjectRefKey)

  eventmainclass:="Create"
  eventclass:="File"

  workunit:=Append(Trim(SapEvent.sessionnr)," ",SapEvent.processid)
  program:=SapEvent.programname
  logrecordtype:=SapEvent.eventcode
  miscellaneous:=Append(miscellaneous, "; ",
                       SapEvent.filetype, "-file '", objectname,
                       "', ", SapEvent.filesize, " bytes",
                       " created on ", objectpath)

  event:=GetEvent(EventDetailKey)

  //

endstate 

final state Signature 

  objectname:=Trim(SapEvent.identifier,"Unavailable")
  objectpath:="."
  objecttype:="SIGNATURE"
  object:=GetObject(ObjectRefKey)

  eventmainclass:="Authenticate"
  eventclass:="User"
  successclass:="Failure"

  workunit:=Append(Trim(SapEvent.sessionnr)," ",SapEvent.processid)
  program:=SapEvent.programname
  logrecordtype:=SapEvent.eventcode
  case SapEvent.reason of
   "PROFILE LOCKED":
     miscellaneous:=Append(miscellaneous,"; The SSF profile is locked")
   "INVALID PASSWORD":
     miscellaneous:=Append(miscellaneous,"; Incorrect password specified")
   "VERIFY FAILED":
     miscellaneous:=Append(miscellaneous,"; User signature could not be verified by the system")
  endcase
  event:=GetEvent(EventDetailKey)

  //
endstate 

final state Audit

  objectname:="Audit"
  objectpath:=sapsrv
  objecttype:="SYSTEM"
  object:=GetObject(ObjectRefKey)

  if SapEvent.action eq "1" then
    eventmainclass:="Start"
  else
    eventmainclass:="Stop"
  endif
  eventclass:="Audit"

  workunit:=Append(Trim(SapEvent.sessionnr)," ",SapEvent.processid)
  program:=""
  logrecordtype:=SapEvent.eventcode

  event:=GetEvent(EventDetailKey)

endstate 

final state AuditPolicy

  objectname:="Audit"
  objectpath:=sapsrv
  objecttype:="SYSTEM"
  object:=GetObject(ObjectRefKey)

  eventmainclass:="Modify"
  eventclass:="Auditpolicy"

  workunit:=Append(Trim(SapEvent.sessionnr)," ",SapEvent.processid)
  program:=""
  logrecordtype:=SapEvent.eventcode
  if SapEvent.aud_class eq "0" then
    miscellaneous:=Append
        ( miscellaneous,"; "
        , "Slot "  ,SapEvent.aud_slot
        ,": Weight ",SapEvent.aud_weight
        ,", User "  ,Trim(SapEvent.aud_user)
        ,", Client ",SapEvent.aud_client
        ,", Msg. "  ,Trim(SapEvent.aud_msg))
  else
    miscellaneous:=Append
        (miscellaneous,"; "
        ,  "Slot "  ,SapEvent.aud_slot
        ,": Class " ,SapEvent.aud_class
        ,", Weight ",SapEvent.aud_weight
        ,", User "  ,Trim(SapEvent.aud_user)
        ,", Client ",SapEvent.aud_client
        )
  endif

  event:=GetEvent(EventDetailKey)

endstate 

