/// snort.src.gml - Licensed Materials ­ Property of IBM
/// snort.src.gml - 5724-S67
/// snort.src.gml - © Copyright IBM Corp. 1998, 2007   All Rights Reserved.
/// snort.src.gml - US Government Users Restricted Right ­ Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

key PlatformRefKey(platformtype, platformname)
key ObjectRefKey(objecttype, objectpath, objectname)
key StreamRefKey(description)
key SourceRefKey(realname, logonname)
key OriginRefKey(platformname, platformtype)
key TargetRefKey(platformname, platformtype)
key EventDetailKey(classification, message, priority, interface)

first state Setup
  description := "Aggregator"
  equality    := "platform,oip,tip,op,tp,msg,proto"
  period      := "300000"
  threshold   := "0"
  aggregator  := GetListener(Aggregator)
  Listen(aggregator,Setup)
  MoveTo(ListenState)
endstate

state ListenState(((platformeventcount eq null) and (platformevent ne "repeatable")))
  Listen(aggregator,ListenState)

  Remember("ev", "platform", ListenState.platform)
  Remember("ev", "oip", ListenState.oip)
  Remember("ev", "tip", ListenState.tip)
  Remember("ev", "op", ListenState.op)
  Remember("ev", "tp", ListenState.tp)
  Remember("ev", "msg", ListenState.msg)
  Remember("ev", "proto", ListenState.proto)

endstate

last state Shutdown
  if aggregator ne null then 
    StopListening(aggregator)
  endif
endstate

initial final state Finish((
                                (platformeventcount ne null) and 
                                (platformevent eq "normal") and 
                                (description eq "Aggregator")
                           )) 
  ListenerId := Append(Finish.platform,Finish.oip,Finish.tip,Finish.op,Finish.tp,Finish.msg,Finish.proto)
  count := Recall("eventcounts", ListenerId)
  Forget("eventcounts", ListernerId)
  eventcount := Sum(Finish.platformeventcount, Trim(count, "0"))

  platformname:=Finish.platform
  platformtype:="Snort"
  platform    := GetPlatform(PlatformRefKey)

  objectname  := Trim(Finish.tpname,Trim(Finish.tp,"*"))
  objectpath  := Append(Finish.proto,":",Finish.tip)
  objecttype  := Finish.objecttype
  object      := GetObject(ObjectRefKey)
  message     := Finish.msg

  description := Append(Finish.platformeventtime, message) 
  stream      := GetStream(StreamRefKey)

  logonname   := "System"  
  realname    := logonname
  source      := GetSource(SourceRefKey)

  platformname := Finish.oip
  platformtype := "-"
  origin := GetOrigin(OriginRefKey)

  platformname := Finish.tip
  target := GetTarget(TargetRefKey)

  eventmainclass := Finish.emainclass
  eventclass     := Finish.eclass
  successclass   := Finish.success
  eventtimestamp := Finish.platformeventtime
  chunk          := Finish
  classification := Finish.classification
  priority       := Trim(Finish.priority,null)
  interface	 := Trim(Finish.interface,null)
  event := GetEvent(EventDetailKey)
endstate

initial state Repeatable(platformevent eq "repeatable")

  platform := Recall("ev", "platform")
  oip := Recall("ev", "oip")
  tip := Recall("ev", "tip")
  op := Recall("ev", "op")
  tp := Recall("ev", "tp")
  msg := Recall("ev", "msg")
  proto := Recall("ev", "proto")

  ListenerId := Append(platform,oip,tip,op,tp,msg,proto)
  count := Recall("eventcounts", ListenerId)
  count := Sum(Repeatable.eventcount, Trim(count, "0"))
  Remember("eventcounts", ListenerId, count)
endstate

