/// solaris.src.gml - Licensed Materials ­ Property of IBM
/// solaris.src.gml - 5724-S67
/// solaris.src.gml - © Copyright IBM Corp. 1998, 2007   All Rights Reserved.
/// solaris.src.gml - US Government Users Restricted Right ­ Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

key PlatformRefKey(platformname,platformtype)

key ObjectRefKey(objectname,objectpath,objecttype)

key SourceRefKey(logonname,realname)
key SourceAspectKey(originator)

key StreamRefKey(description)

key EventDetailKey(miscellaneous, logrecordtype, id, pid, EventName, EventClass, command, reason, oldvalue, newvalue, newname, value, Source, Target, pid, id, user, queue, modification, priority)

first state Setup
  description:="Aggregator"
  equality:= "command,name,eventclass,eventmainclass,objecttype,objectpath,objectname,success_expected,success_code,success_condition,username,origin,target,host"
  period:="1000"
  aggregator := GetListener("Aggregator")
  Listen(aggregator,Setup)
  MoveTo(Listening)
endstate

state Listening (platformeventcount eq null)
  Listen(aggregator,Listening)
endstate

last state Shutdown
  if aggregator ne null then
    StopListening(aggregator)
  endif
endstate

initial state Solaris (platformeventcount ne null)
  case Solaris.logtype of
    "utmp":
      MoveTo(Utmp)
    "trail":
      MoveTo(Trail)

  endcase
endstate

final state Utmp
  eventcount := Solaris.platformeventcount

  id := Solaris.id
  pid := Solaris.pid
  logrecordtype := Solaris.logrecordtype

  miscellaneous := Solaris.msgadd
  _platformtype := "Solaris"

  platformname := Trim(Uppercase(GetChunkProperty(Solaris,"platformname")),"Unavailable")
  platformtype := _platformtype
  platform := GetPlatform(PlatformRefKey)

  target:=GetTarget(PlatformRefKey)

  objecttype := Trim(Solaris.objecttype, "SYSTEM")
  objectname := Trim(Solaris.objectname, platformname)
  objectpath := "-"

  originator := Solaris.uid
  logonname := Trim(Solaris.username, "Anonymous")

  if (logrecordtype eq "DEAD_PROCESS") then 
    objectname := Trim(Forget("pid2objectname", id), "-")
    logonname := Trim(Forget("pid2logonname", id), "SYSTEM")
  else if (id ne null) then 
    Remember("pid2objectname", id, objectname)
    Remember("pid2logonname", id, logonname)
  endif endif

  realname := logonname

  object := GetObject(ObjectRefKey)
  source := GetSource(SourceRefKey, SourceAspectKey)

  entrypoint := Trim(Solaris.host, platformname)
  if (entrypoint eq "0.0.0.0" or entrypoint eq ":0") then
    entrypoint:=platformname
  else
    entrypoint := RegexReplace(entrypoint, ":\\d+$", "")
  endif
  terminal := Trim(Solaris.device, "-")
  description:=Append(logonname,"@",entrypoint,"/",terminal)
  stream := GetStream(StreamRefKey)

  if (Solaris.host ne null and Solaris.host ne "") then
    platformtype := "-"
  else 
    platformtype := _platformtype
  endif
  platformname:=Uppercase(entrypoint)
  origin:=GetOrigin(PlatformRefKey)

  eventmainclass := Solaris.eventmainclass
  eventclass := Solaris.eventclass
  successclass := "Success"  
  eventtimestamp := Solaris.platformeventtime

  chunk := Solaris
  GetEvent(EventDetailKey)
endstate

state Trail

  _platformname:= Uppercase(Trim(Solaris.host,GetChunkProperty(Solaris,"platformname")))
  _platformtype:= "Solaris"

  platformname := _platformname
  platformtype := _platformtype
  platform := GetPlatform(PlatformRefKey)

  if (Solaris.target ne null and Solaris.target ne "") then
    platformname := Uppercase(Solaris.target)
    platformtype := "-"
  else 
    platformname := _platformname
    platformtype := _platformtype
  endif
  target := GetTarget(PlatformRefKey)

  if (Solaris.origin ne null and Solaris.origin ne "") then
    platformname := Uppercase(Solaris.origin)
    platformtype := "-"
  else 
    platformname := _platformname
    platformtype := _platformtype
  endif
  origin := GetOrigin(PlatformRefKey)

  originator := Solaris.uid
  logonname := GetLogonName(Trim(Solaris.username, "Unavailable"))
  if (logonname = "-1" or
      logonname = "-2") then
    logonname:="nobody"
  endif
  realname := logonname
  source := GetSource(SourceRefKey, SourceAspectKey)

  description:=Append(logonname, "@", platformname)
  stream := GetStream(StreamRefKey)

  objecttype := Solaris.objecttype
  if (objecttype eq "SYSTEM") then
    objectname := _platformname
  else if (Solaris.eventclass eq "Su") then
    objectpath := _platformname
    objectname := Solaris.objectname
  else if (Solaris.objectpath ne null) then
    objectpath := Solaris.objectpath
    objectname := Solaris.objectname
  else if (Solaris.objectname ne null) then
    objectname := RegexReplace(Solaris.objectname, "/{2,}", "/") 
    objectpath := Path(objectname)
    objectpath := Trim(objectpath,"-")  
    objectname := Filename(objectname)
  endif endif endif endif

  objectpath := Trim(objectpath,".")
  objectname := Trim(objectname,"-") 

  if (Solaris.name eq "AUE_EXEC" or Solaris.name eq "AUE_EXECVE") then
    if (objectname eq "useradd") then
      objecttype := "USER"
      if (Solaris.exec_args eq "") then
        objectname := "Unavailable"
      else
        objectname := Token(Solaris.exec_args, " ", "1000")
      endif
      objectpath := "."
      eventmainclass := "Create"
      eventclass := "User"
    else if (objectname eq "usermod") then
      objecttype := "USER"
      if (Solaris.exec_args eq "") then
        objectname := "Unavailable"
      else
        objectname := Token(Solaris.exec_args, " ", "1000")
      endif
      objectpath := "."
      eventmainclass := "Modify"
      eventclass := "User"
    else if (objectname eq "userdel") then
      objecttype := "USER"
      if (Solaris.exec_args eq "") then
        objectname := "Unavailable"
      else
        objectname := Token(Solaris.exec_args, " ", "1000")
      endif
      objectpath := "."
      eventmainclass := "Delete"
      eventclass := "User"
    endif endif endif
  endif

  object:=GetObject(ObjectRefKey)

  case Solaris.success_condition of
    "==": 
      if (Solaris.success_code = Solaris.success_expected) then
        successclass := "Success"
      else
        successclass := "Failure"
      endif
    "!=":
      if (Solaris.success_code ne Solaris.success_expected) then
        successclass := "Success"
      else
        successclass := "Failure"
      endif
    ">=": 
      if (Solaris.success_code >= Solaris.success_expected) then
        successclass := "Success"
      else
        successclass := "Failure"
      endif
    "const":
      successclass := Solaris.success_expected

  endcase

  if (eventclass ne "User") then
    eventmainclass := Solaris.eventmainclass
    eventclass := Solaris.eventclass
  endif

  eventtimestamp:=Solaris.platformeventtime

  eventcount := Solaris.platformeventcount

  chunk := Solaris
  logrecordtype := Solaris.systemcall
  miscellaneous := Trim(RegexReplace(Solaris.msgadd, "invert\\((.)(.)(.)(.)(.)(.)(.)(.)(.)\\)", "\\9\\8\\7\\6\\5\\4\\3\\2\\1"), null)
  EventName := Solaris.name
  EventClass := Solaris.class
  reason := Trim(Solaris.reason, null) 
  oldvalue := Solaris.oldvalue
  newvalue := Trim(RegexReplace(Solaris.newvalue, "invert\\((.)(.)(.)(.)(.)(.)(.)(.)(.)\\)", "\\9\\8\\7\\6\\5\\4\\3\\2\\1"), null)
  newname := Trim(RegexReplace(Solaris.newname, "/{2,}", "/"), null) 
  value := Solaris.user
  Source := Solaris.source
  Target := Trim(RegexReplace(Solaris.Target, "/{2,}", "/"), null) 
  pid := Solaris.pid
  id := Solaris.id
  queue := Solaris.queue
  modification := Solaris.modification
  priority := Solaris.priority
  command := Solaris.exec_args
  GetEvent(EventDetailKey)
endstate

final state TrailFinish
endstate

