/// sun.src.gml - Licensed Materials ­ Property of IBM
/// sun.src.gml - 5724-S67
/// sun.src.gml - © Copyright IBM Corp. 1998, 2007   All Rights Reserved.
/// sun.src.gml - US Government Users Restricted Right ­ Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

key PlatformRefKey(platformname,platformtype)

key ObjectRefKey(objectname,objectpath,objecttype)

key SourceRefKey(logonname)
key SourceAspectKey(originator)

key StreamRefKey(description)

key EventDetailKey(miscellaneous,logrecordtype,comment,program,terminal)

initial state Solaris

  case Solaris.logtype of
    "utmp":
      MoveTo(Utmp)
    "trail":
      MoveTo(Finish)

  endcase
endstate

final state Finish

  platformname := Uppercase(Trim(Solaris.host,GetChunkProperty(Solaris,"platformname")))
  platformtype := "Solaris"
  platform := GetPlatform(PlatformRefKey)

  comment := Trim(Solaris.comment,null)
  if (Solaris.objectname ne null and Solaris.objectname ne "") then
    objectname := Solaris.objectname
    objectpath := Solaris.object
  else
    objectpath := Path(Solaris.object)
    objectname := Filename(Solaris.object)
  endif
  objectpath:=Trim(objectpath,"-")
  objectname:=Trim(objectname,".") 

  objecttype := Solaris.objecttype

  if objecttype eq "PROCESS" then
    case Solaris.emclass of
      "Start":
        program := Solaris.object
        if Solaris.execargs ne "" then
          command := Solaris.execargs
        else
          command:=Solaris.object
        endif
        object := GetObject(ObjectRefKey)
        Remember("pid2object", Solaris.pid, object)
      "Stop":
        object := Forget("pid2object", Solaris.pid)
        if object eq "" then
          objectpath := "Unavailable"
          objectname := "Process"
          object := GetObject(ObjectRefKey)
          comment := "Process started before current trail"
        endif
      otherwise
        object := Recall("pid2object", Solaris.pid)
        if object eq "" then
          objectpath := "Unavailable"
          objectname := Process
          object := GetObject(ObjectRefKey)
          comment := "Process started before current trail"
        endif
    endcase
  else
    if Solaris.action eq "create_user" then
      Remember("uid2name", Solaris.newuid, Solaris.objectname)
    endif
    if Solaris.action eq "user" then
      newname := Recall("uid2name", objectname)
      if newname ne "" then
        objectname := newname
      endif
    endif
    object:=GetObject(ObjectRefKey)
  endif

  originator := Solaris.auid
  logonname := Solaris.logonname
  if (logonname eq "-1" or
      logonname eq "-2") then
    logonname:="nobody"
  endif
  realname := logonname
  source := GetSource(SourceRefKey, SourceAspectKey)

  entrypoint := Filename(" ", Solaris.tid)
  if (entrypoint eq "0.0.0.0" or entrypoint eq ":0" or entrypoint eq "") then
    entrypoint:=platformname
  endif

  target:=GetTarget(PlatformRefKey)
  platformname:=Uppercase(entrypoint)
  origin:=GetOrigin(PlatformRefKey)

  description:=Append(entrypoint,":",logonname)
  stream := GetStream(StreamRefKey)

  if Solaris.error eq "success" then
    successclass := "Success"
  else
    successclass:="Failure"
  endif 

  eventmainclass := Solaris.emclass
  eventclass := Solaris.eclass

  eventtimestamp:=Solaris.platformeventtime

  chunk:=Solaris
  logrecordtype:=Solaris.eventtype
  miscellaneous := Solaris.msgadd

  GetEvent(EventDetailKey)
endstate

state Utmp
  platformname := Trim(Uppercase(GetChunkProperty(Solaris,"platformname")),"Unavailable")
  platformtype := "Solaris"

  platform := GetPlatform(PlatformRefKey)
  platformtype:="-" 
  case Solaris.logrectype of
    "DEAD_PROCESS":
      MoveTo(UtmpDeadProcess)
    "OLD_TIME":
      MoveTo(Time)
    otherwise
      MoveTo(UtmpGeneric)
  endcase
endstate

final state Time (logrectype eq "NEW_TIME")
  objectname := "System"
  objecttype := "SYSTEM"
  objectpath := Solaris.host
  object := GetObject(ObjectRefKey)

  subject := "System"
  subjectname := subject
  originator := subject
  logonname := subjectname
  realname := subjectname
  source := GetSource(SourceRefKey, SourceAspectKey)

  entrypoint := Solaris.host

  description:=Append(entrypoint,":",logonname)
  stream := GetStream(StreamRefKey)

  target:=GetTarget(PlatformRefKey)
  platformname:=Uppercase(entrypoint)
  origin:=GetOrigin(PlatformRefKey)

  eventmainclass := "Change"
  eventclass := "Time"
  successclass := "Success"
  eventtimestamp := Solaris.platformeventtime

  chunk := Solaris
  logrecordtype := Solaris.logrectype
  miscellaneous := Append("New time: ", Time.newtime)
  GetEvent(EventDetailKey)
endstate

final state UtmpGeneric

  objecttype := Solaris.objecttype
  objectname := Solaris.objectname
  objectpath := platformname
  object := GetObject(ObjectRefKey)

  subject := Solaris.subject
  if Solaris.logrectype eq "USER_PROCESS" then
    subjectname := Solaris.logonname
  else
    subjectname := subject
  endif
  if (subjectname eq "Unavailable" or
      subjectname eq ""
     )
  then 
    subjectname:="Anonymous"
  endif
  originator := subject
  logonname := subjectname
  realname := subjectname
  source := GetSource(SourceRefKey,SourceAspectKey)

  entrypoint := Solaris.host
  terminal := Solaris.entryadd

  description:=Append(logonname,"@",entrypoint,"/",terminal)
  stream := GetStream(StreamRefKey)

  target:=GetTarget(PlatformRefKey)
  platformname:=Uppercase(entrypoint)
  origin:=GetOrigin(PlatformRefKey)

  if (Solaris.pid <> "0") then
    Remember("pid2object", Solaris.pid, object)
    Remember("pid2source", Solaris.pid, source)
    Remember("pid2stream", Solaris.pid, stream)
    Remember("pid2subject", Solaris.pid, subject)
    Remember("pid2subjectname", Solaris.pid, subjectname)
  endif

  eventmainclass := Solaris.emclass
  eventclass := Solaris.eclass
  successclass := "Success"  
  eventtimestamp := Solaris.platformeventtime

  chunk := Solaris
  workunit := Solaris.pid
  command := Solaris.command
  program := Solaris.program

  if (Solaris.pid <> "0") then
    Remember("pid2command", Solaris.pid, command)
    Remember("pid2program", Solaris.pid, program)
  endif
  logrecordtype := Solaris.logrectype
  comment := Trim(Solaris.comment,null)
  miscellaneous := Trim(Solaris.msgadd,null)
  GetEvent(EventDetailKey)
endstate

final state UtmpDeadProcess
  subjectname := Forget("pid2subjectname", Solaris.pid)

  target:=GetTarget(PlatformRefKey)
  origin:=GetOrigin(PlatformRefKey)

  if subjectname eq "" then
    objecttype := "PROCESS"
    objectpath := "-"
    objectname := "Dead Process"
    object := GetObject(ObjectRefKey)
    logonname := Solaris.logonname
    realname := Solaris.logonname
    source := GetSource(SourceRefKey,SourceAspectKey)
    entrypoint := "Unavailable"
    description := "Unavailable"
    stream := GetStream(StreamRefKey)
  else
    object := Forget("pid2object", Solaris.pid)
    source := Forget("pid2source", Solaris.pid)
    stream := Forget("pid2stream", Solaris.pid)
  endif

  eventmainclass := "Stop"
  eventclass := "Process"
  successclass := "Success"
  eventtimestamp := Solaris.platformeventtime
  chunk := Solaris
  workunit := Solaris.pid

  command := Forget("pid2command", Solaris.pid)
  if command eq null then
    command := " "
    program := " "
  else
    program := Forget("pid2program", Solaris.pid)
  endif
  logrecordtype := Solaris.logrectype
  GetEvent(EventDetailKey)
endstate
