/// zOS.src.gml - Licensed Materials ­ Property of IBM
/// zOS.src.gml - 5724-S67
/// zOS.src.gml - © Copyright IBM Corp. 1998, 2007   All Rights Reserved.
/// zOS.src.gml - US Government Users Restricted Right ­ Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

//

//

//

//

//

key PlatformRefKey(platformtype, platformname)
key ObjectRefKey(objecttype, objectpath, objectname)
key StreamRefKey(description)
key SourceRefKey(realname, logonname)
key SourceAspectKey(originator)
key OriginRefKey(platformtype, platformname)
key TargetRefKey(platformtype, platformname)
key EventDetailKey(logrecordtype, program, command, workunit, miscellaneous, recorddesc, reason)
key ObjectDetailKey(sysplex,box_serial,origtype,profile,volser_or_sms)

first state Setup
  strategy    := "list"
  targets     := "logrecordtype"
  separator   := ";"
  equality    := "eventmainclass,eventclass,successclass,objecttype,objectpath,objectname,streamid"
  description := "DatasetRename"
  period      := "200"         
  glue_rename := GetListener("Aggregator")
  MoveTo(ListenState)
  description := "OpenCloseVSAM"
  period      := "10"
  glue_vsam   := GetListener("Aggregator")
  MoveTo(ListenState)
  description := "VSAMcluster"
  period      := "100"
  glue_cluster:= GetListener("Aggregator")
  MoveTo(ListenState)
endstate

last state Shutdown
  if glue_rename ne null then StopListening(glue_rename) endif
  if glue_vsam ne null then StopListening(glue_vsam) endif
  if glue_cluster ne null then StopListening(glue_cluster) endif
endstate

state ListenState ((glue ne null and glue ne "" and platformeventcount eq null))
  case ListenState.glue of
  "rename" : Listen(glue_rename,ListenState)
  "vsamOC" : Listen(glue_vsam,ListenState)
  "cluster": Listen(glue_cluster,ListenState)
  endcase
endstate

initial final state Zos ((glue eq null or glue eq "" or platformeventcount ne null))

  if Zos.remember_telnet_session eq "yes" then
    Remember("terminal2ip", Zos.terminal, Zos.client_ip)
  endif

  platformname := Trim(Zos.epiname, "z/OS System")      
  platformtype := "z/OS"
  target   := GetTarget(TargetRefKey)              

  if RegexMatch(Zos.logrecordtype,"SMF,102") <> "0" then    
    Append(platformtype," DB2")
    Append(platformname,Append(" ",Zos.objectpath))
  else 
    if Trim(Zos.platformtype) ne "" then

      platformtype := Append(platformtype,Append(" ",Zos.platformtype))
      platformname :=Append(platformname,Append(" ",Zos.objectpath))

    endif
  endif 
  platform := GetPlatform(PlatformRefKey)

  platformname := Zos.epiname                  
  platformtype := "z/OS"
  if Trim(Zos.source_ip) ne "" then
    platformtype := "host"
    platformname := Zos.source_ip
  else 
    if Trim(Zos.source_is_lu) eq "LU" then
      platformtype := Zos.source_is_lu
      if Trim(Zos.utoken_poe_network) ne "" then
        platformname := Append(Zos.utoken_poe_network,Append(".",Zos.utoken_poe))
      else 
        platformname := Append(Zos.network,Append(".",Zos.utoken_poe))
      endif 
    else 
      if Trim(Zos.terminal) ne "" then
        platformtype := "LU"
        platformname := Append(Zos.network,Append(".",Zos.terminal))
      endif
    endif
  endif
  origin   := GetOrigin(OriginRefKey)

  if Trim(Zos.objectname) eq "" then               
    objectname := Trim(Zos.volser_or_sms,"-")
    objectpath := "-"
    objecttype := Trim(Zos.objecttype, "VOLUME")
  else
    objectname := Trim(Zos.objectname,"-")
    objectpath := Trim(Zos.objectpath,"-")
    objecttype := Trim(Zos.objecttype, "DATASET")
  endif
  if Trim(Zos.unix_pathname) ne "" then
    objectpath := Trim(Path("/",Zos.unix_pathname),"-")  
    objectname := Trim(Filename("/",Zos.unix_pathname),"-")
  endif 
  if Trim(Zos.rename_private) ne "" then
    if Trim(Zos.rename_private) eq "Y" then
      objecttype := Append(objecttype,"-P")
    endif
  else
    if Trim(Zos.priv_dataset) ne "" then
      objecttype := Append(objecttype,"-P")
    endif
  endif

  if Trim(Zos.box_serial) ne "" then
    box_serial := Zos.box_serial
  endif
  if Trim(Zos.sysplex) ne "" then
    sysplex    := Zos.sysplex  
  endif   
  if Trim(Zos.profile) ne "" then
    profile    := Zos.profile
  endif
  if Trim(Zos.origtype) ne "" then
    origtype   := Zos.origtype
  endif
  if Trim(Zos.volser_or_sms) ne "" then
    volser_or_sms := Zos.volser_or_sms
  endif
  object := GetObject(ObjectRefKey,ObjectDetailKey)

  logonname  := Zos.logonname
  realname   := Trim(Zos.realname,Zos.logonname)    
  originator := logonname                           
  source     := GetSource(SourceRefKey,SourceAspectKey)

  description := Zos.streamid
  if Trim(Zos.subsys_type) ne "" then
    description := Append(description,Append(" ",RegexReplace(Zos.logonname,".*\\.(.*)",$1)))
  endif 
  stream := GetStream(StreamRefKey)

  if Zos.recall_telnet_session eq "yes" then
    miscellaneous:=Recall("terminal2ip",Zos.terminal)
  else
    if Trim(Zos.misc) ne "" then
      miscellaneous:=Zos.misc
    endif
  endif
  logrecordtype := Zos.logrecordtype
  if Trim(Zos.unixprogram) ne "" then
      program := Zos.unixprogram
  else
    if Trim(Zos.program) ne "" then
      program := Zos.program
    endif
  endif
  if Trim(Zos.command) ne "" then
    command := Zos.command
  endif
  if Trim(Zos.workunit) ne "" then
    workunit := Zos.workunit
  endif
  if RegexMatch(Zos.recorddesc,"(Unrecognized SMF type)") eq "0" then
    recorddesc := Zos.recorddesc
  endif
  if Trim(Zos.reason) ne "" then
    reason := Zos.reason
  endif

  eventmainclass := NormalizeString(Zos.eventmainclass)
  eventclass     := Zos.eventclass
  if Trim(Zos.dbsuccessclass) ne "" then
    successclass := Zos.dbsuccessclass
  else
    successclass   := Zos.successclass
  endif
  if (Trim(Zos.s_auth) ne "" and successclass ne "Failure" )then  
    successclass := Zos.s_auth      
  else                                                            
    if Trim(Zos.u_auth) ne "" then  
      successclass := "UserPriv"
    else
      if Trim(Zos.p_auth) ne "" then
        successclass := "ProfAttr"
      endif
    endif
  endif
  chunk          := "Zos"             
  eventtimestamp := Zos.platformeventtime
  eventcount     := Trim(Zos.platformeventcount,"1")
  event          := GetEvent(EventDetailKey)

  if Zos.forget_telnet_session eq "yes" then
    Forget("terminal2ip", Zos.terminal)
  else
    if Zos.forget_all_telnet_session eq "yes" then
      ForgetAll("terminal2ip")
    endif
  endif	  

endstate
